/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * bernimmt die Kommunikation mit dem DaV und enthlt Funktionen zum Abfragen
 * der im DaV hinterlegten Berechtigungen.
 * 
 * Diese Klasse implementiert das Singleton-Entwurfsmuster.
 * 
 * @version $Revision: 1.3 $ / $Date: 2010/02/04 06:12:25 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public final class SystemObjekteSystem {

	/**
	 * Liefert die einzigste Instanz der Klasse {@link SystemObjekteSystem}.
	 */
	private static SystemObjekteSystem instanz;

	/**
	 * Verbindung zum DaV.
	 */
	private static ClientDavInterface connection;

	/**
	 * Die Liste mit allen Systemobjekten.
	 */
	private final List<SystemObject> systemObjekte = new ArrayList<SystemObject>();

	/**
	 * Die Liste mit den dazugehrigen Pid's.
	 */
	private final List<String> systemObjektePid = new ArrayList<String>();

	/**
	 * Liste der {@link IOberflaechenBerechtigungZuhoerer}, die ber die
	 * Berechtigung informiert werden mchten.
	 */
	private final LinkedList<ISystemObjekteZuhoerer> systemObjekteZuhoererListe = new LinkedList<ISystemObjekteZuhoerer>();

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link SystemObjekteSystem}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link SystemObjekteSystem}.
	 */
	public static SystemObjekteSystem getInstanz() {
		if (SystemObjekteSystem.instanz == null) {
			SystemObjekteSystem.instanz = new SystemObjekteSystem();
		}

		return SystemObjekteSystem.instanz;
	}

	/**
	 * Privater Konstruktor des Singletons.
	 * 
	 */
	private SystemObjekteSystem() {
		SystemObjekteSystem.connection = RahmenwerkService.getService()
				.getRahmenWerk().getDavVerbindung();

		/*
		 * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern
		 * gehrt werden.
		 */
		final DavVerbindungsListener zuhoerer = new DavVerbindungsListener() {

			@Override
			public void verbindungHergestellt(final DavVerbindungsEvent event) {
				SystemObjekteSystem.connection = event.getDavVerbindung();
			}

			@Override
			public boolean verbindungHalten(final DavVerbindungsEvent event) {
				return false;
			}

			@Override
			public void verbindungGetrennt(final DavVerbindungsEvent event) {
				SystemObjekteSystem.connection = null;
			}
		};

		RahmenwerkService.getService().getRahmenWerk()
				.addDavVerbindungsListener(zuhoerer);

		holeSystemObjekte();

	}

	/**
	 * Fgt einen Listener zur Liste der SystemObjekteListener hinzu.
	 * 
	 * @param listener
	 */
	public void hinzufuegenSystemObjekteZuhoerer(
			final ISystemObjekteZuhoerer listener) {
		if (!systemObjekteZuhoererListe.contains(listener)) {
			systemObjekteZuhoererListe.add(listener);
		}

	}

	/**
	 * Entfernt einen {@link ISystemObjekteZuhoerer} von der Liste der Zuhrer.
	 * 
	 * @param systemObjekteZuhoerer
	 *            Zu entfernender {@link ISystemObjekteZuhoerer}.
	 */
	public void entferneSystemObjekteZuhoerer(
			final ISystemObjekteZuhoerer listener) {
		if (systemObjekteZuhoererListe.contains(listener)) {
			systemObjekteZuhoererListe.remove(listener);
		}

	}

	/**
	 * Benachrichtigt alle registrierten {@link ISystemObjekteZuhoerer} darber,
	 * dass ein neuer Benutzer sich angemeldet hat.
	 * 
	 */
	public void benachrichtigeSystemObjekteZuhoerer(
			final SystemObjekteEreignis ereignis) {

		final Iterator<ISystemObjekteZuhoerer> it = systemObjekteZuhoererListe
				.iterator();
		while (it.hasNext()) {
			final ISystemObjekteZuhoerer listener = it.next();

			if (ereignis.getAenderung()) {
				listener.aenderung(ereignis);
			}

		}
	}

	/**
	 * Holt das Verbindungsobjekt.
	 * 
	 * @return
	 */
	public ClientDavInterface getConnection() {
		return SystemObjekteSystem.connection;
	}

	/**
	 * Holt alle SystemObjekte aus der Konfiguration und fgt sie der Liste der
	 * SystemObjekte hinzu. Zustzlich wird berwacht, ob dynamische Objekte der
	 * Konfiguration hinzugefgt bzw. aus der Konfiguration entfernt werden. Ist
	 * dies der Fall, werden alle Zuhoerer ber die neuen bzw. veralteten
	 * Objekte informiert.
	 */
	private void holeSystemObjekte() {

		if (SystemObjekteSystem.connection != null) {
			final DataModel dataModel = SystemObjekteSystem.connection
					.getDataModel();

			try {
				final SystemObjectType type = dataModel
						.getType("typ.konfigurationsBereich");
				// Liste aller Systemobjekte
				final List<SystemObject> objects = type.getObjects();

				for (final SystemObject so1 : objects) {
					final ConfigurationArea element = (ConfigurationArea) so1;
					final Collection<SystemObject> currentObjects = element
							.getCurrentObjects();

					for (final SystemObject so2 : currentObjects) {
						systemObjekte.add(so2);
						systemObjektePid.add(so2.getPid());
					}
				}

				final List<SystemObjectType> baseTypes = SystemObjekteSystem.connection
						.getDataModel().getBaseTypes();
				for (final SystemObjectType sot : baseTypes) {

					if (sot.isOfType("typ.dynamischesObjekt")) {

						final DynamicObjectType dot = (DynamicObjectType) sot;

						dot.addObjectCreationListener(new DynamicObjectType.DynamicObjectCreatedListener() {

							@Override
							public void objectCreated(
									final DynamicObject createdObject) {
								systemObjekte.add(createdObject);
								systemObjektePid.add(createdObject.getPid());

								final SystemObjekteEreignis ereignins = new SystemObjekteEreignis(
										this, true, createdObject, null);

								benachrichtigeSystemObjekteZuhoerer(ereignins);
							}
						});

						dot.addInvalidationListener(new InvalidationListener() {

							@Override
							public void invalidObject(
									final DynamicObject dynamicObject) {
								systemObjekte.remove(dynamicObject);
								systemObjektePid.remove(dynamicObject.getPid());

								final SystemObjekteEreignis ereignins = new SystemObjekteEreignis(
										this, false, null, dynamicObject);

								benachrichtigeSystemObjekteZuhoerer(ereignins);

							}
						});
					}
				}
			} catch (final RuntimeException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Holt die Liste der SystemObjekte.
	 * 
	 * @return Liste mit SystemObjekten
	 */
	public List<SystemObject> getSystemObjekte() {
		return systemObjekte;
	}

	/**
	 * Holt die Liste der Pids der SystemObjekte.
	 * 
	 * @return Liste mit Pids
	 */
	public List<String> getSystemObjektePid() {
		return systemObjektePid;
	}

	public static void setConnection(final ClientDavInterface connection) {
		SystemObjekteSystem.connection = connection;
	}

}
