/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse speichert Einstellungen als Parameter innerhalb einer
 * XML-Struktur. <br>
 * {@link EinstellungenSpeicher}
 * <table>
 * <th>Methode:</th>
 * <th>Aktion:</th>
 * <tr>
 * <td>{@link EinstellungenSpeicher#hinzufuegenParameter(String, String)}</td>
 * <td>Hinzufgen eines Parameters zu den Einstellungen.</td>
 * </tr>
 * <tr>
 * <td>{@link EinstellungenSpeicher#editiereParameter(String, String)}</td>
 * <td>Editieren eines vorhandenen Parameters.</td>
 * </tr>
 * <tr>
 * <td>{@link EinstellungenSpeicher#ueberschreibeParameter(String, String)}</td>
 * <td>berschreiben eines vorhandenen Parameters, legt diesen neu an sofern
 * noch nicht vorhanden.</td>
 * </tr>
 * <tr>
 * <td>{@link EinstellungenSpeicher#loescheParameter(String)}</td>
 * <td>Lscht einen Parameter.</td>
 * </tr>
 * <tr>
 * <td>{@link EinstellungenSpeicher#getParameter(String)}</td>
 * <td>Liefert einen bestimmten Parameter.</td>
 * </tr>
 * <tr>
 * <td>{@link EinstellungenSpeicher#getAlleParameter()}</td>
 * <td>Liefert eine {@link HashMap} mit allen vorhandenen Parametern.</td>
 * </tr>
 * </table>
 * <br>
 * <b>Achtung:</b> Ein Parameter darf maximal <i>32000</i> Zeichen lang sein.
 * Wurde ein Parameter editiert, ist der {@link EinstellungenSpeicher} durch die
 * entsprechenden Methoden der Klasse {@link EinstellungenWaehler} zu sichern. <br>
 * <br>
 * Durch Synchronisation sind die ffentlichen Methoden dieser Klasse
 * Threadsicher!
 * 
 * @author Dambach Werke GmbH
 * 
 */
public class EinstellungenSpeicher {

	private static final Debug LOGGER = Debug.getLogger();
	private final SpeicherKey key;

	EinstellungenSpeicher(final SpeicherKey key) {
		this.key = key;
	}

	/**
	 * Fgt einen Parameter zu den Voereinstellungen hinzu, sofern dieser dort
	 * noch nicht vorhanden ist.
	 * 
	 * @param id
	 *            Id des Parameters
	 * @param wert
	 *            Wert des Parameters, maximal 2 * 1024 ^ 2Zeichen.
	 * @return Liefert <code>true</code>, wenn der Parameter gesetzt werden
	 *         konnte und <code>false</code>, falls dieser schon vorhanden war
	 *         oder Aufgrund der Lnge des Wertes (max 2 * 1024 ^ 2 Zeichen)
	 *         nicht gesetzt werden konnte.
	 */
	public boolean hinzufuegenParameter(final String id, final String wert) {
		final Einstellungen einstellungen = RahmenwerkService.getService()
				.getEinstellungen();

		try {
			final EinstellungsAdresse adresse = new EinstellungsAdresse(null,
					id, key.getOwnertype(), key.getPid(), key.getLocation());
			einstellungen.setValue(adresse, wert);
			return true;
		} catch (final IOException e) {
			EinstellungenSpeicher.LOGGER.error(e.getLocalizedMessage());
		}

		return false;
	}

	/**
	 * Editiert einen vorhandenen Parameter.
	 * 
	 * @param id
	 *            Id des Parameters.
	 * @param wert
	 *            Wert des Parameters, maximal 2 * 1024 ^ 2 Zeichen.
	 * @return Liefert <code>true</code>, wenn der Parameter editiert werden
	 *         konnte, sonnst <code>false</code>.
	 */
	public boolean editiereParameter(final String id, final String wert) {
		return hinzufuegenParameter(id, wert);
	}

	/**
	 * berschreibt einen Parameter. Existiert dieser noch nicht, wird er
	 * angelegt.
	 * 
	 * @param id
	 *            Id des Parameters
	 * @param wert
	 *            Wert des Parameters maximal 2 * 1024 ^ 2 Zeichen.
	 */
	public boolean ueberschreibeParameter(final String id, final String wert) {
		return hinzufuegenParameter(id, wert);
	}

	/**
	 * Lscht einen Parameter.
	 * 
	 * @param id
	 *            Id des Parameters, der gelscht werden soll.
	 * @return Liefert <code>true</code>, wenn der Parameter gelscht werden
	 *         konnte, sonst <code>false</code>.
	 */
	public boolean loescheParameter(final String id) {
		return hinzufuegenParameter(id, null);
	}

	/**
	 * Liefert den Wert eines Parameters.
	 * 
	 * @param id
	 *            Id des Parameters.
	 * @return Wert des Parameters falls dieser existiert, sonst
	 *         <code>null</code> .
	 */
	public synchronized String getParameter(final String id) {

		final Einstellungen einstellungen = RahmenwerkService.getService()
				.getEinstellungen();
		String result = null;

		try {
			final EinstellungsAdresse adresse = new EinstellungsAdresse(null,
					id, key.getOwnertype(), key.getPid(), key.getLocation());
			final Object resultObject = einstellungen.getValue(adresse);
			if (resultObject instanceof String) {
				result = (String) resultObject;
			} else if (resultObject != null) {
				result = resultObject.toString();
			}
		} catch (final IOException e) {
			EinstellungenSpeicher.LOGGER.error(e.getLocalizedMessage());
		}

		return result;
	}

	public Map<EinstellungsAdresse, Einstellung> getAlleEinstellungen() {
		// TODO Auto-generated method stub
		return null;
	}
}
