/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.util.RahmenwerkUtil;
import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse liefert den Zugriff auf die Einstellungen fr:<br>
 * <ul>
 * <li>Benutzer Lokal</li>
 * <li>Benutzer Netzweit</li>
 * <li>Allgemein Lokal</li>
 * <li>Allgemein Netzweit</li>
 * </ul>
 * <br>
 * Der Zugriff wird als Instanz der Klasse {@link EinstellungenWaehler}
 * geliefert. Wurden Parameter verndert <b>muss</b> darauf geachtet werden, mit
 * den Methoden {@link EinstellungenWaehler#sichereSpeicherAllgemeinLokal()},
 * {@link EinstellungenWaehler#sichereSpeicherAllgemeinNetzweit()},
 * {@link EinstellungenWaehler#sichereSpeicherBenutzerLokal()} bzw.
 * {@link EinstellungenWaehler#sichereSpeicherBenutzerNetzweit()} die jeweiligen
 * {@link EinstellungenSpeicher} zu sichern.
 * 
 * @deprecated auf die Einstellungen sollte ber die Schnittstelle
 *             {@link Einstellungen} zugegriffen werden
 * 
 * @author Dambach Werke GmbH
 * 
 */
@Deprecated
public final class EinstellungenWaehler {
	public enum EinstellungsArt {
		/**
		 * benutzerspezifische, lokale Einstellungen.
		 */
		benutzerLokal,
		/**
		 * benutzerspezifische, netzweite Einstellungen.
		 */
		benutzerNetzweit,
		/**
		 * allgemeine, lokale Einstellungen.
		 */
		allgemeinLokal,
		/**
		 * allgemeine, netzweite Einstellungen.
		 */
		allgemeinNetzweit
	}

	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Speichert den Namen des letzten Benutzers des
	 * {@link EinstellungenSpeicher} . Damit lsst sich dieser fr den Benutzer
	 * sichern, wenn sich der Benutzer abgemeldet hat.
	 */
	private String benutzername;

	private boolean sendenOhneUrlasserDialog;

	/** Privater Konstruktor. */
	private EinstellungenWaehler() {
	}

	public static EinstellungenWaehler getInstanz() {
		return Innen.INSTANZ;
	}

	private static class Innen {
		private static final EinstellungenWaehler INSTANZ = new EinstellungenWaehler();
	}

	/**
	 * Liefert den einen {@link EinstellungenSpeicher} nach Wahl ber den
	 * Enumerator {@link EinstellungsArt}. In der Regel sollte mit dieser
	 * Methode ein {@link EinstellungenSpeicher} gewhlt werden, auf dem
	 * Einstellungen vorgenommen werden knnen (hinzufgen, bearbeiten, lschen
	 * und auslesen).<br>
	 * <br>
	 * <b>Achtung:</b> Es ist darauf zu Achten, dass nderungen jeweils
	 * gelieferten {@link EinstellungenSpeicher} auch gesichert werden
	 * <i>mssen</i>. Die entsprechenden Methoden befinden sich weiter unten in
	 * dieser Klasse.
	 * 
	 * @param einstellungArt
	 *            Gibt an, welcher {@link EinstellungenSpeicher} geliefert
	 *            werden soll.
	 * @return Liefert den {@link EinstellungenSpeicher} oder <code>null</code>,
	 *         wenn der Einstellungenspeicher nicht bezogen werden konnte. Dies
	 *         kann bei netzweiten allgemeinen oder benutzerspezifischen
	 *         Einstellungen der Fall sein, wenn keine Verbindung zum DaV
	 *         vorhanden ist.
	 */
	public EinstellungenSpeicher getEinstellungen(
			final EinstellungsArt einstellungArt) {

		final Rahmenwerk rahmenWerk = RahmenwerkService.getService()
				.getRahmenWerk();

		if (benutzername == null) {
			benutzername = rahmenWerk.getBenutzerName();
		}

		final SystemObject benutzer = RahmenwerkUtil.getBenutzer(benutzername);

		SpeicherKey key = null;

		switch (einstellungArt) {
		case allgemeinLokal:
			key = new SpeicherKey(EinstellungOwnerType.SYSTEM,
					EinstellungLocation.LOKAL);
			break;
		case allgemeinNetzweit:
			if (rahmenWerk.isOnline()) {
				key = new SpeicherKey(EinstellungOwnerType.SYSTEM,
						EinstellungLocation.NETZWERKWEIT);
			}
			break;
		case benutzerLokal:
			if (benutzer != null) {
				key = new SpeicherKey(EinstellungOwnerType.BENUTZER,
						EinstellungLocation.LOKAL, benutzer.getPid());
			}
			break;
		case benutzerNetzweit:
			if (benutzer != null) {
				key = new SpeicherKey(EinstellungOwnerType.BENUTZER,
						EinstellungLocation.NETZWERKWEIT, benutzer.getPid());
			}
			break;
		default:
			break;
		}

		if (key == null) {
			return null;
		}

		return new EinstellungenSpeicher(key);
	}

	/**
	 * Sichert die Speicher fr Benutzer lokal und Benutzer netzweit, wenn das
	 * Rahmenwerk beendet wird. Die Speicher fr allgemeine netzweite und
	 * allgemein Lokal <b>mssen</b> manuell durch den Plug-In Entwickler
	 * gesichert werden.
	 * 
	 * @deprecated die Einstellungen werden beim Setzen unmittelbar gespeichert,
	 *             die Funktion entfllt daher
	 * 
	 */
	@Deprecated
	public void sichereSpeicher() {
		EinstellungenWaehler.LOGGER
				.warning("Unntige Speicherung der Einstellungen!");
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} fr allgemeine Einstellungen
	 * lokal.
	 * 
	 * @return Liefert <code>true</code>, wenn erfolgreich gespeichert werden
	 *         konnte.
	 * @deprecated die Einstellungen werden beim Setzen unmittelbar gespeichert,
	 *             die Funktion entfllt daher
	 */
	@Deprecated
	public boolean sichereSpeicherAllgemeinLokal() {
		EinstellungenWaehler.LOGGER
				.warning("Unntige Speicherung der Einstellungen!");
		return true;
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} fr allgemeine Einstellungen
	 * netzweit.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn die Einstellungen
	 *         erfolgreich gespeichert werden konnten, sonst <code>false</code>.
	 * @deprecated die Einstellungen werden beim Setzen unmittelbar gespeichert,
	 *             die Funktion entfllt daher
	 */
	@Deprecated
	public boolean sichereSpeicherAllgemeinNetzweit() {
		EinstellungenWaehler.LOGGER
				.warning("Unntige Speicherung der Einstellungen!");
		return true;
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} lokal fr den aktuell
	 * angemeldeten BenutzerBenutzer.
	 * 
	 * @deprecated die Einstellungen werden beim Setzen unmittelbar gespeichert,
	 *             die Funktion entfllt daher
	 */
	@Deprecated
	public void sichereSpeicherBenutzerLokal() {
		EinstellungenWaehler.LOGGER
				.warning("Unntige Speicherung der Einstellungen!");
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} netzweit fr den Benutzer.
	 * 
	 * @return Liefert <code>true</code>, wenn die Einstellungen fr den
	 *         Benutzer gesichert werden konnten, sonnst <code>false</code>.
	 * @deprecated die Einstellungen werden beim Setzen unmittelbar gespeichert,
	 *             die Funktion entfllt daher
	 */
	@Deprecated
	public boolean sichereSpeicherBenutzerNetzweit() {
		EinstellungenWaehler.LOGGER
				.warning("Unntige Speicherung der Einstellungen!");
		return true;
	}

	/**
	 * Diese Methode liefert die Einstellungen fr einen bestimmten Benutzer in
	 * Form eines {@link EinstellungenSpeicher} zurck.<br>
	 * <b>Achtung:</b> Diese Funktion sollten nur fr administrative Aufgaben
	 * verwendet werden, wenn der betreffende Benutzer nicht angemeldet ist.
	 * I.d.R. ist die Methode
	 * {@link EinstellungenWaehler#getEinstellungen(de.bsvrz.buv.rw.basislib.einstellungen.einstellungen.EinstellungenWaehler.EinstellungsArt)}
	 * zu bevorzugen.
	 * 
	 * @param benutzerName
	 *            Name des Benutzers
	 * @return Liefert den {@link EinstellungenSpeicher} fr den Benutzer
	 *         zurck. Existieren fr diesen Benutzer keine Einstellungen auf
	 *         dem DaV, wird ein leerer {@link EinstellungenSpeicher}
	 *         zurckgeliefert. Existiert keine Verbindung zum DaV, wird
	 *         <code>null</code> zurckgeliefert.
	 */
	public EinstellungenSpeicher getEinstellungenSpeicherBenzutzerBestimmtNetzweit(
			final String benutzerName) {

		final SystemObject benutzer = RahmenwerkUtil.getBenutzer(benutzerName);
		if (benutzer != null) {
			return new EinstellungenSpeicher(new SpeicherKey(
					EinstellungOwnerType.BENUTZER,
					EinstellungLocation.NETZWERKWEIT, benutzer.getPid()));
		}

		return null;
	}

	/**
	 * Liefert den EinstellungenSpeicher fr einen bestimmten, lokalen Benutzer.
	 * Existiert der Benutzer oder dessen {@link EinstellungenSpeicher} noch
	 * nicht, so wird ein leerer {@link EinstellungenSpeicher} zurckgeliefert.<br>
	 * 
	 * <b>Achtung:</b> Diese Funktion sollten nur fr administrative Aufgaben
	 * verwendet werden, wenn der betreffende Benutzer nicht angemeldet ist.
	 * I.d.R. ist die Methode
	 * {@link EinstellungenWaehler#getEinstellungen(de.bsvrz.buv.rw.basislib.einstellungen.einstellungen.EinstellungenWaehler.EinstellungsArt)}
	 * zu bevorzugen.
	 * 
	 * @param benutzerName
	 *            Name des Benutzers, dessen {@link EinstellungenSpeicher}
	 *            geliefert werden soll.
	 * @return Den {@link EinstellungenSpeicher} fr den Benutzer. Dieser kann
	 *         leer sein, wenn noch kein {@link EinstellungenSpeicher} fr den
	 *         Benutzer existent war.
	 */
	public EinstellungenSpeicher getEinstellungenSpeicherBenutzerBestimmtLokal(
			final String benutzerName) {

		final SystemObject benutzer = RahmenwerkUtil.getBenutzer(benutzerName);
		if (benutzer != null) {
			return new EinstellungenSpeicher(new SpeicherKey(
					EinstellungOwnerType.BENUTZER, EinstellungLocation.LOKAL,
					benutzer.getPid()));
		}

		return null;
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} netzweit fr einen bestimmten
	 * Benutzer.<br>
	 * <b>Achtung:</b> Diese Funktion sollten nur fr administrative Aufgaben
	 * verwendet werden, wenn der betreffende Benutzer nicht angemeldet ist.
	 * I.d.R. sind die Methoden {@link EinstellungenWaehler#sichereSpeicher()}
	 * bzw. {@link EinstellungenWaehler#sichereSpeicherBenutzerNetzweit()} zu
	 * bevorzugen.
	 * 
	 * @param benutzerName
	 *            Name des Benutzers, fr den der {@link EinstellungenSpeicher}
	 *            gesichert werden soll.<br>
	 *            <b>Achtung:</b> Dieser Benutzer muss existieren, da sonst die
	 *            Einstellungen nicht gespeichert werden knnen!
	 * @param speicher
	 *            Der {@link EinstellungenSpeicher} des Benutzers, der gesichert
	 *            werden soll. Darf nicht <code>null</code> sein.
	 * @return Liefert <code>true</code>, wenn der {@link EinstellungenSpeicher}
	 *         eines Benutzers erfolgreich gesichert werden konnte. Liefert
	 *         <code>false</code>, wenn keine Verbindung zum DaV existiert oder
	 *         der angegebene Benutzer nicht existiert.
	 * @deprecated die Einstellungen werden beim Setzen unmittelbar gespeichert,
	 *             die Funktion entfllt daher
	 */
	@Deprecated
	public boolean sichereEinstellungenSpeicherBenutzerBestimmtNetzweit() {
		EinstellungenWaehler.LOGGER
				.warning("Unntige Speicherung der Einstellungen!");
		return true;
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} fr einen bestimmten Benutzer
	 * LOKAL.<br>
	 * <b>Achtung:</b> Diese Funktion sollten nur fr administrative Aufgaben
	 * verwendet werden, wenn der betreffende Benutzer nicht angemeldet ist.
	 * I.d.R. sind die Methoden {@link EinstellungenWaehler#sichereSpeicher()}
	 * bzw. {@link EinstellungenWaehler#sichereSpeicherBenutzerLokal()} zu
	 * bevorzugen.
	 * 
	 * @param benutzerName
	 *            Name des Benutzers, fr den der {@link EinstellungenSpeicher}
	 *            gesichert werden soll.
	 * @param speicher
	 *            Zu sichernder {@link EinstellungenSpeicher}.
	 * @return Liefert <code>true</code>, wenn der {@link EinstellungenSpeicher}
	 *         erfolgreich gesichert werden konnte, sonst <code>false</code>.
	 * @deprecated die Einstellungen werden beim Setzen unmittelbar gespeichert,
	 *             die Funktion entfllt daher
	 */
	@Deprecated
	public boolean sichereEinstellungenSpeicherBenutzerBestimmtLokal(
			final String benutzerName, final EinstellungenSpeicher speicher) {
		EinstellungenWaehler.LOGGER
				.warning("Unntige Speicherung der Einstellungen!");
		return true;
	}

	/**
	 * Legt fest, ob mit oder ohne UrlasserDialog gesendet werden soll.<br>
	 * <b>WICHTIG:</b> Dieses Flag <b><i>muss</i></b> nach dem Speichern wieder
	 * zurckgesetzt werden!!!
	 * 
	 * @param sendenOhneUrlasserDialog
	 *            Wenn <code>true</code>, dann wird mit UrlasserDialog gesendet,
	 *            sonst nicht.
	 * @deprecated die Abfrage des Urlasserdialogs sollte bei Bedarf vom Anwener
	 *             bernommen werden, die UrlasserUnformationen werden dann beim
	 *             Setzen eines Parameters mit bergeben
	 */
	@Deprecated
	public void setSendenOhneUrlasserDialog(
			final boolean sendenOhneUrlasserDialog) {
		this.sendenOhneUrlasserDialog = sendenOhneUrlasserDialog;
	}

	/**
	 * Prft, ob mit oder ohne UrlasserDialog gesendet wird.
	 * 
	 * @return Wenn <code>true</code>, dann wird mit UrlasserDialog gesendet,
	 *         sonst nicht.
	 * @deprecated die Abfrage des Urlasserdialogs sollte bei Bedarf vom Anwener
	 *             bernommen werden, die UrlasserUnformationen werden dann beim
	 *             Setzen eines Parameters mit bergeben
	 */
	@Deprecated
	public boolean isSendenOhneUrlasserDialog() {
		return sendenOhneUrlasserDialog;
	}
}
