/*
 * SWIS Benachrichtigung der Meistereien
 * Copyright (C) 2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.event.EventListenerList;

import org.eclipse.jface.preference.IPersistentPreferenceStore;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.internal.RahmenwerkService;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserInfo;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Implementierung eines PreferenceStores, das seine Daten mit dem
 * Einstellungsspeicher des Rahmenwerks persistiert.
 * 
 * @author BitCtrl Systems GmbH, Ch. Hsel
 * 
 * @version $$Id$$
 *
 */
public class RahmenwerkPreferenceStore implements IPreferenceStore, IPersistentPreferenceStore {

	private Properties store = new Properties();
	private final Map<String, String> defaultMap = new HashMap<String, String>();
	private final SpeicherKey speicherKey;
	private Einstellungen einstellungen;
	private String qualifier;
	private AtomicBoolean needsSaving = new AtomicBoolean(false);

	private EventListenerList listenerList = new EventListenerList();

	public RahmenwerkPreferenceStore(final Einstellungen einstellungen, final String qualifier) {
		this(einstellungen, qualifier, EinstellungOwnerType.SYSTEM, EinstellungLocation.NETZWERKWEIT);
	}

	public RahmenwerkPreferenceStore(final Einstellungen einstellungen, final String qualifier,
			final EinstellungOwnerType typ, final EinstellungLocation location) {
		this.einstellungen = einstellungen;
		this.qualifier = qualifier;
		speicherKey = new SpeicherKey(typ, location);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPersistentPreferenceStore#save()
	 */
	@Override
	public void save() throws IOException {
		final Writer writer = new StringWriter();
		store.store(writer, "");

		final Rahmenwerk rahmenWerk = RahmenwerkService.getService().getRahmenWerk();
		final UrlasserInfo info = new UrlasserInfo(rahmenWerk.getBenutzer(), rahmenWerk.getPasswort(), "", "");
		einstellungen.setValue(new EinstellungsAdresse(null, qualifier, speicherKey), writer.toString(), info);
		needsSaving.set(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#addPropertyChangeListener(
	 * org.eclipse.jface.util.IPropertyChangeListener)
	 */
	@Override
	public void addPropertyChangeListener(final IPropertyChangeListener listener) {
		listenerList.add(IPropertyChangeListener.class, listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#contains(java.lang.String)
	 */
	@Override
	public boolean contains(final String name) {
		return store.containsKey(name);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#firePropertyChangeEvent(
	 * java.lang.String, java.lang.Object, java.lang.Object)
	 */
	@Override
	public void firePropertyChangeEvent(final String name, final Object oldValue, final Object newValue) {
		needsSaving.set(true);
		final PropertyChangeEvent event = new PropertyChangeEvent(this, name, oldValue, newValue);
		for (IPropertyChangeListener l : listenerList.getListeners(IPropertyChangeListener.class)) {
			l.propertyChange(event);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferenceStore#getBoolean(java.lang.
	 * String)
	 */
	@Override
	public boolean getBoolean(final String name) {
		if (!store.containsKey(name)) {
			loadEinstellungen();
		}

		if (store.get(name) == null) {
			return getDefaultBoolean(name);
		}
		return Boolean.getBoolean(store.getProperty(name));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getDefaultBoolean(java.lang
	 * .String)
	 */
	@Override
	public boolean getDefaultBoolean(final String name) {
		if (defaultMap.containsKey(name)) {
			return Boolean.getBoolean(defaultMap.get(name));
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getDefaultDouble(java.lang.
	 * String)
	 */
	@Override
	public double getDefaultDouble(final String name) {
		if (defaultMap.containsKey(name)) {
			return Double.parseDouble(defaultMap.get(name));
		}
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getDefaultFloat(java.lang.
	 * String)
	 */
	@Override
	public float getDefaultFloat(final String name) {
		if (defaultMap.containsKey(name)) {
			return Float.parseFloat(defaultMap.get(name));
		}
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getDefaultInt(java.lang.
	 * String)
	 */
	@Override
	public int getDefaultInt(final String name) {
		if (defaultMap.containsKey(name)) {
			return Integer.parseInt(defaultMap.get(name));
		}
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getDefaultLong(java.lang.
	 * String)
	 */
	@Override
	public long getDefaultLong(final String name) {
		if (defaultMap.containsKey(name)) {
			return Long.parseLong(defaultMap.get(name));
		}
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getDefaultString(java.lang.
	 * String)
	 */
	@Override
	public String getDefaultString(final String name) {
		if (defaultMap.containsKey(name)) {
			return defaultMap.get(name);
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getDouble(java.lang.String)
	 */
	@Override
	public double getDouble(final String name) {
		if (!store.containsKey(name)) {
			loadEinstellungen();
		}

		if (store.get(name) == null) {
			return getDefaultDouble(name);
		}
		return Double.parseDouble(store.getProperty(name));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getFloat(java.lang.String)
	 */
	@Override
	public float getFloat(final String name) {
		if (!store.containsKey(name)) {
			loadEinstellungen();
		}

		if (store.get(name) == null) {
			return getDefaultFloat(name);
		}
		return Float.parseFloat(store.getProperty(name));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getInt(java.lang.String)
	 */
	@Override
	public int getInt(final String name) {
		if (!store.containsKey(name)) {
			loadEinstellungen();
		}

		if (store.get(name) == null) {
			return getDefaultInt(name);
		}
		return Integer.parseInt(store.getProperty(name));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getLong(java.lang.String)
	 */
	@Override
	public long getLong(final String name) {
		if (!store.containsKey(name)) {
			loadEinstellungen();
		}

		if (store.get(name) == null) {
			return getDefaultLong(name);
		}
		return Long.parseLong(store.getProperty(name));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#getString(java.lang.String)
	 */
	@Override
	public String getString(final String name) {
		if (!store.containsKey(name)) {
			loadEinstellungen();
		}

		if (store.get(name) == null) {
			return getDefaultString(name);
		}
		return store.getProperty(name);
	}

	private void loadEinstellungen() {
		try {
			final String value = (String) einstellungen.getValue(new EinstellungsAdresse(null, qualifier, speicherKey));
			if (value != null) {
				store.load(new StringReader(value));
			}
		} catch (final IOException e) {
			Debug.getLogger().warning("Einstellung konnten nicht gelesen werden.", e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#isDefault(java.lang.String)
	 */
	@Override
	public boolean isDefault(final String name) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferenceStore#needsSaving()
	 */
	@Override
	public boolean needsSaving() {
		return needsSaving.get();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#putValue(java.lang.String,
	 * java.lang.String)
	 */
	@Override
	public void putValue(final String name, final String value) {
		store.put(name, value);
		needsSaving.set(true);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferenceStore#
	 * removePropertyChangeListener(org.eclipse.jface.util.
	 * IPropertyChangeListener)
	 */
	@Override
	public void removePropertyChangeListener(final IPropertyChangeListener listener) {
		listenerList.remove(IPropertyChangeListener.class, listener);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferenceStore#setDefault(java.lang.
	 * String, double)
	 */
	@Override
	public void setDefault(final String name, final double value) {
		defaultMap.put(name, Double.toString(value));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferenceStore#setDefault(java.lang.
	 * String, float)
	 */
	@Override
	public void setDefault(final String name, final float value) {
		defaultMap.put(name, Float.toString(value));

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferenceStore#setDefault(java.lang.
	 * String, int)
	 */
	@Override
	public void setDefault(final String name, final int value) {
		defaultMap.put(name, Integer.toString(value));

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferenceStore#setDefault(java.lang.
	 * String, long)
	 */
	@Override
	public void setDefault(final String name, final long value) {
		defaultMap.put(name, Long.toString(value));

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferenceStore#setDefault(java.lang.
	 * String, java.lang.String)
	 */
	@Override
	public void setDefault(final String name, final String defaultObject) {
		defaultMap.put(name, defaultObject);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferenceStore#setDefault(java.lang.
	 * String, boolean)
	 */
	@Override
	public void setDefault(final String name, final boolean value) {
		defaultMap.put(name, Boolean.toString(value));

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#setToDefault(java.lang.
	 * String)
	 */
	@Override
	public void setToDefault(final String name) {
		final String oldValue = getString(name);
		final String defaultValue = getDefaultString(name);
		if (oldValue != defaultValue) {
			setValue(name, defaultValue);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
	 * double)
	 */
	@Override
	public void setValue(final String name, final double value) {
		final double oldValue = getDouble(name);
		if (value != oldValue) {
			setValue(name, Double.toString(value));
			firePropertyChangeEvent(name, oldValue, value);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
	 * float)
	 */
	@Override
	public void setValue(final String name, final float value) {
		final float oldValue = getFloat(name);
		if (value != oldValue) {
			setValue(name, Float.toString(value));
			firePropertyChangeEvent(name, oldValue, value);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
	 * int)
	 */
	@Override
	public void setValue(final String name, final int value) {
		final int oldValue = getInt(name);
		if (value != oldValue) {
			setValue(name, Integer.toString(value));
			firePropertyChangeEvent(name, oldValue, value);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
	 * long)
	 */
	@Override
	public void setValue(final String name, final long value) {
		final long oldValue = getLong(name);
		if (value != oldValue) {
			setValue(name, Long.toString(value));
			firePropertyChangeEvent(name, oldValue, value);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
	 * java.lang.String)
	 */
	@Override
	public void setValue(final String name, final String value) {
		final Object oldValue = store.put(name, value);
		firePropertyChangeEvent(name, oldValue, value);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
	 * boolean)
	 */
	@Override
	public void setValue(final String name, final boolean value) {
		final boolean oldValue = getBoolean(name);
		if (value != oldValue) {
			setValue(name, Boolean.toString(value));
			firePropertyChangeEvent(name, oldValue, value);
		}

	}

}
