/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.einstellungen;

import java.util.ArrayList;
import java.util.Collection;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.internal.RahmenwerkService;

/**
 * Schlssel fr die Zuordnung einer Rahmenwerkseinstellung.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: SpeicherKey.java 46953 2013-11-05 10:05:57Z peuker $
 */
public final class SpeicherKey {

	private final EinstellungOwnerType ownerType;
	private final EinstellungLocation location;
	private final String pid;

	/**
	 * Konstruktor.
	 * 
	 * @param ownerType
	 *            der Typ des Besitzers der Einstellung
	 * @param location
	 *            der logische Ort, an dem die Einstellung gespeichert werden
	 *            soll
	 * @param pid
	 *            die PID fr die Speicherung von Nutzereinstellungen
	 */
	public SpeicherKey(final EinstellungOwnerType ownerType,
			final EinstellungLocation location, final String pid) {
		this.ownerType = ownerType;
		this.location = location;
		this.pid = pid;
	}

	/**
	 * Konstruktor.
	 * 
	 * @param ownertype
	 *            der Typ des Besitzers der Einstellung
	 * @param location
	 *            der logische Ort, an dem die Einstellung gespeichert werden
	 *            soll
	 */
	public SpeicherKey(final EinstellungOwnerType ownertype,
			final EinstellungLocation location) {
		this(ownertype, location, null);
	}

	/**
	 * Konstruktor, erzeugt einen Schlssel aus der bergebenen
	 * Einstellungsadresse, die neben der logsichen Zuordnung der Einstellung
	 * zutzlich Informationen zu einer konkreten Einstellung enthlt.
	 * 
	 * @param adresse
	 *            die {@link EinstellungsAdresse}
	 */
	public SpeicherKey(final EinstellungsAdresse adresse) {
		this(adresse.getOwnerType(), adresse.getLocation(), adresse.getPid());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result)
				+ ((location == null) ? 0 : location.hashCode());
		result = (prime * result)
				+ ((ownerType == null) ? 0 : ownerType.hashCode());
		result = (prime * result) + ((pid == null) ? 0 : pid.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		boolean result = false;
		if (this == obj) {
			return true;
		} else if (obj instanceof SpeicherKey) {
			final SpeicherKey key = (SpeicherKey) obj;
			result = ownerType == key.ownerType;
			result &= location == key.location;
			result &= (((pid == null) && (key.pid == null)))
					|| ((pid != null) && pid.equals(key.pid));
		}
		return result;
	}

	/**
	 * ermittelt, ob der Schlssel eine lokale Einstellung beschreibt.
	 * 
	 * @return der Status
	 */
	public boolean isLocal() {
		return location == EinstellungLocation.LOKAL;
	}

	/**
	 * liefert den Besitzertyp fr den der Schlssel gltig ist.
	 * 
	 * @return den Typ
	 * @deprecated Namen vereinheitlicht, ersetzt durh {@link #getOwnerType()}
	 */
	@Deprecated
	public EinstellungOwnerType getOwnertype() {
		return ownerType;
	}

	/**
	 * liefert den Besitzertyp fr den der Schlssel gltig ist.
	 * 
	 * @return den Typ
	 */
	public EinstellungOwnerType getOwnerType() {
		return ownerType;
	}

	/**
	 * liefert den logischen Ort fr den EInstellungen mit deisem Schlssel
	 * gespeichert werden sollen.
	 * 
	 * @return den Ort
	 */
	public EinstellungLocation getLocation() {
		return location;
	}

	/**
	 * liefert die Pid des Schlssels.
	 * 
	 * Die Funktion kann den Wert <code>null</code> liefern, wenn es sich um
	 * einen Schlssel fr allgemeine Einstellungen liefert.
	 * 
	 * @return die PID oder <code>null</code>
	 */
	public String getPid() {
		return pid;
	}

	/**
	 * liefert eine Klartextbeschreibung des Keys.
	 * 
	 * @return die Beschreibung
	 */
	public String getTypBezeichnung() {
		final StringBuilder buffer = new StringBuilder();

		switch (ownerType) {
		case BENUTZER:
			buffer.append("Benutzer ");
			break;
		case BENUTZERKLASSE:
			buffer.append("Berechtigungsklasse ");
			break;
		case SYSTEM:
		default:
			buffer.append("Allgemein ");
			break;
		}

		switch (location) {
		case NETZWERKWEIT:
			buffer.append("netzwerkweit");
			break;
		case LOKAL:
		default:
			buffer.append("lokal");
			break;
		}

		return buffer.toString();
	}

	@Override
	public String toString() {
		if (pid == null) {
			return getTypBezeichnung();
		}

		return getTypBezeichnung();
	}

	/**
	 * liefert die Standard-Speicherkeys, die den ursprnglichen
	 * Einstellungszielen des RW1 entsprechen.
	 * 
	 * @return die Liste der mglichen Standardspeicherorte
	 */
	public static Collection<SpeicherKey> getDefaultKeys() {
		final Rahmenwerk rahmenwerk = RahmenwerkService.getService()
				.getRahmenWerk();
		final Collection<SpeicherKey> result = new ArrayList<SpeicherKey>();
		if (rahmenwerk.isOnline()) {
			result.add(SpeicherKey.benutzerNetzweit());
			result.add(SpeicherKey.benutzerLokal());
			result.add(SpeicherKey.allgemeinNetzweit());
			result.add(SpeicherKey.allgemeinLokal());
		} else {
			result.add(SpeicherKey.allgemeinLokal());
		}

		return result;
	}

	/**
	 * liefert den {@link SpeicherKey} fr die netzwerkweiten Einstellungen des
	 * angemeldeten Nutzers. (nur verfgbar, wenn eine Datenverteilerverbindung
	 * besteht!)
	 * 
	 * @return den Key
	 */
	public static SpeicherKey benutzerNetzweit() {
		final Rahmenwerk rahmenwerk = RahmenwerkService.getService()
				.getRahmenWerk();
		if (rahmenwerk.isOnline()) {
			return new SpeicherKey(EinstellungOwnerType.BENUTZER,
					EinstellungLocation.NETZWERKWEIT, rahmenwerk.getBenutzer()
							.getPid());
		}
		throw new IllegalStateException(
				"Es ist kein Benutzer angemeldet, deshalb knnen keine netzwerkweiten Einstellungen abgerufen werden!");
	}

	/**
	 * liefert den {@link SpeicherKey} fr die lokalen Einstellungen des
	 * angemeldeten Nutzers. (nur verfgbar, wenn eine Datenverteilerverbindung
	 * besteht!)
	 * 
	 * @return den Key
	 */
	public static SpeicherKey benutzerLokal() {
		final Rahmenwerk rahmenwerk = RahmenwerkService.getService()
				.getRahmenWerk();
		if (rahmenwerk.isOnline()) {
			return new SpeicherKey(EinstellungOwnerType.BENUTZER,
					EinstellungLocation.LOKAL, rahmenwerk.getBenutzer()
							.getPid());
		}
		throw new IllegalStateException(
				"Es ist kein Benutzer angemeldet, deshalb knnen keine lokalen Einstellungen abgerufen werden!");
	}

	/**
	 * liefert den {@link SpeicherKey} fr die netzwerkweiten allgemeinen
	 * Einstellungen. (nur verfgbar, wenn eine Datenverteilerverbindung
	 * besteht!)
	 * 
	 * @return den Key
	 */
	public static SpeicherKey allgemeinNetzweit() {
		final Rahmenwerk rahmenwerk = RahmenwerkService.getService()
				.getRahmenWerk();
		if (rahmenwerk.isOnline()) {
			return new SpeicherKey(EinstellungOwnerType.SYSTEM,
					EinstellungLocation.NETZWERKWEIT);
		}
		throw new IllegalStateException(
				"Es besteht keine Datenverteilerverbindung, deshalb knnen netzwerkweiten Einstellungen abgerufen werden!");
	}

	/**
	 * liefert den {@link SpeicherKey} fr die netzwerkweiten lokalen
	 * Einstellungen. (Einstellungen fr den Offline-Betrieb)
	 * 
	 * @return den Key
	 */
	public static SpeicherKey allgemeinLokal() {
		return new SpeicherKey(EinstellungOwnerType.SYSTEM,
				EinstellungLocation.LOKAL);
	}

}
