/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Datenstrukturklasse, enthlt jeweils eine Zeitgruppe mit Start- und
 * Endzeitpunkt.
 * 
 * @author HofmannJ
 * 
 */
public class KalenderZeitFilterDatenStrukturZeitGruppe implements
		Comparable<KalenderZeitFilterDatenStrukturZeitGruppe> {
	/**
	 * Startzeitpunkt als long.
	 */
	private long startzeitpunkt;

	/**
	 * Endzeitpunkt als long.
	 */
	private long endzeitpunkt;

	/**
	 * Konstruktor setzt die Parametern fr Start- und Endzeitpunkt.
	 * 
	 * @param startzeitpunkt
	 *            Startzeit als long (Zeit in MS seit 1.1.1970)
	 * @param endzeitpunkt
	 *            Endzeit als long (Zeit in MS seit 1.1.1970)
	 */
	public KalenderZeitFilterDatenStrukturZeitGruppe(final long startzeitpunkt,
			final long endzeitpunkt) {
		this.startzeitpunkt = startzeitpunkt;
		this.endzeitpunkt = endzeitpunkt;
	}

	/**
	 * Konstruktor, setzt Start- und Endzeitpunkt.
	 * 
	 * @param startzeitpunkt
	 *            Startzeitpunkt als String. Dieser String muss dem Format
	 *            <code>HH:mm:s.S</code> gengen.
	 * @param endzeitpunkt
	 *            Endzeitpunkt als String. Dieser String muss dem Format
	 *            <code>HH:mm:s.S</code> gengen.
	 * @throws ParseException
	 *             Wird geworfen wenn der bergebene String von Start- oder
	 *             Endzeitpunkt nicht dem Format <code>HH:mm:s.S</code> gengt.
	 */
	public KalenderZeitFilterDatenStrukturZeitGruppe(
			final String startzeitpunkt, final String endzeitpunkt)
			throws ParseException {
		setStartzeitpunkt(startzeitpunkt);
		setEndzeitpunkt(endzeitpunkt);
	}

	/**
	 * Liefert den Endzeitpunkt.
	 * 
	 * @return Endzeitpunkt (Zeit in MS seit 1.1.1970)
	 */
	public long getEndzeitpunkt() {
		return endzeitpunkt;
	}

	/**
	 * Liefert den Endzeitpunkt als String.
	 * 
	 * @return Endzeitpunkt als String im Format <code>HH:mm:s.S</code>
	 */
	public String getEndzeitpunktAlsString() {
		String re = "";
		final DateFormat df = new SimpleDateFormat("HH:mm:s.S");
		re = df.format(new Date(endzeitpunkt));
		return re;
	}

	/**
	 * Setzt den Endzeitpunkt.
	 * 
	 * @param endzeitpunkt
	 *            Endzeitpunkt (Zeit in MS seit 1.1.1970)
	 */
	public void setEndzeitpunkt(final long endzeitpunkt) {
		this.endzeitpunkt = endzeitpunkt;
	}

	/**
	 * Setzt den Endzeitpunkt.
	 * 
	 * @param endzeitpunkt
	 *            Endzeitpunkt als String. Dieser String muss dem Format
	 *            <code>HH:mm:s.S</code> gengen.
	 * @throws ParseException
	 *             Wird geworfen wenn der bergebene String nicht dem Format
	 *             <code>HH:mm:s.S</code> gengt.
	 */
	public void setEndzeitpunkt(final String endzeitpunkt)
			throws ParseException {
		final DateFormat df = new SimpleDateFormat("HH:mm:s.S");
		final Date endzeitDate = df.parse(endzeitpunkt);
		this.endzeitpunkt = endzeitDate.getTime();
	}

	/**
	 * Liefert den Startzeitpunkt.
	 * 
	 * @return Startzeitpunkt (Zeit in MS seit 1.1.1970)
	 */
	public long getStartzeitpunkt() {
		return startzeitpunkt;
	}

	public String getStartzeitpunktAlsString() {
		String re = "";
		final DateFormat df = new SimpleDateFormat("HH:mm:s.S");
		re = df.format(new Date(startzeitpunkt));
		return re;
	}

	/**
	 * Setzt den Startzeitpunkt.
	 * 
	 * @param startzeitpunkt
	 *            Startzeitpunkt (Zeit in MS seit 1.1.1970)
	 */
	public void setStartzeitpunkt(final long startzeitpunkt) {
		this.startzeitpunkt = startzeitpunkt;
	}

	/**
	 * Setzt den Startzeitpunkt.
	 * 
	 * @param startzeitpunkt
	 *            Startzeitpunkt als String. Dieser String muss dem Format
	 *            <code>HH:mm:s.S</code> gengen.
	 * @throws ParseException
	 *             Wird geworfen wenn der bergebene String nicht dem Format
	 *             <code>HH:mm:s.S</code> gengt.
	 */
	public void setStartzeitpunkt(final String startzeitpunkt)
			throws ParseException {
		final DateFormat df = new SimpleDateFormat("HH:mm:s.S");
		final Date startZeitDate = df.parse(startzeitpunkt);
		this.startzeitpunkt = startZeitDate.getTime();
	}

	@Override
	public int compareTo(final KalenderZeitFilterDatenStrukturZeitGruppe o) {
		int re = 0;

		if (this.startzeitpunkt == o.getStartzeitpunkt()) {
			if (endzeitpunkt == o.getEndzeitpunkt()) {
				re = 0;
			} else if (endzeitpunkt < o.getEndzeitpunkt()) {
				re = -1;
			} else if (endzeitpunkt > o.getEndzeitpunkt()) {
				re = 1;
			}
		} else if (this.startzeitpunkt < o.getStartzeitpunkt()) {
			re = -1;
		} else if (this.startzeitpunkt > o.getStartzeitpunkt()) {
			re = 1;
		}

		return re;
	}

	@Override
	public String toString() {
		String re = "";

		re = re + "Zeitgruppe von: " + startzeitpunkt + " ("
				+ getStartzeitpunktAlsString() + ") bis: " + endzeitpunkt
				+ " (" + getEndzeitpunktAlsString() + ")";

		return re;
	}
}
