/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.kupzog.ktable.KTable;

/**
 * Klasse zum Speichern und Lesen der benutzerspezifischen, netzweiten
 * Einstellungen.
 * 
 * @author HofmannJ
 * 
 */
public class DavEmpfangenSendenAenderungsDarstellung implements
		ClientSenderInterface, ClientReceiverInterface {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Verbindung zum DaV.
	 */
	private ClientDavConnection verbindung;

	/**
	 * Display.
	 */
	private Display display;

	/**
	 * {@link TabellenModell}, welches die darzustellenden Daten aufnimmt.
	 */
	private TabellenModell tabellenModell;

	/**
	 * {@link KTable}, welche das {@link TabellenModell} anzeigt. Hier muss die
	 * redraw-Methode aufgerufen werden, wenn sich etwas am
	 * {@link TabellenModell} gendert hat.
	 */
	private KTable kTable;

	/**
	 * Datenmodell des DaV.
	 */
	private DataModel dataModell;

	/**
	 * SystemObjekt.
	 */
	private SystemObject systemObject;

	/**
	 * Bezeichnung der Attributgruppe.
	 */
	private static final String ATG_PID = "atg.benutzerEinstellungenOberflcheNetzweit";

	/**
	 * Item Benutzereinstellungen.
	 */
	private static final String ITEM_BENUTZEREINSTELLUNGEN = "Benutzereinstellungen";

	/**
	 * Item Voreinstellungen.
	 */
	private static final String ITEM_VOREINSTELLUNGEN = "Voreinstellungen";

	/**
	 * Item Benutzer.
	 */
	private static final String ITEM_BENUTZER = "benutzer";

	/**
	 * Bezeichnung des Aspektes soll (zum Empfangen).
	 */
	private static final String ASP_SOLL_PID = "asp.parameterSoll";

	/**
	 * Bezeichnung des Aspektes vorgabe (zum Senden).
	 */
	private static final String ASP_VORGABE_PID = "asp.parameterVorgabe";

	/**
	 * Typ Autarke Organisationseinheit.
	 */
	private static final String TYP_AOE = "typ.autarkeOrganisationsEinheit";

	/**
	 * Attributgruppe.
	 */
	private AttributeGroup atg;

	/**
	 * Aspekt.
	 */
	private Aspect asp;

	/**
	 * Datenbeschreibung.
	 */
	private DataDescription dataDescription;

	/**
	 * Objekt zum Synchronisieren, dass der Thread die Empfangenen Daten
	 * verarbeitet hat (auch wenn hier keine groartige Verarbeitung
	 * stattfindet). Mit notify wird eine Methode zur Rckgabe der gelesenen
	 * Daten benachrichtigt.
	 */
	private final Object endeVerarbeitungEmpfangLock = new Object();

	/**
	 * Synchronisation der Sende Steuerung.
	 */
	private final Object sendeSteuerungsMutex = new Object();

	/**
	 * Max. Gre der Queue.
	 */
	private static final int MAXQUEUE = 10;

	/**
	 * Queue.
	 */
	private final Queue<Data> produkte = new LinkedList<Data>();

	/**
	 * Lock der Queue.
	 */
	private final Lock lock = new ReentrantLock();

	/**
	 * Bedingung fr Queue nicht voll.
	 */
	private final Condition nichtVoll = lock.newCondition();

	/**
	 * Bedingung fr Queue nicht leer.
	 */
	private final Condition nichtLeer = lock.newCondition();

	private boolean sendeSteuerung;

	/**
	 * Konstruktor, bernimmt die Verbindung zum DaV.
	 * 
	 * @param verbindung
	 *            DaV-Verbindung
	 */
	public DavEmpfangenSendenAenderungsDarstellung(
			final ClientDavConnection verbindung) {
		this.verbindung = verbindung;
		this.dataModell = verbindung.getDataModel();
	}

	/**
	 * Setzt das Display. Der Thread der DAV-Kommunikation bentigt den Zugriff
	 * hierauf, um asynchron die Darstellung der Tabelle anpassen zu knnen.
	 * 
	 * @param display
	 *            Display.
	 */
	public void setDisplay(final Display display) {
		this.display = display;
	}

	/**
	 * Setzt die zu aktualisierende {@link KTable}.
	 * 
	 * @param table
	 */
	public void setKTable(final KTable table) {
		this.kTable = table;
	}

	/**
	 * Setzt das zu aktualisierende {@link TabellenModell}.
	 * 
	 * @param tabellenModell
	 */
	public void setTabellenModell(final TabellenModell tabellenModell) {
		this.tabellenModell = tabellenModell;
	}

	/**
	 * Anmelden zum Senden von Daten.
	 */
	private void anmeldenSenden() {
		DavEmpfangenSendenAenderungsDarstellung.LOGGER
				.fine("Anmelden Senden...");
		dataModell = verbindung.getDataModel();

		final List<SystemObject> objekte = bestimmeObjekte("typ.autarkeOrganisationsEinheit");
		if (objekte.size() == 1) {
			systemObject = objekte.get(0);
			atg = dataModell
					.getAttributeGroup(DavEmpfangenSendenAenderungsDarstellung.ATG_PID);

			asp = dataModell
					.getAspect(DavEmpfangenSendenAenderungsDarstellung.ASP_VORGABE_PID);
			dataDescription = new DataDescription(atg, asp);

			final SenderRole senderRole = SenderRole.sender();

			try {
				verbindung.subscribeSender(this, systemObject, dataDescription,
						senderRole);
			} catch (final OneSubscriptionPerSendData e) {
				e.printStackTrace();
			}

		}

	}

	protected void senden() {
		anmeldenSenden();
		atg = dataModell
				.getAttributeGroup(DavEmpfangenSendenAenderungsDarstellung.ATG_PID);
		final Data data = verbindung.createData(atg);
		final Array array = data.getArray("Benutzereinstellungen");
		array.setLength(2);

		Data item = null;

		for (int i = 0; i < 2; i++) {
			item = array.getItem(i);
			// TextArray textArray = item.getTextArray(_itemVoreinstellungen);

			final Array arrayText = item
					.getArray(DavEmpfangenSendenAenderungsDarstellung.ITEM_VOREINSTELLUNGEN);

			final ReferenceValue referenceValue = item
					.getReferenceValue(DavEmpfangenSendenAenderungsDarstellung.ITEM_BENUTZER);

			arrayText.setLength(2);

			referenceValue.setSystemObject(bestimmeSOEinesBenutzers("Tester"));
		}

		final ResultData datenTelegramm = new ResultData(systemObject,
				dataDescription, new Date().getTime(), data);

		try {
			verbindung.sendData(datenTelegramm);
		} catch (final DataNotSubscribedException e) {
			DavEmpfangenSendenAenderungsDarstellung.LOGGER.warning(e
					.getLocalizedMessage());
		} catch (final SendSubscriptionNotConfirmed e) {
			DavEmpfangenSendenAenderungsDarstellung.LOGGER.warning(e
					.getLocalizedMessage());
		}

		abmeldenSenden();
	}

	/**
	 * Berarbeitet einen benutzerspezifischen Eintrag.
	 * 
	 * @param benutzerName
	 *            Benutzer, fr den ein Eintrag angepasst werden soll.
	 * @param xmlStruktur
	 *            String der XML-Struktur, die gespeichert werden soll.
	 * @return Liefert <code>true</code>, wenn erfolgreich gespeichert werden
	 *         konnte.
	 */
	public boolean editEintrag(final String benutzerName,
			final String xmlStruktur) {
		boolean erfolgreich = false;

		/*
		 * Daten Holen
		 */
		final Data data = empfangeDatenBenutzereinstellungenNetzweit();

		/*
		 * Benutzerdaten suchen
		 */
		final Data benutzerDaten = sucheBenutzerspezifischenEintrag(
				benutzerName, data);

		/*
		 * Existiert Benutzer bzw. dessen Daten?
		 */
		if (benutzerDaten != null) {
			setBenutzersepzifischeEinstellungen(benutzerDaten, xmlStruktur);
			erfolgreich = true;
		} else {
			if (anlegenBenutzerspezifischenEintrag(data, benutzerName)) {
				final Data neuangelegterBenutzereintrag = sucheBenutzerspezifischenEintrag(
						benutzerName, data);
				setBenutzersepzifischeEinstellungen(
						neuangelegterBenutzereintrag, xmlStruktur);
				erfolgreich = true;
			}
		}

		if (erfolgreich) {
			anmeldenSenden();
			final ResultData datenTelegramm = new ResultData(systemObject,
					dataDescription, new Date().getTime(), data);

			try {
				synchronized (sendeSteuerungsMutex) {
					try {
						while (!sendeSteuerung) {
							DavEmpfangenSendenAenderungsDarstellung.LOGGER
									.fine("=====>_sendenEnde.wait");
							sendeSteuerungsMutex.wait();
							DavEmpfangenSendenAenderungsDarstellung.LOGGER
									.fine("=====>_sendenEnde notified");
						}
						verbindung.sendData(datenTelegramm);
					} catch (final InterruptedException e) {
						DavEmpfangenSendenAenderungsDarstellung.LOGGER
								.warning(
										getClass().getName()
												+ "::editEintrag Wurde unterbrochen\n\t=> ",
										e);
					}
				}
			} catch (final DataNotSubscribedException e) {
				e.printStackTrace();
			} catch (final SendSubscriptionNotConfirmed e) {
				e.printStackTrace();
			}
			abmeldenSenden();
		}

		return erfolgreich;
	}

	public String getEintrag(final String benutzerName) {
		String re = null;

		/*
		 * Daten Holen
		 */
		final Data data = empfangeDatenBenutzereinstellungenNetzweit();

		final Data eintrag = sucheBenutzerspezifischenEintrag(benutzerName,
				data);
		if (eintrag != null) {
			// TextArray textArray =
			// eintrag.getTextArray(_itemVoreinstellungen);
			final Array textArray = eintrag
					.getArray(DavEmpfangenSendenAenderungsDarstellung.ITEM_VOREINSTELLUNGEN);
			if (textArray.getLength() > 0) {
				final StringBuffer buf = new StringBuffer();

				for (int i = 0; i < textArray.getLength(); i++) {
					buf.append(textArray.getTextValue(i).getText());
					buf.append('\n');
				}

				re = buf.toString();
			}
		}
		return re;
	}

	/**
	 * Fgt einen neuen Eintrag fr einen Benutzer hinzu, so dass
	 * Benutzerspezifische Einstellungen fr den Benutzer gespeichert werden
	 * knnen.
	 * 
	 * atg.benutzerEinstellungenOberflcheNetzweit
	 * 
	 * @param data
	 *            Daten, an die der Benutzer angehngt werden soll (in
	 *            Attributliste Benutzereinstellungen)
	 * @param benutzerName
	 *            Benutzername
	 * @return Liefert <code>true</code>, wenn der Benutzereintrag den
	 *         bergebenen Daten angefgt werden kann, sonnst <code>false</code>
	 *         .
	 */
	private boolean anlegenBenutzerspezifischenEintrag(final Data data,
			final String benutzerName) {
		boolean erfolgreich = false;

		final SystemObject soBenutzer = bestimmeSOEinesBenutzers(benutzerName);
		if (soBenutzer != null) {
			/*
			 * Lnge des Arrays um 1 erhhen
			 */
			data.getArray(
					DavEmpfangenSendenAenderungsDarstellung.ITEM_BENUTZEREINSTELLUNGEN)
					.setLength(
							data.getArray(
									DavEmpfangenSendenAenderungsDarstellung.ITEM_BENUTZEREINSTELLUNGEN)
									.getLength() + 1);

			final int pos = data
					.getArray(
							DavEmpfangenSendenAenderungsDarstellung.ITEM_BENUTZEREINSTELLUNGEN)
					.getLength() - 1;

			final Data benutzerItem = data
					.getArray(
							DavEmpfangenSendenAenderungsDarstellung.ITEM_BENUTZEREINSTELLUNGEN)
					.getItem(pos);

			/*
			 * Benutzerreferenz setzen
			 */
			benutzerItem.getReferenceValue(
					DavEmpfangenSendenAenderungsDarstellung.ITEM_BENUTZER)
					.setSystemObject(soBenutzer);

			erfolgreich = true;

		}

		return erfolgreich;
	}

	/**
	 * Liefert das ITEM der Benutzereinstellungen zum angegebenen Benutzer.
	 * 
	 * atg.benutzerEinstellungenOberflcheNetzweit
	 * 
	 * @param benutzer
	 *            Name des Benutzers
	 * @param data
	 *            Daten, aus denen der Benutzer herausgeholt werden soll.
	 * @return Liefert das "ITEM" als {@link Data}, welches zu dem jeweiligen
	 *         Benutzer gehrt oder <code>null</code>, falls dieser Benutzer
	 *         nicht existiert.
	 */
	private Data sucheBenutzerspezifischenEintrag(final String benutzer,
			final Data data) {
		Data re = null;

		final SystemObject soBenutzer = bestimmeSOEinesBenutzers(benutzer);
		if (soBenutzer != null) {
			final Array array = data
					.getArray(DavEmpfangenSendenAenderungsDarstellung.ITEM_BENUTZEREINSTELLUNGEN);
			final int laenge = array.getLength();
			for (int i = 0; i < laenge; i++) {
				final Data item = array.getItem(i);

				final ReferenceValue referenceValue = item
						.getReferenceValue(DavEmpfangenSendenAenderungsDarstellung.ITEM_BENUTZER);

				if (referenceValue.getSystemObject() != null) {
					if (referenceValue.getSystemObject().equals(soBenutzer)) {
						re = item;
						break;
					}
				} else {
					DavEmpfangenSendenAenderungsDarstellung.LOGGER
							.warning("Undefinierte Referenz");
				}
			}
		}

		return re;
	}

	/**
	 * Setzt die Benutzersperzifischen Einstellungen in das Item der
	 * Benutzereinstellungen, die von der Funktion
	 * {@link EinstellungenBenutzerSpezifisch#sucheBenutzerspezifischenEintrag(String, Data)}
	 * geliefert wurde.
	 * 
	 * atg.benutzerEinstellungenOberflcheNetzweit -> Benutzereinstellungen
	 * 
	 * @param data
	 *            "Item" (aus der Attributliste Benutzereinstellungen), in dem
	 *            die Benutzerspezifischen Einstellungen gesetzt werden sollen.
	 * @param xmlStruktur
	 *            Zu setzende XML-Struktur.
	 */
	private void setBenutzersepzifischeEinstellungen(final Data data,
			final String xmlStruktur) {
		// TextArray textArray = data.getTextArray(_itemVoreinstellungen);
		// logger.fine("Data: " + data.toString());
		final Array textArray = data
				.getArray(DavEmpfangenSendenAenderungsDarstellung.ITEM_VOREINSTELLUNGEN);
		final String[] split = xmlStruktur.split("\\n");
		textArray.setLength(split.length);

		for (int i = 0; i < split.length; i++) {
			textArray.getTextValue(i).setText(split[i]);
		}
	}

	/**
	 * Abmeldung.
	 */
	private void abmeldenSenden() {
		DavEmpfangenSendenAenderungsDarstellung.LOGGER
				.fine("Abmelden Senden..");
		verbindung.unsubscribeSender(this, systemObject, dataDescription);

	}

	/**
	 * Bestimmt die PID des zu einem Benutzer zugehrigen Objektes vom Typ
	 * <code>typ.benutzer</code>.
	 * 
	 * @param benutzer
	 *            Name des Benutzer-Objektes
	 * @return {@link SystemObject} des Benutzers oder <code>null</code>, wenn
	 *         kein entsprechender Benutzer existiert.
	 */
	private SystemObject bestimmeSOEinesBenutzers(final String benutzer) {
		SystemObject reSo = null;

		final List<SystemObject> bestimmeObjekte = bestimmeObjekte("typ.benutzer");
		final Iterator<SystemObject> it = bestimmeObjekte.iterator();
		while (it.hasNext()) {
			final SystemObject so = it.next();
			if (so.getName().equals(benutzer)) {
				reSo = so;
				// System.out.println("Benutzer gefunden: " + so.getName() + "("
				// + so.getPid() + ")");
				break;
			}
		}

		return reSo;
	}

	/**
	 * Bestimmt eine Liste von zu Benutzern (typ.benutzer) zugehrigen
	 * {@link SystemObject}s.
	 * 
	 * @return Liste von zu Benutzern (typ.benutzer) zugehrigen
	 *         {@link SystemObject}s
	 */
	public List<SystemObject> bestimmeAlleBenutzerSO() {
		final List<SystemObject> bestimmeObjekte = bestimmeObjekte("typ.benutzer");
		return bestimmeObjekte;
	}

	/**
	 * Methode zum Bestimmen der Objekte, die zu einer bestimmten PID gehren
	 * (z.B. typ.de)
	 * 
	 * @param objPid
	 *            Pid der Objekttypen
	 * @return Liste mit den Systemobjekten
	 */
	public List<SystemObject> bestimmeObjekte(final String objPid) {
		// System.out.println("bestimmeObjekte --> " + objPid);

		final List<SystemObject> objekte = new ArrayList<SystemObject>();

		// Systemobjekt erzeugen

		SystemObjectType typeSysObj = null;

		typeSysObj = verbindung.getDataModel().getType(objPid);

		if (typeSysObj == null) {
			return objekte;
		}

		// Elemente dieses Systemobjekts einlesen

		final List<SystemObject> typobjekte = typeSysObj.getElements();

		final Iterator<SystemObject> iteratorTypen = typobjekte.iterator();

		while (iteratorTypen.hasNext()) {
			final SystemObject sysObj = iteratorTypen.next();

			objekte.add(sysObj);
		}

		return objekte;
	}

	@Override
	public void dataRequest(final SystemObject object,
			final DataDescription dataDesc, final byte state) {
		if (state == 0) {
			synchronized (sendeSteuerungsMutex) {
				sendeSteuerung = true;
				sendeSteuerungsMutex.notifyAll();
			}
			DavEmpfangenSendenAenderungsDarstellung.LOGGER
					.fine("=====>_sendenEnde.notify durch Methode dataRequest");
		}
	}

	@Override
	public boolean isRequestSupported(final SystemObject object,
			final DataDescription dataDesc) {
		return true;
	}

	/**
	 * Ruft die netzweiten, benutzerspezifischen Einstellungen vom DaV ab und
	 * liefert diese als modifiableCopy eines {@link Data}-Objektes zurck.
	 */
	private Data empfangeDatenBenutzereinstellungenNetzweit() {

		synchronized (endeVerarbeitungEmpfangLock) {
			anmeldenEmpfang();
		}

		return null;
	}

	/**
	 * Anmelden zum Empfang der netzweiten, benutzerspezifischen Einstellungen.
	 */
	private void anmeldenEmpfang() {
		final List<SystemObject> objekte = bestimmeObjekte(DavEmpfangenSendenAenderungsDarstellung.TYP_AOE);
		if (objekte.size() == 1) {
			systemObject = objekte.get(0);
			atg = dataModell
					.getAttributeGroup(DavEmpfangenSendenAenderungsDarstellung.ATG_PID);
			asp = dataModell
					.getAspect(DavEmpfangenSendenAenderungsDarstellung.ASP_SOLL_PID);
			dataDescription = new DataDescription(atg, asp);
			final ReceiveOptions receiveOptions = ReceiveOptions.normal();
			final ReceiverRole receiverRole = ReceiverRole.receiver();

			verbindung.subscribeReceiver(this, systemObject, dataDescription,
					receiveOptions, receiverRole);
		}
	}

	/**
	 * Anmelden zum Empfang der netzweiten, benutzerspezifischen Einstellungen.
	 * bernimmt verschiedene Parameter.
	 * 
	 * @param pidSystemObject
	 *            Pid des {@link SystemObject}s, auf welches die Anmeldung
	 *            erfolgen soll.
	 * @param pidAtg
	 *            Pid der Attributgruppe, unter der die Anmeldung erfolgen soll.
	 * @param pidAsp
	 *            Pid des Aspekts, unter dem die Anmeldung erfolgen soll.
	 */
	public void anmeldenEmpfang(final String pidSystemObject,
			final String pidAtg, final String pidAsp) {
		final AttributeGroup anmeldeAtg = dataModell.getAttributeGroup(pidAtg);
		final Aspect anmeldeAsp = dataModell.getAspect(pidAsp);
		final DataDescription anmeldeDesc = new DataDescription(anmeldeAtg,
				anmeldeAsp);
		final ReceiveOptions receiveOptions = ReceiveOptions.normal();
		final ReceiverRole receiverRole = ReceiverRole.receiver();

		verbindung.subscribeReceiver(this,
				dataModell.getObject(pidSystemObject), anmeldeDesc,
				receiveOptions, receiverRole);

	}

	/**
	 * Abmelden vom Empfang der netzweiten, benutzerspezifischen Einstellungen.
	 */
	public void abmeldenEmpfang() {
		verbindung.unsubscribeReceiver(this, systemObject, dataDescription);
		DavEmpfangenSendenAenderungsDarstellung.LOGGER
				.fine("Abgemeldet Empfang");
	}

	/**
	 * Meldet vom Empfang ab.
	 * 
	 * @param pidSystemObject
	 *            Pid des {@link SystemObject}, von dem der Empfang abgemeldet
	 *            werden soll.
	 * @param pidAtg
	 *            Pid der Attributgruppe, von welcher der Empfang abgemeldet
	 *            werden soll.
	 * @param pidAsp
	 *            Pid des Aspekts, unter dem die Empfangsabmeldung erfolgen
	 *            soll.
	 */
	public void abmeldenEmpfang(final String pidSystemObject,
			final String pidAtg, final String pidAsp) {
		final SystemObject anmeldeObject = dataModell
				.getObject(pidSystemObject);
		final AttributeGroup anmeldeAtg = dataModell.getAttributeGroup(pidAtg);
		final Aspect anmeldeAsp = dataModell.getAspect(pidAsp);
		final DataDescription anmeldeDesc = new DataDescription(anmeldeAtg,
				anmeldeAsp);
		verbindung.unsubscribeReceiver(this, anmeldeObject, anmeldeDesc);
		DavEmpfangenSendenAenderungsDarstellung.LOGGER
				.fine("Abgemeldet Empfang");
	}

	@Override
	public void update(final ResultData[] results) {
		// System.out.println("=> " + results.length + " Elmente empfangen");

		for (final ResultData resultData : results) {
			// System.out.println("ResultData " + resultData);

			final DataTableObjectErweitert updateDto = new DataTableObjectErweitert(
					resultData);
			// System.out.println("=> DataTableObject erzeugt");

			/*
			 * TabellenModell aktualisieren
			 */
			if (tabellenModell != null) {
				tabellenModell.addDTO(updateDto);
				tabellenModell.reInit();

				/*
				 * Darstellung der Tabelle aktualisieren
				 */
				if ((kTable != null) && (display != null)) {
					display.asyncExec(new Runnable() {

						@Override
						public void run() {

							kTable.redraw();
						}

					});

				} else {
					DavEmpfangenSendenAenderungsDarstellung.LOGGER
							.warning("Keine aktualisierung mglich!");
				}
			}
		}
	}

	/**
	 * Einfgen von {@link Data}s in die Queue.
	 * 
	 * @param data
	 *            Einzufgendes {@link Data}
	 * @throws InterruptedException
	 *             Wird geworfen, wenn eine Unterbrechung eintritt.
	 */
	public void rein(final Data data) throws InterruptedException {
		lock.lock();
		try {
			while (produkte.size() == DavEmpfangenSendenAenderungsDarstellung.MAXQUEUE) {
				nichtVoll.await();
			}

			produkte.add(data);

			nichtLeer.signalAll();
		} finally {
			lock.unlock();
		}
	}

	/**
	 * Rausholen von {@link Data}s aus der Queue.
	 * 
	 * @return Jeweils das erste {@link Data} in der Queue
	 * @throws InterruptedException
	 *             Wird geworfen, wenn eine Unterbrechung eintritt.
	 */
	public Data raus() throws InterruptedException {
		lock.lock();
		try {
			while (produkte.size() == 0) {
				nichtLeer.await();
			}

			final Data elem = produkte.poll();

			nichtVoll.signalAll();
			return elem;
		} finally {
			lock.unlock();
		}
	}

	/**
	 * Setzt die DaV-Verbindung.
	 * 
	 * @param connection
	 *            DaV-Verbindung
	 */
	public void setDaVVerbindung(final ClientDavConnection connection) {
		this.verbindung = connection;
		if (connection != null) {
			dataModell = connection.getDataModel();
		}
	}

}
