/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.swt.graphics.Point;

import de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen.CellRenderer;
import de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen.FaltStatusSpalten;
import de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen.FixedCellRendererFaltbar;
import de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen.IFaltStautsSpalten;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.sys.funclib.debug.Debug;
import de.kupzog.ktable.KTableCellEditor;
import de.kupzog.ktable.KTableCellRenderer;
import de.kupzog.ktable.KTableDefaultModel;
import de.kupzog.ktable.editors.KTableCellEditorText;

/**
 * Erweitertes Tabellenmodell fr die Anzeige von SystemObjekten mit Ihren
 * Attributgruppen in einer KTable. Hierzu sind Attributgruppe und Aspekt
 * vorzugeben. Die Methode
 * {@link TabellenModell#addDTO(DataTableObjectErweitert)} ermglicht es,
 * weitere {@link DataTableObjectErweitert} zur Liste der TabellenObjekte
 * hinzuzufgen. Diese beinhalten die notwendigen Daten und knnen auch im
 * nachhinein zum Modell hinzugefgt werden.
 * 
 * @author HofmannJ
 * 
 */
public class TabellenModell extends KTableDefaultModel implements
		IFaltStautsSpalten {

	private static final Debug LOGGER = Debug.getLogger();

	public static final int STYLE_FLAT = 1 << 2;
	private final FixedCellRendererFaltbar fixedCellRendererDambach = new FixedCellRendererFaltbar(
			TabellenModell.STYLE_FLAT);

	/**
	 * Speichert die Faltungen der Spalten.
	 */
	private final FaltStatusSpalten faltStatusSpalten = new FaltStatusSpalten();

	private ListenDarstellungBestimmeKopfzeilen kopfzeilen;

	/**
	 * Wurzelknoten der Kopfzeile.
	 */
	private Knoten wurzelKopfZeile;

	/**
	 * Tiefste Ebene der Kopfzeile.
	 */
	private int tiefsteEbeneKopfzeile;

	/**
	 * Offset, wann die Spalten beginnen. Auf der linken Seite werden die
	 * Objekte mit Datum dargestellt (Spalte 0, 1 und 2).
	 */
	private static final int SPALTEN_OFFSET = 3;

	/**
	 * Lister aller Knoten, zusammengestellt via Tiefensuche.
	 */
	private LinkedList<Knoten> listeAllerKnotenDfs;

	/**
	 * Liste aller Objekte, die in der Tabelle dargestellt werden sollen.
	 */
	private LinkedList<DataTableObjectErweitert> tabellenObjekte = new LinkedList<DataTableObjectErweitert>();

	private int fixedHeaderRowCount;

	/**
	 * Anzahl der Zeilen im Datenbereich.
	 */
	private int rowCount;

	/**
	 * Der Renderer fr die Zellen.
	 */
	private CellRenderer cellRenderer;

	/**
	 * Definiert die Art der Darstellung. Die Voreinstellung ist
	 * {@link EnumDarstellungsArt#zustandsDarstellung}.
	 */
	private EnumDarstellungsArt darstellungsArt = EnumDarstellungsArt.zustandsDarstellung;

	/**
	 * Konstruktor.
	 * 
	 * @param dav
	 *            Verbindung zum Datenverteiler
	 * @param typ
	 *            Typ der Objekte.
	 * @param atg
	 *            Attributgruppe
	 * @param asp
	 *            Aspekt
	 * @param tabellenObjekte
	 *            Liste von Tabellenobjekten, die zur Darstellung gebracht
	 *            werden sollen. Ihr Typ, Attributruppe und Aspekt mssen den
	 *            Angaben der drei vorhergehenden Parameter entsprechen. Kann
	 *            auch <code>null</code> sein, dann wird eine leere Liste
	 *            erzeugt, welche nachtrglich durch
	 *            {@link TabellenModell#addDTO(DataTableObjectErweitert)}
	 *            befllbar ist.
	 */
	public TabellenModell(final ClientDavInterface dav, final String typ,
			final String atg, final String asp,
			final LinkedList<DataTableObjectErweitert> tabellenObjekte) {

		super();

		if (tabellenObjekte != null) {
			this.tabellenObjekte = tabellenObjekte;
		}

		/* Initialisierung des TabellenKopfes */
		initialisiereKopf(dav, typ, atg, asp);

		/* Initialisierung des Datenbereiches. */
		initialisiereDaten();
	}

	/**
	 * Setzt die Darstellungsart.
	 * 
	 * @param darstellungsArt
	 *            {@link EnumDarstellungsArt}
	 */
	public void setDarstellungsArt(final EnumDarstellungsArt darstellungsArt) {
		this.darstellungsArt = darstellungsArt;
	}

	/**
	 * Liefert die Darstellungsart.
	 * 
	 * @return {@link EnumDarstellungsArt}
	 */
	public EnumDarstellungsArt getDarstellungsArt() {
		return darstellungsArt;
	}

	/**
	 * Initialisiert den Kopf der Tabelle.
	 * 
	 * @param dav
	 *            Verbindung zum Datenverteiler
	 * @param typ
	 *            Objekt Typ
	 * @param atg
	 *            Attributgruppe
	 * @param asp
	 *            Aspekt
	 */
	private void initialisiereKopf(final ClientDavInterface dav,
			final String typ, final String atg, final String asp) {

		kopfzeilen = new ListenDarstellungBestimmeKopfzeilen(dav);
		kopfzeilen.setDaten(typ, atg, asp);
		kopfzeilen.bestimmeAttributeEinerAttributGruppe();
		wurzelKopfZeile = kopfzeilen.getWurzelKnoten();
		tiefsteEbeneKopfzeile = wurzelKopfZeile.bestimmeTiefsteEbene();
		berechneBreitenKopf();
		berecheKombiBereicheKopf();
	}

	/**
	 * Berechtnet die Breiten aller einzeiligen Zellen.
	 */
	private void berechneBreitenKopf() {
		for (int i = 0; i <= tiefsteEbeneKopfzeile; i++) {
			// logger.fine("Berechne Breiten der Ebene: " + i);

			final LinkedList<Knoten> erwKnoten = new LinkedList<Knoten>();
			Knoten vorgaenger = null;

			final Iterator<Knoten> it = wurzelKopfZeile.bestimmeKnotenAufEbene(
					i).iterator();
			while (it.hasNext()) {
				final Knoten knoten = it.next();

				final int breite = knoten.bestimmeKnotenOhneNachfolgerRek(
						knoten).size();

				// int tiefe = knoten.getTiefe();
				// System.out.println("Knoten:" + knoten.getSpaltenBezeichnung()
				// + " Tiefe: " + tiefe + " Breite: " + breite);

				if (vorgaenger == null) {
					final int von = 0;
					final int bis = (von + breite) - 1;

					knoten.setVon(von);
					knoten.setBis(bis);
					erwKnoten.add(knoten);

					vorgaenger = knoten;

				} else {

					final int von = vorgaenger.getBis() + 1;
					final int bis = (von + breite) - 1;
					knoten.setVon(von);
					knoten.setBis(bis);

					erwKnoten.add(knoten);
					vorgaenger = knoten;
				}
			}
		}
	}

	/**
	 * Berechnet die Bereiche (vonX, vonY, bisX, bisY) aller Zellen.
	 */
	private void berecheKombiBereicheKopf() {
		for (int ebene = 0; ebene <= tiefsteEbeneKopfzeile; ebene++) {
			final LinkedList<Knoten> knotenAufEbene = wurzelKopfZeile
					.bestimmeKnotenAufEbene(ebene);
			final Iterator<Knoten> it = knotenAufEbene.iterator();
			while (it.hasNext()) {
				final Knoten knoten = it.next();
				if (!knoten.hatNachfolger()) {
					knoten.setVonX(knoten.getVon());
					knoten.setVonY(ebene);
					knoten.setBisX(knoten.getBis());
					knoten.setBisY(tiefsteEbeneKopfzeile);

					// System.out.println("Knoten vonX: " + knoten.getVonX() + "
					// vonY: "
					// + knoten.getVonY() + " bisX:" + knoten.getBisX() + "
					// bisY: "
					// + knoten.getBisY());

				} else {
					// System.out.println("Knoten " +
					// knoten.getSpaltenBezeichnung()
					// + " hat Nachfolger");

					knoten.setVonX(knoten.getVon());
					knoten.setVonY(ebene);
					knoten.setBisX(knoten.getBis());
					knoten.setBisY(ebene);

					// System.out.println("Knoten vonX: " + knoten.getVonX() + "
					// vonY: "
					// + knoten.getVonY() + " bisX:" + knoten.getBisX() + "
					// bisY: "
					// + knoten.getBisY());

				}
			}
		}

		/**
		 * Bestimmung aller Knoten via DFS
		 */
		listeAllerKnotenDfs = wurzelKopfZeile.getListeAllerKnotenDfs();

	}

	protected void reInit() {
		initialisiereDaten();
	}

	/**
	 * Fhrt eine komplette Initialisierung der Daten dieser Tabelle durch.
	 * Diese Methode sollte initial aufgerufen werden, wenn eine Liste von
	 * {@link DataTableObjectErweitert} zur Anzeige bergeben wird.<br>
	 * Wird mittels der Methode
	 * {@link TabellenModell#addDTO(DataTableObjectErweitert)} nachtrglich ein
	 * {@link DataTableObjectErweitert} hinzugefgt, ist diese Methode
	 * <b>nicht</b> aufzurufen.
	 */
	private void initialisiereDaten() {
		/* Datenbereich, Auswertung der Zellen */

		int startOffsetZeile = 0;

		if ((tabellenObjekte != null) && (tabellenObjekte.size() > 0)) {
			final Iterator<DataTableObjectErweitert> it = tabellenObjekte
					.iterator();
			while (it.hasNext()) {
				final DataTableObjectErweitert dataTableObjectErweitert = it
						.next();

				startOffsetZeile = initialisiereEinzelnesDTO(
						dataTableObjectErweitert, startOffsetZeile);
			}

		}

		if (wurzelKopfZeile != null) {
			fixedHeaderRowCount = wurzelKopfZeile.bestimmeTiefsteEbene() + 1;
		}

		/*
		 * Initialisierung nach hinzufgen eines DataTableObjecterweitert
		 */
		initialisiereNachHinzufuegenDTO();
	}

	/**
	 * Fhrt abschlieende Initialisierungsoperationen nach dem Hinzufgen eines
	 * oder mehrerer {@link DataTableObjectErweitert} durch. Dazu gehren:<br>
	 * <ul>
	 * <li>Bestimmung der Anzahl der Zeilen (rowCount)</li>
	 * <li>Bestimmung von geraden und ungeraden Zeilen zwecks Frbung</li>
	 * </ul>
	 */
	private void initialisiereNachHinzufuegenDTO() {
		/*
		 * Berechnung der Anzahl der Zeilen, damit dies nicht stets aufs neue
		 * von doGetRowCount durchgefhrt werden muss.
		 */
		rowCount = berechneAnzahlZeilen();

		/*
		 * Bestimmen von geraden und ungeraden DataTableObjectErweitert zwecks
		 * Zeileneinfrbung.
		 */
		bestimmeGeradeUndUngeradeDataTableObjectErweitertZeilen();
	}

	/**
	 * Initialisiert ein einzelnes {@link DataTableObjectErweitert}.
	 * 
	 * @param dataTableObjectErweitert
	 *            Zu initialisierendes {@link DataTableObjectErweitert}.
	 * @param startOffsetZeile
	 *            Startoffset der Zeilen des zu initialisierenden
	 *            {@link DataTableObjectErweitert}.
	 * @return Liefert das StartOffset der Zeilen fr ein nchstes
	 *         {@link DataTableObjectErweitert}. Dies kann natrlich nur dann
	 *         verwendet werden, wenn dieses {@link DataTableObjectErweitert} in
	 *         der Liste der TabellenObjekte aufgenommen wurde.
	 */
	private int initialisiereEinzelnesDTO(
			final DataTableObjectErweitert dataTableObjectErweitert,
			final int startOffsetZeile) {

		/*
		 * Setzen des Start-Offsets
		 */
		dataTableObjectErweitert.setStartOffsetZeile(startOffsetZeile);
		// System.out.println("\t==>StartOffsetZeile: " +
		// startOffsetZeile);

		/*
		 * Besorgen des Data zwecks Auswertung der Zellenstruktur
		 */
		final Data data = dataTableObjectErweitert.getData();

		/*
		 * Erzeugen & Setzen der Wurzelzelle
		 */
		final Zelle wurzelZelle = new Zelle(data);
		dataTableObjectErweitert.setWurzelZelle(wurzelZelle);

		if (data != null) {
			if (data.isList()) {
				wurzelZelle.setListe(true);
			} else if (data.isArray()) {
				wurzelZelle.setArray(true);
			} else if (data.isPlain()) {
				wurzelZelle.setPlain(true);
			}
		} else {
			wurzelZelle.setPlain(true);
			wurzelZelle.setWert(dataTableObjectErweitert.getText());
			wurzelZelle.setBreite(wurzelKopfZeile
					.bestimmeKnotenOhneNachfolger().size());
		}

		/*
		 * Starten der Auswertung. Nachfolgende Kindzellen werden
		 * bercksichtigt.
		 */

		auswertenZelle(wurzelZelle);

		// System.err.println("Wurzlbreite: " +
		// wurzelZelle.getBreite(wurzelZelle));

		/*
		 * Bestimmen aller Plain-Zellen
		 */
		dataTableObjectErweitert.setPlainZellen(wurzelZelle
				.bestimmeAllePlainZellen());

		final Iterator<Zelle> itPlainZellen = dataTableObjectErweitert
				.getPlainZellen().iterator();

		while (itPlainZellen.hasNext()) {
			final Zelle zelle = itPlainZellen.next();

			zelle.setStartX(zelle.bestimmeStartPosX(zelle));
			zelle.setStartY(zelle.bestimmeStartPosY(zelle));
			zelle.setEndX(zelle.bestimmeEndPosX(zelle));

			// System.out.println("ZELLE(" + zelle.getDarstellung() + "): SX: "
			// + zelle.getStartX() + " SY: " + zelle.getStartY() + " EX: "
			// + zelle.getEndX());

		}

		/*
		 * Bestimmen der EndPosY
		 */
		dataTableObjectErweitert.getWurzelZelle().bestimmeEndPosY(
				dataTableObjectErweitert.getWurzelZelle());

		/*
		 * Setzen der tiefsten Ebene. Damit ist auch das Endoffset der Zeilen
		 * bestimmt
		 */
		final int tiefsteEbene = wurzelZelle
				.bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(wurzelZelle);
		// System.out.println("Tiefer: " + tiefsteEbene);
		dataTableObjectErweitert.setTiefsteEbene(tiefsteEbene);

		/*
		 * Erhhen des StartOffsets der Zeilen
		 */
		final int naechsteStartOffsetZeile = dataTableObjectErweitert
				.getEndOffsetZeile() + 1;

		// System.out.println("DATA: "
		// + dataTableObjectErweitert.getTiefsteEbene());
		// System.out.println("<======");

		// dataTableObjectErweitert.initialisiereMap();

		/*
		 * Ausgabe Plain-Zellen
		 */
		// itPlainZellen = dataTableObjectErweitert.getPlainZellen().iterator();
		// while (itPlainZellen.hasNext())
		// {
		// Zelle zelle = itPlainZellen.next();
		//
		// System.out.println("ZELLE(" + zelle.getDarstellung() + "): SX: "
		// + zelle.getStartX() + " SY: " + zelle.getStartY() + " EX: "
		// + zelle.getEndX() + " EY: " + zelle.getEndY());
		//
		// }
		return naechsteStartOffsetZeile;
	}

	/*
	 * private void analysiere(Data data, String rand) { Iterator it =
	 * data.iterator();
	 * 
	 * while (it.hasNext()) { Data data1 = (Data) it.next();
	 * 
	 * if (data1.isArray()) { analysiere(data1, rand + " "); }
	 * 
	 * if (data1.isList()) { analysiere(data1, rand + " "); }
	 * 
	 * if (data1.isArray()) { System.out.println("Achtung Array"); }
	 * 
	 * System.out.println(rand + "ANALYSIERE " + data1.toString()); } }
	 */

	/**
	 * Luft ber die Liste der TabellenObjekte vom Typ
	 * {@link DataTableObjectErweitert} und prft, ob die aktuelle
	 * Listenposition gerade oder ungerade ist. Im jeweiligen
	 * {@link DataTableObjectErweitert} wird das entsprechende Flag gesetzt. Auf
	 * diese Weise lassen sich die Zellen bzw. Zeilen von geraden/ungeraden
	 * {@link DataTableObjectErweitert} unterschiedlich einfrben.
	 */
	private void bestimmeGeradeUndUngeradeDataTableObjectErweitertZeilen() {
		int i = 0;
		final Iterator<DataTableObjectErweitert> it = tabellenObjekte
				.iterator();
		while (it.hasNext()) {
			final DataTableObjectErweitert objectErweitert = it.next();
			if ((i % 2) == 0) {
				objectErweitert.setGeradeZeile(true);
			} else {
				objectErweitert.setGeradeZeile(false);
			}

			i++;
		}
	}

	/**
	 * Hinzufgen eines {@link DataTableObjectErweitert} zur aktuellen
	 * Darstellung. Die Daten werden an die Liste der TabellenObjekte angehngt
	 * und es wird eine Initialisierung fr das entsprechende
	 * {@link DataTableObjectErweitert} durchgefhrt.<br>
	 * <br>
	 * <b>Achtung:</b> Auf das KTable-Widget ist hiernach ein redraw aufzurufen,
	 * das sonst keine Aktualisierung der Darstellung der Tabelle erfolgt.
	 * 
	 * @param wert
	 *            {@link DataTableObjectErweitert}, welches zur Liste der der
	 *            TabellenObjekte hinzugefgt wird.
	 */
	public void addDTO(final DataTableObjectErweitert wert) {

		// System.out.println("addDTO => " + wert.getData());

		if (tabellenObjekte.size() == 0) {
			wert.setAenderungsDarstellung(true);
		} else {
			final DataTableObjectErweitert letztesAenderungsDarstellungsObjekt = bestimmeLetztesAenderungsDarstellungsObjekt();

			String dataAlt = "";
			if (letztesAenderungsDarstellungsObjekt.getData() != null) {
				dataAlt = letztesAenderungsDarstellungsObjekt.getData()
						.toString();
			} else {
				dataAlt = letztesAenderungsDarstellungsObjekt.getText();
			}

			String dataNeu = "";
			if (wert.getData() != null) {
				dataNeu = wert.getData().toString();
			} else {
				dataNeu = wert.getText();
			}

			if (!dataNeu.equals(dataAlt)) {
				wert.setAenderungsDarstellung(true);
			}
		}

		if ((darstellungsArt.equals(EnumDarstellungsArt.zustandsDarstellung))
				|| ((darstellungsArt
						.equals(EnumDarstellungsArt.aenderungsDarstellung)) && (wert
						.isAenderungsDarstellung()))) {
			tabellenObjekte.add(wert);
			initialisiereNachHinzufuegenDTO();
		}

	}

	public DataTableObjectErweitert bestimmeLetztesAenderungsDarstellungsObjekt() {
		DataTableObjectErweitert re = null;
		final Iterator<DataTableObjectErweitert> it = tabellenObjekte
				.iterator();
		while (it.hasNext()) {
			final DataTableObjectErweitert dto = it.next();
			if (dto.isAenderungsDarstellung()) {
				// System.err.println("DTO letzte Zeile: " +
				// dto.getEndOffsetZeile());

				re = dto;
			}
		}

		return re;
	}

	/**
	 * In dieser Methode erfolgt die Auswertung der jeweiligen Zelle, dabei wird
	 * das jeweilige Data-Objekt der Zelle geprft. Ist die Zelle vom Typ:<br>
	 * <ul>
	 * <li>Plain, sind keine weiteren Aktionen notwendig.</li>
	 * <li>List, dann werden die Nachfolger Elemente der Liste ausgewertet und
	 * neue Instanzen der Klasse {@link Zelle} erzeugt und zur Liste der Zelle
	 * hinzugefgt. Die Eltern-Zelle der neu erzeugten Zelle wird automatisch
	 * gesetzt.</li>
	 * <li>Array, dann werden die Nachfolger Elemente des Arrays ausgewertet und
	 * neue Instanzen der Klasse {@link Zelle} erzeugt und zum Array der Zelle
	 * hinzugefgt. Die Elternzellen der neu erzeugten Zellen werden dabei
	 * automatisch gesetzt.</li>
	 * </ul>
	 * 
	 * @param zelle
	 *            {@link Zelle}, die ausgewertet werden soll.
	 */
	private void auswertenZelle(final Zelle zelle) {
		// System.out.println("Zelle: " + zelle.getWert() + " ISPLAIN: "
		// + zelle.isPlain());

		if (zelle.getWert() instanceof Data) {
			final Data data = (Data) zelle.getWert();

			final LinkedList<Zelle> vorgaenger = zelle.getVorgaenger();

			String liste = "";
			final Iterator<Zelle> itVorg = vorgaenger.iterator();
			while (itVorg.hasNext()) {
				final Zelle vorg = itVorg.next();
				liste = liste + "|" + vorg.getDarstellung() + "|";
			}

			if (data.isPlain()) {
				TabellenModell.LOGGER
						.finest("Plain-Data. Keine weitere Behandlung!");
			} else if (data.isList()) {
				final Iterator<Data> it = data.iterator();
				while (it.hasNext()) {
					final Data nextDataInList = it.next();
					Zelle listZelle = null;
					if (nextDataInList.isArray()) {
						listZelle = new Zelle(nextDataInList);
						listZelle.setArray(true);

					} else if (nextDataInList.isList()) {
						listZelle = new Zelle(nextDataInList);
						listZelle.setListe(true);
					} else if (nextDataInList.isPlain()) {
						listZelle = new Zelle(nextDataInList);
						listZelle.setPlain(true);
					}

					if (listZelle != null) {
						zelle.addToList(listZelle);
						auswertenZelle(listZelle);
					}
				}
			} else if (data.isArray()) {

				final Array dataArray = data.asArray();
				final int laengeArray = dataArray.getLength();

				if (laengeArray > 0) {
					for (int i = 0; i < laengeArray; i++) {
						final Data itemAusArray = dataArray.getItem(i);

						Zelle addZelle = null;
						if (itemAusArray.isList()) {
							addZelle = new Zelle(itemAusArray);
							addZelle.setListe(true);
						} else if (itemAusArray.isArray()) {
							if (/* itemAusArray.asArray().getLength() > 0 */true) {
								addZelle = new Zelle(itemAusArray);
								addZelle.setArray(true);
							}
						} else if (itemAusArray.isPlain()) {
							addZelle = new Zelle(itemAusArray);
							addZelle.setPlain(true);
						}

						if (addZelle != null) {
							zelle.addToArray(addZelle);
							auswertenZelle(addZelle);
						}
					}
				} else {
					initLeereArrayZelle(zelle);
				}
			}

		}

	}

	/**
	 * Fhrt die Initialisierung einer leeren Array-Zelle durch. Es wird hierzu
	 * folgende Strategie verwendet: Es wird ein Pfad aufgebaut, sowohl fr die
	 * Daten im Metamodell (Kopfzeilen) als auch fr die eigentlichen Daten
	 * (unter den Kopfzeilen) - fr die jeweils aktuelle Zelle. Ist der Pfad
	 * gleich, kann die maximale Breite der Nachfolgeknoten der bestimmten
	 * Kopfknotens bestimmt werden und bestimmt damit die Breite der leeren
	 * Datenzelle.
	 */
	protected void initLeereArrayZelle(final Zelle zelle) {
		final LinkedList<Knoten> listeArrayKnoten = new LinkedList<Knoten>();

		/*
		 * 1. Bestimmen aller Konten aus dem Kopf, die zu Arrays gehren
		 */
		final Iterator<Knoten> itArrayKnotenBestimmen = listeAllerKnotenDfs
				.iterator();
		while (itArrayKnotenBestimmen.hasNext()) {
			final Knoten knoten = itArrayKnotenBestimmen.next();
			if (knoten.isArray()) {
				listeArrayKnoten.add(knoten);
			}
		}

		if (zelle.getWert() instanceof Data) {
			final Data data = (Data) zelle.getWert();
			final String nameData = data.getName();

			/*
			 * Liste von gleich lautenden Knoten
			 */
			final LinkedList<Knoten> gleichLautendeKnoten = new LinkedList<Knoten>();

			final Iterator<Knoten> it = listeArrayKnoten.iterator();
			while (it.hasNext()) {
				final Knoten pruefKnoten = it.next();
				final Object wert = pruefKnoten.getWert();
				if (wert instanceof Attribute) {
					final Attribute att = (Attribute) wert;
					if (att.getName().equals(nameData)) {
						gleichLautendeKnoten.add(pruefKnoten);
					}
				}
			}

			Knoten gemerkterVergleichsKnoten = null;

			/*
			 * Pfade der gleich lautenden Knoten mit denen der Zellen
			 * vergleichen
			 */
			final Iterator<Knoten> itVergleich = gleichLautendeKnoten
					.iterator();
			while (itVergleich.hasNext()) {
				final Knoten vergleichsKnoten = itVergleich.next();
				final Object wert = vergleichsKnoten.getWert();
				if (wert instanceof Attribute) {
					/*
					 * Attribute att = (Attribute) wert;
					 * System.err.println("NAME: " + att.getName());
					 */
					if (vergleichePfad(zelle, vergleichsKnoten)) {
						gemerkterVergleichsKnoten = vergleichsKnoten;
						break;
					}
				}
			}

			/*
			 * Wenn ein Vergleichsknoten bestimmt werden konnte
			 */
			if (gemerkterVergleichsKnoten != null) {
				final int breite = gemerkterVergleichsKnoten
						.bestimmeKnotenOhneNachfolgerRek(
								gemerkterVergleichsKnoten).size();

				final Zelle leereZelle = new Zelle("");
				leereZelle.setPlain(true);
				leereZelle.setBreite(breite);
				zelle.addToArray(leereZelle);
				// System.err.println("Leere Zelle ergnzt, Breite: " + breite);
			}

		}

	}

	/**
	 * Vergleicht, ob Zelle und Knoten im gleichen Pfad liegen, wenn es sich bei
	 * den beiden um Arrays handelt.
	 * 
	 * @param zelle
	 *            Zu berprfende Zelle.
	 * @param knoten
	 *            Zu berprfender Knoten.
	 * @return Liefert <code>true</code>, wenn Zelle und Knoten im gleichen Pfad
	 *         liegen, sonst <code>false</code>.
	 */
	private boolean vergleichePfad(final Zelle zelle, final Knoten knoten) {
		boolean re = false;

		if (zelle.isArray() && knoten.isArray()) {
			final LinkedList<Zelle> pfadZelle = zelle.getVorgaenger();
			final LinkedList<Knoten> pfadKnoten = knoten.getVorgaenger();

			// System.err.println("PfadZelle Elmente: " + pfadZelle.size());
			// System.err.println("PfadKnoten Elemente: " + pfadKnoten.size());

			String zellPfad = "";
			final Iterator<Zelle> itZelle = pfadZelle.iterator();
			while (itZelle.hasNext()) {
				String add = "";
				final Zelle pruefZelle = itZelle.next();
				if (pruefZelle.getWert() instanceof Data) {
					final Data dataPruefZelle = (Data) pruefZelle.getWert();

					add = dataPruefZelle.getName();
					if (dataPruefZelle.isArray()) {
						add = add + " [Array]";
					}
				}

				// Wenn im Pfad keine Zahlen vorhanden sind (bezeichnen die
				// Nummer
				// des aktuellen Array-Elementes im Array, dann wird sie zum
				// Pfad
				// hinzugefgt. Auf diesee Weise lassen sich der Pfad der
				// Kopfzeilen
				// (reprsentieren das Modell) und der der Datenzellen
				// (reprsentieren
				// das Data) auf einander angleichen zur Bestimmung der
				// maximalen Breite
				// einer leeren Zelle. Evtl. gibt es hier eine bessere Lsung?!?

				try {
					Integer.parseInt(add);
				} catch (final NumberFormatException e) {
					TabellenModell.LOGGER.finer(e.getLocalizedMessage());
					zellPfad = zellPfad + " | " + add;
				}

			}

			String knotenPfad = "";
			final Iterator<Knoten> itKnoten = pfadKnoten.iterator();
			while (itKnoten.hasNext()) {
				final Knoten pruefKnoten = itKnoten.next();
				String add = "";

				if (pruefKnoten.getWert() instanceof Attribute) {
					add = ((Attribute) pruefKnoten.getWert()).getName();

					if (pruefKnoten.isArray()) {
						add = add + " [Array]";
					}
				} else if (pruefKnoten.getWert() instanceof AttributeGroup) {
					// add = ((AttributeGroup) pruefKnoten.getWert()).getName()
					// + "
					// [ATG]";
					add = ((AttributeGroup) pruefKnoten.getWert()).getPid();
				}

				knotenPfad = knotenPfad + " | " + add;

			}

			// System.err.println("Pfad Zellen: " + zellPfad);
			// System.err.println("Pfad Knoten: " + knotenPfad);

			if (zellPfad.equals(knotenPfad)) {
				// System.err.println("==>Pfad ist gleich!");
				re = true;
			}
		}

		return re;
	}

	/**
	 * Bestimmt das {@link DataTableObjectErweitert}, welches sich im
	 * angegebenen Zeilenbereich befindet.
	 * 
	 * @param zeilenNummer
	 *            Nummer der Zeile, zu der das zugehrige
	 *            {@link DataTableObjectErweitert} bestimmt werden soll.
	 * 
	 * @return Liefert das zur Zeile zugehrige {@link DataTableObjectErweitert}
	 *         oder <code>null</code>, wenn zu der angegebenen Zeile kein
	 *         {@link DataTableObjectErweitert} existiert.
	 */
	public DataTableObjectErweitert getDtoImZeilenBereich(final int zeilenNummer) {
		/*
		 * Bestimmen des DataTableObjectErweitert im entsprechenden
		 * Zeilenbereich
		 */
		DataTableObjectErweitert dtoImZeilenbereich = null;
		final Iterator<DataTableObjectErweitert> itDto = tabellenObjekte
				.iterator();
		while (itDto.hasNext()) {
			final DataTableObjectErweitert dataTableObjectErweitert = itDto
					.next();

			// if
			// ((_darstellungsArt.equals(EnumDarstellungsArt.zustandsDarstellung))
			// || ((_darstellungsArt
			// .equals(EnumDarstellungsArt.aenderungsDarstellung)) &&
			// (dataTableObjectErweitert
			// .isAenderungsDarstellung())))
			// {

			final int startOffsetZeile = dataTableObjectErweitert
					.getStartOffsetZeile();
			final int endOffsetZeile = dataTableObjectErweitert
					.getEndOffsetZeile();
			// System.err.println("Zeile (" + zeilenNummer + ") StartOffset: "
			// + startOffsetZeile + " EndOffset: " + endOffsetZeile);
			if ((zeilenNummer >= startOffsetZeile)
					&& (zeilenNummer <= endOffsetZeile)) {
				// System.err.println(">=<=");
				dtoImZeilenbereich = dataTableObjectErweitert;
				// System.out.println("Zeile (" + zeilenNummer +
				// ")StartOffset: "
				// + startOffsetZeile + " EndOffset: " + endOffsetZeile);
				break;
			}
			// }
		}

		return dtoImZeilenbereich;
	}

	@Override
	public KTableCellEditor doGetCellEditor(final int col, final int row) {
		if (isFixedCell(col, row)) {
			return null;
		}
		return new KTableCellEditorText();
	}

	@Override
	public KTableCellRenderer doGetCellRenderer(final int col, final int row) {
		if (isFixedCell(col, row)) {
			return fixedCellRendererDambach;
		}
		if (cellRenderer == null) {
			cellRenderer = new CellRenderer();
		}
		return cellRenderer;
	}

	@Override
	public int doGetColumnCount() {
		int re = 4;

		if (wurzelKopfZeile != null) {
			re = wurzelKopfZeile.bestimmeKnotenOhneNachfolger().size() + 3;
		}

		return re;
	}

	/**
	 * Diese Methode liefert den Inhalt einer Zelle, an Hand der Parameter
	 * Spalte und Zeile.
	 * 
	 * @param col
	 *            Spalte
	 * @param row
	 *            Zeile
	 * @return Objekt, das i.d. Zelle dargestellt werden soll.
	 */
	@Override
	public Object doGetContentAt(final int col, final int row) {

		Object re = "";

		if (col < TabellenModell.SPALTEN_OFFSET) {
			/*
			 * Bereich der Kopfzeilen
			 */
			if (row < getFixedHeaderRowCount()) {
				/* Spalten Art, Zeit & Objekt. */
				switch (col) {
				case 0:
					re = "Art";
					break;
				case 1:
					re = "Zeit";
					break;
				case 2:
					re = "Objekt";
					break;
				default:
					re = "???";
					break;
				}
			} else if (row >= getFixedHeaderRowCount()) {
				re = "_";

				final int zeile = row - getFixedHeaderRowCount();

				/*
				 * Bestimmen des DataTableObjectErweitert im entsprechenden
				 * Zeilenbereich
				 */
				final DataTableObjectErweitert dtoImZeilenbereich = getDtoImZeilenBereich(zeile);

				/*
				 * Wenn ein entsprechendes Objekt gefunden wurde, dann erfolgt
				 * die Auswertung, um was fr einen ArchiveDataKind es sich
				 * handelt. Diese werden genau wie im GTM dargestellt.
				 */
				if (dtoImZeilenbereich != null) {
					switch (col) {
					case 0:
						final ArchiveDataKind dataKind = dtoImZeilenbereich
								.getDataKind();
						if (dataKind == ArchiveDataKind.ONLINE) {
							re = "OA";

						} else if (dataKind == ArchiveDataKind.ONLINE_DELAYED) {
							re = "ON";
						} else if (dataKind == ArchiveDataKind.REQUESTED) {
							re = "NA";
						} else if (dataKind == ArchiveDataKind.REQUESTED_DELAYED) {
							re = "NN";
						} else {
							re = "??";
						}
						break;
					case 1:
						final DateFormat dfmt = new SimpleDateFormat(
								"dd.MM.yyyy HH.mm.ss,SSS");
						re = dfmt.format(new Date(dtoImZeilenbereich
								.getDataTime()));

						break;
					case 2:
						re = dtoImZeilenbereich.getObject().getNameOrPidOrId();
						break;
					default:
						re = "???";
						break;
					}
				}

			}
		} else if (col >= TabellenModell.SPALTEN_OFFSET) {
			/*
			 * Kopfzeilen
			 */
			if (row < getFixedHeaderRowCount()) {
				final int spalte = col - TabellenModell.SPALTEN_OFFSET;

				final LinkedList<Knoten> bestimmeKnotenAufEbene = wurzelKopfZeile
						.bestimmeKnotenAufEbene(row);
				final Iterator<Knoten> it = bestimmeKnotenAufEbene.iterator();
				while (it.hasNext()) {
					final Knoten knoten = it.next();

					if (knoten.istInBereich(spalte)) {
						re = knoten.getSpaltenBezeichnung();
					}
				}
			} else if (row >= getFixedHeaderRowCount()) {

				/*
				 * Datenbereich
				 */
				final int spalte = col - TabellenModell.SPALTEN_OFFSET;
				final int zeile = row - getFixedHeaderRowCount();
				// System.out.println("=> ZEILE: " + zeile + "SPALTE: " + spalte
				// + " "
				// + _plainZellenData.size());

				/*
				 * Bestimmen des DataTableObjectErweitert im entsprechenden
				 * Zeilenbereich
				 */
				final DataTableObjectErweitert dtoImZeilenbereich = getDtoImZeilenBereich(zeile);

				if (dtoImZeilenbereich != null) {
					/*
					 * Iterator ber alle Plain-Zellen. Nur diese werden
					 * dargestellt.
					 */
					final Iterator<Zelle> it = dtoImZeilenbereich
							.getPlainZellen().iterator();
					while (it.hasNext()) {
						final Zelle zelle = it.next();

						/*
						 * int plainZellenZeile = zeile -
						 * dtoImZeilenbereich.getStartOffsetZeile();
						 * 
						 * System.out.println("ZEILE: " + plainZellenZeile +
						 * " SPALTE: " + spalte);
						 */

						if (zelle.istInBereich(
								zeile
										- dtoImZeilenbereich
												.getStartOffsetZeile(), spalte)) {
							re = zelle.getDarstellung();
							// System.out.println("\t=>" + re);
						}
					}

				} else {
					/*
					 * Leeren Inhalt setzen
					 */
					re = "";
				}

			}

		}

		// System.out.println("doGetContentAt => ROW: " + row + " COL: " + col);
		// System.out.println("\tcontent = " + re);

		return re;
	}

	protected Knoten hatVorgaengerZelleOhneNachfolger(final int ebene,
			final int spalte) {
		Knoten re = null;
		if (ebene > 0) {
			final LinkedList<Knoten> knotenAufEbene = wurzelKopfZeile
					.bestimmeKnotenAufEbene(ebene - 1);
			final Iterator<Knoten> it = knotenAufEbene.iterator();
			while (it.hasNext()) {
				final Knoten knoten = it.next();
				if (!knoten.hatNachfolger()) {
					if (knoten.istInBereich(spalte)) {
						re = knoten;
						break;
					}
				}
			}
			if (re == null) {
				re = hatVorgaengerZelleOhneNachfolger(ebene - 1, spalte);
			}

		}
		return re;
	}

	/**
	 * Mit Hilfe dieser Methode werden die Zellenbereiche bestimmt, die zu einer
	 * Zelle gehren.<br>
	 * Die Fragestellung lautet: Zu welchem Zellenbereich gehrt die Zelle in
	 * Spalte X und Zeile Y? <br>
	 * Als Antwort wird die linke obere Zelle des entsprechenden Bereiches
	 * zurckgeliefert.<br>
	 * KTable erzeugt hieraus eine entsprechend groe, zusammenhngende Zelle.
	 */
	@Override
	public Point doBelongsToCell(final int col, final int row) {

		Point re = null;
		/* Erste Spalte im Kopf == "Objekt". */
		if ((col == 0) && (row < getFixedRowCount())) {
			re = new Point(col, 0);
		} else if ((col == 1) && (row < getFixedRowCount())) {
			re = new Point(col, 0);
		} else if ((col == 2) && (row < getFixedRowCount())) {
			re = new Point(col, 0);
		} else if ((col > 2) && (row <= getFixedHeaderRowCount())) {
			final LinkedList<Knoten> alleKnotenDfs = listeAllerKnotenDfs;
			final Iterator<Knoten> it = alleKnotenDfs.iterator();
			while (it.hasNext()) {
				final Knoten knoten = it.next();

				if (knoten.istInBereich(row, col
						- TabellenModell.SPALTEN_OFFSET)) {
					re = new Point(knoten.getVonX()
							+ TabellenModell.SPALTEN_OFFSET, knoten.getVonY());
				}
			}
		} else if ((col > 2) && (row > getFixedHeaderRowCount())) {
			/*
			 * Eigentliche Daten.
			 * 
			 * Bestimmen des DataTableObjectErweitert im entsprechenden
			 * Zeilenbereich
			 */
			final int zeilenNummer = row - getFixedHeaderRowCount();

			final DataTableObjectErweitert dtoImZeilenbereich = getDtoImZeilenBereich(zeilenNummer);

			if (dtoImZeilenbereich != null) {
				/* Bearbeiten aller Plain-Zellen. */
				final Iterator<Zelle> it = dtoImZeilenbereich.getPlainZellen()
						.iterator();
				while (it.hasNext()) {
					final Zelle zelle = it.next();

					final int zeile = row - getFixedHeaderRowCount()
							- dtoImZeilenbereich.getStartOffsetZeile();
					final int spalte = col - TabellenModell.SPALTEN_OFFSET;

					if (zelle.istInBereich(zeile, spalte)) {
						final int pointCol = zelle.getStartX()
								+ TabellenModell.SPALTEN_OFFSET;

						final int pointRow = zelle.getStartY()
								+ getFixedHeaderRowCount()
								+ dtoImZeilenbereich.getStartOffsetZeile();

						re = new Point(pointCol, pointRow);
					} else {
						TabellenModell.LOGGER.warning("ZELLE NICHT IN BEREICH");
					}
				}
			}
		} else if ((col <= 2) && (row > getFixedHeaderRowCount())) {
			/*
			 * Informationen zu den Objekten
			 */
			/*
			 * Bestimmen des DataTableObjectErweitert im entsprechenden
			 * Zeilenbereich
			 */
			DataTableObjectErweitert dtoImZeilenbereich = null;
			final Iterator<DataTableObjectErweitert> itDto = tabellenObjekte
					.iterator();
			while (itDto.hasNext()) {
				final DataTableObjectErweitert dataTableObjectErweitert = itDto
						.next();
				final int startOffsetZeile = dataTableObjectErweitert
						.getStartOffsetZeile();
				final int endOffsetZeile = dataTableObjectErweitert
						.getEndOffsetZeile();

				if ((row >= (startOffsetZeile + getFixedHeaderRowCount()))
						&& (row <= (endOffsetZeile + getFixedHeaderRowCount()))) {
					// System.out.println("Start: " + startOffsetZeile + " Ende:
					// " +
					// endOffsetZeile);
					dtoImZeilenbereich = dataTableObjectErweitert;
					break;
				}
			}

			if (dtoImZeilenbereich != null) {
				final int startOffsetZeile = dtoImZeilenbereich
						.getStartOffsetZeile();
				re = new Point(col, startOffsetZeile + getFixedHeaderRowCount());
			}

		}

		// System.out.println("POINT: " + re);

		return re;
	}

	public boolean pruefeImBereichNaechsteZeilen(final int zeile,
			final int von, final int bis) {
		boolean re = false;
		final LinkedList<Knoten> knotenAufEbene = wurzelKopfZeile
				.bestimmeKnotenAufEbene(zeile);
		final Iterator<Knoten> it = knotenAufEbene.iterator();
		while (it.hasNext()) {
			final Knoten knoten = it.next();
			if ((knoten.istInBereich(von)) || (knoten.istInBereich(bis))) {
				re = true;
				break;
			}
		}

		return re;
	}

	public int berechneAnzahlZeilen() {
		int calculatedRowCount = tiefsteEbeneKopfzeile + 1;
		final Iterator<DataTableObjectErweitert> it = tabellenObjekte
				.iterator();
		while (it.hasNext()) {
			final DataTableObjectErweitert tabellenObjekt = it.next();

			if (darstellungsArt
					.equals(EnumDarstellungsArt.aenderungsDarstellung)) {
				if (tabellenObjekt.isAenderungsDarstellung()) {
					calculatedRowCount = calculatedRowCount
							+ tabellenObjekt.getTiefsteEbene();
				}
			} else {
				calculatedRowCount = calculatedRowCount
						+ tabellenObjekt.getTiefsteEbene();
			}
		}

		return calculatedRowCount;
	}

	@Override
	public int doGetRowCount() {
		return rowCount;
	}

	@Override
	public void doSetContentAt(final int col, final int row, final Object value) {
		System.out.println("==>Setze - Spalte: " + col + " Zeile: " + row
				+ " Wert: " + value);

		/*
		 * Vorgehensweise: -Suchen des zugehrigen DataTableObject an Hand der
		 * Start- und Endzeile -Anpassen der Zeilen an Hand des Offsets der
		 * Startzeile -Zugehriges Objekt der Zelle suchen -Wert anpassen
		 * -Dirty-Flag setzen
		 */

	}

	@Override
	public int getInitialColumnWidth(final int column) {
		int re = 50;

		switch (column) {
		case 0:
			re = 40;
			break;
		case 1:
			re = 150;
			break;
		case 2:
			re = 150;
			break;
		default:
			re = 10;
			break;

		}

		return re;
	}

	@Override
	public int getInitialRowHeight(final int row) {
		return 20;
	}

	@Override
	public int getFixedHeaderColumnCount() {
		return 3;
	}

	@Override
	public int getFixedHeaderRowCount() {
		return fixedHeaderRowCount;
	}

	@Override
	public int getFixedSelectableColumnCount() {
		return 0;
	}

	@Override
	public int getFixedSelectableRowCount() {
		return 0;
	}

	@Override
	public int getRowHeightMinimum() {
		return 15;
	}

	@Override
	public boolean isColumnResizable(final int col) {
		return true;
	}

	@Override
	public boolean isRowResizable(final int row) {
		return false;
	}

	@Override
	public FaltStatusSpalten getFaltStatusSpalten() {
		return faltStatusSpalten;
	}

	/**
	 * Liefert die Wurzel der Kopfzeile.
	 * 
	 * @return Wurzel der Kopfzeile.
	 */
	public Knoten getWurzelKopfzeile() {
		return wurzelKopfZeile;
	}
}
