/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AeusseresStrassenSegment;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.InneresStrassenSegment;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.KonfigurationNeu;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Strasse;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenSegment;
import de.bsvrz.dav.daf.main.ClientDavConnection;

public class FilterStrassenSegmentOffset implements Cloneable {
	/**
	 * Verbindung zum DaV.
	 */
	private final ClientDavConnection davConnection;

	/**
	 * Konfiguration erzeugen.
	 */
	private final KonfigurationNeu konf;

	/**
	 * Speichert die Pid der selektierten Strae.
	 */
	private String pidStrasse;

	/**
	 * Speichert die Pid des selektierten StraenSegments.
	 */
	private String pidStrassenSegment;

	/**
	 * Speichert die eingegebenen Kilometer als float.
	 */
	private long km = -1;

	/**
	 * Konstruktor.
	 * 
	 * @param connection
	 */
	public FilterStrassenSegmentOffset(final ClientDavConnection connection) {
		this.davConnection = connection;
		konf = KonfigurationNeu.getInstanz();
		konf.bestimmeObjekte(davConnection);
	}

	/**
	 * Liefert eine Liste aller {@link Strasse}en zurck.
	 * 
	 * @return Liste aller {@link Strasse}en
	 */
	public LinkedList<Strasse> getStrassen() {
		final LinkedList<Strasse> strassen = new LinkedList<Strasse>();

		final TreeMap<String, Strasse> tmSortiert = new TreeMap<String, Strasse>();
		tmSortiert.putAll(konf.getStrassen());

		final Iterator<Entry<String, Strasse>> it = tmSortiert.entrySet()
				.iterator();
		while (it.hasNext()) {
			final Entry<String, Strasse> entry = it.next();
			strassen.add(entry.getValue());
		}
		return strassen;
	}

	/**
	 * Liefert eine Liste von {@link StrassenSegment}en (bestehend aus
	 * {@link InneresStrassenSegment}en und {@link AeusseresStrassenSegment}en),
	 * die innerhalb einer {@link Strasse} vorhanden sind, zurck.
	 * 
	 * Gleichzeitig wird die PID der selektierten {@link Strasse} gesetzt.
	 * 
	 * @param pid
	 *            PID der {@link Strasse}, zu der die
	 *            {@link AeusseresStrassenSegment}e und
	 *            {@link InneresStrassenSegment}e geliefert werden sollen.
	 * @return Liste von {@link StrassenSegment}en.
	 */
	public LinkedList<StrassenSegment> getStrassenSegmenteGefiltert(
			final String pid) {

		this.pidStrasse = pid;

		final LinkedList<StrassenSegment> strassenSegmenteGefiltert = new LinkedList<StrassenSegment>();

		/*
		 * Filtern von ueren/Innerenstraensegmenten passend zur selektierten
		 * Strae
		 */
		konf.filtereNachStrasse(pid);

		/*
		 * uereStraenSegmente hinzufgen
		 */
		final Iterator<Entry<String, AeusseresStrassenSegment>> itAeussereStrassenSegmente = konf
				.getAeussereStrassenSegmenteGefiltert().entrySet().iterator();
		while (itAeussereStrassenSegmente.hasNext()) {
			final Entry<String, AeusseresStrassenSegment> entry = itAeussereStrassenSegmente
					.next();
			final AeusseresStrassenSegment aeusseresStrassenSegment = entry
					.getValue();
			strassenSegmenteGefiltert.add(aeusseresStrassenSegment);
		}

		/*
		 * InnereStraenSegmente hinzufgen
		 */
		final Iterator<Entry<String, InneresStrassenSegment>> itInnereStrassenSegmente = konf
				.getInnereStrassenSegmenteGefiltert().entrySet().iterator();
		while (itInnereStrassenSegmente.hasNext()) {
			final Entry<String, InneresStrassenSegment> entry = itInnereStrassenSegmente
					.next();
			final InneresStrassenSegment inneresStrassenSegment = entry
					.getValue();
			strassenSegmenteGefiltert.add(inneresStrassenSegment);
		}

		return strassenSegmenteGefiltert;

	}

	/**
	 * Liefert true zurck, wenn die angegebene Kilometerzahl kleiner oder
	 * gleich der der Lnge des jeweiligen Segmentes ist. <br>
	 * Passt der angegebene KM, wird dieser gleichzeitig gesetzt.
	 * 
	 * @param pruefKm
	 *            in Metern als float
	 * @param pidSegment
	 *            Schlssel des jeweiligen Segments
	 * @return true, falls die Lnge des Segmentes nicht berschritten wird,
	 *         sonst false
	 */
	public boolean pruefeKilometerSegment(final long pruefKm,
			final String pidSegment) {
		// String pidSegment = _pidStrassenSegment;

		boolean re = false;

		if (pidSegment != null) {
			final Map<String, InneresStrassenSegment> innen = konf
					.getInnereStrassenSegmenteGefiltert();
			final Map<String, AeusseresStrassenSegment> aussen = konf
					.getAeussereStrassenSegmenteGefiltert();

			StrassenSegment segment = null;

			// Bei den inneren Segmenten nachsehen
			if (innen.containsKey(pidSegment)) {
				segment = innen.get(pidSegment);
			}

			// Bei den aeusseren Segmenten nachsehen
			if (segment == null) {
				if (aussen.containsKey(pidSegment)) {
					segment = aussen.get(pidSegment);
				}
			}

			if (segment != null) {
				final long laenge = segment.getLaenge();

				if (pruefKm > laenge) {
					re = false;
				} else if ((pruefKm <= laenge) && (pruefKm >= 0)) {
					re = true;
					this.km = pruefKm;
				} else {
					re = false;
				}

			}

		}
		return re;
	}

	/**
	 * Liefert true zurck, wenn die angegebene Kilometerzahl kleiner oder
	 * gleich der der Lnge des jeweiligen Segmentes ist. <br>
	 * Passt der angegebene KM, wird dieser gleichzeitig gesetzt.
	 * 
	 * @param pruefKm
	 *            in Metern als float
	 * @param pidSegment
	 *            Schlssel des jeweiligen Segments
	 * @return true, falls die Lnge des Segmentes nicht berschritten wird,
	 *         sonst false
	 */
	public boolean pruefeKilometerSegment(final float pruefKm,
			final String pidSegment) {
		final Long l = (long) (pruefKm * 1000.0);

		boolean re = false;

		if (pidSegment != null) {
			final Map<String, InneresStrassenSegment> innen = konf
					.getInnereStrassenSegmenteGefiltert();
			final Map<String, AeusseresStrassenSegment> aussen = konf
					.getAeussereStrassenSegmenteGefiltert();

			StrassenSegment segment = null;

			// Bei den inneren Segmenten nachsehen
			if (innen.containsKey(pidSegment)) {
				segment = innen.get(pidSegment);
			}

			// Bei den aeusseren Segmenten nachsehen
			if (segment == null) {
				if (aussen.containsKey(pidSegment)) {
					segment = aussen.get(pidSegment);
				}
			}

			if (segment != null) {
				final long laenge = segment.getLaenge();

				if (l > laenge) {
					re = false;
				} else if ((l <= laenge) && (l >= 0)) {
					re = true;
					this.km = l;
				} else {
					re = false;
				}

			}

		}
		return re;
	}

	/**
	 * Liefert die PID der selektierten {@link Strasse}.
	 * 
	 * @return PID der selektierten {@link Strasse}
	 */
	public String getPidSelektierteStrasse() {
		return pidStrasse;
	}

	/**
	 * Setzt die Pid der selektierten Strae.
	 * 
	 * @param pid
	 *            Pid der selektierten Strae
	 */
	public void setPidSelektierteStrasse(final String pid) {
		this.pidStrasse = pid;
	}

	/**
	 * Liefert die PID des selektierten {@link StrassenSegment}es.
	 * 
	 * @return PID des selektierten {@link StrassenSegment}es
	 */
	public String getPidSelektiertesStrassenSegment() {
		return pidStrassenSegment;
	}

	/**
	 * Setzt die PID des selektierten {@link StrassenSegment}es.
	 * 
	 * @param pid
	 */
	public void setPidSelektiertesStrassenSegment(final String pid) {
		this.pidStrassenSegment = pid;
	}

	/**
	 * Liefert die KM.
	 * 
	 * @return KM
	 */
	public long getKm() {
		return km;
	}

	/**
	 * Setzt die KM.
	 * 
	 * @param km
	 *            Zu setzende KM.
	 */
	public void setKm(final long km) {
		this.km = km;
	}

	/**
	 * Setzt die KM.
	 * 
	 * @param km
	 *            Zu setzende KM.
	 */
	public void setKm(final float km) {
		// Sans 20091124
		// km*=1000.0;
		this.km = (long) (km * 1000.0);
	}

	@Override
	public String toString() {
		String re = "";
		re = re + "===" + getClass().getName() + "===\n";

		re = re + "PID Strae:         " + pidStrasse + "\n";
		re = re + "PID StraenSegment: " + pidStrassenSegment + "\n";
		re = re + "KM:                 " + km + "\n";

		for (int i = 0; i < (6 + getClass().getName().length()); i++) {
			re = re + "=";
		}

		re = re + "\n";

		return re;
	}

	@Override
	public Object clone() {
		try {
			super.clone();
		} catch (final CloneNotSupportedException e) {
			throw new InternalError();
		}
		final FilterStrassenSegmentOffset re = new FilterStrassenSegmentOffset(
				davConnection);
		re.pidStrasse = pidStrasse;
		re.pidStrassenSegment = pidStrassenSegment;
		re.km = km;
		return re;
	}

}
