/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse realisiert ein ueres Straensegment.
 * 
 * @author Dambach Werke GmbH
 */
public class AeusseresStrassenSegment extends StrassenSegment {
	/**
	 * zum Straensegment gehrender Startknoten.
	 */
	private StrassenKnoten vonKnoten;

	/**
	 * zum Straensegment gehrender Zielknoten.
	 */
	private StrassenKnoten nachKnoten;

	/**
	 * zum Straensegment gehrende TMC Richtung.
	 */
	private String tmcRichtung;

	/**
	 * zum Straensegment gehrende Straenbezeichnung.
	 */
	private String strasse;

	/**
	 * Datenmodell des DaV.
	 */
	private final DataModel davDataModel;

	/**
	 * Konstruktor der Klasse.
	 * 
	 * @param dataModel
	 *            Datenmodell
	 * @param objekt
	 *            ueres Straensegment
	 */
	public AeusseresStrassenSegment(final DataModel dataModel,
			final SystemObject objekt) {
		super(dataModel, objekt);

		davDataModel = dataModel;

		final AttributeGroup atg = davDataModel
				.getAttributeGroup("atg.ueresStraenSegment");

		if (atg != null) {
			final Data confData = objekt.getConfigurationData(atg);

			if (confData != null) {
				final SystemObject o1 = confData.getReferenceValue("vonKnoten")
						.getSystemObject();
				if (o1 != null) {
					vonKnoten = KonfigurationNeu.getInstanz()
							.getStrassenKnoten(o1.getPid());
				}

				final SystemObject o2 = confData
						.getReferenceValue("nachKnoten").getSystemObject();
				if (o2 != null) {
					nachKnoten = KonfigurationNeu.getInstanz()
							.getStrassenKnoten(o2.getPid());
				}

				tmcRichtung = confData.getScaledValue("TmcRichtung")
						.getValueText();

				strasse = getStrassenNummer(objekt);
			}
		}
	}

	/**
	 * Methode bestimmt den Strassennamen zu einem Straensegment.
	 * 
	 * @param strassenSegment
	 *            Straensegment
	 * @return String mit der Nummer der Strae
	 */
	private String getStrassenNummer(final SystemObject strassenSegment) {
		String nr = null;

		if (strassenSegment == null) {
			return nr;
		}

		final AttributeGroup atg = davDataModel
				.getAttributeGroup("atg.straenSegment");

		final Data confDataSS = strassenSegment.getConfigurationData(atg);

		if (confDataSS != null) {
			final String strassenObject = confDataSS.getReferenceValue(
					"gehrtZuStrae").getValueText();
			if (strassenObject != null) {
				final SystemObject soStr = davDataModel
						.getObject(strassenObject);
				if (soStr != null) {
					final Data confDataStr = soStr
							.getConfigurationData(davDataModel
									.getAttributeGroup("atg.strae"));
					if (confDataStr != null) {
						nr = confDataStr.getTextValue("Nummer").getValueText();
					}
				}
			}
		}

		return nr;
	}

	/**
	 * @return liefert die Variable nachKnoten zurck
	 */
	public StrassenKnoten getNachKnoten() {
		return nachKnoten;
	}

	/**
	 * Setzt fr den nachKnoten den {@link StrassenKnoten}.
	 * 
	 * @param nachKnoten
	 *            {@link StrassenKnoten}
	 */
	public void setNachKnoten(final StrassenKnoten nachKnoten) {
		this.nachKnoten = nachKnoten;
	}

	/**
	 * @return liefert die Variable strasse zurck.
	 */
	public String getStrasse() {
		return strasse;
	}

	/**
	 * @return liefert die Variable tmcRichtung zurck
	 */
	public String getTmcRichtung() {
		return tmcRichtung;
	}

	/**
	 * @return liefert die Variable vonKnoten zurck
	 */
	public StrassenKnoten getVonKnoten() {
		return vonKnoten;
	}

	/**
	 * Setzt fr den <code>vonKnoten</code> den {@link StrassenKnoten}.
	 * 
	 * @param vonKnoten
	 *            {@link StrassenKnoten}
	 */
	public void setVonKnoten(final StrassenKnoten vonKnoten) {
		this.vonKnoten = vonKnoten;
	}

	/**
	 * Methode prft, ob zwei ussere Straensegmente auf der selben Strae
	 * liegen.
	 * 
	 * @param segment
	 *            zu prfendes Straensegment
	 * @return selbe Strae: true, sonst false
	 */
	public boolean liegtAufSelberStrasse(final AeusseresStrassenSegment segment) {
		if (segment != null) {
			if (tmcRichtung.equals(segment.getTmcRichtung())
					&& strasse.equals(segment.getStrasse())) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Methode liefert bei ueren Straensegmenten die Pid des Systemobjekts,
	 * erweitert um die TMC-Richtung und die Nummer der Strasse.
	 * 
	 * @return erweiterte Pid, im Fehlerfall null
	 */
	@Override
	public String getPidRichtungStrasse() {
		if (super.getSystemObject() != null) {
			return super.getPid() + " (" + strasse + "/" + tmcRichtung + ")";
		}

		return null;
	}

	/**
	 * Liefert die TMC-Richtung der Strae.
	 * 
	 * @return TMC-Richtung der Strae
	 */
	public String getRichtungStrasse() {
		if (super.getSystemObject() != null) {
			return tmcRichtung;
		}
		return null;
	}

	/**
	 * Methode prft ob es sich bei dem Straensegment um ein ueres
	 * Straensegment handelt.
	 * 
	 * @return bei uerem Straensegment: true, sonst: false
	 */
	@Override
	public boolean isAeusseresStrassenSegment() {
		return true;
	}

	/**
	 * Methode prft das Straensegmenten in einem Autobahnkreuz oder
	 * Autobahndreieck beginnt.
	 * 
	 * @return Autobahnkreuz, Autobahndreick: true, sonst false
	 */
	@Override
	public boolean beginntInAutobahnKreuzOderDreieck() {
		if (vonKnoten != null) {
			return (vonKnoten.isAutobahnKreuz() || vonKnoten
					.isAutobahnDreieck());
		}

		return false;
	}

	/**
	 * Methode prft das Straensegmenten in einem Autobahnkreuz oder
	 * Autobahndreieck endet.
	 * 
	 * @return Autobahnkreuz, Autobahndreick: true, sonst false
	 */
	@Override
	public boolean endetInAutobahnKreuzOderDreieck() {
		if (nachKnoten != null) {
			return (nachKnoten.isAutobahnKreuz() || nachKnoten
					.isAutobahnDreieck());
		}

		return false;
	}

	/**
	 * Methode prft ob das Straensegmenten am Autobahnanfang beginnt.
	 * 
	 * @return Autobahnanfang: true, sonst false
	 */
	@Override
	public boolean beginntAmAutobahnAnfang() {
		if (vonKnoten != null) {
			return (vonKnoten.isAutobahnEnde());
		}

		return false;
	}

	/**
	 * Methode prft ob das Straensegmenten am Autobahnende endet.
	 * 
	 * @return Autobahnende: true, sonst false
	 */
	@Override
	public boolean endetAmAutobahnEnde() {
		if (nachKnoten != null) {
			return nachKnoten.isAutobahnEnde();
		}

		return false;
	}

	public String toStringLang() {
		String re = "";
		re = re + "ueresStraenSegment: " + getSystemObject().getPid() + "\n";
		re = re + "Richtung: " + tmcRichtung + "\n";
		re = re + "Nach Knoten: ";
		if (nachKnoten != null) {
			re = re + nachKnoten.getPid() + "\n";
		} else {
			re = re + "null\n";
		}

		re = re + "Von Knoten: ";
		if (vonKnoten != null) {
			re = re + vonKnoten.getPid() + "\n";
		} else {
			re = re + "null\n";
		}

		return re;
	}

	/**
	 * Liefert eine Kurzdarstellung dieses {@link AeusseresStrassenSegment}es.
	 * 
	 * @return Kurzdarstellung dieses {@link AeusseresStrassenSegment}es
	 */
	@Override
	public String toString() {
		String re = "";
		re = re + getSystemObject().getName() + " (PID: "
				+ getSystemObject().getPid() + ")";

		return re;
	}

}
