/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenTeilSegment.AsbStationierung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse stellt smtliche Informationen bereit die notwendig sind, um
 * Ortsangaben ber uere Straensegmente durchzufhren. Es werden geliefert:<br>
 * <ul>
 * <li>Straen</li>
 * <li>Alle Knoten der ASB-Ortsangabe, die auf dieser Strae verfgbar sind</li>
 * <li>Informationen ber die zu einem ASB-Knoten zugehrigen, erreichbaren
 * Knoten</li>
 * <li>Prfung, ob eine ASB-Stationierung im Bereich zwischen zwei Knoten liegt</li>
 * </ul>
 * 
 * @author Dambach Werke GmbH
 * 
 */
public class AsbOrtsangabe {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Verbindung zum DaV.
	 */
	private final ClientDavConnection verbindung;

	/**
	 * Enthlt die Instanz der Klasse {@link KonfigurationNeu}, welche die hier
	 * bentigten Datenstrukturen zur Verfgung stellt.
	 */
	private KonfigurationNeu konf;

	/**
	 * {@link Struktur} in positiver TMC-Richtung.
	 */
	private Struktur strukturPos;

	/**
	 * {@link Struktur} in negativer TMC-Richtung.
	 */
	private Struktur strukturNeg;

	/**
	 * Referenz auf die {@link AsbKnotenVerwaltung}, positive Richtung.
	 */
	private AsbKnotenVerwaltung asbKnotenVerwaltungPositiv;

	/**
	 * Referenz auf die {@link AsbKnotenVerwaltung}, negative Richtung.
	 */
	private AsbKnotenVerwaltung asbKnotenVerwaltungNegativ;

	private AsbKnotenVerwaltungKombiniert asbKnotenVerwaltungKombiniert;

	/**
	 * Speichert eine Liste aller zu einer Strae verfgbaren AsbKnoten. Wird
	 * durch die Methode {@link AsbOrtsangabe#filtereAsbKnoten()}, die von der
	 * Methode {@link AsbOrtsangabe#initialisiereStrukturNachStrasse(String)}
	 * aufgerufen wird, gefllt.
	 */
	private List<AsbKnoten> listeKnotenAsbGesamt = new LinkedList<AsbKnoten>();

	/**
	 * Konstruktor. bernimmt die Verbindung zum DaV und fhrt die
	 * Basis-Initialisierung durch.
	 * 
	 * @param verbindung
	 */
	public AsbOrtsangabe(final ClientDavConnection verbindung) {
		this.verbindung = verbindung;
		einlesenBasisStruktur();
	}

	/**
	 * Liest die Basisstruktur ein.
	 */
	private void einlesenBasisStruktur() {
		if (verbindung != null) {
			konf = KonfigurationNeu.getInstanz();
			konf.bestimmeObjekte(verbindung);
		} else {
			AsbOrtsangabe.LOGGER
					.error(getClass().getName()
							+ "::einlesenBasisStruktur \n\t=>Es wurde keine Verbindung zum DaV bergeben. Kann die Datenstrukturen nicht aus dem DaV einlesen.");
		}

	}

	/**
	 * Baut die Struktur auf.
	 * 
	 * @param pidSelektierteStrasse
	 *            PID der Strae, zu der die Struktur aufgebaut werden soll.
	 */
	public void initialisiereStrukturNachStrasse(
			final String pidSelektierteStrasse) {
		konf.filtereNachStrasse(pidSelektierteStrasse);

		/*
		 * Liste der uerenStraenSegmente in positiver Richtung
		 * (atg.ueresStraenSegment Attribut TmcRichtung)
		 */
		final LinkedList<AeusseresStrassenSegment> aeussereStrassenSegmentePositiv = new LinkedList<AeusseresStrassenSegment>();

		/*
		 * Liste der uerenStraenSegmente in negativer Richtung
		 * (atg.ueresStraenSegment Attribut TmcRichtung)
		 */
		final LinkedList<AeusseresStrassenSegment> aeussereStrassenSegmenteNegativ = new LinkedList<AeusseresStrassenSegment>();

		/*
		 * Liste der Knoten der uerenStraenSegmente in positiver Richtung
		 */
		final LinkedList<StrassenKnoten> knotenPositiv = new LinkedList<StrassenKnoten>();

		/*
		 * Liste der Knoten der uerenStraenSegmente in negativer Richtung
		 */
		final LinkedList<StrassenKnoten> knotenNegativ = new LinkedList<StrassenKnoten>();

		final Iterator<Entry<String, AeusseresStrassenSegment>> it = konf
				.getAeussereStrassenSegmenteGefiltert().entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, AeusseresStrassenSegment> entry = it.next();
			final AeusseresStrassenSegment aeusseresStrassenSegment = entry
					.getValue();
			if ("positiv".equals(aeusseresStrassenSegment.getRichtungStrasse())) {
				aeussereStrassenSegmentePositiv.add(aeusseresStrassenSegment);
				// VON-Knoten
				if ((aeusseresStrassenSegment.getVonKnoten() != null)
						&& (!knotenPositiv.contains(aeusseresStrassenSegment
								.getVonKnoten()))) {
					knotenPositiv.add(aeusseresStrassenSegment.getVonKnoten());
				}
				// NACH-Knoten
				if ((aeusseresStrassenSegment.getNachKnoten() != null)
						&& (!knotenPositiv.contains(aeusseresStrassenSegment
								.getNachKnoten()))) {
					knotenPositiv.add(aeusseresStrassenSegment.getNachKnoten());
				}

			} else if ("negativ".equals(aeusseresStrassenSegment
					.getRichtungStrasse())) {
				aeussereStrassenSegmenteNegativ.add(aeusseresStrassenSegment);
				// VON-Knoten
				if ((aeusseresStrassenSegment.getVonKnoten() != null)
						&& (!knotenNegativ.contains(aeusseresStrassenSegment
								.getVonKnoten()))) {
					knotenNegativ.add(aeusseresStrassenSegment.getVonKnoten());
				}

				// NACH-KNOTEN
				if ((aeusseresStrassenSegment.getNachKnoten() != null)
						&& (!knotenNegativ.contains(aeusseresStrassenSegment
								.getNachKnoten()))) {
					knotenNegativ.add(aeusseresStrassenSegment.getNachKnoten());
				}
			}

		}

		AsbOrtsangabe.LOGGER.fine("uereStraenSegmente (positiv): "
				+ aeussereStrassenSegmentePositiv.size());
		AsbOrtsangabe.LOGGER.fine("uereStraenSegmente (negativ): "
				+ aeussereStrassenSegmenteNegativ.size());
		AsbOrtsangabe.LOGGER.fine("StraenKnoten (positiv): "
				+ knotenPositiv.size());
		AsbOrtsangabe.LOGGER.fine("StraenKnoten (negativ): "
				+ knotenNegativ.size());

		/*
		 * Struktur (TmcRichtung positiv) erzeugen
		 */
		strukturPos = new Struktur("positiv");

		// uereStraenSegmente hinzufgen
		final Iterator<AeusseresStrassenSegment> itAePos = aeussereStrassenSegmentePositiv
				.iterator();
		while (itAePos.hasNext()) {
			final AeusseresStrassenSegment aeusseresStrassenSegment = itAePos
					.next();
			strukturPos.erzeugeStrukturElement(aeusseresStrassenSegment);
		}

		// StraenKnoten hinzufgen
		final Iterator<StrassenKnoten> itSKPos = knotenPositiv.iterator();
		while (itSKPos.hasNext()) {
			final StrassenKnoten strassenKnoten = itSKPos.next();
			strukturPos.erzeugeStrukturElement(strassenKnoten);
		}

		// Bestimmen der Struktur in TmcRichtung positiv
		strukturPos.bestimmeVorgaengerNachfolger();

		/*
		 * Struktur (TmcRichtung negativ) erzeugen
		 */
		strukturNeg = new Struktur("negativ");

		// urereStraenSegmente zur negativen Struktur hinzufgen
		final Iterator<AeusseresStrassenSegment> itAeNeg = aeussereStrassenSegmenteNegativ
				.iterator();
		while (itAeNeg.hasNext()) {
			final AeusseresStrassenSegment aeusseresStrassenSegment = itAeNeg
					.next();
			strukturNeg.erzeugeStrukturElement(aeusseresStrassenSegment);
		}

		// StraenKnoten zur negativen Struktur hizufgen.
		final Iterator<StrassenKnoten> itSKNeg = knotenNegativ.iterator();
		while (itSKNeg.hasNext()) {
			final StrassenKnoten strassenKnoten = itSKNeg.next();
			strukturNeg.erzeugeStrukturElement(strassenKnoten);
		}

		// Bestimmen der Struktur in TmcRichtung negativ
		strukturNeg.bestimmeVorgaengerNachfolger();

		final Iterator<StrukturElement> itkombi = getVonAuswahlfeld()
				.iterator();
		while (itkombi.hasNext()) {
			final StrukturElement element = itkombi.next();
			final List<StrukturElementMitPfad> li = bestimmeErreichbareKnotenAeste(element);
			AsbOrtsangabe.LOGGER.fine("SIZE: " + li.size());
		}

		/**
		 * Herausfiltern der ASB-Knoten, passend zur aktuell initialisierten
		 * Strae.
		 */
		filtereAsbKnoten();
	}

	/**
	 * Gibt die Elemente einer {@link Struktur} auf der Konsole aus.
	 * 
	 * @param struktur
	 *            Auszugebende {@link Struktur}
	 */
	protected void gebeStrukturAus(final Struktur struktur) {
		/*
		 * Ermitteln der Elemente ohne Vorgnger
		 */
		final StrukturElement ohneVor = struktur
				.bestimmeStrukturElementOhneVorgaenger();

		// System.out.println("Strukturelement ohne Vorgnger \n\t=> "
		// + ohneVor.toString());

		int i = 0;
		final Iterator<StrukturElement> itAus = struktur
				.bestimmeNachfolgerFuerAusgabe(ohneVor).iterator();
		while (itAus.hasNext()) {
			final StrukturElement element = itAus.next();
			System.out.println("StrukturElement " + i + ": "
					+ element.toString());
			i++;
		}

	}

	/**
	 * Liefert eine Liste der kombinierten AsbVonKnoten.
	 * 
	 * @return Liste der kombinierten AsbVonKnoten
	 */
	public List<AsbKnotenKombiniert> getVonAuswahlfeldKombiniert() {
		List<AsbKnotenKombiniert> re = null;

		re = asbKnotenVerwaltungKombiniert.getListeKombinierteKnoten();

		return re;
	}

	/**
	 * Liefert die nach Key sortierte Kombination der {@link StrukturElement}e
	 * der positiven und negativen {@link Struktur} als {@link LinkedList}
	 * zurck.<br>
	 * <br>
	 * 
	 * Diese Liste bietet sich zum Fllen des <i>ersten Auswahlfeldes</i> an, in
	 * dem ein Knoten bzw. ein Ast (ueres Straensegment) der Strae
	 * ausgewhlt werden kann.
	 * 
	 * @return LinkedList der {@link StrukturElement}e der positiven und
	 *         negativen {@link Struktur}en.
	 */
	public List<StrukturElement> getVonAuswahlfeld() {
		/**
		 * Enthlt alle positiven Strukturelemente, nach KEY sortiert.
		 */
		final Map<String, StrukturElement> kombiPos = new TreeMap<String, StrukturElement>();

		/**
		 * Enthlt alle negativen Strukturelemente, nach KEY sortiert.
		 * 
		 */
		final Map<String, StrukturElement> kombiNeg = new TreeMap<String, StrukturElement>();

		/*
		 * Positive Struktur hinzufgen
		 */
		if ((strukturPos != null) && (strukturPos.size() > 0)) {
			final Iterator<Entry<String, StrukturElement>> itPos = strukturPos
					.getStrukturElemente().entrySet().iterator();

			while (itPos.hasNext()) {
				final Entry<String, StrukturElement> entry = itPos.next();
				if (!kombiPos.containsKey(entry.getKey())) {
					kombiPos.put(entry.getKey(), entry.getValue());
					AsbOrtsangabe.LOGGER
							.fine(getClass().getName()
									+ "::kombiniereStrukturen PUT positives StrukturElement: "
									+ entry.getValue());
				}
			}

		}

		/*
		 * Negative Struktur hinzufgen
		 */
		if ((strukturNeg != null) && (strukturNeg.size() > 0)) {
			final Iterator<Entry<String, StrukturElement>> itNeg = strukturNeg
					.getStrukturElemente().entrySet().iterator();

			while (itNeg.hasNext()) {
				final Entry<String, StrukturElement> entry = itNeg.next();
				if (!kombiNeg.containsKey(entry.getKey())) {
					kombiNeg.put(entry.getKey(), entry.getValue());
					AsbOrtsangabe.LOGGER
							.fine(getClass().getName()
									+ "::kombiniereStrukturen PUT positives StrukturElement: "
									+ entry.getValue());

				}
			}
		}

		/*
		 * Liste erstellen und zurckliefern
		 */
		final LinkedList<StrukturElement> ru = new LinkedList<StrukturElement>();

		final Iterator<Entry<String, StrukturElement>> it = kombiPos.entrySet()
				.iterator();
		while (it.hasNext()) {
			final Entry<String, StrukturElement> entry = it.next();
			if (!ru.contains(entry.getValue())) {
				ru.add(entry.getValue());
			} else {
				AsbOrtsangabe.LOGGER
						.error("Das StrukturElement ("
								+ entry.getValue().toStringLang()
								+ ") ist bereits i.d. sortierten und kombinierten Liste der StrukturElemente vorhanden!");

			}
		}

		final Iterator<Entry<String, StrukturElement>> itNeg = kombiNeg
				.entrySet().iterator();
		while (itNeg.hasNext()) {
			final Entry<String, StrukturElement> entry = itNeg.next();
			if (!ru.contains(entry.getValue())) {
				ru.add(entry.getValue());
			} else {
				AsbOrtsangabe.LOGGER
						.error("Das StrukturElement ("
								+ entry.getValue().toStringLang()
								+ ") ist bereits i.d. sortierten und kombinierten Liste der StrukturElemente vorhanden!");

			}
		}

		return ru;
	}

	/**
	 * Bestimmt die von einem {@link StrukturElement} aus erreichbaren Knoten
	 * und ste. Diese werden in einer Liste vom Typ
	 * {@link StrukturElementMitPfad} zurckgegeben, so dass zu jedem
	 * erreichbaren {@link StrukturElement} (gekapselt als
	 * {@link StrukturElementMitPfad}) eine Pfadangabe vom Ausgangs
	 * {@link StrukturElement} verfgbar ist.
	 * 
	 * @param element
	 *            {@link StrukturElement}, von dem die erreichbaren
	 *            {@link StrukturElement}e bestimmt werden sollen.
	 * @return Liste der erreichbaren {@link StrukturElement}e (gekapselt als
	 *         {@link StrukturElementMitPfad}) inklusive Pfad vom Ausgangs
	 *         {@link StrukturElement} an.
	 */
	public List<StrukturElementMitPfad> bestimmeErreichbareKnotenAeste(
			final StrukturElement element) {
		final Map<String, StrukturElementMitPfad> pos = bestimmeIntern(
				strukturPos, element);
		final Map<String, StrukturElementMitPfad> neg = bestimmeIntern(
				strukturNeg, element);

		final List<StrukturElementMitPfad> liste = new LinkedList<StrukturElementMitPfad>();

		if ((pos != null) && (pos.size() > 0)) {
			final Iterator<Entry<String, StrukturElementMitPfad>> itPos = pos
					.entrySet().iterator();
			while (itPos.hasNext()) {
				final Entry<String, StrukturElementMitPfad> entry = itPos
						.next();
				liste.add(entry.getValue());
			}
		}

		if ((neg != null) && (neg.size() > 0)) {

			final Iterator<Entry<String, StrukturElementMitPfad>> itNeg = neg
					.entrySet().iterator();
			while (itNeg.hasNext()) {
				final Entry<String, StrukturElementMitPfad> entry = itNeg
						.next();
				liste.add(entry.getValue());
			}
		}

		return liste;
	}

	/**
	 * Bestimmt alle von einem {@link StrukturElement} erreichbaren
	 * {@link StrukturElement}e innerhalb einer {@link Struktur}. Der Pfad der
	 * erreichbaren {@link StrukturElement}e wird zusammen mit dem Ausgangs
	 * {@link StrukturElement} in einer Instanz der Klasse
	 * {@link StrukturElementMitPfad} gekapselt.
	 * 
	 * 
	 * @param struktur
	 *            {@link Struktur}, innerhalb der alle von einem
	 *            {@link StrukturElement} erreichbaren {@link StrukturElement}e
	 *            bestimmt werden sollen.
	 * @param element
	 *            Ausgangs {@link StrukturElement}, von dem ausgehend die
	 *            erreichbaren {@link StrukturElement}e innerhalb der
	 *            {@link Struktur} bestimmt werden.
	 * @return Alle von einem Ausgangs- {@link StrukturElement} aus erreichbaren
	 *         {@link StrukturElement}e. Wird gekapselt in einer TreeMap als
	 *         {@link StrukturElementMitPfad} und beinhaltet sowohl den Pfad als
	 *         auch das erreichte {@link StrukturElement}.
	 */
	private Map<String, StrukturElementMitPfad> bestimmeIntern(
			final Struktur struktur, final StrukturElement element) {
		Map<String, StrukturElementMitPfad> map = null;

		if (struktur.containsValue(element)) {
			map = new TreeMap<String, StrukturElementMitPfad>();

			/*
			 * Bestimmen der Vor- und Nachfolgerlisten
			 */
			final List<StrukturElement> nachfolgerListe = struktur
					.bestimmeNachfolgerListe(element);
			final List<StrukturElement> vorgaengerListe = struktur
					.bestimmeVorgaengerListe(element);

			// Nachfolger durchlaufen
			for (int i = 1; i < nachfolgerListe.size(); i++) {
				final List<StrukturElement> subList = nachfolgerListe.subList(
						0, i + 1);
				final List<StrukturElement> tempListNachfolger = new LinkedList<StrukturElement>();

				tempListNachfolger.addAll(subList);

				final StrukturElementMitPfad elementMitPfad = new StrukturElementMitPfad(
						nachfolgerListe.get(i), true, tempListNachfolger);
				if (!map.containsKey(elementMitPfad.getKey())) {
					map.put(elementMitPfad.getKey(), elementMitPfad);
				}
			}

			// Vorgnger durchlaufen
			for (int i = 1; i < vorgaengerListe.size(); i++) {
				final List<StrukturElement> subList = vorgaengerListe.subList(
						0, i + 1);
				final LinkedList<StrukturElement> tempListVorgaenger = new LinkedList<StrukturElement>();
				tempListVorgaenger.addAll(subList);

				final StrukturElementMitPfad elementMitPfad = new StrukturElementMitPfad(
						vorgaengerListe.get(i), false, tempListVorgaenger);
				if (!map.containsKey(elementMitPfad.getKey())) {
					map.put(elementMitPfad.getKey(), elementMitPfad);
				}
			}
		}

		return map;
	}

	/**
	 * Liefert eine Liste aller verfgbaren ASB-Knoten zurck. Wurden die
	 * Strukturen nach keiner Strae initialisiert (vgl. Method
	 * {@link AsbOrtsangabe#initialisiereStrukturNachStrasse(String)}), wird
	 * eine leere Liste zurckgeliefert.
	 * 
	 * @return Liste aller verfgbaren {@link AsbKnoten}. Kann leer sein, wenn
	 *         nach keiner Strae initialisiert wurde.
	 */
	public List<AsbKnoten> getVerfuegbareAsbKnoten() {
		return listeKnotenAsbGesamt;
	}

	/**
	 * Bestimmt eine Liste aller von einem {@link AsbKnoten} aus erreichbaren
	 * {@link AsbKnoten}. Die Liste ist leer, wenn von dem {@link AsbKnoten} aus
	 * keine Knoten erreichbar sind oder keine Knotenverwaltung initialisiert
	 * werden konnte.
	 * 
	 * @param ausgangsKnoten
	 *            Ausgangs {@link AsbKnoten}, von dem aus alle erreichbaren
	 *            {@link AsbKnoten} bestimmt werden sollen.
	 * @return Eine Liste der erreichbaren {@link AsbKnoten}. Kann leer sein,
	 *         wenn keine {@link AsbKnoten} erreichbar sind oder keine
	 *         {@link AsbKnotenVerwaltung} initialisiert werden konnte.
	 */
	public List<AsbKnoten> bestimmeErreichbareAsbKnoten(
			final AsbKnoten ausgangsKnoten) {
		List<AsbKnoten> re = new LinkedList<AsbKnoten>();

		AsbKnotenVerwaltung knotenVerwaltung = null;

		if (ausgangsKnoten.getTmcRichtung().equals(EnumTmcRichtung.POSITIV)) {
			knotenVerwaltung = asbKnotenVerwaltungPositiv;
		} else if (ausgangsKnoten.getTmcRichtung().equals(
				EnumTmcRichtung.NEGATIV)) {
			knotenVerwaltung = asbKnotenVerwaltungNegativ;
		}

		if (knotenVerwaltung != null) {
			re = knotenVerwaltung
					.bestimmeListeAllerErreichbarenKnoten(ausgangsKnoten);
		}

		return re;
	}

	public AsbKnotenVerwaltungKombiniert bestimmeErreichbareAsbKnotenKombiniert(
			final AsbKnotenKombiniert ausgangsKnoten) {
		/*
		 * Rckgabe einer AsbKnotenVerwaltungKobiniert, auf der dann gearbeitet
		 * werden kann.
		 */

		final AsbKnotenVerwaltungKombiniert verwaltungKombiniert = new AsbKnotenVerwaltungKombiniert();

		if (ausgangsKnoten.hatKnotenPositiv()) {
			final List<AsbKnoten> listeErreichbareKnotenPositiv = asbKnotenVerwaltungPositiv
					.bestimmeListeAllerErreichbarenKnoten(ausgangsKnoten
							.getAsbKnotenPositiv());

			verwaltungKombiniert
					.addAsbKnotenPositiv(listeErreichbareKnotenPositiv);

		}

		if (ausgangsKnoten.hatKnotenNegativ()) {
			final List<AsbKnoten> listeErreichbarerKnotenPositiv = asbKnotenVerwaltungNegativ
					.bestimmeListeAllerErreichbarenKnoten(ausgangsKnoten
							.getAsbKnotenNegativ());

			verwaltungKombiniert
					.addAsbKnotenNegativ(listeErreichbarerKnotenPositiv);
		}

		return verwaltungKombiniert;
	}

	/**
	 * Prft, ob eine angegebene Stationierung im Bereich zwischen zwei Knoten
	 * vorhanden ist bzw. gltig ist.
	 * 
	 * @param knotenA
	 *            {@link AsbKnoten} A
	 * @param knotenB
	 *            {@link AsbKnoten} B
	 * @param stationierung
	 *            Stationierung zwischen den beiden Knoten, die geprft werden
	 *            soll. Die Angabe erfolgt in Metern.
	 * @return Liefert <code>true</code>, wenn die Stationierung zwischen den
	 *         beiden Knoten gltig ist, sonst <code>false</code>.
	 */
	public boolean pruefeAsbStationierung(final AsbKnoten knotenA,
			final AsbKnoten knotenB, final long stationierung) {
		boolean re = false;

		if (knotenA.getTmcRichtung().equals(knotenB.getTmcRichtung())) {
			AsbKnotenVerwaltung knotenVerwaltung = null;
			if (knotenA.getTmcRichtung().equals(EnumTmcRichtung.POSITIV)) {
				knotenVerwaltung = asbKnotenVerwaltungPositiv;
			} else if (knotenA.getTmcRichtung().equals(EnumTmcRichtung.NEGATIV)) {
				knotenVerwaltung = asbKnotenVerwaltungNegativ;
			}

			if (knotenVerwaltung != null) {
				// re = knotenVerwaltung.istInBereich(knotenA, knotenB,
				// stationierung);

				re = knotenVerwaltung.istInBereich(knotenA, knotenB,
						stationierung).istInBereich();
			} else {
				AsbOrtsangabe.LOGGER
						.warning(getClass().getName()
								+ "::pruefeAsbStationierung => Es konnte keine Knotenverwaltung "
								+ "zum Prfen der Stationierung bestimmt werden.");
			}

		} else {
			AsbOrtsangabe.LOGGER.warning(getClass().getName()
					+ "::pruefeAsbStationierung => Die TMC-Richtungen der "
					+ "bergebenen Objekte ist nicht gleich. KnotenA: "
					+ knotenA + " KnotenB: " + knotenB);
		}

		return re;
	}

	public BereichsInfoAsbKnotenKombiniert pruefeAsbStationierungKombiniert(
			final AsbKnotenKombiniert knotenA,
			final AsbKnotenKombiniert knotenB, final long stationierung) {
		final BereichsInfoAsbKnotenKombiniert info = asbKnotenVerwaltungKombiniert
				.istInBereichDetailiert(knotenA, knotenB, stationierung);

		return info;
	}

	/**
	 * Filtert die zu einer Strae gehrigen ASB-Knoten an Hand der durch die
	 * Methode {@link AsbOrtsangabe#initialisiereStrukturNachStrasse(String)}
	 * bestimmten Strukturen <code>_sturkturPos</code> und
	 * <code>_strukturNeg</code>.
	 */
	private void filtereAsbKnoten() {
		asbKnotenVerwaltungPositiv = new AsbKnotenVerwaltung(
				EnumTmcRichtung.POSITIV);
		asbKnotenVerwaltungNegativ = new AsbKnotenVerwaltung(
				EnumTmcRichtung.NEGATIV);
		asbKnotenVerwaltungKombiniert = new AsbKnotenVerwaltungKombiniert();

		final StrukturElement ohneVorgaengerPositiveRichtung = strukturPos
				.bestimmeStrukturElementOhneVorgaenger();

		final StrukturElement ohneVorgaengerNegativeRichtung = strukturNeg
				.bestimmeStrukturElementOhneVorgaenger();

		bestimmeAsbKnotenAnfangEnde(ohneVorgaengerPositiveRichtung,
				asbKnotenVerwaltungPositiv);
		bestimmeAsbKnotenAnfangEnde(ohneVorgaengerNegativeRichtung,
				asbKnotenVerwaltungNegativ);

		// ASB Orstangabe Ausgabe
		// System.out.println("===POSITIV START===");
		// System.out.println(asbKnotenVerwaltungPositiv);
		// System.out.println("===POSITIV ENDE===");
		// System.out.println("===Negativ Start===");
		// System.out.println(asbKnotenVerwaltungNegativ);
		// System.out.println("===Negativ Ende===");

		final List<AsbKnoten> listeAllerKnotenPos = asbKnotenVerwaltungPositiv
				.bestimmeListeAllerKnoten();
		final List<AsbKnoten> listeAllerKnotenNeg = asbKnotenVerwaltungNegativ
				.bestimmeListeAllerKnoten();

		/*
		 * Bestimmte, positive ASBKnoten zur Kombinierten ASBKnotenverwaltung
		 * hinzufgen
		 */
		final Iterator<AsbKnoten> itPos = listeAllerKnotenPos.iterator();
		while (itPos.hasNext()) {
			asbKnotenVerwaltungKombiniert.addAsbKnotenPositiv(itPos.next());
		}

		/*
		 * Bestimmte, negative ASBKnoten zur Kombinierten ASBKnotenverwaltung
		 * hinzufgen
		 */
		final Iterator<AsbKnoten> itNeg = listeAllerKnotenNeg.iterator();
		while (itNeg.hasNext()) {
			asbKnotenVerwaltungKombiniert.addAsbKnotenNegativ(itNeg.next());
		}

		listeKnotenAsbGesamt = new LinkedList<AsbKnoten>();
		listeKnotenAsbGesamt.addAll(listeAllerKnotenPos);
		listeKnotenAsbGesamt.addAll(listeAllerKnotenNeg);
		Collections.sort(listeKnotenAsbGesamt);

	}

	private void bestimmeAsbKnotenAnfangEnde(
			final StrukturElement ohneVorgaenger,
			final AsbKnotenVerwaltung asbKnotenVerwaltung) {
		/*
		 * Bestimmen der erreichbaren Knoten und ste
		 */
		final List<StrukturElementMitPfad> erreichbareKnotenAeste = bestimmeErreichbareKnotenAeste(ohneVorgaenger);
		final Iterator<StrukturElementMitPfad> it = erreichbareKnotenAeste
				.iterator();
		while (it.hasNext()) {
			final StrukturElementMitPfad mitPfad = it.next();

			/*
			 * Pfad extrahieren
			 */
			final Iterator<StrukturElement> itPfad = mitPfad.getPfad()
					.iterator();
			while (itPfad.hasNext()) {
				/*
				 * Pfad Element holen
				 */
				final StrukturElement elementImPfad = itPfad.next();

				/*
				 * StrassenKnoten
				 */
				if (elementImPfad.isStrassenKnoten()) {
					final StrassenKnoten stk = (StrassenKnoten) elementImPfad
							.getElement();
					final List<InneresStrassenSegment> iss = stk
							.getInnereStrassenSegmente();

					// System.out.println("==>Innere Strassen Segmente - Anzahl: "
					// + iss.size());

					final Iterator<InneresStrassenSegment> itIss = iss
							.iterator();
					while (itIss.hasNext()) {
						final InneresStrassenSegment inneresStrassenSegment = itIss
								.next();

						// System.out.println("==>ISS: "
						// + inneresStrassenSegment.toStringLang());

						final Iterator<StrassenTeilSegment> itSts = inneresStrassenSegment
								.getStrassenTeilSegmente().iterator();
						while (itSts.hasNext()) {
							final StrassenTeilSegment nextSts = itSts.next();
							final Iterator<AsbStationierung> itAsbIss = nextSts
									.getAsbStationierungen().iterator();
							while (itAsbIss.hasNext()) {
								final AsbStationierung asbStationierung = itAsbIss
										.next();
								asbKnotenVerwaltung
										.addAsbStationierung(asbStationierung);
							}
						}
					}
				}

				/*
				 * ueres Straensegment?
				 */
				if (elementImPfad.isAeusseresStrassenSegment()) {
					final AeusseresStrassenSegment aes = (AeusseresStrassenSegment) elementImPfad
							.getElement();

					// System.out.println("=>AES: " + aes.toStringLang());

					/*
					 * Bestimmen der StraenTeilsegmente des ueren
					 * StraenSegmentes
					 */
					final List<StrassenTeilSegment> strassenTeilSegmente = aes
							.getStrassenTeilSegmente();

					/*
					 * Extrahieren der AsbStationierungen aus den
					 * StraenTeilSegmenten
					 */
					final Iterator<StrassenTeilSegment> itSts = strassenTeilSegmente
							.iterator();
					while (itSts.hasNext()) {
						final StrassenTeilSegment strassenTeilSegment = itSts
								.next();

						final List<AsbStationierung> asbStationierungen = strassenTeilSegment
								.getAsbStationierungen();

						final Iterator<AsbStationierung> itAsb = asbStationierungen
								.iterator();
						while (itAsb.hasNext()) {
							final AsbStationierung asbStationierung = itAsb
									.next();

							/*
							 * Stationierung zur Verwaltung hinzufgen.
							 */
							asbKnotenVerwaltung
									.addAsbStationierung(asbStationierung);
						}
					}
				}
				// if (elementImPfad.isAeusseresStrassenSegment())
				// {
				// AeusseresStrassenSegment aes = (AeusseresStrassenSegment)
				// elementImPfad
				// .getElement();
				//
				// /*
				// * Bestimmen der StraenTeilsegmente des ueren
				// StraenSegmentes
				// */
				// Vector<StrassenTeilSegment> strassenTeilSegmente = aes
				// .getStrassenTeilSegmente();
				//
				// /*
				// * Extrahieren der AsbStationierungen aus den
				// StraenTeilSegmenten
				// */
				// Iterator<StrassenTeilSegment> itSts =
				// strassenTeilSegmente.iterator();
				// while (itSts.hasNext())
				// {
				// StrassenTeilSegment strassenTeilSegment = itSts.next();
				//
				// LinkedList<AsbStationierung> asbStationierungen =
				// strassenTeilSegment
				// .getAsbStationierungen();
				//
				// Iterator<AsbStationierung> itAsb =
				// asbStationierungen.iterator();
				// while (itAsb.hasNext())
				// {
				// AsbStationierung asbStationierung = itAsb.next();
				//
				// /*
				// * Stationierung zur Verwaltung hinzufgen.
				// */
				// asbKnotenVerwaltung.addAsbStationierung(asbStationierung);
				// }
				// }
				// }
			}
		}
	}

	/**
	 * Liefert die Elemente einer Struktur der {@link AeusseresStrassenSegment}e
	 * einer Strae. <br>
	 * Die {@link AeusseresStrassenSegment}e dieser Strae knnen sortiert in
	 * Reihenfolge ausgelesen werden, sofern ein Zusammenhang besteht.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public class Struktur {
		/**
		 * Liste der Elemente dieser Struktur.
		 */
		private final Map<String, StrukturElement> elemente = new HashMap<String, StrukturElement>();

		/**
		 * TMC-Richtung der Elemente dieser Struktur.
		 */
		private final String tmcRichtung;

		private int zaehler;

		/**
		 * Konstruktor. bernimmt die TmcRichtung der Elemente dieser
		 * {@link Struktur}.
		 * 
		 * @param tmcRichtungPositiv
		 *            TmcRichtung dieser Struktur
		 */
		public Struktur(final String tmcRichtungPositiv) {
			AsbOrtsangabe.LOGGER.fine(getClass().getName()
					+ "::Struktur \n\t=> Struktur erzeugt.");

			this.tmcRichtung = tmcRichtungPositiv;
		}

		/**
		 * Erzeugt ein neues {@link StrukturElement} und liefert dieses zurck.
		 * Ist bereits eines mit dem gleichen {@link AeusseresStrassenSegment}
		 * vorhanden, so wird dieses zurckgeliefert.
		 * 
		 * @param aeusseresStrassenSegment
		 *            {@link AeusseresStrassenSegment}, aus dem ein
		 *            {@link StrukturElement} erzeugt werden soll.
		 * @return {@link StrukturElement}
		 */
		public StrukturElement erzeugeStrukturElement(
				final AeusseresStrassenSegment aeusseresStrassenSegment) {
			/*
			 * StrukturElement besorgen
			 */
			StrukturElement strukturElement = null;
			if (containsKey(aeusseresStrassenSegment.getPid())) {
				strukturElement = this
						.getStrukturElement(aeusseresStrassenSegment.getPid());
			} else {
				strukturElement = new StrukturElement(aeusseresStrassenSegment,
						tmcRichtung);
				this.addStrukturElement(strukturElement);
			}

			return strukturElement;
		}

		/**
		 * Erzeugt ein neues {@link StrukturElement} und liefert dieses zurck.
		 * Ist bereits eines mit dem gleichen {@link StrassenKnoten} vorhanden,
		 * so wird dieses zurckgeliefert.
		 * 
		 * @param strassenKnoten
		 *            {@link StrassenKnoten}, aus dem ein
		 *            {@link StrukturElement} erzeugt werden soll.
		 * @return {@link StrukturElement}
		 */
		public StrukturElement erzeugeStrukturElement(
				final StrassenKnoten strassenKnoten) {
			/*
			 * StrukturElement besorgen
			 */
			StrukturElement strukturElement = null;
			if (containsKey(strassenKnoten.getPid())) {
				strukturElement = this.getStrukturElement(strassenKnoten
						.getPid());
			} else {
				strukturElement = new StrukturElement(strassenKnoten,
						tmcRichtung);
				this.addStrukturElement(strukturElement);
			}

			return strukturElement;
		}

		/**
		 * Fgt ein {@link StrukturElement} in die {@link HashMap} ein, sofern
		 * dieses noch nicht enthalten ist.
		 * 
		 * @param element
		 *            Hinzuzufgendes {@link StrukturElement}
		 */
		public void addStrukturElement(final StrukturElement element) {
			if (!elemente.containsKey(element.getPid())) {
				elemente.put(element.getPid(), element);
			}
		}

		/**
		 * Liefert ein {@link StrukturElement} an Hand ihres Keys aus der
		 * {@link HashMap} zurck.
		 * 
		 * @param pid
		 *            Key des StrukturElements, entsprechend der PID des
		 *            {@link AeusseresStrassenSegment}.
		 * @return {@link StrukturElement}, zugehrig zum Key.
		 */
		public StrukturElement getStrukturElement(final String pid) {
			if (elemente.containsKey(pid)) {
				return elemente.get(pid);
			}
			return null;
		}

		/**
		 * Liefert alle {@link StrukturElement}e in einer {@link HashMap}
		 * zurck.
		 * 
		 * @return {@link HashMap} (String als Key, {@link StrukturElement} als
		 *         Value)
		 */
		public Map<String, StrukturElement> getStrukturElemente() {
			return elemente;
		}

		/**
		 * Liefert die TmcRichtung der Elemente dieser {@link Struktur}.
		 * 
		 * @return TmcRichtung der Elemente dieser {@link Struktur}
		 */
		public String getTmcRichtung() {
			return tmcRichtung;
		}

		/**
		 * Prft, ob ein {@link StrukturElement} mit einem bestimmten Key in der
		 * {@link HashMap} der {@link StrukturElement}e vorhanden ist.
		 * 
		 * @param key
		 *            Schlssel des auf Existenz zu prfenden
		 *            {@link StrukturElement}s.
		 * @return Liefert <code>true</code>, wenn das Element in der
		 *         {@link HashMap} vorhanden ist, sonst <code>false</code>.
		 */
		public boolean containsKey(final String key) {
			return elemente.containsKey(key);
		}

		/**
		 * Prft, ob die {@link HashMap} dieser {@link Struktur} ein bestimmtes
		 * Objekt enthlt.
		 * 
		 * @param object
		 *            Objekt, auf dessen Existenz geprft werden soll.
		 * @return Liefert <code>true</code>, wenn das Objekt in der
		 *         {@link HashMap} vorhanden ist, sonst <code>false</code>.
		 */
		public boolean containsValue(final Object object) {
			return elemente.containsValue(object);
		}

		/**
		 * Liefert die Gre der Struktur.
		 * 
		 * @return Gre der Struktur
		 */
		public int size() {
			return elemente.size();
		}

		/**
		 * Bestimmt die Vorgnger und Nachfolger fr jedes
		 * {@link StrukturElement}.
		 */
		protected void bestimmeVorgaengerNachfolger() {
			final Iterator<Entry<String, StrukturElement>> it = elemente
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<String, StrukturElement> entry = it.next();
				final StrukturElement strukturElement = entry.getValue();

				/*
				 * ueresStraenSegment
				 */
				if (strukturElement.isAeusseresStrassenSegment()) {
					// Aussen
					final AeusseresStrassenSegment segment = (AeusseresStrassenSegment) strukturElement
							.getElement();

					final StrassenKnoten nachKnoten = segment.getNachKnoten();
					if (nachKnoten != null) {
						// Nach
						if (elemente.containsKey(nachKnoten.getPid())) {
							final StrukturElement ssNachfolger = elemente
									.get(nachKnoten.getPid());
							ssNachfolger.addVorgaenger(strukturElement);

							strukturElement.addNachfolger(ssNachfolger);
						}
					}

					final StrassenKnoten vonKnoten = segment.getVonKnoten();
					if (vonKnoten != null) {
						// Von
						if (elemente.containsKey(vonKnoten.getPid())) {
							final StrukturElement ssVorgaenger = elemente
									.get(vonKnoten.getPid());
							ssVorgaenger.addNachfolger(strukturElement);

							strukturElement.addVorgaenger(ssVorgaenger);
						}
					}

				}

				/*
				 * StraenKnoten
				 */
				if (strukturElement.isStrassenKnoten()) {
					final StrassenKnoten knoten = (StrassenKnoten) strukturElement
							.getElement();

					/*
					 * Abgehend (Nachfolgend)
					 */
					final Iterator<AeusseresStrassenSegment> itAbgehend = knoten
							.getAbgehendeAeussereStrassenSegmente().iterator();
					while (itAbgehend.hasNext()) {
						// Abgehend
						final AeusseresStrassenSegment segment = itAbgehend
								.next();
						if (elemente.containsKey(segment.getPid())) {
							final StrukturElement ssNachfolger = elemente
									.get(segment.getPid());
							ssNachfolger.addVorgaenger(strukturElement);

							strukturElement.addNachfolger(ssNachfolger);
						}
					}

					/*
					 * Hinzufhrend
					 */
					final Iterator<AeusseresStrassenSegment> itHinzu = knoten
							.getHinfuehrendeAeussereStrassenSegmente()
							.iterator();
					while (itHinzu.hasNext()) {
						// Hinzufhrend
						final AeusseresStrassenSegment segment = itHinzu.next();
						if (elemente.containsKey(segment.getPid())) {
							final StrukturElement ssVorgaenger = elemente
									.get(segment.getPid());
							ssVorgaenger.addNachfolger(strukturElement);

							strukturElement.addVorgaenger(ssVorgaenger);
						}
					}

				}

			}
		}

		/**
		 * Bestimmt das {@link StrukturElement}, welches keinen Vorgnger hat.
		 * 
		 * @return {@link StrukturElement} ohne Vorgnger, sonst
		 *         <code>null</code>.
		 */
		public StrukturElement bestimmeStrukturElementOhneVorgaenger() {
			StrukturElement element = null;

			final Iterator<Entry<String, StrukturElement>> it = elemente
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<String, StrukturElement> entry = it.next();
				final int size = entry.getValue().getVorgaenger().size();
				if (size == 0) {
					element = entry.getValue();
					break;
				}
			}

			AsbOrtsangabe.LOGGER
					.fine(getClass().getName()
							+ "::bestimmeStrukturElementOhneNachfolger \n\t=>Element ohne Vorgaenger: "
							+ (element == null ? "Kein Element" : element
									.getPid()));

			return element;
		}

		/**
		 * Diese Funktion liefert eine geordnete Liste der
		 * {@link AeusseresStrassenSegment}e.
		 * 
		 * @param element
		 *            {@link StrukturElement}, an Hand dessen Nachfolgern (und
		 *            damit auch Vorgngern) die
		 *            {@link AeusseresStrassenSegment} in Reihenfolge bestimmt
		 *            werden sollen.
		 * @return Liste mit in Reihenfolgen gebrachten {@link StrukturElement}
		 *         en.
		 */
		public List<StrukturElement> bestimmeNachfolgerFuerAusgabe(
				final StrukturElement element) {
			AsbOrtsangabe.LOGGER
					.fine(getClass().getName()
							+ "::bestimmeNachfolger \n\tNachfolger werden bestimmt, siehe nachfolgende Liste.");

			final List<StrukturElement> liste = new LinkedList<StrukturElement>();

			final List<StrukturElement> nach = bestimmeNachfolgerRekursivFuerAusgabe(element);
			liste.addAll(nach);

			return liste;
		}

		/**
		 * Bestimmt <i>rekursiv</i> die Nachfolger eines Elementes <br>
		 * <b>Nicht fr ffentlichen Gebrauch bestimmt.</b>
		 * 
		 * @param element
		 * @return
		 */
		private List<StrukturElement> bestimmeNachfolgerRekursivFuerAusgabe(
				final StrukturElement element) {
			final List<StrukturElement> liste = new LinkedList<StrukturElement>();
			liste.add(element);

			AsbOrtsangabe.LOGGER.fine("\t" + zaehler
					+ " Element in Nachfolge: " + element.getPid());
			zaehler++;

			if (element.getNachfolger().size() > 0) {
				final List<StrukturElement> nach = bestimmeNachfolgerRekursivFuerAusgabe(element
						.getNachfolger().get(0));
				liste.addAll(nach);
			}

			return liste;
		}

		@Override
		public String toString() {
			final StringBuffer buffer = new StringBuffer();
			final Iterator<Entry<String, StrukturElement>> it = elemente
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<String, StrukturElement> entry = it.next();
				buffer.append("PID: ");
				buffer.append(entry.getKey());
				buffer.append(" Anz. Nachfolger: ");
				buffer.append(entry.getValue().getNachfolger().size());
				buffer.append(" Anz. Vorgaenger: ");
				buffer.append(entry.getValue().getVorgaenger().size());
				buffer.append('\n');
			}

			return buffer.toString();
		}

		/**
		 * Bestimmt die Liste der Nachfolger eines {@link StrukturElement}es.
		 * Dabei wird das {@link StrukturElement} an erster Stelle der Liste
		 * gesetzt und das letzte, nachfolgende Element an die letzte Stelle. <br>
		 * Der Verlauf der Liste ist linear und reprsentiert den Verlauf der
		 * nachfolgenden {@link StrukturElement}e auf der Strae.
		 * 
		 * @param element
		 *            {@link StrukturElement}, von dem die Liste der
		 *            nachfolgenden {@link StrukturElement}e bestimmt werden
		 *            soll.
		 * @return Liste der nachfolgenden {@link StrukturElement}e mit dem
		 *         Ausgangs {@link StrukturElement} am Anfang und dem letzen,
		 *         nachfolgenden {@link StrukturElement} am Ende.
		 */
		public List<StrukturElement> bestimmeNachfolgerListe(
				final StrukturElement element) {
			final List<StrukturElement> liste = new LinkedList<StrukturElement>();
			final List<StrukturElement> nachfolgerListe = bestimmeNachfolgerListeRekursiv(element);
			if (!nachfolgerListe.isEmpty()) {
				liste.addAll(nachfolgerListe);
			}

			return liste;
		}

		/**
		 * Bestimmt rekursiv die Nachfolger eines {@link StrukturElement}es.
		 * 
		 * @param element
		 *            {@link StrukturElement}, dessen Nachfolger bestimmt werden
		 *            sollen.
		 * @return Liste von Nachfolgern des {@link StrukturElement}es, das
		 *         bergebene Element steht an erster Stelle der Liste, der
		 *         letzte Nachfolger an letzter Stelle.
		 */
		private List<StrukturElement> bestimmeNachfolgerListeRekursiv(
				final StrukturElement element) {
			final List<StrukturElement> liste = new LinkedList<StrukturElement>();
			liste.add(element);

			if (element.getNachfolger().size() > 0) {
				final List<StrukturElement> nach = bestimmeNachfolgerListeRekursiv(element
						.getNachfolger().get(0));
				liste.addAll(nach);
			}

			return liste;
		}

		/**
		 * Bestimmt die Liste der Vorgnger eines {@link StrukturElement}es.
		 * Dabei wird das {@link StrukturElement} an erster Stelle der Liste
		 * gesetzt und das letzte, vorgehende Element an die letzte Stelle. <br>
		 * Der Verlauf der Liste ist linear und reprsentiert den Verlauf der
		 * vorhergehenden {@link StrukturElement}e auf der Strae.
		 * 
		 * @param element
		 *            {@link StrukturElement}, vom dem die Liste der
		 *            vorhergehenden {@link StrukturElement}e bestimmt werden
		 *            soll.
		 * @return Liste der vorhergehenden {@link StrukturElement}e mit dem
		 *         Ausgangs {@link StrukturElement} am Anfang und dem letzten
		 *         vorhergehenden {@link StrukturElement} am Ende.
		 */
		public List<StrukturElement> bestimmeVorgaengerListe(
				final StrukturElement element) {
			final List<StrukturElement> liste = new LinkedList<StrukturElement>();
			final List<StrukturElement> vorgaengerListe = bestimmeVorgaengerListeRekursiv(element);
			if (!vorgaengerListe.isEmpty()) {
				liste.addAll(vorgaengerListe);
			}

			return liste;
		}

		/**
		 * Bestimmt rekursiv die Vorgnger eines {@link StrukturElement}es.
		 * 
		 * @param element
		 *            {@link StrukturElement}, dessen Vorgnger bestimmt werden
		 *            sollen.
		 * @return Liste von Vorgngern des {@link StrukturElement}es, das
		 *         bergebene Element steht an erster Stelle der Liste, der
		 *         letzte Vorgnger an letzter Stelle.
		 */
		private List<StrukturElement> bestimmeVorgaengerListeRekursiv(
				final StrukturElement element) {
			final List<StrukturElement> liste = new LinkedList<StrukturElement>();
			liste.add(element);

			if (element.getVorgaenger().size() > 0) {
				final List<StrukturElement> vorgaenger = bestimmeVorgaengerListeRekursiv(element
						.getVorgaenger().get(0));
				liste.addAll(vorgaenger);
			}

			return liste;
		}

	}

	/**
	 * Reprsentiert ein Element in der Struktur. Dieses verfgt ber je eine
	 * Liste von Vorgngern und Nachfolgern und kann ein
	 * {@link AeusseresStrassenSegment} oder einen {@link StrassenKnoten} als
	 * Wert aufnehmen.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public static class StrukturElement {
		private final Object element;

		/**
		 * TMC-Richtung dieses StrukturElements.
		 */
		private final String tmcRichtung;

		private final List<StrukturElement> vorgaengerListe = new LinkedList<StrukturElement>();

		private final List<StrukturElement> nachfolgerListe = new LinkedList<StrukturElement>();

		/**
		 * Konstruktor, bernimmt ein {@link AeusseresStrassenSegment} als
		 * Element der Klasse {@link StrukturElement}.
		 * 
		 * @param element
		 *            {@link AeusseresStrassenSegment}
		 * 
		 * @param tmcRichtung
		 *            TmcRichtung dieses {@link StrukturElement}es.
		 */
		public StrukturElement(final AeusseresStrassenSegment element,
				final String tmcRichtung) {
			this.element = element;
			this.tmcRichtung = tmcRichtung;
		}

		/**
		 * Konstruktor, bernimmt einen {@link StrassenKnoten} als Element der
		 * Klasse {@link StrukturElement}.
		 * 
		 * @param knoten
		 *            {@link StrassenKnoten}
		 * @param tmcRichtung
		 *            TmcRichtung dieses {@link StrukturElement}es.
		 */
		public StrukturElement(final StrassenKnoten knoten,
				final String tmcRichtung) {
			this.element = knoten;
			this.tmcRichtung = tmcRichtung;
		}

		/**
		 * Liefert das Element dieser Datenkapselungsklasse als {@link Object}
		 * zurck.
		 * 
		 * @return {@link Object}
		 */
		public Object getElement() {
			return element;
		}

		/**
		 * Prft, ob es sich beim Element um einen {@link StrassenKnoten}
		 * handelt.
		 * 
		 * @return Liefert <code>true</code>, wenn es sich um einen
		 *         {@link StrassenKnoten} handelt, sonst <code>false</code>.
		 */
		public boolean isStrassenKnoten() {
			if (element instanceof StrassenKnoten) {
				return true;
			}
			return false;
		}

		/**
		 * Prft, ob es sich beim Element um ein
		 * {@link AeusseresStrassenSegment} handelt.
		 * 
		 * @return Liefert <code>true</code>, wenn es sich um ein
		 *         {@link AeusseresStrassenSegment} handelt, sonst
		 *         <code>false</code>.
		 */
		public boolean isAeusseresStrassenSegment() {
			if (element instanceof AeusseresStrassenSegment) {
				return true;
			}
			return false;
		}

		/**
		 * Liefert die Pid des Elementes.
		 * 
		 * @return Pid Liefert die PID des Elementes (entweder die PID des
		 *         {@link StrassenKnoten}s oder des
		 *         {@link AeusseresStrassenSegment}s, sonst <code>null</code>).
		 */
		public String getPid() {
			if (isAeusseresStrassenSegment()) {
				final AeusseresStrassenSegment as = (AeusseresStrassenSegment) element;
				return as.getPid();
			} else if (isStrassenKnoten()) {
				final StrassenKnoten sk = (StrassenKnoten) element;
				return sk.getPid();
			} else {
				return null;
			}
		}

		/**
		 * Liefert die TmcRichtung dieses {@link StrukturElement}es.
		 * 
		 * @return TmcRichtung dieses {@link StrukturElement}es.
		 */
		public String getTmcRichtung() {
			return tmcRichtung;
		}

		/**
		 * Fgt einen Vorgnger hinzu, falls dieser noch nicht i.d. Liste der
		 * Vorgnger vorhanden ist.
		 * 
		 * @param vorgaenger
		 *            Vorgnger
		 */
		public void addVorgaenger(final StrukturElement vorgaenger) {
			if (!vorgaengerListe.contains(vorgaenger)) {
				vorgaengerListe.add(vorgaenger);
			}
		}

		/**
		 * Liefert die Liste der Vorgnger.
		 * 
		 * @return Liste der Vorgnger
		 */
		public List<StrukturElement> getVorgaenger() {
			return vorgaengerListe;
		}

		/**
		 * Fgt einen Nachfolger vom Typ {@link StrukturElement} zur Liste der
		 * Nachfolger hinzu.
		 * 
		 * @param nachfolger
		 *            {@link StrukturElement} als Nachfolger
		 */
		public void addNachfolger(final StrukturElement nachfolger) {
			if (!nachfolgerListe.contains(nachfolger)) {
				nachfolgerListe.add(nachfolger);
			}
		}

		/**
		 * Liefert die Liste der Nachfolger.
		 * 
		 * @return Liste der Nachfolger.
		 */
		public List<StrukturElement> getNachfolger() {
			return nachfolgerListe;
		}

		@Override
		public String toString() {
			String re = "";

			if (isStrassenKnoten()) {
				final StrassenKnoten sk = (StrassenKnoten) element;
				re = sk.toString();
			} else if (isAeusseresStrassenSegment()) {
				final AeusseresStrassenSegment aeSS = (AeusseresStrassenSegment) element;
				re = aeSS.toString();
			}

			if (tmcRichtung != null) {
				re = re + " (TmcRichtung: " + tmcRichtung + ")";
			}

			return re;
		}

		public String toStringLang() {

			final StringBuilder buffer = new StringBuilder();

			buffer.append(toString());
			buffer.append('\n');

			buffer.append("Vorgnger:  ");
			final Iterator<StrukturElement> itVor = vorgaengerListe.iterator();
			while (itVor.hasNext()) {
				final StrukturElement curElement = itVor.next();
				buffer.append(curElement.toString());
				buffer.append(' ');
			}
			buffer.append('\n');

			buffer.append("Nachfolger: ");
			final Iterator<StrukturElement> itNach = nachfolgerListe.iterator();
			while (itNach.hasNext()) {
				final StrukturElement curElement = itNach.next();
				buffer.append(curElement.toString());
				buffer.append(' ');
			}
			buffer.append('\n');

			return buffer.toString();
		}

	}

	/**
	 * Diese Klasse kapselt ein {@link StrukturElement} und beinhaltet einen
	 * kompletten Pfad von {@link StrukturElement}en bis zu dem hier gekapselten
	 * {@link StrukturElement}. Das erste Element in diesem Pfad gibt das
	 * Ausgangs {@link StrukturElement} an.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public static class StrukturElementMitPfad {
		/**
		 * Speichert das {@link StrukturElement}.
		 */
		private final StrukturElement strukturElement;

		/**
		 * Mit Hilfe dieses Flags wird angezeigt, dass der Verlauf im Pfad in
		 * positiver Richtung, d.h. das erste Element ist der Vorgnger vom
		 * 2.ten ist der Vorgnger vom 3. ten usw, erfolgt <br>
		 * 
		 * Folgende Struktur sei ein Beispiel fr die aus {@link Struktur}
		 * gelieferte Grundstruktur. <br>
		 * X---a---X---b---X---c---X---d---X <br>
		 * <ul>
		 * <li>Soll jetzt ein Pfad von b nach dem Letzten Knoten X gebildet
		 * werden, ist die Richtung positiv un die Gesamtlnge von b geht i.d.
		 * Berechnung mit ein</li>
		 * <li>Soll ein Pfad von c zum ersten Knoten X gebildet werden ist davon
		 * auszugehen, dass wir uns am Anfang von c befinden und c somit nicht
		 * mit seiner vollen Lnge i.d. Berechnung mit ein geht. Die Richtung
		 * ist im Vergleich zu der von {@link Struktur} gelieferten Richtung
		 * negativ. <br>
		 * In dieser Situation muss vom Anfang des
		 * {@link AeusseresStrassenSegment} ausgegangen werden.</li>
		 * <li>3.tes X nach a: Richtung negativ, A geht mit voller Lnge ein</li>
		 * <li>2.tes X nach c: Richtung positiv, C geht mit voller Lnge ein</li>
		 * </ul>
		 * 
		 * <b>Zusammenfassung</b>: Es wird immer vom Angang eines
		 * {@link AeusseresStrassenSegment}es ausgegangen. In <i>positiver</i>
		 * Richtung geht dieses komplett mit ein, in <i>negativer</i> Richtung
		 * nicht (d.h. man geht nur vom Anfang dieses
		 * {@link AeusseresStrassenSegment} aus).
		 * 
		 */
		private boolean richtungPositiv = true;

		/**
		 * Speichert den Pfad. Das erste Element ist das
		 * Ausgangsstrukturelement, das letzte Element ist das Zielelement,
		 * entsprechend dem in {@link StrukturElementMitPfad#_strukturElement}
		 * gespeichertem Element.
		 */
		private List<StrukturElement> pfad;

		/**
		 * Konstruktor. bernimmt ein {@link StrukturElement}.
		 * 
		 * @param strukturElement
		 *            Zu bernehmendes {@link StrukturElement}
		 * @param richtungPositiv
		 *            Gibt an, ob im Vergleich zur Klasse {@link Struktur}, aus
		 *            der der Pfad gebildet wurde, der Pfad in positiver
		 *            Richtung (Vorgnger im Pfad ist auch Vorgnger in
		 *            {@link Struktur}) oder in negativer Richtung (Vorgnger im
		 *            Pfad ist Nachfolger in {@link Struktur}) verluft.
		 * 
		 */
		public StrukturElementMitPfad(final StrukturElement strukturElement,
				final boolean richtungPositiv) {
			this.strukturElement = strukturElement;
			this.richtungPositiv = richtungPositiv;
		}

		/**
		 * Konstruktor. bernimmt ein {@link StrukturElement}, die Richtung
		 * sowie einen Pfad, auf dem das {@link StrukturElement} zu erreichen
		 * ist.
		 * 
		 * @param strukturElement
		 *            Zu bernehmendes {@link StrukturElement}
		 * @param richtungPositiv
		 *            Gibt an, ob im Vergleich zur Klasse {@link Struktur}, aus
		 *            der der Pfad gebildet wurde, der Pfad in positiver
		 *            Richtung (Vorgnger im Pfad ist auch Vorgnger in
		 *            {@link Struktur}) oder in negativer Richtung (Vorgnger im
		 *            Pfad ist Nachfolger in {@link Struktur}) verluft.
		 * @param pfad
		 *            Pfad, auf dem das {@link StrukturElement} zu erreichen
		 *            ist.
		 * @throws IllegalArgumentException
		 *             Wird geworfen, wenn es sich beim letzten Element im Pfad
		 *             nicht um das {@link StrukturElement} handelt.
		 * 
		 */
		public StrukturElementMitPfad(final StrukturElement strukturElement,
				final boolean richtungPositiv, final List<StrukturElement> pfad) {
			this.strukturElement = strukturElement;
			this.richtungPositiv = richtungPositiv;
			setPfad(pfad);
		}

		/**
		 * Liefert das hinterlegte {@link StrukturElement} zurck.
		 * 
		 * @return Liefert das hinterlegte {@link StrukturElement} zurck.
		 */
		public StrukturElement getStrukturElement() {
			return strukturElement;
		}

		/**
		 * Liefert die TmcRichtung des hinterlegten {@link StrukturElement}es
		 * zurck.
		 * 
		 * @return TmcRichtung des hinterlegten {@link StrukturElement}es zurck
		 */
		public String getStrukturElementTmcRichtung() {
			return strukturElement.getTmcRichtung();
		}

		/**
		 * Liefert die PID des in dieser Klasse hinterlegten
		 * {@link StrukturElement}es zurck.
		 * 
		 * @return PID des hinterlegten {@link StrukturElement}es
		 */
		public String getPid() {
			String re = null;

			if (strukturElement != null) {
				re = strukturElement.getPid();
			}

			return re;
		}

		/**
		 * Liefert die PID des in dieser Klasse hinterlegten
		 * {@link StrukturElement}es, erweitert um die TmcRichtung, als
		 * Schlssel zurck.
		 * 
		 * @return
		 */
		public String getKey() {
			String re = null;
			if (strukturElement != null) {
				re = getPid();

				if (richtungPositiv) {
					re = re + " (TmcRichtung: positiv)";
				} else {
					re = re + " (TmcRichtung: negativ)";
				}
			}

			return re;
		}

		/**
		 * Liefert den Pfad, auf dem das {@link StrukturElement} zu erreichen
		 * ist. Beim letzten Element in der Liste handelt es sich um das in
		 * {@link StrukturElementMitPfad} hinterlegte {@link StrukturElement},
		 * beim ersten Element um das Ausgangselement.
		 * 
		 * @return Pfad, auf dem das {@link StrukturElement} zu erreichen ist.
		 */
		public List<StrukturElement> getPfad() {
			return pfad;
		}

		/**
		 * Setzt den Pfad, auf dem das {@link StrukturElement} zu erreichen ist.
		 * Beim letzten Element in der Liste handelt es sich um das in
		 * {@link StrukturElementMitPfad} hinterlegte {@link StrukturElement},
		 * beim ersten Element um das Ausgangselement.
		 * 
		 * @param pfad
		 *            Pfad, auf dem das {@link StrukturElement} zu erreichen
		 *            ist.
		 * 
		 * @throws IllegalArgumentException
		 *             Wird geworfen, wenn es sich beim letzten Element im Pfad
		 *             nicht um das in dieser Klasse bereits hinterlegte
		 *             {@link StrukturElement} handelt.
		 */
		public void setPfad(final List<StrukturElement> pfad) {
			if (pfad.isEmpty()
					|| (pfad.get(pfad.size() - 1) != strukturElement)) {
				throw new IllegalArgumentException(
						"Das letzte Element im Pfad entspricht nicht dem hinterlegten StrukturElement!");
			}
			this.pfad = pfad;
		}

		/**
		 * Prfung auf Lnge.
		 * 
		 * @param stationierung
		 * @return
		 */
		public boolean pruefe(final long stationierung) {
			final boolean re = true;

			/*
			 * Richtungsbereinigung positiv/negativ
			 */
			final LinkedList<StrukturElement> bereinigt = new LinkedList<StrukturElement>();
			for (int i = 0; i < pfad.size(); i++) {
				final StrukturElement curStrukturElement = pfad.get(i);

				if (!((i == 0) && (!richtungPositiv) && (curStrukturElement
						.isAeusseresStrassenSegment()))) {
					if (curStrukturElement.isAeusseresStrassenSegment()) {
						bereinigt.add(curStrukturElement);
					}
				}

			}

			return re;
		}

		@Override
		public String toString() {
			String re = strukturElement.toString() + " ";
			re = re + " Anzahl Elemente im Pfad: " + pfad.size();
			return re;
		}

		/**
		 * Lange Version der <code>toString</code>-Methode mit kompletter
		 * Pfadausgabe.
		 * 
		 * @return Lang-Version von <code>toString</code>.
		 */
		public String toStringLang() {

			final StringBuilder buffer = new StringBuilder();
			buffer.append(toString());

			if (pfad.size() > 0) {
				buffer.append('[');
				final Iterator<StrukturElement> it = pfad.iterator();
				while (it.hasNext()) {
					final StrukturElement element = it.next();
					buffer.append(element.getPid());
					if (it.hasNext()) {
						buffer.append("; ");
					}
				}

				buffer.append(']');
			}

			return buffer.toString();
		}

	}

	/**
	 * Liefert eine dem Namen nach sortierte Liste der {@link Strasse}n zurck.
	 * 
	 * @return Sortierte Liste der {@link Strasse}n.
	 */
	public List<Strasse> getStrassen() {

		final Map<String, Strasse> strassen = konf.getStrassen();

		final Map<String, Strasse> strassenSortiert = new TreeMap<String, Strasse>();

		strassenSortiert.putAll(strassen);

		final List<Strasse> listeStrassen = new LinkedList<Strasse>();
		final Iterator<Entry<String, Strasse>> it = strassenSortiert.entrySet()
				.iterator();
		while (it.hasNext()) {
			final Entry<String, Strasse> entry = it.next();

			if (!"null".equals(entry.getValue().getName())) {
				listeStrassen.add(entry.getValue());
			}

		}

		return listeStrassen;
	}

	/**
	 * Liefert ein StrukturElement an Hand seiner PID zurck.
	 * 
	 * @param pid
	 *            PID des {@link StrukturElement}es
	 * @param tmcRichtung
	 *            TmcRichtung des {@link StrukturElement}es (
	 *            <code>positiv</code> oder <code>negativ</code>)
	 * @return {@link StrukturElement}
	 */
	public StrukturElement getStrukturElement(final String pid,
			final String tmcRichtung) {
		StrukturElement strukturElement = null;

		if (strukturPos.containsKey(pid)) {
			final StrukturElement strukturElementPos = strukturPos
					.getStrukturElement(pid);
			if (strukturElementPos.getTmcRichtung().equals(tmcRichtung)) {
				strukturElement = strukturElementPos;
			}
		}

		if ((strukturElement == null) && (strukturNeg.containsKey(pid))) {
			final StrukturElement strukturElementNeg = strukturNeg
					.getStrukturElement(pid);
			if (strukturElementNeg.getTmcRichtung().equals(tmcRichtung)) {
				strukturElement = strukturElementNeg;
			}
		}

		return strukturElement;
	}

	private static class AsbKnotenVerwaltung {
		/**
		 * TMC-Richtung.
		 */
		private final EnumTmcRichtung tmcRichtung;

		/**
		 * Liste der {@link AsbKnotenAnfangEnde}.
		 */
		private final List<AsbKnotenAnfangEnde> asbKnoten = new LinkedList<AsbKnotenAnfangEnde>();

		/**
		 * Konstruktor, bernimmt die {@link EnumTmcRichtung} als Parameter.
		 * 
		 * @param tmcRichtung
		 *            {@link EnumTmcRichtung}
		 */
		public AsbKnotenVerwaltung(final EnumTmcRichtung tmcRichtung) {
			this.tmcRichtung = tmcRichtung;
		}

		/**
		 * Fgt eine {@link AsbStationierung} zur {@link AsbKnotenVerwaltung}
		 * hinzu. Existiert diese Stationierung nicht, wird diese als neuer
		 * {@link AsbKnotenAnfangEnde} hinzugefgt. Dieser beinhaltet den
		 * Anfangs und den Endknoten, die {@link AsbStationierung} wird in einer
		 * Liste von AsbStationierungsBereichen verwaltet. Wird erneut eine
		 * {@link AsbStationierung} mit gleichem Anfangs- und Endknoten
		 * hinzugefgt, so wird diese nur noch in der Bereichsliste des bereits
		 * existierenden {@link AsbKnotenAnfangEnde} hinzugefgt.
		 * 
		 * @param asbStationierung
		 *            Hinzuzufgende {@link AsbStationierung}.
		 */
		public void addAsbStationierung(final AsbStationierung asbStationierung) {
			AsbKnotenAnfangEnde asbKnotenBereich = null;

			/*
			 * Prfen, ob eine ASB-Stationierung mit gleichen Anfangs und
			 * Endknoten existiert
			 */
			final String anfangsKnoten = asbStationierung.getAnfangsKnoten();
			final String endKnoten = asbStationierung.getEndKnoten();

			final Iterator<AsbKnotenAnfangEnde> it = asbKnoten.iterator();
			while (it.hasNext()) {
				final AsbKnotenAnfangEnde pruefKnotenBereich = it.next();
				if ((pruefKnotenBereich.getAnfangsKnoten().getKnoten()
						.equals(anfangsKnoten))
						&& (pruefKnotenBereich.getEndKnoten().getKnoten()
								.equals(endKnoten))) {
					asbKnotenBereich = pruefKnotenBereich;
					break;
				}
			}

			/*
			 * Wenn asbKnotenBereich == null, dann neu anlegen und zur Liste der
			 * AsbKnotenBereiche hinzufgen
			 */
			if (asbKnotenBereich == null) {
				asbKnotenBereich = new AsbKnotenAnfangEnde(asbStationierung,
						tmcRichtung);
				asbKnoten.add(asbKnotenBereich);
			}

			/*
			 * Neuen Bereich hinzufgen
			 */
			asbKnotenBereich.addBereich(asbStationierung);
		}

		/**
		 * Bestimmt eine kombinierte Liste aller Knoten. Diese wird aus Anfangs-
		 * und EndKnoten erstellt. Jeder Knoten kommt, egal ob Anfangs- oder
		 * Endknoten, nur ein mal in der Liste vor.
		 * 
		 * @return Kombinierte Liste aller Knoten.
		 */
		public List<AsbKnoten> bestimmeListeAllerKnoten() {
			final List<AsbKnoten> re = new LinkedList<AsbKnoten>();
			// Von String auf AsbKnoten umgestellt

			final Iterator<AsbKnotenAnfangEnde> it = asbKnoten.iterator();
			while (it.hasNext()) {
				final AsbKnotenAnfangEnde asbKnotenAnfangEnde = it.next();

				/*
				 * Ist der AnfangsKnoten schon vorhanden? Wenn nein, hinzufgen.
				 */
				if (!istKnotenAsbInListe(re,
						asbKnotenAnfangEnde.getAnfangsKnoten())) {
					re.add(asbKnotenAnfangEnde.getAnfangsKnoten());
				}

				/*
				 * Ist der EndKnoten schon vorhanden? Wenn nein, hinzufgen.
				 */
				// if (!re.contains(asbKnotenAnfangEnde.getEndKnoten()))
				if (!istKnotenAsbInListe(re, asbKnotenAnfangEnde.getEndKnoten())) {
					re.add(asbKnotenAnfangEnde.getEndKnoten());
				}
			}

			// Collections.sort(re, Collator.getInstance());

			Collections.sort(re);

			return re;
		}

		/**
		 * Bestimmt eine Liste aller von einem Knoten aus erreichbaren Knoten.
		 * 
		 * @return Liste aller von einem Knoten aus erreichbaren Knoten
		 */
		public List<AsbKnoten> bestimmeListeAllerErreichbarenKnoten(
				final AsbKnoten ausgangsKnoten) {
			final List<AsbKnoten> re = new LinkedList<AsbKnoten>();
			// Von String auf AsbKnoten umgestellt

			final Iterator<AsbKnotenAnfangEnde> it = asbKnoten.iterator();
			while (it.hasNext()) {
				final AsbKnotenAnfangEnde knotenAnfangEnde = it.next();

				/*
				 * Anfangsknoten == Ausgangsknoten?
				 */
				if (knotenAnfangEnde.getAnfangsKnoten().equals(ausgangsKnoten)) {
					// if (!re.contains(knotenAnfangEnde.getEndKnoten()))
					if (!istKnotenAsbInListe(re,
							knotenAnfangEnde.getEndKnoten())) {
						re.add(knotenAnfangEnde.getEndKnoten());
					}
				}

				/*
				 * Endknoten == Ausgabgsknoten?
				 */
				if (knotenAnfangEnde.getEndKnoten().equals(ausgangsKnoten)) {
					// if (!re.contains(knotenAnfangEnde.getAnfangsKnoten()))
					if (!istKnotenAsbInListe(re,
							knotenAnfangEnde.getAnfangsKnoten())) {
						re.add(knotenAnfangEnde.getAnfangsKnoten());
					}
				}
			}

			// Collections.sort(re, Collator.getInstance());
			Collections.sort(re);

			return re;
		}

		/**
		 * Prft, ob ein {@link AsbKnoten} in einer {@link Collection} von
		 * {@link AsbKnoten} vorhanden ist.
		 * 
		 * @param collection
		 *            {@link Collection}, die zu berprfen ist
		 * @param knoten
		 *            {@link AsbKnoten}, auf dessen vorhanden sein i.d.
		 *            {@link Collection} zu prfen ist.
		 * @return Liefert <code>true</code>, wenn der Knoten in der
		 *         {@link Collection} vorhanden ist, sonst <code>false</code>.
		 */
		private boolean istKnotenAsbInListe(
				final Collection<AsbKnoten> collection, final AsbKnoten knoten) {
			boolean re = false;
			final Iterator<AsbKnoten> it = collection.iterator();
			while (it.hasNext()) {
				final AsbKnoten pruefKnoten = it.next();
				if ((pruefKnoten.getKnoten().equals(knoten.getKnoten()))
						&& (pruefKnoten.getTmcRichtung().equals(knoten
								.getTmcRichtung()))) {
					re = true;
					break;
				}
			}

			return re;
		}

		// public boolean istInBereich(AsbKnoten knotenA, AsbKnoten knotenB,
		// long stationierung)
		public BereichsInfoAsbKnoten istInBereich(final AsbKnoten knotenA,
				final AsbKnoten knotenB, final long stationierung) {
			BereichsInfo bereichsInfo = null;

			/*
			 * Knoten, der durch den Angegebenen Bereich bestimmt ist.
			 */
			AsbKnotenAnfangEnde node = null;

			/*
			 * Vorwrts-Annahme: AnfangsKnoten == knotenA, EndKnoten == knotenB
			 */
			final Iterator<AsbKnotenAnfangEnde> itVor = asbKnoten.iterator();
			while (itVor.hasNext()) {
				final AsbKnotenAnfangEnde pruefKnoten = itVor.next();
				if ((pruefKnoten.getAnfangsKnoten().equals(knotenA))
						&& (pruefKnoten.getEndKnoten().equals(knotenB))) {
					node = pruefKnoten;
					break;
				}
			}

			/*
			 * Rckwrts-Annahme: AnfangsKnoten == knotenB, EndKnoten == knotenA
			 */
			if (node == null) {
				final Iterator<AsbKnotenAnfangEnde> it = asbKnoten.iterator();
				while (it.hasNext()) {
					final AsbKnotenAnfangEnde pruefknoten = it.next();
					if ((pruefknoten.getEndKnoten().equals(knotenA))
							&& (pruefknoten.getAnfangsKnoten().equals(knotenB))) {
						node = pruefknoten;
						break;
					}
				}
			}

			/*
			 * Bereichsprfung durchfhren.
			 */
			if (node != null) {
				// re = node.istInBereichen(stationierung);
				bereichsInfo = node.istInBereichen(stationierung);
			}

			final BereichsInfoAsbKnoten bereichsInfoAsb = new BereichsInfoAsbKnoten(
					knotenA, knotenB, stationierung, bereichsInfo);

			// return istInBereich;
			return bereichsInfoAsb;
		}

		@Override
		public String toString() {
			final StringBuffer buffer = new StringBuffer();
			final Iterator<AsbKnotenAnfangEnde> it = asbKnoten.iterator();
			int i = 0;
			while (it.hasNext()) {
				final AsbKnotenAnfangEnde asbKnotenAnfangEnde = it.next();
				buffer.append('#');
				buffer.append(i);
				buffer.append(":\n");
				buffer.append(asbKnotenAnfangEnde);
				buffer.append('\n');
				i++;
			}
			return buffer.toString();
		}
	}

	/**
	 * Klasse zur Reprsentation der vorhandenen Stationierungsknoten.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class AsbKnotenAnfangEnde {
		/**
		 * AnfangsKnoten.
		 */
		private final AsbKnoten anfangsKnoten;

		/**
		 * EndKnoten.
		 */
		private final AsbKnoten endKnoten;

		private final List<AsbStationierung> asbStationierungsBereiche = new LinkedList<AsbStationierung>();

		/**
		 * Konstruktor, bernimmt Anfangs- und EndKnoten.
		 * 
		 * @param anfangsKnoten
		 *            AnfangsKnoten
		 * @param endKnoten
		 *            EndKnoten
		 */
		public AsbKnotenAnfangEnde(final AsbStationierung asbStationierung,
				final EnumTmcRichtung tmcRichtung) {

			this.anfangsKnoten = new AsbKnoten(
					asbStationierung.getAnfangsKnoten(), tmcRichtung,
					asbStationierung.getVerkehrsRichtung());
			this.endKnoten = new AsbKnoten(asbStationierung.getEndKnoten(),
					tmcRichtung, asbStationierung.getVerkehrsRichtung());
			addBereich(asbStationierung);
		}

		/**
		 * Fgt einen {@link AsbStationierung}s-Bereich (Anfang, Ende) hinzu,
		 * sofern dieser noch nicht vorhanden ist. Vergleiche ATG
		 * AsbStationiereung. An Hand diesem kann dann bestimmt werden, ob eine
		 * Stationierung im gltigen Bereich liegt.
		 * 
		 * @param Hinzuzufgender
		 *            {@link AsbStationierung}s-Bereich.
		 */
		public void addBereich(final AsbStationierung bereich) {

			if (!asbStationierungsBereiche.contains(bereich)) {
				asbStationierungsBereiche.add(bereich);
			}
		}

		/**
		 * Prft, ob eine gegebene Stationierungsangabe in Metern im Bereich
		 * liegt.<br>
		 * 
		 * @param stationierungsangabe
		 *            Stationierungsangabe in Metern
		 * @return Liefert <code>true</code>, wenn die Stationierungsangabe in
		 *         einem der Bereiche liegt, sonst <code>false</code>.
		 */
		// public boolean istInBereichen(long stationierungsangabe)
		public BereichsInfo istInBereichen(final long stationierungsangabe) {
			boolean istInBereich = false;

			long minBereich = -1;
			long maxBereich = -1;

			final Iterator<AsbStationierung> it = asbStationierungsBereiche
					.iterator();
			while (it.hasNext()) {
				final AsbStationierung bereich = it.next();
				final long anfang = bereich.getAnfang();
				final long ende = bereich.getEnde();

				if (minBereich == -1) {
					minBereich = anfang;
				} else {
					if (anfang < minBereich) {
						minBereich = anfang;
					}
				}

				if (maxBereich == -1) {
					maxBereich = ende;
				} else {
					if (ende > maxBereich) {
						maxBereich = ende;
					}
				}

				/*
				 * Liegt die Stationierungsangabe genau auf dem Angang oder dem
				 * Ende eines Bereichs?
				 */

				if ((stationierungsangabe == anfang)
						|| (stationierungsangabe == ende)) {
					istInBereich = true;
					break;
				}

				/*
				 * Anfang < Ende: Ist die Stationierungsangabe zwischen den
				 * Bereichen?
				 */
				if (anfang < ende) {
					if ((stationierungsangabe >= anfang)
							&& (stationierungsangabe <= ende)) {
						istInBereich = true;
						break;
					}
				}

				/*
				 * Anfang > Ende: Ist die Stationierungsangabe zwischen den
				 * Bereichen?
				 */
				if (anfang > ende) {
					if ((stationierungsangabe <= anfang)
							&& (stationierungsangabe >= ende)) {
						istInBereich = true;
						break;
					}
				}

			}

			final BereichsInfo bereichsInfo = new BereichsInfo(istInBereich,
					minBereich, maxBereich);

			// return istInBereich;
			return bereichsInfo;
		}

		/**
		 * Liefert den AnfangsKnoten.
		 * 
		 * @return AnfangsKnoten
		 */
		public AsbKnoten getAnfangsKnoten() {
			return anfangsKnoten;
		}

		/**
		 * Liefert den EndKnoten.
		 * 
		 * @return EndKnoten
		 */
		public AsbKnoten getEndKnoten() {
			return endKnoten;
		}

		@Override
		public String toString() {
			final StringBuffer buffer = new StringBuffer();

			buffer.append("AsbKnotenAnfangEnde: AnfangsKnoten=");
			buffer.append(anfangsKnoten);
			buffer.append(" EndKnoten=");
			buffer.append(endKnoten);
			buffer.append('\n');

			final Iterator<AsbStationierung> it = asbStationierungsBereiche
					.iterator();
			int i = 0;
			while (it.hasNext()) {
				final AsbStationierung asbStationierung = it.next();
				final long anfang = asbStationierung.getAnfang();
				final long ende = asbStationierung.getEnde();
				buffer.append("Eintrag #");
				buffer.append(i);
				buffer.append(": Anfang=");
				buffer.append(anfang);
				buffer.append("m, Ende=");
				buffer.append(ende);
				buffer.append("m. (Sts:");
				buffer.append(asbStationierung
						.getGehoertZuStrassenTeilSegement().getPid());
				buffer.append('\n');
				i++;
			}

			return buffer.toString();
		}

	}

	/**
	 * Klasse zur Erfassung und Darstellung von ASB-Knoten. Erfasst wird die
	 * Kennung des Knotens sowie, in welcher TMC-Richtung dieser Knoten liegt.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public static class AsbKnoten implements Comparable<AsbKnoten> {

		/** Kennung des Knotens. */
		private final String knoten;

		/** TMC-Richtung des Knotens. */
		private final EnumTmcRichtung tmcRichtung;

		/** speichert die ASB Verkehrsrichtung. */
		private final EnumAsbVerkehrsRichtung asbVerkehrsRichtung;

		/**
		 * Konstruktor, bernimmt die Kennung sowie die {@link EnumTmcRichtung}
		 * des Knotens.
		 * 
		 * @param knoten
		 *            Kennung des Knotens
		 * @param tmcRichtung
		 *            {@link EnumTmcRichtung} des Knotens
		 * @param asbVerkehrsRichtung
		 *            {@link EnumAsbVerkehrsRichtung}
		 */
		public AsbKnoten(final String knoten,
				final EnumTmcRichtung tmcRichtung,
				final EnumAsbVerkehrsRichtung asbVerkehrsRichtung) {
			this.knoten = knoten;
			this.tmcRichtung = tmcRichtung;
			this.asbVerkehrsRichtung = asbVerkehrsRichtung;
		}

		/**
		 * Liefert die Kennung des Knotens.
		 * 
		 * @return Knotenkennung.
		 */
		public String getKnoten() {
			return knoten;
		}

		/**
		 * Liefert die {@link EnumTmcRichtung} des Knotens.
		 * 
		 * @return {@link EnumTmcRichtung} des Knotens
		 */
		public EnumTmcRichtung getTmcRichtung() {
			return tmcRichtung;
		}

		/**
		 * Liefert die {@link EnumAsbVerkehrsRichtung}.
		 * 
		 * @return {@link EnumAsbVerkehrsRichtung}.
		 */
		public EnumAsbVerkehrsRichtung getAsbVerkehrsRichtung() {
			return asbVerkehrsRichtung;
		}

		@Override
		public boolean equals(final Object obj) {
			boolean re = false;

			if (obj instanceof AsbKnoten) {
				final AsbKnoten vergleich = (AsbKnoten) obj;

				if ((this.getKnoten().equals(vergleich.getKnoten()))
						&& (this.getTmcRichtung().equals(vergleich
								.getTmcRichtung()))) {
					re = true;
				}

			}

			return re;
		}

		@Override
		public int hashCode() {
			return knoten.hashCode();
		}

		@Override
		public String toString() {
			// String re = _knoten + " (TmcRichtung: " + _tmcRichtung + ")";
			final String re = knoten + " (AsbVerkehrsRichtung: "
					+ asbVerkehrsRichtung + ")";
			return re;
		}

		@Override
		public int compareTo(final AsbKnoten o) {
			final String aktuell = this.getKnoten();
			final String vergleich = o.getKnoten();

			return aktuell.compareTo(vergleich);
		}

	}

	/**
	 * Liefert die kombinierte ASB-Knotenverwaltung zurck.
	 * 
	 * @return
	 */
	public AsbKnotenVerwaltungKombiniert getAsbKnotenVerwaltungKombiniert() {
		return asbKnotenVerwaltungKombiniert;
	}

	public class AsbKnotenVerwaltungKombiniert {
		/**
		 * HashMap der kombinierten ASB-Knoten.
		 */
		private final Map<String, AsbKnotenKombiniert> asbKnotenKombiniert = new TreeMap<String, AsbKnotenKombiniert>();

		/**
		 * Fgt einen {@link AsbKnoten} in positiver TMC-Richtung zur Map der
		 * {@link AsbKnotenKombiniert} hinzu.
		 * 
		 * @param asbKnotenPositiv
		 *            Hinzuzufgender {@link AsbKnoten} in positiver
		 *            TMC-Richtung.
		 */
		public void addAsbKnotenPositiv(final AsbKnoten asbKnotenPositiv) {

			if (asbKnotenKombiniert.containsKey(asbKnotenPositiv.getKnoten())) {
				asbKnotenKombiniert.get(asbKnotenPositiv.getKnoten())
						.setAsbKnotenPositiv(asbKnotenPositiv);
			} else {
				final AsbKnotenKombiniert asbKnotenKombiniertPositiv = new AsbKnotenKombiniert(
						asbKnotenPositiv.getKnoten());
				asbKnotenKombiniertPositiv
						.setAsbKnotenPositiv(asbKnotenPositiv);
				asbKnotenKombiniert.put(asbKnotenPositiv.getKnoten(),
						asbKnotenKombiniertPositiv);
			}
		}

		/**
		 * Fgt eine Liste von {@link AsbKnoten} in positiver TMC-Richtung zur
		 * Map der {@link AsbKnotenKombiniert} hinzu.
		 * 
		 * @param asbKnotenPositiv
		 *            Hinzuzufgende Liste von {@link AsbKnoten} in positiver
		 *            TMC-Richtung.
		 */
		public void addAsbKnotenPositiv(final List<AsbKnoten> asbKnotenPositiv) {
			final Iterator<AsbKnoten> it = asbKnotenPositiv.iterator();
			while (it.hasNext()) {
				addAsbKnotenPositiv(it.next());
			}
		}

		/**
		 * Fgt einen {@link AsbKnoten} in negativer TMC-Richtung zur Map der
		 * {@link AsbKnotenKombiniert} hinzu.
		 * 
		 * @param asbKnotenNegativ
		 *            Hinzuzufgender {@link AsbKnoten} in negativer
		 *            TMC-Richtung.
		 */
		public void addAsbKnotenNegativ(final AsbKnoten asbKnotenNegativ) {
			if (asbKnotenKombiniert.containsKey(asbKnotenNegativ.getKnoten())) {
				asbKnotenKombiniert.get(asbKnotenNegativ.getKnoten())
						.setAsbKnotenNegativ(asbKnotenNegativ);
			} else {
				final AsbKnotenKombiniert asbKnotenKombiniertNegativ = new AsbKnotenKombiniert(
						asbKnotenNegativ.getKnoten());
				asbKnotenKombiniertNegativ
						.setAsbKnotenNegativ(asbKnotenNegativ);
				asbKnotenKombiniert.put(asbKnotenNegativ.getKnoten(),
						asbKnotenKombiniertNegativ);
			}
		}

		/**
		 * Fgt eine Liste von {@link AsbKnoten} in negativer TMC-Richtung zur
		 * Map der {@link AsbKnotenKombiniert} hinzu.
		 * 
		 * @param asbKnotenNegativ
		 *            Hinzuzufgende Liste von {@link AsbKnoten} in negativer
		 *            TMC-Richtung.
		 */
		public void addAsbKnotenNegativ(final List<AsbKnoten> asbKnotenNegativ) {
			final Iterator<AsbKnoten> it = asbKnotenNegativ.iterator();
			while (it.hasNext()) {
				addAsbKnotenNegativ(it.next());
			}
		}

		/**
		 * Liefert eine Liste der {@link AsbKnotenKombiniert}.
		 * 
		 * @return Liste der {@link AsbKnotenKombiniert}
		 */
		public List<AsbKnotenKombiniert> getListeKombinierteKnoten() {
			final List<AsbKnotenKombiniert> re = new LinkedList<AsbKnotenKombiniert>();

			final Iterator<Entry<String, AsbKnotenKombiniert>> it = asbKnotenKombiniert
					.entrySet().iterator();
			while (it.hasNext()) {
				re.add(it.next().getValue());
			}

			return re;
		}

		/**
		 * Prft, ob sich eine Stationierung in einem gltigen Bereich zwischen
		 * zwei ASB-Knoten befindet.
		 * 
		 * @param knotenA
		 *            Erster Knoten
		 * @param knotenB
		 *            Zweiter Knoten
		 * @param stationierung
		 *            Stationierung zwischen den beiden Knoten
		 * @return Liefert <code>true</code>, wenn die Stationierung zwischen
		 *         den beiden Knoten gltig ist, sonst <code>false</code>.
		 */
		public boolean istInBereich(final AsbKnotenKombiniert knotenA,
				final AsbKnotenKombiniert knotenB, final long stationierung) {
			boolean inBereich = false;

			/*
			 * Prfung in TMC-Richtung positiv
			 */
			if ((knotenA.hatKnotenPositiv()) && (knotenB.hatKnotenPositiv())) {
				if (asbKnotenVerwaltungPositiv.istInBereich(
						knotenA.getAsbKnotenPositiv(),
						knotenB.getAsbKnotenPositiv(), stationierung)
						.istInBereich()) {
					inBereich = true;
				}
			}

			/*
			 * Prfung in TMC-Richtung negativ
			 */
			if ((knotenA.hatKnotenNegativ()) && (knotenB.hatKnotenNegativ())) {
				if (asbKnotenVerwaltungNegativ.istInBereich(
						knotenA.getAsbKnotenNegativ(),
						knotenB.getAsbKnotenNegativ(), stationierung)
						.istInBereich()) {
					inBereich = true;
				}
			}

			return inBereich;
		}

		/**
		 * Prft, ob sich eine Stationierung in einem gltigen Bereich zwischen
		 * zwei ASB-Knoten befindet.
		 * 
		 * @param knotenA
		 *            Erster Knoten
		 * @param knotenB
		 *            Zweiter Knoten
		 * @param stationierung
		 *            Stationierung zwischen den beiden Knoten.
		 * @return
		 */
		public BereichsInfoAsbKnotenKombiniert istInBereichDetailiert(
				final AsbKnotenKombiniert knotenA,
				final AsbKnotenKombiniert knotenB, final long stationierung) {
			BereichsInfoAsbKnoten bereichsInfoAsbKnotenTmcPositiv = null;
			BereichsInfoAsbKnoten bereichsInfoAsbKnotenTmcNegativ = null;

			/*
			 * Prfung in TMC-Richtung positiv
			 */
			if ((knotenA.hatKnotenPositiv()) && (knotenB.hatKnotenPositiv())) {
				bereichsInfoAsbKnotenTmcPositiv = asbKnotenVerwaltungPositiv
						.istInBereich(knotenA.getAsbKnotenPositiv(),
								knotenB.getAsbKnotenPositiv(), stationierung);
			}

			/*
			 * Prfung in TMC-Richtung negativ
			 */
			if ((knotenA.hatKnotenNegativ()) && (knotenB.hatKnotenNegativ())) {
				bereichsInfoAsbKnotenTmcNegativ = asbKnotenVerwaltungNegativ
						.istInBereich(knotenA.getAsbKnotenNegativ(),
								knotenB.getAsbKnotenNegativ(), stationierung);
			}

			final BereichsInfoAsbKnotenKombiniert bereichsInfoAsbKnotenKombiniert = new BereichsInfoAsbKnotenKombiniert(
					bereichsInfoAsbKnotenTmcPositiv,
					bereichsInfoAsbKnotenTmcNegativ);

			return bereichsInfoAsbKnotenKombiniert;
		}
	}

	// public class AsbKnotenVerwaltungKombiniertBereichsPruefung
	// {
	// /**
	// * Speichert die Stationierung in Metern.
	// */
	// private long _stationierung = -1;
	//
	// /**
	// * Konstruktor, bernimmt die Stationierung in Metern.
	// *
	// * @param stationierung
	// * Stationierung in Metern
	// */
	// public AsbKnotenVerwaltungKombiniertBereichsPruefung(long stationierung,
	// AsbKnoten knotenATmcPos, AsbKnoten knotenBTmcPos,
	// boolean istInBereichTmcPos, AsbKnoten knotenATmcNeg, AsbKnoten knoten)
	// {
	// this._stationierung = stationierung;
	// }
	//
	// /**
	// * Liefert die Stationierung in Metern.
	// *
	// * @return Stationierung in Metern.
	// */
	// public long getStationierung()
	// {
	// return _stationierung;
	// }
	//
	// }

	/**
	 * Speichert unter der ID eines {@link AsbKnoten}s sowohl den
	 * {@link AsbKnoten} in positiver als auch in negativer TMC-Richtung.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public static class AsbKnotenKombiniert {
		private final String asbKnoten;
		private AsbKnoten asbKnotenPositiv;
		private AsbKnoten asbKnotenNegativ;

		/**
		 * Konstruktor, bernimmt den Titel des kombinieren {@link AsbKnoten}s.
		 * 
		 * @param knoten
		 *            Titel
		 */
		public AsbKnotenKombiniert(final String knoten) {
			this.asbKnoten = knoten;
		}

		/**
		 * Liefert den Titel des kombinierten {@link AsbKnoten}s.
		 * 
		 * @return Titel
		 */
		public String getKnoten() {
			return asbKnoten;
		}

		/**
		 * Liefert den {@link AsbKnoten} ins positiver TMC-Richtung.
		 * 
		 * @return {@link AsbKnoten} ins positiver TMC-Richtung
		 */
		public AsbKnoten getAsbKnotenPositiv() {
			return asbKnotenPositiv;
		}

		/**
		 * Setzt den {@link AsbKnoten} in positiver TMC-Richtung.
		 * 
		 * @param asbKnotenPositiv
		 *            {@link AsbKnoten} in positiver TMC-Richtung
		 */
		public void setAsbKnotenPositiv(final AsbKnoten asbKnotenPositiv) {
			this.asbKnotenPositiv = asbKnotenPositiv;
		}

		/**
		 * Liefert den {@link AsbKnoten} in negativer TMC-Richtung.
		 * 
		 * @return {@link AsbKnoten} in negativer TMC-Richtung.
		 */
		public AsbKnoten getAsbKnotenNegativ() {
			return asbKnotenNegativ;
		}

		/**
		 * Setzt den {@link AsbKnoten} in negativer TMC-Richtung.
		 * 
		 * @param asbKnotenNegativ
		 *            {@link AsbKnoten} in negativer TMC-Richtung
		 */
		public void setAsbKnotenNegativ(final AsbKnoten asbKnotenNegativ) {
			this.asbKnotenNegativ = asbKnotenNegativ;
		}

		/**
		 * Prft, ob ein {@link AsbKnoten} in positiver TMC-Richtung gesetzt
		 * wurde.
		 * 
		 * @return Liefert <code>true</code>, wenn ein {@link AsbKnoten} in
		 *         positiver TMC-Richtung gesetzt wurde, sonst
		 *         <code>false</code>.
		 */
		public boolean hatKnotenPositiv() {
			if (this.asbKnotenPositiv != null) {
				return true;
			}
			return false;
		}

		/**
		 * Prft, ob ein {@link AsbKnoten} in negativer TMC-Richtung gesetzt
		 * wurde.
		 * 
		 * @return Liefert <code>true</code>, wenn ein {@link AsbKnoten} in
		 *         negativer TMC-Richtung gesetzt wurde, sonst
		 *         <code>false</code>.
		 */
		public boolean hatKnotenNegativ() {
			if (this.asbKnotenNegativ != null) {
				return true;
			}
			return false;
		}

		@Override
		public String toString() {
			return asbKnoten;
		}
	}

	/**
	 * Enumerator fr die Richtung.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum EnumTmcRichtung {
		/**
		 * Positive TMC-Richtung.
		 */
		POSITIV,

		/**
		 * Negative TMC-Richtung.
		 */
		NEGATIV;
	}

	/**
	 * Speichert Informationen ber einen Bereich. Dazu gehrt:
	 * <ul>
	 * <li>Befinden wir uns im Bereich?</li>
	 * <li>Minimaler, gltiger Wert</li>
	 * <li>Maximaler, gltiger Wert</li>
	 * </ul>
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class BereichsInfo {
		/**
		 * Flag, zeigt an ob der Bereich gltig oder nicht gltig ist.
		 */
		private final boolean istInBereich;

		/**
		 * Minimaler Wert des Bereiches.
		 */
		private long bereichMin = -1;

		/**
		 * Maximaler Wert des Bereiches.
		 */
		private long bereichMax = -1;

		/**
		 * Konstruktor.
		 * 
		 * @param istInBereich
		 *            Muss <code>true</code> sein, wenn wir uns im Bereich
		 *            befinden, sonst <code>false</code>.
		 * @param bereichMin
		 *            Minimaler Wert des Bereiches
		 * @param bereichMax
		 *            Maximaler Wert des Bereiches
		 */
		public BereichsInfo(final boolean istInBereich, final long bereichMin,
				final long bereichMax) {
			this.istInBereich = istInBereich;
			this.bereichMin = bereichMin;
			this.bereichMax = bereichMax;
		}

		/**
		 * Prft, ob wir uns im Bereich befinden.
		 * 
		 * @return Liefert <code>true</code>, wenn wir uns im Bereich befinden,
		 *         sonst <code>false</code>.
		 */
		public boolean istInBereich() {
			return istInBereich;
		}

		/**
		 * Liefert den minimalsten Wert des Bereichs.
		 * 
		 * @return Minimalster Wert des Bereichs.
		 */
		public long getBereichMin() {
			return bereichMin;
		}

		/**
		 * Liefert den maximalsten Wert des Bereichs.
		 * 
		 * @return Maximalster Wert des Bereichs.
		 */
		public long getBereichMax() {
			return bereichMax;
		}

		@Override
		public String toString() {
			String re = "";

			re = re + "IstInBereich: " + istInBereich + " ";
			re = re + "BereichMin: " + bereichMin + " ";
			re = re + "BereichMax: " + bereichMax + " ";

			return re;
		}
	}

	/**
	 * Liefert die Information zum Bereich eines ASB-Knotens.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public static class BereichsInfoAsbKnoten {
		/**
		 * Von-{@link AsbKnoten}.
		 */
		private final AsbKnoten von;

		/**
		 * Bis-{@link AsbKnoten}.
		 */
		private final AsbKnoten bis;

		/**
		 * Stationierung zwischen den beiden Knoten.
		 */
		private long stationierung = -1;

		/**
		 * {@link BereichsInfo}.
		 */
		private final BereichsInfo bereichsInfo;

		/**
		 * Konstruktor. bernimmt verschiedene Parameter zur Definition der
		 * Bereichsinformation eines {@link AsbKnoten}s.
		 * 
		 * @param von
		 *            Von-{@link AsbKnoten} des Bereichs.
		 * @param bis
		 *            Bis-{@link AsbKnoten} des Bereichs.
		 * @param stationierung
		 *            Stationierung zwischen den beiden Knoten in Metern.
		 * @param bereichsInfo
		 *            {@link BereichsInfo}.
		 */
		public BereichsInfoAsbKnoten(final AsbKnoten von, final AsbKnoten bis,
				final long stationierung, final BereichsInfo bereichsInfo) {
			this.von = von;
			this.bis = bis;
			this.stationierung = stationierung;
			this.bereichsInfo = bereichsInfo;
		}

		/**
		 * Liefert den Von-{@link AsbKnoten}.
		 * 
		 * @return Von-{@link AsbKnoten}
		 */
		public AsbKnoten getAsbKnotenVon() {
			return von;
		}

		/**
		 * Liefert den Bis-{@link AsbKnoten}.
		 * 
		 * @return Bis-{@link AsbKnoten}
		 */
		public AsbKnoten getAsbKnotenBis() {
			return bis;
		}

		/**
		 * Liefert die Stationierung zwischen Von- und Bis-{@link AsbKnoten}.
		 * 
		 * @return Von- und Bis-{@link AsbKnoten}
		 */
		public long getStationierung() {
			return stationierung;
		}

		/**
		 * "Prft", ob sich die Stationierung im Bereich zwischen den beiden
		 * {@link AsbKnoten} befindet.
		 * 
		 * @return Liefert <code>true</code>, wenn sich die Stationierung im
		 *         Bereich zwischen den beiden {@link AsbKnoten} befindet, sonst
		 *         <code>false</code>.
		 */
		public boolean istInBereich() {
			return bereichsInfo.istInBereich();
		}

		/**
		 * Liefert den minimalen, gltigen Wert fr den Bereich zwischen den
		 * beiden {@link AsbKnoten}.
		 * 
		 * @return Min-Wert
		 */
		public long getBereichMin() {
			return bereichsInfo.getBereichMin();
		}

		/**
		 * Liefert den maximalen, gltigen Wert fr den Bereich zwischen den
		 * beiden {@link AsbKnoten}.
		 * 
		 * @return Max-Wert
		 */
		public long getBereichMax() {
			return bereichsInfo.getBereichMax();
		}

		/**
		 * Liefert die {@link BereichsInfo} zu den {@link AsbKnoten}.
		 * 
		 * @return {@link BereichsInfo} zu den {@link AsbKnoten}
		 */
		public BereichsInfo getBereichsInfo() {
			return bereichsInfo;
		}

		@Override
		public String toString() {
			String re = "";

			re = re + "Von-Knoten: " + von + " Bis-Knoten: " + bis
					+ " Stationierung: " + stationierung + "m Bereich: "
					+ bereichsInfo.toString();

			return re;
		}
	}

	/**
	 * Kombiniert die {@link BereichsInfoAsbKnoten} fr die {@link AsbKnoten}
	 * aus positiver und neg.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public static class BereichsInfoAsbKnotenKombiniert {
		private final BereichsInfoAsbKnoten bereichsInfoAsbKnotenTmcPos;
		private final BereichsInfoAsbKnoten bereichsInfoAsbKnotenTmcNeg;

		public BereichsInfoAsbKnotenKombiniert(
				final BereichsInfoAsbKnoten bereichsInfoAsbKnotenTmcPos,
				final BereichsInfoAsbKnoten bereichsInfoAsbKnotenTmcNeg) {
			this.bereichsInfoAsbKnotenTmcPos = bereichsInfoAsbKnotenTmcPos;
			this.bereichsInfoAsbKnotenTmcNeg = bereichsInfoAsbKnotenTmcNeg;
		}

		public BereichsInfoAsbKnoten getBereichsInfoAsbKnotenTmcPos() {
			return bereichsInfoAsbKnotenTmcPos;
		}

		public BereichsInfoAsbKnoten getBereichsInfoAsbKnotenTmcNeg() {
			return bereichsInfoAsbKnotenTmcNeg;
		}

		public boolean istInBereich() {
			boolean istInBereich = false;

			/*
			 * Positive TMC-Richtung
			 */
			if (bereichsInfoAsbKnotenTmcPos != null) {
				if (bereichsInfoAsbKnotenTmcPos.istInBereich()) {
					istInBereich = true;
				}
			}

			/*
			 * Negative TMC-Richtung
			 */
			if (bereichsInfoAsbKnotenTmcNeg != null) {
				if (bereichsInfoAsbKnotenTmcNeg.istInBereich()) {
					istInBereich = true;
				}
			}

			return istInBereich;
		}

		/**
		 * Prft, ob sich der gewhlte Bereich zwischen Von- und Bis-
		 * {@link AsbKnoten} gltig ist. Des weitern wird nach der
		 * {@link EnumAsbVerkehrsRichtung} gefiltert. {@link AsbKnoten}-Paare,
		 * die nicht der Vorgabe entsprechen, werden nicht bercksichtigt.
		 * 
		 * @param asbVerkehrsRichtung
		 *            {@link EnumAsbVerkehrsRichtung}, welche filternd auf die
		 *            {@link AsbKnoten}-Paare wirkt.
		 * @return
		 */
		public boolean[] istInBereichGefiltert(
				final EnumAsbVerkehrsRichtung asbVerkehrsRichtung) {
			boolean istInBereichTmcPositiv = false;
			boolean istInBereichTmcNegativ = false;

			/*
			 * Positive TMC-Richtung
			 */
			if (bereichsInfoAsbKnotenTmcPos != null) {
				if (pruefe(asbVerkehrsRichtung,
						bereichsInfoAsbKnotenTmcPos.getAsbKnotenVon(),
						bereichsInfoAsbKnotenTmcPos.getAsbKnotenBis())) {
					if (bereichsInfoAsbKnotenTmcPos.istInBereich()) {
						istInBereichTmcPositiv = true;
					}
				}
			}

			/*
			 * Negative TMC-Richtung
			 */
			if (bereichsInfoAsbKnotenTmcNeg != null) {

				if (pruefe(asbVerkehrsRichtung,
						bereichsInfoAsbKnotenTmcNeg.getAsbKnotenVon(),
						bereichsInfoAsbKnotenTmcNeg.getAsbKnotenBis())) {
					if (bereichsInfoAsbKnotenTmcNeg.istInBereich()) {
						istInBereichTmcNegativ = true;
					}
				}

			}

			final boolean[] re = new boolean[] { istInBereichTmcPositiv,
					istInBereichTmcNegativ };

			return re;
		}

		/**
		 * Prft, ob die Richtung von zwei Knoten einer vorgegebenen
		 * Stationierungsrichtung entspricht.
		 * 
		 * @param richtung
		 *            {@link EnumAsbVerkehrsRichtung}, gegen die geprft werden
		 *            soll.
		 * @param von
		 *            Von-{@link AsbKnoten}, der geprft werden soll.
		 * @param bis
		 *            Bis-{@link AsbKnoten}, der geprft werden soll.
		 * @return Liefert <code>true</code>, wenn beide {@link AsbKnoten} von
		 *         der selben {@link EnumAsbVerkehrsRichtung} sind oder
		 *         <code>false</code>, wenn dem nicht so ist.
		 */
		private boolean pruefe(final EnumAsbVerkehrsRichtung richtung,
				final AsbKnoten von, final AsbKnoten bis) {
			boolean re = false;

			/*
			 * In Stationierungsrichtung
			 */
			if (richtung
					.equals(EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG)) {
				if (von.getAsbVerkehrsRichtung().equals(
						EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG)) {
					if (bis.getAsbVerkehrsRichtung().equals(
							EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG)) {
						re = true;
					}
				}
			} else if (richtung
					.equals(EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG)) {
				/* Gegen Stationierungsrichtung */
				if (von.getAsbVerkehrsRichtung().equals(
						EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG)) {
					if (von.getAsbVerkehrsRichtung()
							.equals(EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG)) {
						re = true;
					}
				}
			} else if (richtung.equals(EnumAsbVerkehrsRichtung.BEIDERICHTUNGEN)) {
				/* In beide Richtungen */
				if (von.getAsbVerkehrsRichtung().equals(
						EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG)
						|| von.getAsbVerkehrsRichtung()
								.equals(EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG)) {
					if ((bis.getAsbVerkehrsRichtung()
							.equals(EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG))
							|| (bis.getAsbVerkehrsRichtung()
									.equals(EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG))) {
						re = true;
					}
				}
			} else if (richtung.equals(EnumAsbVerkehrsRichtung.UNBEKANNT)) {
				/* unbekannt */
				if (von.getAsbVerkehrsRichtung().equals(
						EnumAsbVerkehrsRichtung.UNBEKANNT)) {
					if (bis.getAsbVerkehrsRichtung().equals(
							EnumAsbVerkehrsRichtung.UNBEKANNT)) {
						re = true;
					}
				}
			}

			return re;
		}

		@Override
		public String toString() {
			String re = "";

			re = re + "Kombiniert:\n";

			if (bereichsInfoAsbKnotenTmcPos != null) {
				re = re + "\t TMC Pos: "
						+ bereichsInfoAsbKnotenTmcPos.toString() + "\n";
			}

			if (bereichsInfoAsbKnotenTmcNeg != null) {
				re = re + "\t TMC Neg: "
						+ bereichsInfoAsbKnotenTmcNeg.toString() + "\n";
			}

			return re;
		}

	}

}
