/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenTeilSegment.BetriebsKilometerEintrag;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.sys.funclib.debug.Debug;

public class Betriebskilometer {

	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Verbindung zum DaV.
	 */
	private final ClientDavConnection verbindung;

	/**
	 * Enthlt die Instanz der Klasse {@link KonfigurationNeu}, welche die hier
	 * bentigten Datenstrukturen zur Verfgung stellt.
	 */
	private KonfigurationNeu konf;

	/**
	 * Diese {@link Struktur} enthlt die der Reihenfolge nach geordneten
	 * uerenStraenSegmente einer Strae, deren Richtung <b>positiv</b> ist.
	 */
	private Struktur strukturPositiv;

	/**
	 * Diese {@link Struktur} enthlt die der Reihenfolge nach geordneten
	 * uerenStraenSegmente einer Strae, deren Richtung <b>negativ</b> ist.
	 */
	private Struktur strukturNegativ;

	/**
	 * Enthlt die Liste der Reihenfolge nach geordneten ueren StraenSegmente
	 * in <b>positiver</b> Richtung.
	 */
	private List<StrukturElement> listePosAeussereStrassenSegmente;

	/**
	 * Enthlt die Liste der Reihenfolge nach geordneten ueren StraenSegmente
	 * in <b>negativer</b> Richtung.
	 */
	private List<StrukturElement> listeNegAeussereStrassenSegmente;

	/**
	 * In diesem String wird die PID der aktuellen Strae hinterlegt. <br>
	 * Id die Pid der alten Strae mit der Pid der aktuellen Strae identisch,
	 * so muss nicht neu nach den uerenStraenSegmenten der Strae gefiltert
	 * werden.
	 */
	private String pidAktuelleStrasse;

	/**
	 * In diesem String wird die PID der alten Strae hinterlegt. <br>
	 * Id die Pid der alten Strae mit der Pid der aktuellen Strae identisch,
	 * so muss nicht neu nach den uerenStraenSegmenten der Strae gefiltert
	 * werden.
	 */
	private String pidAlteStrasse;

	/**
	 * Konstruktor. bernimmt die Verbindung zum DaV und fhrt die
	 * Basis-Initialisierung durch.
	 * 
	 * @param verbindung
	 *            {@link ClientDavConnection}
	 */
	public Betriebskilometer(final ClientDavConnection verbindung) {
		this.verbindung = verbindung;
		einlesenBasisStruktur();
	}

	/**
	 * Prft ob ein Betriebskilometer vorkommt und liefert eine Liste von
	 * BlockNummern zurck.
	 * 
	 * @param pidStrasse
	 *            PID der Strae von der geprft werden soll, ob der angegebene
	 *            Betriebskilometer in Ihr vorkommt.
	 * @param betriebsKilometer
	 *            Anzugebender Betriebskilometer, von dem das Vorkommen geprft
	 *            wird.
	 * @return Liste von BlockNummern, innerhalb deren Bereich der
	 *         Betriebskilometer liegt.<br>
	 *         Enthlt die Liste den Wert <code>-1</code>, dann existiert zwar
	 *         ein Bereich in dem der Betriebskilometer liegt, jedoch wird keine
	 *         eindeutige Zuordnung zu einem Block getroffen.<br>
	 *         Ist die Liste leer, liegt der Betriebkilometer
	 *         <b><u>nicht</u></b> im LCL-Segment.
	 */
	public List<String> pruefeBetriebsKilometerVorkommen(
			final String pidStrasse, final long betriebsKilometer) {
		List<String> zusammengefuehrt = null;

		if (konf != null) {

			pidAktuelleStrasse = pidStrasse;

			if (!((pidAlteStrasse != null) && (pidAktuelleStrasse
					.equals(pidAlteStrasse)))) {
				/*
				 * AlteStrae auf die Pid der aktuell bearbeiteten Strae
				 * setzen. Auf diese Weise wird verhindert, dass eine bereits in
				 * die Struktur berfhrte Strae erneut eingelesen und
				 * verarbeitet werden muss.
				 */
				pidAlteStrasse = pidStrasse;

				konf.filtereNachStrasse(pidStrasse);

				/*
				 * Durchlaufen aller uerenStraenSegmente, sortieren nach
				 * positiver und negativer Richtung.
				 */
				final List<AeusseresStrassenSegment> strassenSegmentePositiv = new LinkedList<AeusseresStrassenSegment>();
				final List<AeusseresStrassenSegment> strassenSegmenteNegativ = new LinkedList<AeusseresStrassenSegment>();
				final Iterator<Entry<String, AeusseresStrassenSegment>> it = konf
						.getAeussereStrassenSegmenteGefiltert().entrySet()
						.iterator();
				while (it.hasNext()) {
					final Entry<String, AeusseresStrassenSegment> entry = it
							.next();
					final AeusseresStrassenSegment aeusseresStrassenSegment = entry
							.getValue();
					if ("positiv".equals(aeusseresStrassenSegment
							.getRichtungStrasse())) {
						strassenSegmentePositiv.add(aeusseresStrassenSegment);
					} else {
						strassenSegmenteNegativ.add(aeusseresStrassenSegment);
					}
				}

				strukturPositiv = erzeugeStruktur(strassenSegmentePositiv);
				final StrukturElement element = strukturPositiv
						.bestimmeStrukturElementOhneNachfolger();
				listePosAeussereStrassenSegmente = strukturPositiv
						.bestimmeNachfolger(element);

				strukturNegativ = erzeugeStruktur(strassenSegmenteNegativ);
				final StrukturElement elementNeg = strukturNegativ
						.bestimmeStrukturElementOhneNachfolger();
				listeNegAeussereStrassenSegmente = strukturNegativ
						.bestimmeNachfolger(elementNeg);
			} else {
				Betriebskilometer.LOGGER
						.fine(getClass().getName()
								+ "::pruefeBetriebsKilometerVorkommen \n\t=> NICHT EINLESEN");

			}

			/*
			 * Prfung in positiver und negativer Richtung
			 */
			final List<String> pruefeBetriebsKilometerVorkommenInListePos = pruefeBetriebsKilometerVorkommenInListe(
					listePosAeussereStrassenSegmente, betriebsKilometer);
			final List<String> pruefeBetriebsKilometerVorkommenInListeNeg = pruefeBetriebsKilometerVorkommenInListe(
					listeNegAeussereStrassenSegmente, betriebsKilometer);

			/*
			 * Zusammenfhren der Ergebnisse in der nachfolgenden Liste
			 */
			zusammengefuehrt = new LinkedList<String>();

			// Hinzufgen BetriebsKilometer in positiver Richtung
			final Iterator<String> itPos = pruefeBetriebsKilometerVorkommenInListePos
					.iterator();
			while (itPos.hasNext()) {
				final String stringPos = itPos.next();
				if (!zusammengefuehrt.contains(stringPos)) {
					zusammengefuehrt.add(stringPos);
				}
			}

			// Hinzufgen Betriebskilometer in negativer Richtung
			final Iterator<String> itNeg = pruefeBetriebsKilometerVorkommenInListeNeg
					.iterator();
			while (itNeg.hasNext()) {
				final String stringNeg = itNeg.next();
				if (!zusammengefuehrt.contains(stringNeg)) {
					zusammengefuehrt.add(stringNeg);
				}
			}

		}

		return zusammengefuehrt;
	}

	/**
	 * Liest die Basisstruktur ein.
	 */
	private void einlesenBasisStruktur() {
		if (verbindung != null) {
			konf = KonfigurationNeu.getInstanz();
			konf.bestimmeObjekte(verbindung);
		} else {
			Betriebskilometer.LOGGER
					.error(getClass().getName()
							+ "::einlesenBasisStruktur \n\t=>Es wurde keine Verbindung zum DaV bergeben. Kann die Datenstrukturen nicht aus dem DaV einlesen.");
		}

	}

	/**
	 * Mit Hilfe dieser Methode lt sich erfragen, zu welcher Blocknummer ein
	 * Betriebskilometer innerhalb einer nach Reihenfolge sortierten Liste von
	 * ueren Straensegmenten gehrt.
	 * 
	 * @param strukturElemente
	 *            Liste der {@link StrukturElement}e, innerhalb derer die
	 *            Prfung vorgenommen werden soll.
	 * @param betriebsKilometer
	 *            Betriebskilometer in Metern zu dem bestimmt werden soll, zu
	 *            welcher Blocknummer er zuzuordnen ist.
	 * @return Liste von BlockNummern, in deren Bereich der bergebende
	 *         Betriebskilometer liegt. Es kann vorkommen, dass eine konkrete
	 *         Zuordnung zu einem Block nicht mglich ist, wenn keine
	 *         BlockNummer bei den Betriebskilometern angegeben wurde. In diesem
	 *         Fall wird <code>-1</code> als Wert in der Liste erscheinen. <br>
	 *         Mehrfach vorkommende Blocknummern werden nur <b>1x</b> in der
	 *         Liste aufgenommen.
	 */
	private List<String> pruefeBetriebsKilometerVorkommenInListe(
			final List<StrukturElement> strukturElemente,
			final long betriebsKilometer) {
		/**
		 * Betriebskilometer aufsteigend oder fallend?
		 */
		boolean aufsteigend = true;

		/**
		 * Liste der BlockNummern, fr welche die Betriebskilometerangabe gltig
		 * ist.
		 */
		final List<String> blockNummern = new LinkedList<String>();

		/**
		 * Wert des BetriebsKilometers des aktuellen Blocks
		 */
		long bkWertAktuell = -1;

		/**
		 * Wert des BetriebsKilometers des nachfolgenden Blocks
		 */
		long bkWertNachfolger = -1;

		/*
		 * StraenSegmente
		 */
		for (int i = 0; i < strukturElemente.size(); i++) {
			final AeusseresStrassenSegment aeusseresStrassenSegment = strukturElemente
					.get(i).getElement();

			final List<StrassenTeilSegment> strassenTeilSegmente = aeusseresStrassenSegment
					.getStrassenTeilSegmente();

			/*
			 * StraenTeilSegmente
			 */
			if (strassenTeilSegmente.size() > 0) {
				for (int j = 0; j < strassenTeilSegmente.size(); j++) {
					/*
					 * Zurcksetzen
					 */
					bkWertAktuell = -1;
					bkWertNachfolger = -1;

					/*
					 * Temporre BlockNummer zum Speichern einer erfassten
					 * Nummer
					 */
					String tempBlockNummer = "KeineAngabe";

					/*
					 * Bestimmung des Wertes des aktuellen Betriebskilometers
					 */
					final List<BetriebsKilometerEintrag> bkAktuell = strassenTeilSegmente
							.get(j).getBetriebsKilometer();
					if ((bkAktuell != null) && (bkAktuell.size() > 0)) {
						bkWertAktuell = bkAktuell.get(0).getWert();
						tempBlockNummer = bkAktuell.get(0).getBlockNummer();
					} else {
						bkWertAktuell = -1;
					}

					/*
					 * Existiert ein zweiter Betriebskilometereintrag im
					 * StraenTeilSegment, dessen Offset grer 0 ist, so
					 * definiert dieser das Ende.
					 */
					if ((bkAktuell != null) && (bkAktuell.size() > 1)) {
						if (bkAktuell.get(1).getOffset() > 0) {
							bkWertNachfolger = bkAktuell.get(1).getWert();
							if ((bkAktuell.get(1).wurdeBerechnet())) {
								Betriebskilometer.LOGGER
										.fine(getClass().getName()
												+ "::pruefeBetriebsKilometerVorkommenInListe \n\t=> BERECHNETEN GEFUNDEN");
							}
						}
					}

					/*
					 * Suche des Wertes des nachfolgenden Betriebskilometers,
					 * wenn nicht ein zweiter Betriebskilometereintrag im
					 * StraenTeilSegment vorhanden ist.
					 */
					if (bkWertNachfolger < 0) {

						if ((j + 1) < strassenTeilSegmente.size()) {
							/*
							 * Innerhalb der StraenTeilSegmente des gleichen
							 * StraenSegments
							 */
							final List<BetriebsKilometerEintrag> bkNachfolger = strassenTeilSegmente
									.get(j + 1).getBetriebsKilometer();
							if ((bkNachfolger != null)
									&& (bkNachfolger.size() > 0)) {
								bkWertNachfolger = bkNachfolger.get(0)
										.getWert();
							}

						} else if ((i + 1) < strukturElemente.size()) {
							/*
							 * Keine weiteren StraenTeilSegmente im gleichen
							 * Segment? Dann wird im ersten StraenTeilSegment
							 * des nachfolgenden StraenSegments gesucht.
							 */

							// Nachfolgendes StraenSegment
							final List<StrukturElement> nachfolger = strukturElemente
									.get(i + 1).getNachfolger();
							if ((nachfolger != null) && (nachfolger.size() > 0)) {
								final StrukturElement strukturElement = nachfolger
										.get(0);
								final List<StrassenTeilSegment> sst = strukturElement
										.getElement().getStrassenTeilSegmente();
								if (!sst.isEmpty()) {
									/*
									 * Suche im nachfolgenden StraenTeilSegment
									 */
									final List<BetriebsKilometerEintrag> betriebsKilometerNachfolgend = sst
											.get(0).getBetriebsKilometer();
									if ((betriebsKilometerNachfolgend != null)
											&& (betriebsKilometerNachfolgend
													.size() > 0)) {
										bkWertNachfolger = betriebsKilometerNachfolgend
												.get(0).getWert();
									} else {
										bkWertNachfolger = -1;
									}

								} else {
									bkWertNachfolger = -1;
								}
							}
						} else {
							bkWertNachfolger = -1;
						}

						/*
						 * BetriebsKilometerEintrag mit berechntetem
						 * Betriebskilometer generieren und zum
						 * StraenTeilSegment hinzufgen.
						 */
						if (bkWertNachfolger > 0) {
							final StrassenTeilSegment strassenTeilSegment = strassenTeilSegmente
									.get(j);

							final BetriebsKilometerEintrag bke = new BetriebsKilometerEintrag(
									strassenTeilSegment.getLaenge(),
									bkWertNachfolger, tempBlockNummer, true);

							strassenTeilSegment
									.hinzufuegenBetriebsKilometerEintrag(bke);
						}

					}

					/*
					 * Bestimmung ob AUF- oder ABSTEIGEND
					 */
					if (bkWertNachfolger > -1) {
						if (bkWertNachfolger > bkWertAktuell) {
							aufsteigend = true;
						} else if (bkWertAktuell > bkWertNachfolger) {
							aufsteigend = false;
						}
					}

					/*
					 * Wenn Aufsteigend, dann wird die Lnge des
					 * StraenTeilSegments zum Wert des Betriebskilometers
					 * hinzuaddiert. Der zu prfende Betriebskilometer muss
					 * zwischen Betriebskilometerwert und Betriebskilometerwert
					 * + Lnge StraenTeilSegment liegen.
					 */
					if (aufsteigend) {
						if (betriebsKilometer >= bkWertAktuell) {
							if (betriebsKilometer <= (bkWertAktuell + strassenTeilSegmente
									.get(j).getLaenge())) {
								if (!blockNummern.contains(tempBlockNummer)) {
									blockNummern.add(tempBlockNummer);
									Betriebskilometer.LOGGER
											.fine(":::=>PASST AUFSTEIGEND");

								}
							}
						}
					}

					/*
					 * Wenn Absteigend, dann wird die Lnge des
					 * StraenTeilSegments vom Wert des Betriebskilometers
					 * abgezogen. Der zu prfende Betriebskilometer muss
					 * zwischen Betriebskilometerwert und Betriebskilometerwert
					 * - Lnge StraenTeilSegment liefen.
					 */
					if (!aufsteigend) {
						if (betriebsKilometer <= bkWertAktuell) {
							if (betriebsKilometer >= (bkWertAktuell - strassenTeilSegmente
									.get(j).getLaenge())) {
								if (!blockNummern.contains(tempBlockNummer)) {
									blockNummern.add(tempBlockNummer);
									Betriebskilometer.LOGGER
											.fine(":::=>PASST FALLEND");

								}
							}
						}
					}

					String ausg = "bkWertAktuell: " + bkWertAktuell
							+ " bkWertNachfolger: " + bkWertNachfolger + "\n";
					if (aufsteigend) {
						ausg = ausg + "Richtung: Aufsteigend" + "\n";
					} else {
						ausg = ausg + "Richtung: Fallend" + "\n";
					}
					ausg = ausg + "Lnge StraenTeilSegment: "
							+ strassenTeilSegmente.get(j).getLaenge() + " m.\n";
					Betriebskilometer.LOGGER.fine("=>\n" + ausg + "\n");

				}
			}

		}

		/*
		 * Blocknummern ausgeben
		 */
		final Iterator<String> it = blockNummern.iterator();
		while (it.hasNext()) {
			final String blockNummer = it.next();
			Betriebskilometer.LOGGER
					.fine(getClass().getName()
							+ "::pruefeBetriebsKilometerVorkommenInListe \n\t=> BLOCKNUMMER: "
							+ blockNummer);

		}

		return blockNummern;

	}

	/**
	 * Erzeugt aus einer Liste von {@link AeusseresStrassenSegment} eine
	 * {@link Struktur}.<br>
	 * In dieser Struktur knnen Vorgnger und Nachfolger leicht gefunden
	 * werden.
	 * 
	 * @param strassenSegmente
	 * @return
	 */
	private Struktur erzeugeStruktur(
			final List<AeusseresStrassenSegment> strassenSegmente) {
		final Struktur struktur = new Struktur();

		final Iterator<AeusseresStrassenSegment> it = strassenSegmente
				.iterator();
		while (it.hasNext()) {
			final AeusseresStrassenSegment aeusseresStrassenSegment = it.next();

			// _debug.fine("AES:" + aeusseresStrassenSegment.getPid());
			final AeusseresStrassenSegment vorgaenger = bestimmeVorgaenger(
					aeusseresStrassenSegment,
					aeusseresStrassenSegment.getTmcRichtung());
			final AeusseresStrassenSegment nachfolger = bestimmeNachfolger(
					aeusseresStrassenSegment,
					aeusseresStrassenSegment.getTmcRichtung());

			final StrukturElement strukturElement = struktur
					.erzeugeStrukturElement(aeusseresStrassenSegment);

			if (nachfolger != null) {
				final StrukturElement strukturNachfolger = struktur
						.erzeugeStrukturElement(nachfolger);
				strukturElement.addNachfolger(strukturNachfolger);
				strukturNachfolger.addVorgaenger(strukturElement);
			}

			if (vorgaenger != null) {
				final StrukturElement strukturVorgaenger = struktur
						.erzeugeStrukturElement(vorgaenger);
				strukturElement.addVorgaenger(strukturVorgaenger);
				strukturVorgaenger.addNachfolger(strukturElement);
			}

		}

		Betriebskilometer.LOGGER.fine(getClass().getName()
				+ "::erzeugeStruktur \n\t=> Gre: " + struktur.size());

		return struktur;
	}

	/**
	 * Bestimmt den Vorgnger eines {@link AeusseresStrassenSegment}.
	 * 
	 * @param ass
	 *            {@link AeusseresStrassenSegment}, dessen Vorgnger
	 *            {@link AeusseresStrassenSegment} bestimmt werden soll.
	 * @param richtung
	 *            Richtung (<code>positiv</code> oder <code>negativ</code>) der
	 *            {@link AeusseresStrassenSegment}e.
	 * @return Vorgnger des {@link AeusseresStrassenSegment}es.
	 */
	private AeusseresStrassenSegment bestimmeVorgaenger(
			final AeusseresStrassenSegment ass, final String richtung) {
		String vorg = "\t=>Vorgnger: ";
		String vorgAES = "";
		AeusseresStrassenSegment re = null;

		final Iterator<StrassenSegment> itSSVorgaenger = ass
				.getVorgaengerAufStrasse().iterator();

		while (itSSVorgaenger.hasNext()) {
			final StrassenSegment issVorgaenger = itSSVorgaenger.next();
			vorg = vorg + " PID: " + issVorgaenger.getPid() + " ANZ: "
					+ issVorgaenger.getVorgaengerAufStrasse().size();

			final Iterator<StrassenSegment> itAesVorg = issVorgaenger
					.getVorgaengerAufStrasse().iterator();
			while (itAesVorg.hasNext()) {
				final StrassenSegment aesVorg = itAesVorg.next();
				if (aesVorg instanceof AeusseresStrassenSegment) {
					final AeusseresStrassenSegment vorgaengerAss = (AeusseresStrassenSegment) aesVorg;
					if (vorgaengerAss.getTmcRichtung().equals(richtung)) {
						// _debug.fine("POS: " + vorgaengerAss.getPid());
						vorgAES = vorgAES + "POS: " + vorgaengerAss.getPid()
								+ "\n";
						re = vorgaengerAss;
					}
				}
			}
		}
		// _debug.fine(vorg + "\n" + vorgAES);
		return re;
	}

	/**
	 * Bestimmt den Nachfolger eines ueren StraenSegments.
	 * 
	 * @param ass
	 *            {@link AeusseresStrassenSegment}, dessen Nachfolger bestimmt
	 *            werden soll.
	 * @param richtung
	 *            Richtung des {@link AeusseresStrassenSegment}, in der der
	 *            Nachfolger bestimmt werden soll (sollte <code>positiv</code>
	 *            oder <code>negativ</code> sein).
	 * @return Nachfolger des {@link AeusseresStrassenSegment}es.
	 */
	private AeusseresStrassenSegment bestimmeNachfolger(
			final AeusseresStrassenSegment ass, final String richtung) {
		AeusseresStrassenSegment re = null;

		final Iterator<StrassenSegment> itISSVorgaenger = ass
				.getNachfolgerAufStrasse().iterator();
		while (itISSVorgaenger.hasNext()) {
			final StrassenSegment strassenSegment = itISSVorgaenger.next();
			final Iterator<StrassenSegment> itAESVorgaenger = strassenSegment
					.getNachfolgerAufStrasse().iterator();
			while (itAESVorgaenger.hasNext()) {
				final StrassenSegment aus = itAESVorgaenger.next();
				if (aus instanceof AeusseresStrassenSegment) {
					final AeusseresStrassenSegment aeusseresStrassenSegmentNachfolger = (AeusseresStrassenSegment) aus;
					if (aeusseresStrassenSegmentNachfolger.getTmcRichtung()
							.equals(richtung)) {
						re = aeusseresStrassenSegmentNachfolger;
						// _debug.fine("PID NACHFOLGER: " + re.getPid());
					}
				}
			}
		}

		return re;
	}

	/**
	 * Liefert die Elemente einer Struktur der {@link AeusseresStrassenSegment}e
	 * einer Strae. <br>
	 * Die {@link AeusseresStrassenSegment}e dieser Strae knnen sortiert in
	 * Reihenfolge ausgelesen werden, sofern ein Zusammenhang besteht.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Struktur {
		/**
		 * Liste der Elemente dieser Struktur.
		 */
		private final Map<String, StrukturElement> elemente = new HashMap<String, StrukturElement>();

		private int zaehler;

		/**
		 * Konstruktor.
		 */
		public Struktur() {
			Betriebskilometer.LOGGER.fine(getClass().getName()
					+ "::Struktur \n\t=> Struktur erzeugt.");

		}

		/**
		 * Erzeugt ein neues {@link StrukturElement} und liefert dieses zurck.
		 * Ist bereits eiens mit dem gleichen {@link AeusseresStrassenSegment}
		 * vorhanden, so wird dieses zurckgeliefert.
		 * 
		 * @param aeusseresStrassenSegment
		 * @return
		 */
		public StrukturElement erzeugeStrukturElement(
				final AeusseresStrassenSegment aeusseresStrassenSegment) {
			/*
			 * StrukturElement besorgen
			 */
			StrukturElement strukturElement = null;
			if (containsKey(aeusseresStrassenSegment.getPid())) {
				strukturElement = this
						.getStrukturElement(aeusseresStrassenSegment.getPid());
			} else {
				strukturElement = new StrukturElement(aeusseresStrassenSegment);
				this.addStrukturElement(strukturElement);
			}

			return strukturElement;
		}

		/**
		 * Fgt ein {@link StrukturElement} in die {@link HashMap} ein, sofern
		 * dieses noch nicht enthalten ist.
		 * 
		 * @param element
		 *            Hinzuzufgendes {@link StrukturElement}
		 */
		public void addStrukturElement(final StrukturElement element) {
			if (!elemente.containsKey(element.getPid())) {
				elemente.put(element.getPid(), element);
			}
		}

		/**
		 * Liefert ein {@link StrukturElement} an Hand ihres Keys aus der
		 * {@link HashMap} zurck.
		 * 
		 * @param pid
		 *            Key des StrukturElements, entsprechend der PID des
		 *            {@link AeusseresStrassenSegment}.
		 * @return {@link StrukturElement}, zugehrig zum Key.
		 */
		public StrukturElement getStrukturElement(final String pid) {
			if (elemente.containsKey(pid)) {
				return elemente.get(pid);
			}
			return null;
		}

		/**
		 * Prft, ob ein {@link StrukturElement} mit einem bestimmten Key in der
		 * {@link HashMap} der {@link StrukturElement}e vorhanden ist.
		 * 
		 * @param key
		 *            Schlssel des auf Existenz zu prfenden
		 *            {@link StrukturElement}s.
		 * @return Liefert <code>true</code>, wenn das Element in der
		 *         {@link HashMap} vorhanden ist, sonst <code>false</code>.
		 */
		public boolean containsKey(final String key) {
			return elemente.containsKey(key);
		}

		/**
		 * Prft, ob die {@link HashMap} ein bestimmtes Objekt enthlt.
		 * 
		 * @param object
		 *            Objekt, auf dessen Existenz geprft werden soll.
		 * @return Liefert <code>true</code>, wenn das Objekt in der
		 *         {@link HashMap} vorhanden ist, sonst <code>false</code>.
		 */
		@SuppressWarnings("unused")
		public boolean containsValue(final Object object) {
			boolean re = false;
			final Iterator<Entry<String, StrukturElement>> it = elemente
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<String, StrukturElement> entry = it.next();
				if (entry.getValue().getElement().equals(object)) {
					re = true;
				}
			}

			return re;
		}

		/**
		 * Liefert die Gre der Struktur.
		 * 
		 * @return Gre der Struktur
		 */
		public int size() {
			return elemente.size();
		}

		/**
		 * Bestimmt das {@link StrukturElement}, welches keinen Nachfolger hat.
		 * 
		 * @return {@link StrukturElement} ohne Nachfolger, sonst
		 *         <code>null</code>.
		 */
		public StrukturElement bestimmeStrukturElementOhneNachfolger() {
			StrukturElement element = null;

			final Iterator<Entry<String, StrukturElement>> it = elemente
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<String, StrukturElement> entry = it.next();
				final int size = entry.getValue().getVorgaenger().size();
				if (size == 0) {
					element = entry.getValue();
					break;
				}
			}

			Betriebskilometer.LOGGER
					.fine(getClass().getName()
							+ "::bestimmeStrukturElementOhneNachfolger \n\t=>Element ohne Vorgaenger: "
							+ (element == null ? null : element.getPid()));

			return element;
		}

		/**
		 * Diese Funktion liefert eine geordnete Liste der
		 * {@link AeusseresStrassenSegment}e.
		 * 
		 * @param element
		 *            {@link StrukturElement}, an Hand dessen Nachfolgern (und
		 *            damit auch Vorgngern) die
		 *            {@link AeusseresStrassenSegment} in Reihenfolge bestimmt
		 *            werden sollen.
		 * @return Liste mit in Reihenfolgen gebrachten {@link StrukturElement}
		 *         en.
		 */
		public List<StrukturElement> bestimmeNachfolger(
				final StrukturElement element) {
			Betriebskilometer.LOGGER
					.fine(getClass().getName()
							+ "::bestimmeNachfolger \n\tNachfolger werden bestimmt, siehe nachfolgende Liste.");

			final List<StrukturElement> liste = new LinkedList<StrukturElement>();

			final List<StrukturElement> nach = bestimmeNachfolgerRekursiv(element);
			liste.addAll(nach);

			return liste;
		}

		/**
		 * Bestimmt <i>rekursiv</i> die Nachfolger eines Elementes <br>
		 * <b>Nicht fr ffentlichen Gebrauch bestimmt.</b>
		 * 
		 * @param element
		 * @return
		 */
		private List<StrukturElement> bestimmeNachfolgerRekursiv(
				final StrukturElement element) {
			final List<StrukturElement> liste = new LinkedList<StrukturElement>();
			liste.add(element);

			Betriebskilometer.LOGGER.fine("\t" + zaehler
					+ " Element in Nachfolge: " + element.getPid());
			zaehler++;

			if (element.getNachfolger().size() > 0) {
				final List<StrukturElement> nach = bestimmeNachfolgerRekursiv(element
						.getNachfolger().get(0));
				liste.addAll(nach);
			}

			return liste;
		}

		@Override
		public String toString() {
			String re = "";
			final Iterator<Entry<String, StrukturElement>> it = elemente
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<String, StrukturElement> entry = it.next();
				re = re + "PID: " + entry.getKey() + " Anz. Nachfolger: "
						+ entry.getValue().getNachfolger().size()
						+ " Anz. Vorgaenger: "
						+ entry.getValue().getVorgaenger().size() + "\n";

			}

			return re;
		}

	}

	/**
	 * Reprsentiert ein Element in der Struktur.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class StrukturElement {
		private final AeusseresStrassenSegment element;

		private final List<StrukturElement> vorgaengerListe = new LinkedList<StrukturElement>();

		private final List<StrukturElement> nachfolgerListe = new LinkedList<StrukturElement>();

		/**
		 * Konstruktor, bernimmt ein {@link AeusseresStrassenSegment} als
		 * Element der Klasse {@link StrukturElement}.
		 * 
		 * @param element
		 *            {@link AeusseresStrassenSegment}
		 */
		public StrukturElement(final AeusseresStrassenSegment element) {
			this.element = element;
		}

		/**
		 * Liefert das {@link AeusseresStrassenSegment} als Element dieses
		 * {@link StrukturElement}s zurck.
		 * 
		 * @return {@link AeusseresStrassenSegment}
		 */
		public AeusseresStrassenSegment getElement() {
			return element;
		}

		/**
		 * Liefert die Pid des {@link AeusseresStrassenSegment}.
		 * 
		 * @return Pid
		 */
		public String getPid() {
			return element.getPid();
		}

		/**
		 * Fgt einen Vorgnger hinzu, falls dieser noch nicht i.d. Liste der
		 * Vorgnger vorhanden ist.
		 * 
		 * @param vorgaenger
		 *            Vorgnger
		 */
		public void addVorgaenger(final StrukturElement vorgaenger) {
			if (!vorgaengerListe.contains(vorgaenger)) {
				vorgaengerListe.add(vorgaenger);
			}
		}

		/**
		 * Liefert die Liste der Vorgnger.
		 * 
		 * @return Liste der Vorgnger
		 */
		public List<StrukturElement> getVorgaenger() {
			return vorgaengerListe;
		}

		/**
		 * Fgt einen Nachfolger vom Typ {@link StrukturElement} zur Liste der
		 * Nachfolger hinzu.
		 * 
		 * @param nachfolger
		 *            {@link StrukturElement} als Nachfolger
		 */
		public void addNachfolger(final StrukturElement nachfolger) {
			if (!nachfolgerListe.contains(nachfolger)) {
				nachfolgerListe.add(nachfolger);
			}
		}

		/**
		 * Liefert die Liste der Nachfolger.
		 * 
		 * @return Liste der Nachfolger.
		 */
		public List<StrukturElement> getNachfolger() {
			return nachfolgerListe;
		}

	}

	/**
	 * Liefert eine dem Namen nach sortierte Liste der {@link Strasse}n zurck.
	 * 
	 * @return Sortierte Liste der {@link Strasse}n.
	 */
	public List<Strasse> getStrassen() {

		final Map<String, Strasse> strassen = konf.getStrassen();

		final Map<String, Strasse> strassenSortiert = new TreeMap<String, Strasse>();

		strassenSortiert.putAll(strassen);

		final List<Strasse> listeStrassen = new LinkedList<Strasse>();
		final Iterator<Entry<String, Strasse>> it = strassenSortiert.entrySet()
				.iterator();
		while (it.hasNext()) {
			final Entry<String, Strasse> entry = it.next();

			if (!"null".equals(entry.getValue().getName())) {
				listeStrassen.add(entry.getValue());
			}

		}

		return listeStrassen;
	}

}
