/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse zur Realisierung eines Straenknotens.
 * 
 * @author Dambach Werke GmbH
 */
public class StrassenKnoten {
	/**
	 * Zum Strassenknoten gehrendes Systemobjekt.
	 */
	private final SystemObject knotenObject;

	/**
	 * Liste der usseren Straensegmente die von diesem Knoten abgehen.
	 */
	private final List<AeusseresStrassenSegment> abgehendeAeussereStrassensegmente = new ArrayList<AeusseresStrassenSegment>();

	/**
	 * Liste der usseren Straensegmente die zu diesem Knoten hinfhren.
	 */
	private final List<AeusseresStrassenSegment> hinfuehrendeAeussereStrassensegmente = new ArrayList<AeusseresStrassenSegment>();

	/**
	 * Liste der inneren Straensegmente dieses Knotens.
	 */
	private final List<InneresStrassenSegment> innereStrassenSegmente = new ArrayList<InneresStrassenSegment>();

	/**
	 * Typ des Straenknotens.
	 */
	private String typ;

	/**
	 * Konstruktor der Klasse.
	 * 
	 * @param connection
	 *            Verbindung zum DaV
	 * @param systemObject
	 *            Objekt vom Typ "typ.straenKnoten"
	 */
	public StrassenKnoten(final ClientDavConnection connection,
			final SystemObject systemObject) {
		knotenObject = systemObject;

		final DataModel dataModel = connection.getDataModel();

		final AttributeGroup atg = dataModel
				.getAttributeGroup("atg.straenKnoten");

		final Data data = systemObject.getConfigurationData(atg);

		if (data != null) {
			typ = data.getTextValue("Typ").getValueText();
		}
	}

	/**
	 * Methode fgt ein abgehendes ueres Straensegment zum Knoten hinzu.
	 * 
	 * @param objekt
	 *            ueres Straensegment
	 */
	public void addAbgehendesAeusseresStrassenSegment(
			final AeusseresStrassenSegment objekt) {
		if (!abgehendeAeussereStrassensegmente.contains(objekt)) {
			abgehendeAeussereStrassensegmente.add(objekt);
		}
	}

	/**
	 * Methode fgt ein hinfhrendes ueres Straensegment zum Knoten hinzu.
	 * 
	 * @param objekt
	 *            ueres Straensegment
	 */
	public void addHinfuehrendesAeusseresStrassenSegment(
			final AeusseresStrassenSegment objekt) {
		if (!hinfuehrendeAeussereStrassensegmente.contains(objekt)) {
			hinfuehrendeAeussereStrassensegmente.add(objekt);
		}
	}

	/**
	 * Methode liefert eine Liste aller von diesem Knoten abgehender uerer
	 * Straensegmente.
	 * 
	 * @return Liste mit ueren Straensegmenten
	 */
	public List<AeusseresStrassenSegment> getAbgehendeAeussereStrassenSegmente() {
		return abgehendeAeussereStrassensegmente;
	}

	/**
	 * Methode liefert eine Liste aller an diesem Knoten hinfhrender uerer
	 * Straensegmente.
	 * 
	 * @return Liste mit ueren Straensegmenten
	 */
	public List<AeusseresStrassenSegment> getHinfuehrendeAeussereStrassenSegmente() {
		return hinfuehrendeAeussereStrassensegmente;
	}

	/**
	 * Methode liefert alle inneren Staensegmente dieses Knotens.
	 * 
	 * @return Liste mit inneren Straensegmenten
	 */
	public List<InneresStrassenSegment> getInnereStrassenSegmente() {
		return innereStrassenSegmente;
	}

	// public InneresStrassenSegment
	// getInneresStrassenSegmentZwischenAeusserenSegmenten()

	/**
	 * @return liefert die Variable systemObject zurck
	 */
	public SystemObject getSystemObject() {
		return knotenObject;
	}

	/**
	 * Methode fgt ein inneres Straensegment dem Knoten hinzu.
	 * 
	 * @param objekt
	 *            inneres Straensegment
	 */
	public void addInneresStrassenSegment(final InneresStrassenSegment objekt) {
		if (!innereStrassenSegmente.contains(objekt)) {
			innereStrassenSegmente.add(objekt);
		}
	}

	/**
	 * @return liefert die Variable typ zurck
	 */
	public String getTyp() {
		return typ;
	}

	/**
	 * Methode prft ob es sich bei dem Knoten um ein Autobahnkreuz handelt.
	 * 
	 * @return Knoten ist Autobahnkreuz: true, sonst false
	 */
	public boolean isAutobahnKreuz() {
		return "AutobahnKreuz".equals(typ);
	}

	/**
	 * Methode prft ob es sich bei dem Knoten um ein Autobahndreieck handelt.
	 * 
	 * @return Knoten ist Autobahndreieck: true, sonst false
	 */
	public boolean isAutobahnDreieck() {
		return "AutobahnDreieck".equals(typ);
	}

	/**
	 * Methode prft ob es sich bei dem Knoten um ein Autobahnende handelt.
	 * 
	 * @return Knoten ist Autobahnende: true, sonst false
	 */
	public boolean isAutobahnEnde() {
		return "AutobahnEnde".equals(typ);
	}

	/**
	 * Methode prft ob es sich bei dem Knoten um eine Autobahnanschlussstelle
	 * handelt.
	 * 
	 * @return Knoten ist eine Autobahnanschlussstelle: true, sonst false
	 */
	public boolean isAutobahnAnschlussStelle() {
		return "AutobahnAnschlussStelle".equals(typ);
	}

	/**
	 * Methode prft ob es sich bei dem Knoten um einen sonstigen Knoten
	 * handelt.
	 * 
	 * @return Knoten ist ein sonstiger Knoten: true, sonst false
	 */
	public boolean isSonstigerKnoten() {
		return "SonstigerKnoten".equals(typ);
	}

	/**
	 * Methode liefert die Pid des Systemobjekts, das zu diesem Straenknoten
	 * gehrt.
	 * 
	 * @return Pid im Fehlerfall null
	 */
	public String getPid() {
		if (knotenObject != null) {
			return knotenObject.getPid();
		}

		return null;
	}

	public String toStringLang() {
		final StringBuilder buffer = new StringBuilder();

		buffer.append("StraenKnoten: ");
		buffer.append(knotenObject.getPid());
		buffer.append('\n');

		buffer.append("Anzahl abgehender uerer StraenSegmente:     ");
		buffer.append(abgehendeAeussereStrassensegmente.size());
		buffer.append('\n');

		buffer.append("Anzahl hinzufhrender uerer StraenSegmente: ");
		buffer.append(hinfuehrendeAeussereStrassensegmente.size());
		buffer.append('\n');

		buffer.append("Anzahl innerer StraenSegmente:                ");
		buffer.append(innereStrassenSegmente.size());
		buffer.append('\n');

		buffer.append("Abgehend:     ");
		final Iterator<AeusseresStrassenSegment> itAbgehend = abgehendeAeussereStrassensegmente
				.iterator();
		while (itAbgehend.hasNext()) {
			final AeusseresStrassenSegment abgehendesAeusseresStrassenSegment = itAbgehend
					.next();
			buffer.append(abgehendesAeusseresStrassenSegment.getPid());
			if (itAbgehend.hasNext()) {
				buffer.append("; ");
			}
		}
		buffer.append('\n');

		buffer.append("Hinzufhrend: ");
		final Iterator<AeusseresStrassenSegment> itHinzuf = hinfuehrendeAeussereStrassensegmente
				.iterator();
		while (itHinzuf.hasNext()) {
			final AeusseresStrassenSegment hinzufAeusseresStrassenSegment = itHinzuf
					.next();
			buffer.append(hinzufAeusseresStrassenSegment.getPid());
			if (itHinzuf.hasNext()) {
				buffer.append("; ");
			}
		}
		buffer.append('\n');

		return buffer.toString();
	}

	/**
	 * Liefert eine Kurzdarstellung dieses {@link StrassenKnoten}s.
	 * 
	 * @return Kurzdarstellung dieses {@link StrassenKnoten}s
	 */
	@Override
	public String toString() {
		String re = "";
		re = re + knotenObject.getName() + " (PID: " + knotenObject.getPid()
				+ ")";
		return re;
	}

}
