/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse zur Realisierung eines Straensegments, es enthlt die Daten, die
 * sowohl fr die inneren wie auch fr die ueren Straensegmente verwendet
 * werden.
 * 
 * @author Dambach Werke GmbH
 */
public class StrassenSegment {
	/**
	 * Das zum Straensegment gehrende Systemobjekt.
	 */
	private final SystemObject systemObject;

	/**
	 * Vorgnger des Staensegments die nicht auf der selben Strae liegen.
	 */
	private List<StrassenSegment> vorgaenger = new ArrayList<StrassenSegment>();

	/**
	 * Vorgnger des Staensegments die auf der selben Strae liegen.
	 */
	private List<StrassenSegment> vorgaengerAufStrasse = new ArrayList<StrassenSegment>();

	/**
	 * Nachfolger des Staensegments die nicht auf der selben Strae liegen.
	 */
	private List<StrassenSegment> nachfolgerMenge = new ArrayList<StrassenSegment>();

	/**
	 * Nachfolger des Staensegments die auf der selben Strae liegen.
	 */
	private List<StrassenSegment> nachfolgerAufStrasse = new ArrayList<StrassenSegment>();

	/**
	 * Zum Strassensegment gehrende Straenteilsegmente.
	 */
	private final List<StrassenTeilSegment> strassenTeilSegment = new ArrayList<StrassenTeilSegment>();

	/**
	 * Zum Strassensegment gehrende Messquerschnitte.
	 */
	private List<MessQuerschnitt> messquerschnittMenge = new ArrayList<MessQuerschnitt>();

	/**
	 * Datenmodell des DaV.
	 */
	private final DataModel davDataModel;

	/**
	 * Lnge des Segments.
	 */
	private long laenge;

	/**
	 * Konstruktor der Klasse.
	 * 
	 * @param dataModel
	 *            Datenmodell des DaV
	 * @param objekt
	 *            Systemobjekt vom Typ 'typ.straenSegment'
	 */
	public StrassenSegment(final DataModel dataModel, final SystemObject objekt) {
		systemObject = objekt;

		davDataModel = dataModel;
	}

	/**
	 * Methode bestimme zu einem Straensegment die Lnge und die dazugehrenden
	 * Straenteilsegmente. Achtung: rechenintensiv, sollte nur bei den
	 * Segmenten aufgerufen werden die diese Informationen auch bentigten.
	 * Daher erfolgt dieser Aufruf nicht automatisch im Konstruktor der Klasse.
	 */
	public void initialisiere() {
		// Bestimmen der Lnge des Straensegments
		// System.out.println("Bestimmen der Lnge des Straensegments + STS");

		AttributeGroup atg = davDataModel
				.getAttributeGroup("atg.straenSegment");

		if (atg != null) {
			final Data confData = systemObject.getConfigurationData(atg);

			if (confData != null) {
				laenge = confData.getUnscaledValue("Lnge").longValue() / 100;
			}
		}

		// Bestimmen der zum Straensegment gehrenden Straenteilsegmente

		atg = davDataModel.getAttributeGroup("atg.bestehtAusLinienObjekten");

		long offset = 0;

		if (atg != null) {
			final Data confData = systemObject.getConfigurationData(atg);

			if (confData != null) {
				final int anzahl = confData.getReferenceArray("LinienReferenz")
						.getLength();

				for (int j = 0; j < anzahl; j++) {
					final SystemObject objSts = confData
							.getReferenceArray("LinienReferenz")
							.getReferenceValue(j).getSystemObject();

					if (objSts != null) {
						// Anlegen des Objekts und Bestimmen des Offsets des
						// Strassenteilsegments innerhalb des
						// Strassensegments

						final StrassenTeilSegment sts = new StrassenTeilSegment(
								davDataModel, objSts, this);

						sts.setOffset(offset);

						offset += sts.getLaenge();

						strassenTeilSegment.add(sts);
					}
				}
			}
		}
	}

	/**
	 * @return liefert die Variable vorgnger zurck
	 */
	public List<StrassenSegment> getVorgaenger() {
		final List<StrassenSegment> v = new ArrayList<StrassenSegment>();
		final Iterator<StrassenSegment> it = vorgaenger.iterator();
		while (it.hasNext()) {
			v.add(it.next());
		}

		return v;
	}

	/**
	 * @param vorgaengerSegment
	 *            setzt die Variable vorgnger
	 */
	public void setVorgaenger(final List<StrassenSegment> vorgaengerSegment) {
		vorgaenger = vorgaengerSegment;
	}

	/**
	 * @return liefert die Variable nachfolger zurck
	 */
	public List<StrassenSegment> getNachfolger() {
		final List<StrassenSegment> v = new ArrayList<StrassenSegment>();
		final Iterator<StrassenSegment> it = nachfolgerMenge.iterator();
		while (it.hasNext()) {
			v.add(it.next());
		}

		return v;
	}

	/**
	 * @param nachfolger
	 *            setzt die Variable nachfolger
	 */
	public void setNachfolger(final List<StrassenSegment> nachfolger) {
		nachfolgerMenge = nachfolger;
	}

	/**
	 * @return liefert die Variable vorgngerAufStrae zurck
	 */
	public List<StrassenSegment> getVorgaengerAufStrasse() {
		final List<StrassenSegment> v = new ArrayList<StrassenSegment>();
		final Iterator<StrassenSegment> it = vorgaengerAufStrasse.iterator();
		while (it.hasNext()) {
			v.add(it.next());
		}

		return v;
	}

	/**
	 * Methode liefert das Vorgngerstraensegment auf der selben Strae zurck,
	 * das die krzeste Lnge hat. Bei mehreren gleichen Lngen wird das zuerst
	 * gefundene Segment zurckgeliefert.
	 * 
	 * @return krzestes Straenteilsegment, im Fehlerfall null
	 */
	public StrassenSegment getKuerzestenVorgaengerAufStrasse() {
		StrassenSegment ks = null;

		final Iterator<StrassenSegment> it = vorgaengerAufStrasse.iterator();
		while (it.hasNext()) {
			final StrassenSegment s = it.next();
			if (ks == null) {
				ks = s;
			} else {
				if (s.getLaenge() < ks.getLaenge()) {
					ks = s;
				}
			}
		}

		return ks;
	}

	/**
	 * @param vorgaengerAufStrasse
	 *            setzt die Variable vorgngerAufStrae
	 */
	public void setVorgaengerAufStrasse(
			final List<StrassenSegment> vorgaengerAufStrasse) {
		this.vorgaengerAufStrasse = vorgaengerAufStrasse;
	}

	/**
	 * @param segment
	 *            setzt die Variable vorgngerAufStrae
	 */
	public void setVorgaengerAufStrasse(final StrassenSegment segment) {
		vorgaengerAufStrasse.clear();
		vorgaengerAufStrasse.add(segment);
	}

	/**
	 * @return liefert die Variable systemObject zurck
	 */
	public SystemObject getSystemObject() {
		return systemObject;
	}

	/**
	 * @return liefert die Variable nachfolgerAufStrae zurck
	 */
	public List<StrassenSegment> getNachfolgerAufStrasse() {
		final List<StrassenSegment> v = new ArrayList<StrassenSegment>();
		final Iterator<StrassenSegment> it = nachfolgerAufStrasse.iterator();
		while (it.hasNext()) {
			v.add(it.next());
		}

		return v;
	}

	/**
	 * @param nachfolgerAufStrae
	 *            setzt die Variable nachfolgerAufStrae
	 */
	public void setNachfolgerAufStrasse(
			final List<StrassenSegment> nachfolgerAufStrasse) {
		this.nachfolgerAufStrasse = nachfolgerAufStrasse;
	}

	/**
	 * @param nachfolgerAufStrae
	 *            setzt die Variable nachfolgerAufStrae
	 */
	public void setNachfolgerAufStrasse(final StrassenSegment segment) {
		this.nachfolgerAufStrasse.clear();
		this.nachfolgerAufStrasse.add(segment);
	}

	/**
	 * @return liefert die Variable messquerschnitte zurck
	 */
	public List<MessQuerschnitt> getMessquerschnitte() {
		final List<MessQuerschnitt> v = new ArrayList<MessQuerschnitt>();
		final Iterator<MessQuerschnitt> it = messquerschnittMenge.iterator();
		while (it.hasNext()) {
			v.add(it.next());
		}

		return v;
	}

	/**
	 * @param messquerschnitte
	 *            setzt die Variable messquerschnitte
	 */
	public void setMessquerschnitte(final List<MessQuerschnitt> messquerschnitte) {
		messquerschnittMenge = messquerschnitte;
	}

	/**
	 * Methode liefet innerhalb des Straensegments den letzten Messquerschnitt,
	 * der vor einem bergebenen Offset liegt.
	 * 
	 * @param offset
	 *            Offset innerhalb des Staensegments
	 * @return letzter Messquerschnitt vor Offset, im Fehlerfall, bzw. wenn
	 *         nicht gefunden 'null'
	 */
	public MessQuerschnitt getMessQuerschnittVorOffset(final long offset) {
		MessQuerschnitt mq = null;

		final Iterator<MessQuerschnitt> it = messquerschnittMenge.iterator();
		while (it.hasNext()) {
			final MessQuerschnitt m = it.next();

			if (m.getOffset() < offset) {
				mq = m;
			}
		}

		return mq;
	}

	/**
	 * Methode liefet innerhalb des Straensegments den ersten Messquerschnitt,
	 * der in oder nach einem bergebenen Offset liegt.
	 * 
	 * @param offset
	 *            Offset innerhalb des Staensegments
	 * @return letzter Messquerschnitt nach Offset, im Fehlerfall, bzw. wenn
	 *         nicht gefunden 'null'
	 */
	public MessQuerschnitt getMessQuerschnittNachOffset(final long offset) {
		final Iterator<MessQuerschnitt> it = messquerschnittMenge.iterator();
		while (it.hasNext()) {
			final MessQuerschnitt m = it.next();
			if (m.getOffset() >= offset) {
				return m;
			}
		}

		return null;
	}

	/**
	 * Methode liefert die Pid des Systemobjekts, das zu diesem Staensegment
	 * gehrt.
	 * 
	 * @return Pid im Fehlerfall null
	 */
	public String getPid() {
		if (systemObject != null) {
			return systemObject.getPid();
		}

		return null;
	}

	/**
	 * Methode liefert bei ueren Straensegmenten die Pid des Systemobjekts,
	 * erweitert um die TMC-Richtung und die Nummer der Strasse. Bei inneren
	 * Straensegmenten wird nur die Pid zurckgegben.
	 * 
	 * @return erweiterte Pid, im Fehlerfall null
	 */
	public String getPidRichtungStrasse() {
		if (systemObject != null) {
			return systemObject.getPid();
		}

		return null;
	}

	/**
	 * Methode prft ob es sich bei dem Straensegment um ein inneres
	 * Straensegment handelt (Methode wird in den abgeleiteten Klassen
	 * berschrieben).
	 * 
	 * @return bei innerem Straensegment: true, sonst: false
	 */
	public boolean isInneresStrassenSegment() {
		return false;
	}

	/**
	 * Methode prft ob es sich bei dem Straensegment um ein ueres
	 * Straensegment handelt. (Methode wird in den abgeleiteten Klassen
	 * berschrieben).
	 * 
	 * @return bei uerem Straensegment: true, sonst: false
	 */
	public boolean isAeusseresStrassenSegment() {
		return false;
	}

	/**
	 * Methode prft bei ueren Straensegmenten ob das Straensegment in einem
	 * Autobahnkreuz oder Autobahndreieck beginnt, bei inneren Straensegmenten
	 * ob der Straenknoten in dem das Straensegment liegt ein Autobahnkreuz
	 * oder Autobahndreieck ist. (Methode wird in den abgeleiteten Klassen
	 * berschrieben).
	 * 
	 * @return Autobahnkreuz, Autobahndreick: true, sonst false
	 */
	public boolean beginntInAutobahnKreuzOderDreieck() {
		return false;
	}

	/**
	 * Methode prft bei ueren Straensegmenten ob das Straensegment in einem
	 * Autobahnkreuz oder Autobahndreieck endet, bei inneren Straensegmenten ob
	 * der Straenknoten in dem das Straensegment liegt ein Autobahnkreuz oder
	 * Autobahndreieck ist. (Methode wird in den abgeleiteten Klassen
	 * berschrieben).
	 * 
	 * @return Autobahnkreuz, Autobahndreick: true, sonst false
	 */
	public boolean endetInAutobahnKreuzOderDreieck() {
		return false;
	}

	/**
	 * Methode prft bei ueren Straensegmenten ob das Segment am
	 * Autobahnanfang beginnt. Bei inneren Straensegmenten wird geprft, ob der
	 * Straenknoten in dem das Straensegment liegt ein Autobahnanfang ist.
	 * 
	 * @return Autobahnanfang: true, sonst false
	 */
	public boolean beginntAmAutobahnAnfang() {
		return false;
	}

	/**
	 * Methode prft bei ueren Straensegmenten ob das Segment am Autobahnende
	 * endet. Bei inneren Straensegmenten wird geprft, ob der Straenknoten in
	 * dem das Straensegment liegt ein Autobahnende ist.
	 * 
	 * @return Autobahnende: true, sonst false
	 */
	public boolean endetAmAutobahnEnde() {
		return false;
	}

	/**
	 * Methode prft ob es sich bei dem Straensegment um eine Einfahrt handelt.
	 * Diese Information wird dadurch gebildet, dass geprft wird ob es auf
	 * diesem Segment einen Messquerschnitt von Typ Einfahrt gibt.
	 * 
	 * @return Einfahrt: true, sonst false
	 */
	public boolean isEinfahrt() {
		return (getMessQuerschnitteTypEinfahrt().size() > 0);
	}

	/**
	 * Methode prft ob es sich bei dem Straensegment um eine Hauptfahrbahn
	 * handelt. Diese Information wird dadurch gebildet, dass geprft wird ob es
	 * auf diesem Segment einen Messquerschnitt von Typ Hauptfahrbahn gibt.
	 * 
	 * @return Hauptfahrbahn: true, sonst false
	 */
	public boolean isHauptFahrbahn() {
		return (getMessQuerschnitteTypHauptFahrbahn().size() > 0);
	}

	/**
	 * Methode prft ob es sich bei dem Straensegment um eine Nebenfahrbahn
	 * handelt. Diese Information wird dadurch gebildet, dass geprft wird ob es
	 * auf diesem Segment einen Messquerschnitt von Typ Nebenfahrbahn gibt.
	 * 
	 * @return Nebenfahrbahn: true, sonst false
	 */
	public boolean isNebenFahrbahn() {
		return (getMessQuerschnitteTypNebenFahrbahn().size() > 0);
	}

	/**
	 * Methode prft ob es sich bei dem Straensegment um eine sonstige Fahrbahn
	 * handelt. Diese Information wird dadurch gebildet, dass geprft wird ob es
	 * auf diesem Segment einen Messquerschnitt von Typ sonstige Fahrbahn gibt.
	 * 
	 * @return sonstige Fahrbahn: true, sonst false
	 */
	public boolean isSonstigeFahrbahn() {
		return (getMessQuerschnitteTypSonstigeFahrbahn().size() > 0);
	}

	/**
	 * Methode prft ob es sich bei dem Straensegment um eine Ausfahrt handelt.
	 * Diese Information wird dadurch gebildet, dass geprft wird ob es auf
	 * diesem Segment einen Messquerschnitt von Typ Ausfahrt gibt.
	 * 
	 * @return Ausfahrt: true, sonst false
	 */
	public boolean isAusfahrt() {
		return (getMessQuerschnitteTypAusfahrt().size() > 0);
	}

	/**
	 * Methode liefert die Straensegmente, die als Einfahrt in dieses
	 * Straensegment hineinfhren (am Anfang des Straensegments).
	 * 
	 * @return List mit Einfahrten des Straensegments
	 */
	public List<StrassenSegment> getEinfahrendeStrassenSegmente() {
		final List<StrassenSegment> einfahrten = new ArrayList<StrassenSegment>();

		Iterator<StrassenSegment> it = vorgaenger.iterator();
		while (it.hasNext()) {
			final StrassenSegment s = it.next();

			if (s.isEinfahrt()) {
				einfahrten.add(s);
			}
		}

		it = vorgaengerAufStrasse.iterator();
		while (it.hasNext()) {
			final StrassenSegment s = it.next();

			if (s.isEinfahrt()) {
				einfahrten.add(s);
			}
		}

		return einfahrten;
	}

	/**
	 * Methode liefert die Straensegmente, die als Ausfahrt aus diesem
	 * Straensegment hinausfhren (am Ende des Straensegments).
	 * 
	 * @return List mit Einfahrten des Straensegments
	 */
	public List<StrassenSegment> getAusfahrendeStrassenSegmente() {
		final List<StrassenSegment> ausfahrten = new ArrayList<StrassenSegment>();

		Iterator<StrassenSegment> it = nachfolgerMenge.iterator();
		while (it.hasNext()) {
			final StrassenSegment s = it.next();

			if (s.isAusfahrt()) {
				ausfahrten.add(s);
			}
		}

		it = nachfolgerAufStrasse.iterator();
		while (it.hasNext()) {
			final StrassenSegment s = it.next();

			if (s.isAusfahrt()) {
				ausfahrten.add(s);
			}
		}

		return ausfahrten;
	}

	/**
	 * Methode liefert die Messquerschnitte, die in das Segment hineinfhren
	 * (nicht die zu diesem Segment definierten Messquerschnitte die als
	 * Einfahrt definiert sind).
	 * 
	 * @return Liste mit Messquerschnitten
	 */
	public List<MessQuerschnitt> getEinfahrendeMessQuerschnitte() {
		final List<MessQuerschnitt> messQuerschnitt = new ArrayList<MessQuerschnitt>();

		Iterator<StrassenSegment> it = vorgaenger.iterator();
		while (it.hasNext()) {
			final StrassenSegment s = it.next();

			if (s.isEinfahrt()) {
				messQuerschnitt.addAll(s.getMessQuerschnitteTypEinfahrt());
			}
		}

		it = vorgaengerAufStrasse.iterator();
		while (it.hasNext()) {
			final StrassenSegment s = it.next();

			if (s.isEinfahrt()) {
				messQuerschnitt.addAll(s.getMessQuerschnitteTypEinfahrt());
			}
		}

		return messQuerschnitt;
	}

	/**
	 * Methode liefert die Messquerschnitte, die die als Ausfahrt aus diesem
	 * Straensegment hinausfhren (nicht die zu diesem Segment definierten
	 * Messquerschnitte die als Ausfahrt definiert sind).
	 * 
	 * @return Liste mit Messquerschnitten
	 */
	public List<MessQuerschnitt> getAusfahrendeMessQuerschnitte() {
		final List<MessQuerschnitt> messQuerschnitt = new ArrayList<MessQuerschnitt>();

		Iterator<StrassenSegment> it = nachfolgerMenge.iterator();
		while (it.hasNext()) {
			final StrassenSegment s = it.next();

			if (s.isAusfahrt()) {
				messQuerschnitt.addAll(s.getMessQuerschnitteTypAusfahrt());
			}
		}

		it = nachfolgerAufStrasse.iterator();
		while (it.hasNext()) {
			final StrassenSegment s = it.next();

			if (s.isAusfahrt()) {
				messQuerschnitt.addAll(s.getMessQuerschnitteTypAusfahrt());
			}
		}

		return messQuerschnitt;
	}

	/**
	 * Methode liefert die Messquerschnitt des Strassensegments, die in diesem
	 * Segment als Einfahrt definiert sind.
	 * 
	 * @return Liste mit Messquerschnitten
	 */
	public List<MessQuerschnitt> getMessQuerschnitteTypEinfahrt() {
		final List<MessQuerschnitt> messQuerschnitte = new ArrayList<MessQuerschnitt>();
		final Iterator<MessQuerschnitt> it = messquerschnittMenge.iterator();
		while (it.hasNext()) {
			final MessQuerschnitt mq = it.next();
			if (mq.isEinfahrt()) {
				messQuerschnitte.add(mq);
			}
		}

		return messQuerschnitte;
	}

	/**
	 * Methode liefert die Messquerschnitt des Strassensegments, die in diesem
	 * Segment als Ausfahrt definiert sind.
	 * 
	 * @return Liste mit Messquerschnitten
	 */
	public List<MessQuerschnitt> getMessQuerschnitteTypAusfahrt() {
		final List<MessQuerschnitt> messQuerschnitte = new ArrayList<MessQuerschnitt>();
		final Iterator<MessQuerschnitt> it = messquerschnittMenge.iterator();
		while (it.hasNext()) {
			final MessQuerschnitt mq = it.next();
			if (mq.isAusfahrt()) {
				messQuerschnitte.add(mq);
			}
		}

		return messQuerschnitte;
	}

	/**
	 * Methode liefert die Messquerschnitt des Strassensegments, die in diesem
	 * Segment als Hauptfahrbahn definiert sind.
	 * 
	 * @return Liste mit Messquerschnitten
	 */
	public List<MessQuerschnitt> getMessQuerschnitteTypHauptFahrbahn() {
		final List<MessQuerschnitt> messQuerschnitte = new ArrayList<MessQuerschnitt>();
		final Iterator<MessQuerschnitt> it = messquerschnittMenge.iterator();
		while (it.hasNext()) {
			final MessQuerschnitt mq = it.next();
			if (mq.isHauptFahrbahn()) {
				messQuerschnitte.add(mq);
			}
		}

		return messQuerschnitte;
	}

	/**
	 * Methode liefert die Messquerschnitt des Strassensegments, die in diesem
	 * Segment als Nebenfahrbahn definiert sind.
	 * 
	 * @return Liste mit Messquerschnitten
	 */
	public List<MessQuerschnitt> getMessQuerschnitteTypNebenFahrbahn() {
		final List<MessQuerschnitt> messQuerschnitte = new ArrayList<MessQuerschnitt>();
		final Iterator<MessQuerschnitt> it = messquerschnittMenge.iterator();
		while (it.hasNext()) {
			final MessQuerschnitt mq = it.next();
			if (mq.isNebenFahrbahn()) {
				messQuerschnitte.add(mq);
			}
		}

		return messQuerschnitte;
	}

	/**
	 * Methode liefert die Messquerschnitt des Strassensegments, die in diesem
	 * Segment als sonstige Fahrbahn definiert sind.
	 * 
	 * @return Liste mit Messquerschnitten
	 */
	public List<MessQuerschnitt> getMessQuerschnitteTypSonstigeFahrbahn() {
		final List<MessQuerschnitt> messQuerschnitte = new ArrayList<MessQuerschnitt>();
		final Iterator<MessQuerschnitt> it = messquerschnittMenge.iterator();
		while (it.hasNext()) {
			final MessQuerschnitt mq = it.next();
			if (mq.isSonstigeFahrbahn()) {
				messQuerschnitte.add(mq);
			}
		}

		return messQuerschnitte;
	}

	/**
	 * Methode liefert den letzten Messquerschnitt (in Fahrtrichtung) auf diesem
	 * Strassensegment.
	 * 
	 * @return letzter Messquerschnitt, im Fehlerfall null
	 */
	public MessQuerschnitt getLetzterMessQuerschnitt() {
		final int anzahl = messquerschnittMenge.size();
		if (anzahl > 0) {
			return messquerschnittMenge.get(anzahl - 1);
		}

		return null;
	}

	/**
	 * Methode liefert den ersten Messquerschnitt (in Fahrtrichtung) auf diesem
	 * Strassensegment.
	 * 
	 * @return erster Messquerschnitt, im Fehlerfall null
	 */
	public MessQuerschnitt getErsterMessQuerschnitt() {
		final int anzahl = messquerschnittMenge.size();
		if (anzahl > 0) {
			return messquerschnittMenge.get(0);
		}

		return null;
	}

	/**
	 * Methode fgt einen Messquerschnitt zum Strassensegment hinzu.
	 * 
	 * @param messQuerschnitt
	 */
	public void addMessQuerschnitt(final MessQuerschnitt messQuerschnitt) {
		messquerschnittMenge.add(messQuerschnitt);

		Collections.sort(messquerschnittMenge, new MyComparator());
	}

	/**
	 * Klasse zum vergleichen zweier Messquerschnitte anhand des Offsets.
	 * 
	 * @author Dambach Werke GmbH
	 */
	static class MyComparator implements Comparator<MessQuerschnitt> {
		@Override
		public int compare(final MessQuerschnitt m1, final MessQuerschnitt m2) {
			if (m1.getOffset() < m2.getOffset()) {
				return 0;
			}
			return 1;
		}
	}

	/**
	 * Methode liefert die Lnge des Straensegments zurck.
	 * 
	 * @return Lnge des Straensegments
	 */
	public long getLaenge() {
		return laenge;
	}

	/**
	 * Methode liefert die Straenteilsegmente eines Straensegmente von einem
	 * bestimmten Offset ab, bis zu einem bestimmten Abstand (relativ zu diesem
	 * Offset. Sollte die Summe von Offset plus Abstand grsser als die Lnge
	 * des Straensegments sein, so wird in den nachfolgenden Straensegmenten
	 * weitergesucht. Methode ruft sich rekursiv selber auf.
	 * 
	 * @param offset
	 *            Offset im Straensegment (int Meter)
	 * @param abstand
	 *            Abstand relativ zum Offset (in Meter)
	 * @return Liste mit den Straenteilsegmenten
	 */
	public List<StrassenTeilSegment> getStrassenTeilSegmente(final long offset,
			final long abstand) {
		// System.out.println(this.getPid() + " Lnge = " + getLnge() +
		// " Offset = " + offset + " Abstand = " + abstand);

		final List<StrassenTeilSegment> v = new ArrayList<StrassenTeilSegment>();

		final Iterator<StrassenTeilSegment> it = strassenTeilSegment.iterator();
		while (it.hasNext()) {
			final StrassenTeilSegment sts = it.next();

			final long anfang = sts.getOffset();
			final long ende = anfang + sts.getLaenge();

			// Es werden alle Segmente gesucht deren Ende oder deren Anfang im
			// gesuchten Bereich liegt.

			if ((ende > offset) && (anfang < (offset + abstand))) {
				if (!v.contains(sts)) {
					v.add(sts);
					// System.out.println(" - " + sts.getPid() + " Offset: " +
					// sts.getOffset() + " Lnge: " + sts.getLnge());
				}
			}
		}

		// Offset geht noch in das nchste Segment hinein

		final long abstandNeu = abstand - (getLaenge() - offset);
		if ((offset + abstand) > getLaenge()) {
			final Iterator<StrassenSegment> itn = getNachfolgerAufStrasse()
					.iterator();
			while (itn.hasNext()) {
				final StrassenSegment s = itn.next();

				final List<StrassenTeilSegment> stsNachfolger = s
						.getStrassenTeilSegmente(0, abstandNeu);
				v.addAll(stsNachfolger);
			}
		}

		return v;
	}

	/**
	 * Methode liefert die Straenteilsegmente eines Straensegments.
	 * 
	 * @return Liste mit den Straenteilsegmenten
	 */
	public List<StrassenTeilSegment> getStrassenTeilSegmente() {
		final List<StrassenTeilSegment> v = new ArrayList<StrassenTeilSegment>();

		final Iterator<StrassenTeilSegment> it = strassenTeilSegment.iterator();
		while (it.hasNext()) {
			v.add(it.next());
		}

		return v;
	}

	/**
	 * Methode liefert die Straenteilsegmente eines Straensegments ab einem
	 * bestimmten Offset.
	 * 
	 * @return Liste mit den Straenteilsegmenten
	 */
	public List<StrassenTeilSegment> getStrassenTeilSegmenteAbOffset(
			final long offset) {
		final List<StrassenTeilSegment> v = new ArrayList<StrassenTeilSegment>();

		final Iterator<StrassenTeilSegment> it = strassenTeilSegment.iterator();
		while (it.hasNext()) {
			final StrassenTeilSegment sts = it.next();
			if ((sts.getOffset() + sts.getLaenge()) > offset) {
				if (!v.contains(sts)) {
					v.add(sts);
				}
			}
		}

		return v;
	}

	/**
	 * Methode liefert die Straenteilsegmente eines Straensegments bis zu
	 * einem bestimmten Offset.
	 * 
	 * @return Liste mit den Straenteilsegmenten
	 */
	public List<StrassenTeilSegment> getStrassenTeilSegmenteBisOffset(
			final long offset) {
		final List<StrassenTeilSegment> v = new ArrayList<StrassenTeilSegment>();

		final Iterator<StrassenTeilSegment> it = strassenTeilSegment.iterator();
		while (it.hasNext()) {
			final StrassenTeilSegment sts = it.next();
			if (sts.getOffset() < offset) {
				if (!v.contains(sts)) {
					v.add(sts);
				}
			}
		}

		return v;
	}

	/**
	 * Methode fgt einen Nachfolger zum Straensegment hinzu.
	 * 
	 * @param segment
	 *            inneres Straensegment
	 */
	public void addNachfolger(final StrassenSegment segment) {
		if (!nachfolgerMenge.contains(segment)) {
			nachfolgerMenge.add(segment);
		}
	}

	/**
	 * Methode fgt einen Nachfolger auf der Strae zum Straensegment hinzu.
	 * 
	 * @param segment
	 *            inneres Straensegment
	 */
	public void addNachfolgerAufStrasse(final StrassenSegment segment) {
		if (!nachfolgerAufStrasse.contains(segment)) {
			nachfolgerAufStrasse.add(segment);
		}
	}

	/**
	 * Methode fgt einen Vorgnger zum Straensegment hinzu.
	 * 
	 * @param segment
	 *            inneres Straensegment
	 */
	public void addVorgaenger(final StrassenSegment segment) {
		if (!vorgaenger.contains(segment)) {
			vorgaenger.add(segment);
		}
	}

	/**
	 * Methode fgt einen Vorgnger auf der Strae zum Straensegment hinzu.
	 * 
	 * @param segment
	 *            inneres Straensegment
	 */
	public void addVorgaengerAufStrasse(final StrassenSegment segment) {
		if (!vorgaengerAufStrasse.contains(segment)) {
			vorgaengerAufStrasse.add(segment);
		}
	}

	@Override
	public String toString() {
		String re = "";
		re = systemObject.getName() + " (PID:" + getPid() + ")";
		return re;
	}
}
