/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class StrassenTeilSegment {

	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Das zum Straenteilsegment gehrende Systemobjekt.
	 */
	private final SystemObject systemObject;

	/**
	 * Lnge des Straenteilsegments.
	 */
	private long laenge;

	/**
	 * Anzahl Fahrstreifen.
	 */
	private int anzahlFahrstreifen;

	/**
	 * Steigung/Geflle.
	 */
	private String steigungGefaelle;

	/**
	 * Offset des Straenteilsegmentes Innerhalb des Straensegments.
	 */
	private long segmentOffset;

	/**
	 * Liste der Betriebskilometer dieses StraenTeilSegments.
	 */
	private final List<BetriebsKilometerEintrag> betriebsKilometer = new LinkedList<BetriebsKilometerEintrag>();

	/**
	 * Liste der ASB-Stationierungen.
	 */
	private final List<AsbStationierung> asbStationierungen = new LinkedList<AsbStationierung>();

	/**
	 * Referenz auf das zugehrige StraenSegment.
	 */
	private final StrassenSegment strassenSegment;

	/**
	 * Konstruktor der Klasse.
	 * 
	 * @param dataModel
	 *            Datenmodell des DaV
	 * @param object
	 *            Systemobjekt vom Typ 'typ.straenTeilSegment'
	 * @param strassenSegment
	 *            Referenz auf das StraenSegment, zu dem dieses
	 *            StraenTeilsegment gehrt
	 */
	public StrassenTeilSegment(final DataModel dataModel,
			final SystemObject object, final StrassenSegment strassenSegment) {
		systemObject = object;

		this.strassenSegment = strassenSegment;

		final AttributeGroup atg = dataModel
				.getAttributeGroup("atg.straenTeilSegment");

		if (atg != null) {
			final Data confData = object.getConfigurationData(atg);

			if (confData != null) {
				laenge = confData.getUnscaledValue("Lnge").longValue() / 100;
				anzahlFahrstreifen = confData.getUnscaledValue(
						"AnzahlFahrStreifen").intValue();
				steigungGefaelle = confData.getScaledValue("SteigungGeflle")
						.getValueText();
			}
		}

		/*
		 * Infos zu den Betriebskilometern herausziehen
		 */
		final AttributeGroup atgBK = dataModel
				.getAttributeGroup("atg.betriebsKilometer");
		if (atgBK != null) {
			final Data data = object.getConfigurationData(atgBK);
			if (data != null) {
				final Data itemBetriebsKilometer = data
						.getItem("BetriebsKilometer");
				if (itemBetriebsKilometer != null) {
					final Array arrayBetriebsKilometer = data
							.getArray("BetriebsKilometer");

					for (int i = 0; i < arrayBetriebsKilometer.getLength(); i++) {
						final Data item = arrayBetriebsKilometer.getItem(i);
						final long bkOffset = item.getUnscaledValue("Offset")
								.longValue();
						final long bkWert = item.getUnscaledValue("Wert")
								.longValue();

						final String blockNummer = item.getTextValue(
								"BlockNummer").getText();

						final BetriebsKilometerEintrag bk = new BetriebsKilometerEintrag(
								bkOffset, bkWert, blockNummer);
						betriebsKilometer.add(bk);
					}

				}
			}
		}

		/*
		 * Infos zu den ASB-Stationierungen herausziehen
		 */
		final AttributeGroup atgAsb = dataModel
				.getAttributeGroup("atg.asbStationierung");
		if (atgAsb != null) {
			final Data data = object.getConfigurationData(atgAsb);
			if (data != null) {
				final Data itemAsbStationierung = data
						.getItem("AsbStationierung");
				if (itemAsbStationierung != null) {
					final Array arrayAsbStationierung = data
							.getArray("AsbStationierung");
					for (int i = 0; i < arrayAsbStationierung.getLength(); i++) {
						final Data item = arrayAsbStationierung.getItem(i);
						final long offsetValue = item
								.getUnscaledValue("Offset").longValue();
						final String anfangsKnoten = item.getTextValue(
								"AnfangsKnoten").getText();
						final String endKnoten = item.getTextValue("EndKnoten")
								.getText();
						final long anfang = item.getUnscaledValue("Anfang")
								.longValue();
						final long ende = item.getUnscaledValue("Ende")
								.longValue();

						final int verkehrsRichtung = item.getUnscaledValue(
								"VerkehrsRichtung").intValue();

						EnumAsbVerkehrsRichtung richtung = null;
						switch (verkehrsRichtung) {
						case 0:
							richtung = EnumAsbVerkehrsRichtung.UNBEKANNT;
							break;
						case 1:
							richtung = EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG;
							break;
						case 2:
							richtung = EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG;
							break;
						default:
							richtung = EnumAsbVerkehrsRichtung.UNBEKANNT;
							break;
						}

						final AsbStationierung asbStationierung = new AsbStationierung(
								offsetValue, anfangsKnoten, endKnoten, anfang,
								ende, richtung, this);
						asbStationierungen.add(asbStationierung);
					}
				}
			}

		}
	}

	/**
	 * @return liefert die Variable systemObject zurck
	 */
	public SystemObject getSystemObject() {
		return systemObject;
	}

	/**
	 * Methode liefert die Pid des Straenteilsegments.
	 * 
	 * @return Pid des Straenteilsegments
	 */
	public String getPid() {
		if (systemObject != null) {
			return systemObject.getPid();
		}

		return null;
	}

	/**
	 * @return liefert die Variable anzahlFahrstreifen zurck.
	 */
	public int getAnzahlFahrstreifen() {
		return anzahlFahrstreifen;
	}

	/**
	 * @return liefert die Variable lnge zurck
	 */
	public long getLaenge() {
		return laenge;
	}

	/**
	 * @return liefert die Variable steigungGeflle zurck
	 */
	public String getSteigungGefaelle() {
		return steigungGefaelle;
	}

	/**
	 * Liefert den Offset des Straenteilsegmentes innerhalb des
	 * Straensegmentes zurck.
	 * 
	 * @return liefert die Variable offset zurck
	 */
	public long getOffset() {
		return segmentOffset;
	}

	/**
	 * Setzt den Offset des Straenteilsegmentes innerhalb des Straensegmentes.
	 * 
	 * @param offset
	 *            setzt die Variable offset
	 */
	public void setOffset(final long offset) {
		this.segmentOffset = offset;
	}

	/**
	 * Liefert die Liste der Betriebskilometer dieses StraenTeilSegmentes
	 * zurck.
	 * 
	 * @return Liste der Betriebskilometer.
	 */
	public List<BetriebsKilometerEintrag> getBetriebsKilometer() {
		return betriebsKilometer;
	}

	public List<AsbStationierung> getAsbStationierungen() {
		return asbStationierungen;
	}

	/**
	 * Fgt einen {@link BetriebsKilometerEintrag} zur Liste der
	 * BetriebsKilometerEintrge hinzu.
	 * 
	 * @param bke
	 *            Hinzuzufgender Eintrag
	 */
	public void hinzufuegenBetriebsKilometerEintrag(
			final BetriebsKilometerEintrag bke) {
		if (!betriebsKilometer.contains(bke)) {
			betriebsKilometer.add(bke);
		}
	}

	@Override
	public String toString() {

		final StringBuilder buffer = new StringBuilder();

		buffer.append("StraenTeilSegment (Pid: ");
		buffer.append(systemObject.getPid());
		buffer.append(") Lnge = ");
		buffer.append(laenge);
		buffer.append(" m :\n");

		final Iterator<BetriebsKilometerEintrag> itBK = betriebsKilometer
				.iterator();
		while (itBK.hasNext()) {
			buffer.append("\t=> ");
			buffer.append(itBK.next().toString());
			buffer.append('\n');
		}

		return buffer.toString();
	}

	/**
	 * Bestimmt die Bereiche dieses StraenTeilsegmentes.
	 */
	public String bestimmeBereiche() {
		final StringBuffer sb = new StringBuffer();

		final List<StrassenTeilSegment> strassenTeilSegmente = strassenSegment
				.getStrassenTeilSegmente();

		final Iterator<BetriebsKilometerEintrag> it = betriebsKilometer
				.iterator();
		while (it.hasNext()) {
			final BetriebsKilometerEintrag bk = it.next();

			sb.append("BKE: ");
			sb.append(bk.toString());
			sb.append('\n');

			sb.append("StrassenSegment: ");
			sb.append(strassenSegment.getPid());
			sb.append(" Lnge: ");
			sb.append(strassenSegment.getLaenge());
			sb.append('\n');

			if (!it.hasNext()) {
				// Nachsehen in den nachfolgenden StraenTeilSegmenten dieses
				// StraenSegments
				if (strassenTeilSegmente.contains(this)) {
					/*
					 * Index dieses STS innerhalb der Liste der
					 * StraenTeilSegmente
					 */
					final int idxThis = strassenTeilSegmente.indexOf(this);
					final int laengeStrassenTeilSegmente = strassenTeilSegmente
							.size();

					sb.append("IDX THIS: ");
					sb.append(idxThis);
					sb.append(" SIZE: ");
					sb.append(laengeStrassenTeilSegmente);
					sb.append('\n');

					if (idxThis <= (laengeStrassenTeilSegmente - 2)) {
						sb.append("\tBestimmen innerhalb der Betriebskilometer des gleichen STS\n");

						final StrassenTeilSegment strassenTeilSegment = strassenTeilSegmente
								.get(idxThis + 1);

						if (strassenTeilSegment.getBetriebsKilometer().size() > 0) {
							final BetriebsKilometerEintrag bkNext = strassenTeilSegment
									.getBetriebsKilometer().get(0);

							sb.append(bkNext.toString());
							sb.append('\n');
						}

					} else {
						/*
						 * Ansonsten: im ersten Straensegment des nchsten STS
						 * den ersten BK bestimmen
						 */
						final long bestimmeVomNaechstenStrassenSegmentDenErstenStSBkWert = bestimmeVomNaechstenStrassenSegmentDenErstenStSBkWert();
						sb.append("VomNaechstenStrassenSegmentDenErstenStSBkWert:");
						sb.append(bestimmeVomNaechstenStrassenSegmentDenErstenStSBkWert);
						sb.append('\n');

					}
				}

			}
		}

		return sb.toString();
	}

	/**
	 * Bestimmt vom Nachfolger des StraenSegmentes (in dem sich dieses
	 * StraenTeilSegment befindet, ber einen StraenKnoten hinweg) vom ersten
	 * StaenTeilSegment den Wert des ersten Betriebskilometereintrag.
	 * 
	 * @return Wert des ersten Betriebskilometereintrags des nachfolgenden
	 *         Straensegments oder <code>-1</code>, falls dieser nicht
	 *         ermittelt werden konnte.
	 */
	private long bestimmeVomNaechstenStrassenSegmentDenErstenStSBkWert() {
		long re = -1;

		StrassenTeilSegment.LOGGER
				.fine("Bestimmen innerhalb des ersten BKs des nchsten STS");
		final AeusseresStrassenSegment aeusseresStrassenSegment = (AeusseresStrassenSegment) strassenSegment;
		final int groesseNachfolgerAufStrasse = aeusseresStrassenSegment
				.getNachfolgerAufStrasse().size();
		StrassenTeilSegment.LOGGER.fine("ANZAHL NACHFOLGER: "
				+ groesseNachfolgerAufStrasse);
		StrassenTeilSegment.LOGGER.fine("RICHTUNG         : "
				+ aeusseresStrassenSegment.getTmcRichtung());

		// if (groesseNachfolgerAufStrasse == 1)
		// {
		//
		// } else if (groesseNachfolgerAufStrasse > 1)
		// {
		final Iterator<StrassenSegment> it = aeusseresStrassenSegment
				.getNachfolgerAufStrasse().iterator();
		while (it.hasNext()) {
			final StrassenSegment curStrassenSegment = it.next();
			StrassenTeilSegment.LOGGER
					.fine("===>InneresStraenSegment in einem Knoten");
			StrassenTeilSegment.LOGGER.fine("===>TYP:"
					+ curStrassenSegment.getClass().getName());
			if (curStrassenSegment instanceof AeusseresStrassenSegment) {
				final AeusseresStrassenSegment intern = (AeusseresStrassenSegment) curStrassenSegment;
				StrassenTeilSegment.LOGGER.fine("===>A: " + intern.getPid()
						+ " RICHTUNG: " + intern.getRichtungStrasse());
			} else if (curStrassenSegment instanceof InneresStrassenSegment) {
				final InneresStrassenSegment inneresStrassenSegment = (InneresStrassenSegment) curStrassenSegment;
				StrassenTeilSegment.LOGGER.fine("===>InnersesStraenSegment: "
						+ inneresStrassenSegment.getPidRichtungStrasse()
						+ "Nachfolger auf Strae, Anzahl Nachfolger: "
						+ inneresStrassenSegment.getNachfolgerAufStrasse()
								.size());
				final Iterator<StrassenSegment> itNachfolgerInneresseStrassenSegment = inneresStrassenSegment
						.getNachfolgerAufStrasse().iterator();
				while (itNachfolgerInneresseStrassenSegment.hasNext()) {
					final StrassenSegment ssnachfolger = itNachfolgerInneresseStrassenSegment
							.next();
					if (ssnachfolger instanceof AeusseresStrassenSegment) {
						StrassenTeilSegment.LOGGER
								.fine("====>Nachfolgendes ueres StraenSegment");
						final AeusseresStrassenSegment ausseresStrassenSegmentsNachInnerenSegment = (AeusseresStrassenSegment) ssnachfolger;
						StrassenTeilSegment.LOGGER.fine("====>AUSSEN: "
								+ ausseresStrassenSegmentsNachInnerenSegment
										.getPid());
						StrassenTeilSegment.LOGGER.fine("====>RICHTUNG: "
								+ ausseresStrassenSegmentsNachInnerenSegment
										.getTmcRichtung());

						final List<StrassenTeilSegment> strassenTeilSegmente = aeusseresStrassenSegment
								.getStrassenTeilSegmente();
						if (!strassenTeilSegmente.isEmpty()) {
							final StrassenTeilSegment strassenTeilSegment = strassenTeilSegmente
									.get(0);
							final List<BetriebsKilometerEintrag> stsBetriebsKilometer = strassenTeilSegment
									.getBetriebsKilometer();
							if ((stsBetriebsKilometer != null)
									&& (!stsBetriebsKilometer.isEmpty())) {
								final long wert = stsBetriebsKilometer.get(0)
										.getWert();

								if (re == -1) {
									re = wert;
								} else if (re != wert) {
									StrassenTeilSegment.LOGGER
											.fine("Problem mit Paralellfahrbahn");
								}

								StrassenTeilSegment.LOGGER
										.fine("========>STS: "
												+ strassenTeilSegment.getPid()
												+ "Lnge:"
												+ strassenTeilSegment
														.getLaenge() + "<===");
								StrassenTeilSegment.LOGGER
										.fine("========>WERT: " + wert
												+ " <========");
							}
						}

					}

				}

			}
		}
		// }

		return re;
	}

	/**
	 * Diese Klasse reprsentiert einen Eintrag in der ArrayLiest der
	 * atg.betriebsKilometer. <br>
	 * Mit Hilfe des Konstruktors wird der Eintrag angelegt, die entsprechenden
	 * Getter-Methoden liefern die Werte. Setter-Methoden sind nicht vorgesehen.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public static class BetriebsKilometerEintrag {
		private final long offset;

		private final long wert;

		private final String blockNummer;

		/**
		 * Gibt an, ob dieser Betriebskilometereintrag berechnet wurde.
		 */
		private boolean berechnet;

		/**
		 * Konstruktor fr den {@link BetriebsKilometerEintrag}. bernimmt
		 * Offset, Wert und BlockNummer als Parameter.
		 * 
		 * @param offset
		 *            Offset Offset Offset, ab dem der Betriebskilometereintrag
		 *            in diesem StraenTeilSegment gltig ist.
		 * @param wert
		 *            Wert des Betriebskilometers (in Metern)
		 * @param blockNummer
		 *            Blocknummer Blocknummer des fr dieses StraenTeilSegments
		 *            gltigen Betriebskilometers.
		 */
		public BetriebsKilometerEintrag(final long offset, final long wert,
				final String blockNummer) {
			this.offset = offset;
			this.wert = wert;
			this.blockNummer = blockNummer;
		}

		/**
		 * Konstruktor fr den {@link BetriebsKilometerEintrag}. bernimmt
		 * Offset, Wert und BlockNummer als Parameter.
		 * 
		 * @param offset
		 *            Offset Offset, ab dem der Betriebskilometereintrag in
		 *            diesem StraenTeilSegment gltig ist.
		 * @param wert
		 *            Wert des Betriebskilometers (in Metern)
		 * @param blockNummer
		 *            Blocknummer Blocknummer des fr dieses StraenTeilSegments
		 *            gltigen Betriebskilometers.
		 * @param berechnet
		 *            Gibt an, ob dieser Betriebskilometereintrag berechnet
		 *            wurde oder nicht. Muss <code>true</code> sein, wenn er
		 *            berechnet wurde, sonst <code>false</code>.
		 */
		public BetriebsKilometerEintrag(final long offset, final long wert,
				final String blockNummer, final boolean berechnet) {
			this.offset = offset;
			this.wert = wert;
			this.blockNummer = blockNummer;
			this.berechnet = berechnet;
		}

		/**
		 * Liefert den Offset zurck.
		 * 
		 * @return Offset.
		 */
		public long getOffset() {
			return offset;
		}

		/**
		 * Liefert den Wert zurck. Dieser wird in Metern geliefert.
		 * 
		 * @return Wert in Metern.
		 */
		public long getWert() {
			return wert;
		}

		/**
		 * Liefert die BlockNummer zurck.
		 * 
		 * @return Liefert die BlockNummer oder -1, wenn kein Eintrag vorhanden
		 *         ist.
		 */
		public String getBlockNummer() {
			return blockNummer;
		}

		/**
		 * Erfragt, ob dieser {@link BetriebsKilometerEintrag} berechnet wurde
		 * oder nicht.
		 * 
		 * @return Liefert <code>true</code>, wenn dieser
		 *         {@link BetriebsKilometerEintrag} berechnet wurde, sonst
		 *         <code>false</code>.
		 */
		public boolean wurdeBerechnet() {
			return berechnet;
		}

		@Override
		public String toString() {
			String re = "";
			re = re + "Offset: " + offset + " m Wert: " + wert
					+ " m BlockNummer: " + blockNummer;
			return re;
		}
	}

	/**
	 * Speichert die Daten der atg.asbStationierung.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public class AsbStationierung {
		/**
		 * Abstand des ASB-Stationierungs-Punktes vom Anfang des Linienobjekts.
		 */
		private long offset = -1;

		/**
		 * Anfangsknoten mit eindeutiger Kennung (Anfangsnullpunkt) des
		 * Teilabschnittes oder Astes.
		 */
		private String anfangsKnoten;

		/**
		 * Endknoten mit eindeutiger Kennung (Endnullpunkt) des Teilabschnittes
		 * oder Astes.
		 */
		private String endKnoten;

		/**
		 * Anfangsstationierung. Stationierungsangabe in Metern relativ zum
		 * Abschnitt oder Ast.
		 */
		private long anfang = -1;

		/**
		 * Endstationierung. Stationierungsangabe in Metern relativ zum
		 * Abschnitt oder Ast.
		 */
		private long ende = -1;

		/**
		 * Referenz auf das StrassenTeilSegment, zu dem diese ASB-Stationierung
		 * gehrt.
		 */
		private final StrassenTeilSegment gehoertZuStrassenTeilSegment;

		/**
		 * Angabe des Richtungsbezugs der ASB-Stationierung relativ zur Richtung
		 * des Straensegments.
		 */
		private EnumAsbVerkehrsRichtung verkehrsRichtung;

		/**
		 * Konstruktor. bernimmt die nachfolgend aufgelisteten Parameter zur
		 * Initialisierung eines {@link AsbStationierung}-Objektes:
		 * 
		 * @param offset
		 *            Abstand des ASB-Stationierungs-Punktes vom Anfang des
		 *            Linienobjekts.
		 * @param anfangsKnoten
		 *            Anfangsknoten mit eindeutiger Kennung (Anfangsnullpunkt)
		 *            des Teilabschnittes oder Astes. Die Nullpunktbezeichung
		 *            hat die Form TTTTnnnB, wobei TTTT die vierstellige
		 *            TK25-Blattnummer und nnn die dreistellige laufende Nummer
		 *            ist, die zusammen die bundesweit eindeutige
		 *            Netzknotennummer darstellen. Durch die Kennung B (ein
		 *            Zeichen) wird zustzlich der Nullpunkt des Abschnitts oder
		 *            Astes eindeutig festgelegt. Eine nicht vorhandene Kennung
		 *            wird als o(hne) eingetragen.
		 * @param endKnoten
		 *            Endknoten mit eindeutiger Kennung (Endnullpunkt) des
		 *            Teilabschnittes oder Astes. Die Nullpunktbezeichung hat
		 *            die Form TTTTnnnB, wobei TTTT die vierstellige
		 *            TK25-Blattnummer und nnn die dreistellige laufende Nummer
		 *            ist, die zusammen die bundesweit eindeutige
		 *            Netzknotennummer darstellen. Durch die Kennung B (ein
		 *            Zeichen) wird zustzlich der Nullpunkt des Abschnitts oder
		 *            Astes eindeutig festgelegt. Eine nicht vorhandene Kennung
		 *            wird als o(hne) eingetragen.
		 * @param anfang
		 *            Anfangsstationierung. Stationierungsangabe in Metern
		 *            relativ zum Abschnitt oder Ast.
		 * @param ende
		 *            Endstationierung. Stationierungsangabe in Metern relativ
		 *            zum Abschnitt oder Ast.
		 * @param verkehrsRichtung
		 *            Angabe des Richtungsbezugs der ASB-Stationierung relativ
		 *            zur Richtung des Straensegments.
		 * @param gehoertZuStrassenTeilSegment
		 *            Referenz auf das {@link StrassenTeilSegment}, zus dem
		 *            diese {@link AsbStationierung} gehrt.s
		 */
		public AsbStationierung(final long offset, final String anfangsKnoten,
				final String endKnoten, final long anfang, final long ende,
				final EnumAsbVerkehrsRichtung verkehrsRichtung,
				final StrassenTeilSegment gehoertZuStrassenTeilSegment) {
			this.offset = offset;
			this.anfangsKnoten = anfangsKnoten;
			this.endKnoten = endKnoten;
			this.anfang = anfang;
			this.ende = ende;
			this.verkehrsRichtung = verkehrsRichtung;
			this.gehoertZuStrassenTeilSegment = gehoertZuStrassenTeilSegment;
		}

		/**
		 * Liefert den Abstand des ASB-Stationierungs-Punktes vom Anfang des
		 * Linienobjekts.
		 * 
		 * @return Abstand des ASB-Stationierungs-Punktes vom Anfang des
		 *         Linienobjekts
		 */
		public long getOffset() {
			return offset;
		}

		/**
		 * Setzt den Abstand des ASB-Stationierungs-Punktes vom Anfang des
		 * Linienobjekts.
		 * 
		 * @param offset
		 *            Abstand des ASB-Stationierungs-Punktes vom Anfang des
		 *            Linienobjekts
		 */
		public void setOffset(final long offset) {
			this.offset = offset;
		}

		/**
		 * Liefert den String, der den Anfangsknoten definiert.
		 * 
		 * @return Anfangsknotendefinition.
		 */
		public String getAnfangsKnoten() {
			return anfangsKnoten;
		}

		/**
		 * Setzt den String, der den Anfangsknoten definiert.
		 * 
		 * @param anfangsKnoten
		 *            Anfangsknotendefinition.
		 */
		public void setAnfangsKnoten(final String anfangsKnoten) {
			this.anfangsKnoten = anfangsKnoten;
		}

		/**
		 * Liefert vom Anfangsknoten die (4-stellige) <b>TK25-Blattnummer</b>.
		 * 
		 * @return TK25-Blattnummer oder <code>-1</code>, falls diese nicht
		 *         ermittelt werden konnte.
		 */
		public int getAnfangsKnotenBlattNr() {
			int re = -1;

			if (anfangsKnoten != null) {
				final String sub = anfangsKnoten.substring(0, 4);
				re = Integer.valueOf(sub);
			}

			return re;
		}

		/**
		 * Liefert die laufende Nummer vom Anfangsknoten.
		 * 
		 * @return Laufende Nummer vom Anfangsknoten oder <code>-1</code>, falls
		 *         diese nicht ermittelt werden konnte.
		 */
		public int getAnfangsKnotenLaufendeNummer() {
			int re = -1;

			if (anfangsKnoten != null) {
				final String sub = anfangsKnoten.substring(4, 7);
				re = Integer.valueOf(sub);
			}

			return re;
		}

		/**
		 * Prft, ob es sich bei dem Angangsknoten um einen Nullpunkt handelt.
		 * 
		 * @return Liefert <code>true</code>, wenn es sich bei dem Angangsknoten
		 *         um einen Nullpunkt handelt, sonst <code>false</code>.<br>
		 *         Wenn kein Anfangsknoten ermittelt werden konnte, wird
		 *         <code>false</code> geliefert.
		 */
		public boolean isAnfangsKnotenNullpunkt() {
			boolean re = false;

			if (anfangsKnoten != null) {
				try {
					final char c = anfangsKnoten.charAt(7);
					if (c == 'B') {
						re = true;
					} else {
						re = false;
					}
				} catch (final IndexOutOfBoundsException e) {
					StrassenTeilSegment.LOGGER.fine(getClass().getName()
							+ "::isAnfangsKnotenNullpunkt \n\t=> "
							+ e.toString());
				}

			}

			return re;
		}

		/**
		 * Liefert den String, der den Endknoten definiert.
		 * 
		 * @return Endknotendefinition.
		 */
		public String getEndKnoten() {
			return endKnoten;
		}

		/**
		 * Setzt den String, der den Endknoten definiert.
		 * 
		 * @param endKnoten
		 *            Endknotendefinition
		 */
		public void setEndKnoten(final String endKnoten) {
			this.endKnoten = endKnoten;
		}

		/**
		 * Liefert vom Endknoten die (4-stellige) <b>TK25-Blattnummer</b>.
		 * 
		 * @return TK25-Blattnummer oder <code>-1</code>, falls diese nicht
		 *         ermittelt werden konnte.
		 */
		public int getEndKnotenBlattNr() {
			int re = -1;

			if (endKnoten != null) {
				final String sub = endKnoten.substring(0, 4);
				re = Integer.valueOf(sub);
			}

			return re;
		}

		/**
		 * Liefert die laufende Nummer vom Endknoten.
		 * 
		 * @return Laufende Nummer vom Endsknoten oder <code>-1</code>, falls
		 *         diese nicht ermittelt werden konnte.
		 */
		public int getEndKnotenLaufendeNummer() {
			int re = -1;

			if (endKnoten != null) {
				final String sub = endKnoten.substring(4, 7);
				re = Integer.valueOf(sub);
			}

			return re;
		}

		/**
		 * Liefert das {@link StrassenTeilSegment}, zu dem diese
		 * {@link AsbStationierung} gehrt.
		 * 
		 * @return {@link StrassenTeilSegment}, zu dem diese
		 *         {@link AsbStationierung} gehrt
		 */
		public StrassenTeilSegment getGehoertZuStrassenTeilSegement() {
			return gehoertZuStrassenTeilSegment;
		}

		/**
		 * Prft, ob es sich bei dem Endknoten um einen Nullpunkt handelt.
		 * 
		 * @return Liefert <code>true</code>, wenn es sich bei dem Endknoten um
		 *         einen Nullpunkt handelt, sonst <code>false</code>.<br>
		 *         Wenn kein Endknoten ermittelt werden konnte, wird
		 *         <code>false</code> geliefert.
		 */
		public boolean isEndKnotenNullpunkt() {
			boolean re = false;

			if (endKnoten != null) {
				try {
					final char c = endKnoten.charAt(7);
					if (c == 'B') {
						re = true;
					} else {
						re = false;
					}
				} catch (final IndexOutOfBoundsException e) {
					StrassenTeilSegment.LOGGER.error(getClass().getName()
							+ "::isEndKnotenNullpunkt \n\t=> " + e.toString());
				}
			}

			return re;
		}

		/**
		 * Liefert die Anfangsstationierung (Stationierungsangabe in Metern
		 * relativ zum Abschnitt oder Ast).
		 * 
		 * @return Anfangsstationierung (Stationierungsangabe in Metern relativ
		 *         zum Abschnitt oder Ast)
		 */
		public long getAnfang() {
			return anfang;
		}

		/**
		 * Setzt die Anfangsstationierung (Stationierungsangabe in Metern
		 * relativ zum Abschnitt oder Ast).
		 * 
		 * @param anfang
		 *            Anfangsstationierung (Stationierungsangabe in Metern
		 *            relativ zum Abschnitt oder Ast)
		 */
		public void setAnfang(final long anfang) {
			this.anfang = anfang;
		}

		/**
		 * Liefert die Endstationierung (Stationierungsangabe in Metern relativ
		 * zum Abschnitt oder Ast).
		 * 
		 * @return Endstationierung (Stationierungsangabe in Metern relativ zum
		 *         Abschnitt oder Ast)
		 */
		public long getEnde() {
			return ende;
		}

		/**
		 * Setzt die Endstationierung (Stationierungsangabe in Metern relativ
		 * zum Abschnitt oder Ast).
		 * 
		 * @param ende
		 *            Endstationierung (Stationierungsangabe in Metern relativ
		 *            zum Abschnitt oder Ast)
		 */
		public void setEnde(final long ende) {
			this.ende = ende;
		}

		/**
		 * Liefert die Verkehrsrichtung als Instanz des Enumerators
		 * {@link EnumAsbVerkehrsRichtung}.
		 * 
		 * @return {@link EnumAsbVerkehrsRichtung}
		 */
		public EnumAsbVerkehrsRichtung getVerkehrsRichtung() {
			return verkehrsRichtung;
		}

		/**
		 * Setzt die Verkehrsrichtung als Instanz des Enumerators
		 * {@link EnumAsbVerkehrsRichtung}.
		 * 
		 * @param verkehrsRichtung
		 *            {@link EnumAsbVerkehrsRichtung}
		 */
		public void setVerkehrsRichtung(
				final EnumAsbVerkehrsRichtung verkehrsRichtung) {
			this.verkehrsRichtung = verkehrsRichtung;
		}

		@Override
		public String toString() {
			final StringBuilder buffer = new StringBuilder();

			buffer.append("===");
			buffer.append(getClass().getName());
			buffer.append("===\n");

			buffer.append("Offset:           ");
			buffer.append(offset);
			buffer.append('\n');

			buffer.append("AnfangsKnoten:    ");
			buffer.append(anfangsKnoten);
			buffer.append('\n');

			buffer.append("EndKnoten:        ");
			buffer.append(endKnoten);
			buffer.append('\n');

			buffer.append("Anfang:           ");
			buffer.append(anfang);
			buffer.append('\n');

			buffer.append("Ende:             ");
			buffer.append(ende);
			buffer.append('\n');

			buffer.append("Verkehrsrichtung: ");
			if (verkehrsRichtung != null) {
				switch (verkehrsRichtung) {
				case UNBEKANNT:
					buffer.append("unbekannt");
					break;
				case INSTATIONIERUNGSRICHTUNG:
					buffer.append("inStationierungsRichtung");
					break;
				case GEGENSTATIONIERUNGSRICHTUNG:
					buffer.append("gegenStationierungsRichtung");
					break;
				default:
					buffer.append("unbekannt");
					break;
				}
			} else {
				buffer.append("null");
			}
			buffer.append('\n');

			for (int i = 0; i < (6 + getClass().getName().length()); i++) {
				buffer.append('=');
			}
			buffer.append('\n');

			return buffer.toString();
		}

	}

}
