/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse mit Methoden zum Bestimmen der aktuellen Verkehrsstre eines
 * Strassenteilsegments.
 * 
 * @author Dambach Werke GmbH
 */
public class Verkehrsstaerke implements ClientReceiverInterface {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Verbindung zum DaV.
	 */
	private final ClientDavConnection davConnection;

	/**
	 * Definition der Attributgruppe.
	 */
	private static final String ATG = "atg.verkehrsDatenKurzZeitMq";

	/**
	 * bentigter Aspekt.
	 */
	private static final String ASP = "asp.analyse";

	/**
	 * Aktuelle StauEigenschaften (QB).
	 */
	private long qb = -1;

	/**
	 * Aktueller Gteindex.
	 */
	private String gueteIndex;

	/**
	 * Pid des Messquerschnitts.
	 */
	private final String pidMq;

	/**
	 * Vector mit empfangenen Daten.
	 */
	private final List<ResultData> resultInput = new ArrayList<ResultData>();

	/**
	 * Konstruktor der Klasse. Konstruktor meldet sich beim DaV zum Empfang der
	 * Daten an.
	 * 
	 * @param connection
	 *            Verbindung zum DaV
	 * @param pidMessQuerschnitt
	 *            Pid eines Objekts vom Typ "typ.messQuerschnitt"
	 */
	public Verkehrsstaerke(final ClientDavConnection connection,
			final String pidMessQuerschnitt) {
		davConnection = connection;

		pidMq = pidMessQuerschnitt;

		anmeldeLeseDaV(pidMessQuerschnitt, Verkehrsstaerke.ATG,
				Verkehrsstaerke.ASP, ReceiverRole.receiver());
	}

	/**
	 * Update Methode des ClientReceiverInterface.
	 */
	@Override
	public void update(final ResultData[] results) {
		try {
			for (final ResultData result : results) {
				synchronized (resultInput) {
					resultInput.add(result);
				}
			}
			verarbeiten();
		} catch (final Exception e) {
			Verkehrsstaerke.LOGGER.warning(e.getLocalizedMessage());
		}
	}

	/**
	 * Methode zum eigentlichen Verarbeiten der empfangenen Daten des DaV.
	 */
	protected void verarbeiten() {
		ResultData[] results = null;

		int anzahl;

		while (true) {
			anzahl = 0;

			// Daten aus Eingangsqueue holen

			synchronized (resultInput) {
				if (!resultInput.isEmpty()) {
					anzahl = resultInput.size();

					results = new ResultData[anzahl];

					for (int i = 0; i < anzahl; i++) {
						results[i] = resultInput.remove(0);
					}
				}
			}

			if ((results == null) || (anzahl == 0)) {
				break;
			}

			// Daten aus Eingangsqueue bearbeiten

			for (int i = 0; i < anzahl; i++) {
				final ResultData dat = results[i];

				// ----------------------------------------------------------------------
				// Auslesen der "atg.verkehrsDatenKurzZeitMq"
				// ----------------------------------------------------------------------

				if (dat.getDataDescription().getAttributeGroup().getPid()
						.equals(Verkehrsstaerke.ATG)) {
					// Absender feststellen

					final String absenderPid = dat.getObject().getPid();

					if (dat.hasData()) {

						Verkehrsstaerke.LOGGER.fine("Daten fr " + absenderPid
								+ " empfangen");

						final Data d = dat.getData().getItem("QB");

						qb = d.getUnscaledValue("Wert").longValue();

						final Data d1 = d.getItem("Gte");

						gueteIndex = d1.getTextValue("Index").getValueText();

						Verkehrsstaerke.LOGGER.fine("QB = " + qb
								+ " GteIndex = " + gueteIndex);

					} else {

						Verkehrsstaerke.LOGGER.fine("Keine Daten fr "
								+ absenderPid + " definiert");

						qb = -1;
						gueteIndex = "";
					}
				}
			}
		}
	}

	/**
	 * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet
	 * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
	 * 
	 * @param objPid
	 *            Pid des Objekts
	 * @param atgPid
	 *            Attributgruppe die angemeldet werden soll
	 * @param aspPid
	 *            Apekt der angemeldet werden soll
	 * @param role
	 *            Rolle des Empfngers (siehe
	 *            stauma.dav.clientside.ReceiveOptions)
	 */
	private void anmeldeLeseDaV(final String objPid, final String atgPid,
			final String aspPid, final ReceiverRole role) {
		final String buffer = "Anmelden am DaV (Lesen): " + objPid + " "
				+ atgPid + "  " + aspPid;

		Verkehrsstaerke.LOGGER.fine(buffer);

		final ClientDavInterface verb = davConnection;

		final AttributeGroup atg = verb.getDataModel()
				.getAttributeGroup(atgPid);

		final Aspect asp = verb.getDataModel().getAspect(aspPid);

		final DataDescription dd = new DataDescription(atg, asp);

		final SystemObject obj = davConnection.getDataModel().getObject(objPid);

		davConnection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(),
				role);
	}

	/**
	 * Methode die das Objekt mit der Pid objPid beim Datenverteiler abmeldet
	 * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
	 * 
	 * @param objPid
	 *            Pid des Objekts
	 * @param atgPid
	 *            Attributgruppe die abgemeldet werden soll
	 * @param aspPid
	 *            Apekt der abgemeldet werden soll
	 */
	private void abmeldeLeseDaV(final String objPid, final String atgPid,
			final String aspPid) {
		final String buffer = "Abmelden am DaV (Lesen): " + objPid + " "
				+ atgPid + "  " + aspPid;

		Verkehrsstaerke.LOGGER.fine(buffer);

		final AttributeGroup atg = davConnection.getDataModel()
				.getAttributeGroup(atgPid);
		final Aspect asp = davConnection.getDataModel().getAspect(aspPid);

		final DataDescription dd = new DataDescription(atg, asp);

		final SystemObject obj = davConnection.getDataModel().getObject(objPid);

		davConnection.unsubscribeReceiver(this, obj, dd);
	}

	/**
	 * Methode liefert die aktuelle StauEigenschaften des Messquerschnitts
	 * zurck.
	 * 
	 * @return aktuelle StauEigenschaften, falls nicht bestimmbar wird -1
	 *         zurckgeliefert.
	 */
	public long getVerkehrsstaerke() {
		return qb;
	}

	/**
	 * Methode prft ob der QB Wert plausibel ist.
	 * 
	 * @return plausibel: true, sonst false
	 */
	public boolean isQBPlausibel() {
		return qb >= 0;
	}

	// ----------------------------------------------------------------------------------------------
	// Klasse aufrumen
	// ----------------------------------------------------------------------------------------------

	/**
	 * Methode mit der die Klasse veranlasst wird, ihre Objekte am DaV
	 * abzumelden un die angemeldeten Listener abzumelden.
	 */
	public void dispose() {
		// Messquerschnitte abmelden

		abmeldeLeseDaV(pidMq, Verkehrsstaerke.ATG, Verkehrsstaerke.ASP);
	}
}
