/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.codec.binary.Base64;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse zur Verwaltung von XML-Filtern, welche Listen von regulren Ausdrcken
 * enthalten. Liefert Funktionen zum Einlesen und Schreiben der entsprechenden
 * XML-Strukturen.
 * 
 * @author HofmannJ
 * 
 */
public class XmlFilterVerwaltung implements Cloneable {

	/**
	 * XML-Dateicodierung.
	 */
	public static final String XML_CODIERUNG = "ISO-8859-1";

	/**
	 * Bezeichnung des Wurzelelementes der XML-Struktur.
	 */
	public static final String XML_ELEMENT_WURZEL = "filterdefinition";

	/**
	 * Bezeichnung eines Filterelementes.
	 */
	public static final String XML_ELEMENT_FILTER = "filter";

	/**
	 * Regulrer Ausdruck innerhalb des Filterelementes.
	 */
	public static final String XML_ELEMENT_AUSDRUCK = "ausdruck";

	/**
	 * Bezeichnung des Attributes ID eines Filterelementes.
	 */
	public static final String XML_ATT_ID = "id";

	/**
	 * Bezeichnung eines Filterelementes.
	 */
	public static final String XML_ATT_BEZEICHNUNG = "bezeichnung";

	/**
	 * JDOM Document, in dem die Verarbeitung erfolgt.
	 */
	private Document document;

	/**
	 * Wurzelelement.
	 */
	private Element elementWurzel;

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Konstruktor, erzeugt eine leere XML-Struktur.
	 */
	public XmlFilterVerwaltung() {
		elementWurzel = new Element(XmlFilterVerwaltung.XML_ELEMENT_WURZEL);
		document = new Document(elementWurzel);
	}

	/**
	 * Liefert die zu dieser Struktur zugehrigen {@link ElementFilter} nach ID
	 * sortiert zurck.
	 * 
	 * @return Zugehrige XML-Filter
	 */
	public LinkedList<ElementFilter> getElementFilter() {
		LinkedList<ElementFilter> re = null;

		final List<Element> filter = elementWurzel
				.getChildren(XmlFilterVerwaltung.XML_ELEMENT_FILTER);
		if ((filter != null) && (filter.size() > 0)) {
			re = new LinkedList<ElementFilter>();
			final Iterator<Element> it = filter.iterator();
			while (it.hasNext()) {
				final Element element = it.next();
				final ElementFilter elementFilter = new ElementFilter(element);
				re.add(elementFilter);
			}
		}

		/*
		 * Sortieren nach ID
		 */
		if ((re != null) && (re.size() > 1)) {
			Collections.sort(re);
		}

		return re;
	}

	/**
	 * Fgt ein {@link Element} vom Typ {@link ElementFilter} zur XML-Struktur
	 * hinzu. Ist ein {@link Element} mit der gleichen ID vorhanden, so wird
	 * dieses entfernt und durch das neue {@link Element} ersetzt.
	 * 
	 * @param elementFilter
	 */
	public void addElementFilter(final ElementFilter elementFilter) {
		final Element addElement = elementFilter.getElement();

		/*
		 * Prfen ob das Element schon vorhanden ist & in Lschliste schreiben
		 */
		final LinkedList<Element> loeschListe = new LinkedList<Element>();
		final Iterator<Element> itPruefung = elementWurzel.getChildren(
				XmlFilterVerwaltung.XML_ELEMENT_FILTER).iterator();
		while (itPruefung.hasNext()) {
			final Element pruefElement = itPruefung.next();
			if (pruefElement.getAttributeValue(XmlFilterVerwaltung.XML_ATT_ID)
					.equals(addElement
							.getAttributeValue(XmlFilterVerwaltung.XML_ATT_ID))) {
				loeschListe.add(pruefElement);
			}
		}

		/*
		 * Elemente in der Lschliste vom Elternelement ablsen
		 */
		final Iterator<Element> itLoeschen = loeschListe.iterator();
		while (itLoeschen.hasNext()) {
			final Element loeschElement = itLoeschen.next();
			loeschElement.detach();
		}

		/*
		 * Neues / gendertes Element anhngen
		 */
		elementWurzel.addContent(addElement);
	}

	public void removeElementFilter(final ElementFilter elementFilter) {
		/*
		 * Speichert zu lschende Elemente
		 */
		final LinkedList<Element> loeschListe = new LinkedList<Element>();

		/*
		 * Suchen der Lschelemente
		 */
		final Iterator<Element> itPruefen = elementWurzel.getChildren(
				XmlFilterVerwaltung.XML_ELEMENT_FILTER).iterator();
		while (itPruefen.hasNext()) {
			final Element element = itPruefen.next();
			final ElementFilter pruefElementFilter = new ElementFilter(element);
			if (pruefElementFilter.getId() == elementFilter.getId()) {
				loeschListe.add(element);
			}
		}

		/*
		 * Lschen der Elemente aus der Lschliste
		 */
		final Iterator<Element> itLoeschen = loeschListe.iterator();
		while (itLoeschen.hasNext()) {
			final Element loeschElement = itLoeschen.next();
			elementWurzel.removeContent(loeschElement);
		}

	}

	/**
	 * Bestimmt die nchste, freie ID fr Elemente vom Typ {@link ElementFilter}
	 * .
	 * 
	 * @return ID
	 */
	public int bestimmeNaechsteFreieIdElementFilter() {
		int re = -1;

		final List<Element> filter = elementWurzel
				.getChildren(XmlFilterVerwaltung.XML_ELEMENT_FILTER);
		if ((filter != null) && (filter.size() > 0)) {

			final Iterator<Element> it = filter.iterator();
			while (it.hasNext()) {
				final Element element = it.next();
				final ElementFilter elementFilter = new ElementFilter(element);
				if (elementFilter.getId() > re) {
					re = elementFilter.getId();
				}
			}
		}

		re = re + 1;

		return re;
	}

	/**
	 * Liest eine XML-Datei ein.
	 * 
	 * @param dateiPfad
	 *            Kompletter Pfad zur Datei
	 * @throws IllegalArgumentException
	 *             Wird geworfen, wenn die Datei nicht existiert
	 */
	public void einlesenXmlAusDatei(final String dateiPfad) {
		final SAXBuilder builder = new SAXBuilder();

		final File file = new File(dateiPfad);
		if (file.exists()) {
			try {
				builder.setValidation(false);
				document = builder.build(file);
				elementWurzel = document.getRootElement();
			} catch (final JDOMException e) {
				e.printStackTrace();
			} catch (final IOException e) {
				e.printStackTrace();
			}

		} else {
			throw new IllegalArgumentException("Die Datei " + dateiPfad
					+ " existiert nicht!");
		}
	}

	/**
	 * Liest die XML-Struktur aus einem String ein.
	 * 
	 * @param xmlString
	 *            String der XML-Struktur, die eingelesen werden soll.
	 */
	public void einlesenXmlAusString(final String xmlString) {
		final SAXBuilder builder = new SAXBuilder();
		builder.setValidation(false);
		try {
			final StringReader reader = new StringReader(xmlString);
			document = builder.build(reader);
			elementWurzel = document.getRootElement();

		} catch (final JDOMException e) {
			XmlFilterVerwaltung.LOGGER.error(getClass().getName()
					+ " => JDOMException", e);
		} catch (final IOException e) {
			XmlFilterVerwaltung.LOGGER.error(getClass().getName()
					+ " => IOException", e);
		}
	}

	/**
	 * Liefert eine formatierte Ausgabe der erzeugten XML-Struktur. Setzt das
	 * Dirty-Flag auf <code>false</code>.
	 * 
	 * @return Formatierte XML-Struktur
	 */
	public String ausgabeXmlFormatiert() {
		String returnString = null;
		if (document == null) {
			document = new Document(elementWurzel);
		}

		final Format prettyFormat = Format.getPrettyFormat();
		final XMLOutputter out = new XMLOutputter(prettyFormat);
		final Format f = out.getFormat();
		f.setEncoding(XmlFilterVerwaltung.XML_CODIERUNG);
		out.setFormat(f);

		returnString = out.outputString(document);

		return returnString;
	}

	/**
	 * Schreibt einen String in eine Datei.
	 * 
	 * @param dateiPfad
	 *            Kompletter Pfad zur Datei, in die geschrieben werden soll
	 * @param schreibString
	 *            String, der in die Datei geschrieben werden soll
	 * @param ueberschreiben
	 *            true == berschreiben der Datei, falls schon vorhanden
	 * @throws IllegalArgumentException
	 *             Wir geworfen, wenn die Datei existiert und nicht
	 *             berschrieben werden kann
	 */
	public void schreibeString(final String dateiPfad,
			final String schreibString, final boolean ueberschreiben) {
		final File file = new File(dateiPfad);

		// File existiert, auf berschreiben prfen
		if (file.exists() && ueberschreiben) {
			try {
				final PrintWriter pw = new PrintWriter(file);
				pw.print(schreibString);
				pw.close();
			} catch (final FileNotFoundException e) {
				e.printStackTrace();
			}
		} else if (!file.exists()) {
			// File existiert nicht und kann einfach angelegt werden
			try {
				final PrintWriter pw = new PrintWriter(file);
				pw.print(schreibString);
				pw.close();
			} catch (final FileNotFoundException e) {
				e.printStackTrace();
			}
		} else {
			// File existiert und ueberschreiben == flase
			throw new IllegalArgumentException("Die Datei " + dateiPfad
					+ " existiert und ueberschreiben == flase");
		}
	}

	@Override
	public String toString() {
		String re = "===XmlFilterVerwaltung===\n";
		final Iterator<Element> it = elementWurzel.getChildren(
				XmlFilterVerwaltung.XML_ELEMENT_FILTER).iterator();
		while (it.hasNext()) {
			final Element element = it.next();
			final ElementFilter fi = new ElementFilter(element);
			re = re + "FI: " + fi.toString();
		}

		re = re + "=========================";
		return re;
	}

	/**
	 * Kodiert einen String nach Base64.
	 * 
	 * @param string
	 *            Zu kodierender String
	 * @return Base64-Kodierung des Strings. Liefert <code>null</code>, wenn der
	 *         bergebene String <code>null</code> ist oder die Lnge == 0 hat.
	 */
	public String stringToBase64(final String string) {

		if ((string != null) && (string.length() > 0)) {
			try {
				return new String(Base64.encodeBase64(string.getBytes()),
						"UTF-8");
			} catch (final UnsupportedEncodingException e) {
				throw new IllegalStateException(e);
			}
		}

		return null;
	}

	/**
	 * Decodiert einen Base64-kodierten String.
	 * 
	 * @param base64
	 *            Base64-kodierter String.
	 * @return Klartextreprsentation oder <code>null</code>, wenn
	 *         <code>null</code> bergeben wurde oder eine {@link IOException}
	 *         beim decodieren aufgetreten ist.
	 */
	public String base64ToString(final String base64) {
		if (base64 != null) {
			final byte[] decodeBuffer = Base64.decodeBase64(base64.getBytes());
			return new String(decodeBuffer);
		}

		return null;
	}

	/**
	 * Liefert eine tiefe Kopie der Instanz dieser Klasse.
	 */
	@Override
	public XmlFilterVerwaltung clone() {
		try {
			super.clone();
		} catch (final CloneNotSupportedException e) {
			throw new IllegalStateException(e);
		}

		final XmlFilterVerwaltung clone = new XmlFilterVerwaltung();
		final String xmlString = this.ausgabeXmlFormatiert();
		clone.einlesenXmlAusString(xmlString);
		return clone;
	}

}
