/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.urlasser;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Standard-Implementierung des {@link UrlasserDatenSender}.
 * 
 * Eine Instanz dieser Klasse erhlt eine Menge zu versendender Datenstze. die
 * ber die potentiell erstellte Urlasser-Datenverteilerverbindung versendet
 * werden.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: DefaultDatenSender.java 49030 2014-02-28 15:19:54Z peuker $
 */
public class DefaultDatenSender implements UrlasserDatenSender,
		ClientSenderInterface {

	/** interner Mutex zur Synchronisation des Datenversands. */
	private static class SenderMutex {

		private SystemObject object;
		private DataDescription desc;

		public SystemObject getObject() {
			return object;
		}

		public void setObject(final SystemObject object) {
			this.object = object;
		}

		public DataDescription getDesc() {
			return desc;
		}

		public void setDesc(final DataDescription desc) {
			this.desc = desc;
		}
	}

	private static final Debug LOGGER = Debug.getLogger();
	private final List<ResultData> resultDatas = new ArrayList<ResultData>();
	private ClientDavInterface verbindung;
	private final SenderMutex senderMutex = new SenderMutex();

	/**
	 * Konstruktor.
	 * 
	 * Die bergebene Menge von Datenstzen wird ber die erstellte
	 * Urlasser-Datenverteiler-Verbindung versendet.
	 * 
	 * @param resultDatas
	 *            die Liste der zu versendenden Daten
	 */
	public DefaultDatenSender(final Collection<ResultData> resultDatas) {
		if (resultDatas != null) {
			this.resultDatas.addAll(resultDatas);
		}
	}

	/**
	 * fgt den Datensender zustzliche zu versendende Datenstze hinzu.
	 * 
	 * @param datas
	 *            die neuen Datenstze
	 */
	public void addResultData(final Collection<ResultData> datas) {
		this.resultDatas.addAll(datas);
	}

	@Override
	public void execute(final ClientDavInterface davVerbindung,
			final UrlasserInfo urlasser) {

		this.verbindung = davVerbindung;

		ResultData sendeDaten = null;
		for (final ResultData result : resultDatas) {

			final Data data = result.getData();
			if (data == null) {
				continue;
			}

			if ("asp.parameterVorgabe".equals(result.getDataDescription()
					.getAspect().getPid())) {

				final Data newData = data.createModifiableCopy();
				final Data urlasserItem = newData.getItem("Urlasser");

				/*
				 * Prfen, ob die Daten ein Urlasserelement beinhalten.
				 */
				if (urlasserItem != null) {
					urlasserItem.getReferenceValue("BenutzerReferenz")
							.setSystemObject(urlasser.getBenutzer());
					urlasserItem.getTextValue("Ursache").setText(
							urlasser.getUrsache());
					urlasserItem.getTextValue("Veranlasser").setText(
							urlasser.getVeranlasser());
				}

				sendeDaten = new ResultData(result.getObject(),
						result.getDataDescription(), result.getDataTime(),
						newData);

			} else {
				sendeDaten = result;
			}

			senden(sendeDaten);
		}
	}

	private void senden(final ResultData daten) {

		synchronized (senderMutex) {
			boolean doubledSubscription = false;

			try {
				senderMutex.setObject(daten.getObject());
				senderMutex.setDesc(daten.getDataDescription());
				try {
					verbindung.subscribeSender(this, daten.getObject(),
							daten.getDataDescription(), SenderRole.sender());
					try {
						senderMutex.wait(30000);
					} catch (final InterruptedException e) {
						DefaultDatenSender.LOGGER.warning(
								e.getLocalizedMessage(), e);
					}
				} catch (final OneSubscriptionPerSendData e) {
					doubledSubscription = true;
				}
				verbindung.sendData(daten);
			} catch (final DataNotSubscribedException e) {
				throw new UrlasserVerbindungException(e.getLocalizedMessage(),
						e);
			} catch (final SendSubscriptionNotConfirmed e) {
				throw new UrlasserVerbindungException(e.getLocalizedMessage(),
						e);
			} finally {
				if (!doubledSubscription) {
					verbindung.unsubscribeSender(this, daten.getObject(),
							daten.getDataDescription());
				}
			}
		}
	}

	@Override
	public void dataRequest(final SystemObject object,
			final DataDescription desc, final byte status) {
		synchronized (senderMutex) {
			if ((status == ClientSenderInterface.START_SENDING)
					&& isRequestSupported(object, desc)) {
				senderMutex.notifyAll();
			}
		}
	}

	@Override
	public boolean isRequestSupported(final SystemObject object,
			final DataDescription desc) {
		boolean result = false;
		synchronized (senderMutex) {
			if ((object.equals(senderMutex.getObject()))
					&& (desc.getAttributeGroup().equals(senderMutex.getDesc()
							.getAttributeGroup()))) {
				result = desc.getAspect().equals(
						senderMutex.getDesc().getAspect());
			}
		}
		return result;
	}
}
