/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 */

package de.bsvrz.buv.rw.basislib.urlasser;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.internal.RahmenwerkService;
import de.bsvrz.buv.rw.basislib.internal.RwBasisLibActivator;
import de.bsvrz.buv.rw.basislib.util.RahmenwerkUtil;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;

/**
 * Dialog zur Erfassung von UrlasserInformationen und der Ausfhrung
 * entsprechender Aktionen ber eine Urlasser-Datenverteiler-Verbindung.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: UrlasserDatenDialog.java 50979 2014-07-16 14:40:57Z hoesel $
 */
public class UrlasserDatenDialog extends TitleAreaDialog {

	private Text textBenutzer;
	private Text textPasswort;
	private Text textVeranlasser;
	private Text textUrsache;

	private UrlasserInfo urlasserInfo;
	private final UrlasserDatenSender datenSender;
	private String initialVeranlasser;
	private String initialBenutzer;
	private String initialUrsache;
	private ClientDavInterface verbindung;
	private boolean verbindungIstFuerUrlasser;

	/**
	 * Konstruktor.
	 * 
	 * Erzeugt eine Instanz des Dialogs mit der bergebenen Parent-Shell und dem
	 * angegebenen Datenversender ber den bei erfolgreicher Authentifizierung
	 * die gewnschten Operationen mit dem Datenverteiler ausgefhrt werden.
	 * 
	 * @param parent
	 *            die Parent-Shell
	 * @param datenSender
	 *            der Datensender
	 */
	public UrlasserDatenDialog(final Shell parent,
			final UrlasserDatenSender datenSender) {
		super(parent);
		this.datenSender = datenSender;
	}

	/**
	 * liefert den definierten Datensender zur Ausfhrung der Operationen mit
	 * der Urlasser-Datenverteiler-Verbindung.
	 * 
	 * @return den Datensender
	 */
	protected final UrlasserDatenSender getDatenSender() {
		return datenSender;
	}

	@Override
	protected void cancelPressed() {
		final String userName = textBenutzer.getText().trim();

		urlasserInfo = new UrlasserInfo(RahmenwerkUtil.getBenutzer(userName),
				textPasswort.getText(), textUrsache.getText().trim(),
				textVeranlasser.getText().trim());
		super.cancelPressed();
	}

	@Override
	protected void okPressed() {

		final String userName = textBenutzer.getText().trim();
		final String passWord = textPasswort.getText();

		urlasserInfo = new UrlasserInfo(RahmenwerkUtil.getBenutzer(userName),
				passWord, textUrsache.getText().trim(), textVeranlasser
						.getText().trim());

		try {
			if (datenSender != null) {
				erzeugeVerbindung(userName, passWord);
				setErrorMessage("Daten werden bertragen...");
				final UIJob uiJob = new UIJob("Urlasserkommunikation") {
					@Override
					public IStatus runInUIThread(final IProgressMonitor monitor) {
						try {
							datenSender.execute(verbindung, urlasserInfo);
						} catch (final UrlasserVerbindungException exception) {
							return new Status(IStatus.ERROR,
									RwBasisLibActivator.PLUGIN_ID,
									exception.getLocalizedMessage());
						}
						return Status.OK_STATUS;
					}
				};
				uiJob.addJobChangeListener(new JobChangeAdapter() {
					@Override
					public void done(final IJobChangeEvent event) {
						final IStatus result = event.getResult();
						if (result.getSeverity() == IStatus.OK) {
							setErrorMessage(null);
							UrlasserDatenDialog.super.okPressed();
						} else {
							setErrorMessage(result.getMessage());
						}
						if ((verbindung != null) && verbindungIstFuerUrlasser) {
							verbindung.disconnect(false, "");
						}
					}
				});
				uiJob.schedule();
			}
		} catch (final UrlasserVerbindungException exception) {
			setErrorMessage(exception.getLocalizedMessage());

		}
	}

	private void erzeugeVerbindung(final String userName, final String passWord) {

		verbindungIstFuerUrlasser = true;
		verbindung = null;

		final Rahmenwerk rahmenWerk = RahmenwerkService.getService()
				.getRahmenWerk();
		final ClientDavInterface davVerbindung = rahmenWerk.getDavVerbindung();
		if (davVerbindung == null) {
			throw new UrlasserVerbindungException(
					"Es besteht keine Datenverteilerverbindung!");
		}

		if (rahmenWerk.getBenutzerName().equals(userName)) {
			if (!davVerbindung.checkLoggedUserNameAndPassword(userName,
					passWord)) {
				throw new UrlasserVerbindungException("Ungltige Anmeldedaten!");
			}
			verbindung = davVerbindung;
			verbindungIstFuerUrlasser = false;
		} else {

			try {
				verbindung = new ClientDavConnection(
						davVerbindung.getClientDavParameters());
				verbindung.connect();
				verbindung.login(userName, passWord);
			} catch (final MissingParameterException e) {
				throw new UrlasserVerbindungException(e.getLocalizedMessage(),
						e);
			} catch (final CommunicationError e) {
				throw new UrlasserVerbindungException(e.getLocalizedMessage(),
						e);
			} catch (final ConnectionException e) {
				throw new UrlasserVerbindungException(e.getLocalizedMessage(),
						e);
			} catch (final InconsistentLoginException e) {
				throw new UrlasserVerbindungException(e.getLocalizedMessage(),
						e);
			}
		}
	}

	private void checkOkAvailable() {

		final Button okButton = getButton(IDialogConstants.OK_ID);
		if (okButton != null) {
			if (textBenutzer.getText().trim().isEmpty()) {
				setErrorMessage("Es muss ein Benutzername angegeben werden!");
				okButton.setEnabled(false);
			} else if (textPasswort.getText().trim().isEmpty()) {
				setErrorMessage("Es muss ein Passwort angegeben werden!");
				okButton.setEnabled(false);
			} else {
				setErrorMessage(null);
				okButton.setEnabled(true);
			}
		}
	}

	@Override
	protected Control createDialogArea(final Composite parent) {

		getShell().setText("Urlasserdaten");

		setTitle("Urlasserdaten");
		setTitleImage(RwBasisLibActivator.getDefault().getImage(
				"icons/wizban/bitctrl_wizard.png"));
		setMessage("Geben Sie die Daten zur Authentifizierung der Aktion an!");

		final Composite panel = new Composite(parent, SWT.NONE);
		panel.setLayout(new GridLayout(2, false));
		panel.setLayoutData(GridDataFactory.fillDefaults().grab(true, true)
				.create());

		new Label(panel, SWT.NONE).setText("Benutzer:");
		textBenutzer = new Text(panel, SWT.BORDER);
		textBenutzer.setLayoutData(GridDataFactory.fillDefaults()
				.grab(true, false).create());
		textBenutzer.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(final ModifyEvent e) {
				checkOkAvailable();
			}
		});

		if (initialBenutzer != null) {
			textBenutzer.setText(initialBenutzer);
		}

		new Label(panel, SWT.NONE).setText("Passwort:");
		textPasswort = new Text(panel, SWT.BORDER | SWT.PASSWORD);
		textPasswort.setLayoutData(GridDataFactory.fillDefaults()
				.grab(true, false).create());
		textPasswort.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(final ModifyEvent e) {
				checkOkAvailable();
			}
		});

		new Label(panel, SWT.NONE).setText("Ursache:");
		textUrsache = new Text(panel, SWT.BORDER | SWT.WRAP);
		textUrsache.setLayoutData(GridDataFactory.fillDefaults()
				.grab(true, true).create());
		if (initialUrsache != null) {
			textUrsache.setText(initialUrsache);
		}

		new Label(panel, SWT.NONE).setText("Veranlasser:");
		textVeranlasser = new Text(panel, SWT.BORDER);
		textVeranlasser.setLayoutData(GridDataFactory.fillDefaults()
				.grab(true, false).create());
		if (initialVeranlasser != null) {
			textVeranlasser.setText(initialVeranlasser);
		}

		return panel;
	}

	@Override
	protected void createButtonsForButtonBar(final Composite parent) {
		super.createButtonsForButtonBar(parent);
		checkOkAvailable();
	}

	/**
	 * Liefert den Benutzer.
	 * 
	 * @return Benutzer
	 */
	protected String getBenutzer() {
		return textBenutzer.getText();
	}

	/**
	 * Liefert das Passwort.
	 * 
	 * @return Passwort
	 */
	protected String getPasswort() {
		return textPasswort.getText();
	}

	/**
	 * liefert die Urlasserinformationen aus dem geschlossenen Dialog.
	 * 
	 * @return die Informationen
	 */
	public UrlasserInfo getUrlasserInfo() {
		return urlasserInfo;
	}

	/**
	 * Liefert die Ursache.
	 * 
	 * @return Ursache
	 */
	protected String getUrsache() {
		return textUrsache.getText();
	}

	/**
	 * Liefert den Veranlasser.
	 * 
	 * @return Veranlasser
	 */
	protected String getVeranlasser() {
		return textVeranlasser.getText();
	}

	/**
	 * setzt den initial anzuzeigenden Benutzer.
	 * 
	 * @param benutzer
	 *            der Name des Benutzers
	 */
	public void setInitialBenutzer(final String benutzer) {
		this.initialBenutzer = benutzer;
	}

	/**
	 * setzt die initial anzuzeigende Ursache.
	 * 
	 * @param ursache
	 *            die Beschreibung der Ursache
	 */
	public void setInitialUrsache(final String ursache) {
		this.initialUrsache = ursache;
	}

	/**
	 * setzt den initial anzuzeigenden Veranlasser.
	 * 
	 * @param veranlasser
	 *            der Name des Veranlassers
	 */
	public void setInitialVeranlasser(final String veranlasser) {
		this.initialVeranlasser = veranlasser;
	}
}
