/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.util;

import org.eclipse.jface.window.Window;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

/**
 * Vordefinierte Standardpositionen fr Dialoge.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: DefaultDialogPosition.java 47675 2013-12-18 08:33:08Z peuker $
 */
public enum DefaultDialogPosition implements DialogPosition {

	/** Anzeige in der linken oberen Ecke des primren Monitors. */
	SCREEN_TOP_LEFT(Relative.SCREEN, Align.TOP_LEFT),

	/** Anzeige in der rechten oberen Ecke des primren Monitors. */
	SCREEN_TOP_RIGHT(Relative.SCREEN, Align.TOP_RIGHT),

	/** Anzeige in der linken unteren Ecke des primren Monitors. */
	SCREEN_BOTTOM_LEFT(Relative.SCREEN, Align.BOTTOM_LEFT),

	/** Anzeige in der rechten unteren Ecke des primren Monitors. */
	SCREEN_BOTTOM_RIGHT(Relative.SCREEN, Align.BOTTOM_RIGHT),

	/** zentriert auf dem primren Monitor. */
	SCREEN_CENTER(Relative.SCREEN, Align.CENTER),

	/**
	 * Anzeige in der linken oberen Ecke des Rahmenwerkfensters aus dem der
	 * Dialog aufgerufen wurde.
	 */
	APP_TOP_LEFT(Relative.APP, Align.TOP_LEFT),

	/**
	 * Anzeige in der rechten oberen Ecke des Rahmenwerkfensters aus dem der
	 * Dialog aufgerufen wurde.
	 */
	APP_TOP_RIGHT(Relative.APP, Align.TOP_RIGHT),

	/**
	 * Anzeige in der linken unteren Ecke des Rahmenwerkfensters aus dem der
	 * Dialog aufgerufen wurde.
	 */
	APP_BOTTOM_LEFT(Relative.APP, Align.BOTTOM_LEFT),

	/**
	 * Anzeige in der rechten unteren Ecke des Rahmenwerkfensters aus dem der
	 * Dialog aufgerufen wurde.
	 */
	APP_BOTTOM_RIGHT(Relative.APP, Align.BOTTOM_RIGHT),

	/**
	 * Anzeige zentriert zum Rahmenwerkfensters aus dem der Dialog aufgerufen
	 * wurde.
	 */
	APP_CENTER(Relative.APP, Align.CENTER),

	/** Anzeige in der linken oberen Ecke der bergeordneten Shell. */
	PARENT_TOP_LEFT(Relative.PARENT, Align.TOP_LEFT),

	/** Anzeige in der rechten oberen Ecke der bergeordneten Shell. */
	PARENT_TOP_RIGHT(Relative.PARENT, Align.TOP_RIGHT),

	/** Anzeige in der linken unteren Ecke der bergeordneten Shell. */
	PARENT_BOTTOM_LEFT(Relative.PARENT, Align.BOTTOM_LEFT),

	/** Anzeige in der rechten unteren Ecke der bergeordneten Shell. */
	PARENT_BOTTOM_RIGHT(Relative.PARENT, Align.BOTTOM_RIGHT),

	/** Anzeige zentriert zur bergeordneten Shell. */
	PARENT_CENTER(Relative.PARENT, Align.CENTER);

	/** Definition der Bezugspunkte fr die Dialogpositionierung. */
	private enum Relative {
		/** Primrer Bildschirm. */
		SCREEN,
		/** Rahmenwerkfenster. */
		APP,
		/** bergordnete Shell. */
		PARENT;
	}

	/** Definition der Ausrichtung. */
	private enum Align {
		/** links oben. */
		TOP_LEFT,
		/** rechts oben. */
		TOP_RIGHT,
		/** links unten. */
		BOTTOM_LEFT,
		/** rechts unten. */
		BOTTOM_RIGHT,
		/** zentriert. */
		CENTER;
	}

	private Relative relative;
	private Align align;

	private DefaultDialogPosition(final Relative relative, final Align align) {
		this.relative = relative;
		this.align = align;
	}

	@Override
	public Point getLocation(final Window window, final Point size) {
		Rectangle bounds = null;

		switch (relative) {
		case APP:
			bounds = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
					.getShell().getBounds();
			break;
		case SCREEN:
			bounds = PlatformUI.getWorkbench().getDisplay().getPrimaryMonitor()
					.getBounds();
			break;
		case PARENT:
		default:
			final Shell shell = window.getShell();
			if (shell != null) {
				final Composite parent = shell.getParent();
				if (parent != null) {
					bounds = parent.getBounds();
				}
			}

			if (bounds == null) {
				bounds = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
						.getShell().getBounds();
			}
			break;
		}

		return getPoint(bounds, size);
	}

	private Point getPoint(final Rectangle bounds, final Point size) {

		switch (align) {
		case BOTTOM_LEFT:
			return new Point(bounds.x, (bounds.y + bounds.height) - size.y);

		case BOTTOM_RIGHT:
			return new Point((bounds.x + bounds.width) - size.x,
					(bounds.y + bounds.height) - size.y);

		case TOP_LEFT:
			return new Point(bounds.x, bounds.y);

		case TOP_RIGHT:
			return new Point((bounds.x + bounds.width) - size.x, bounds.y);

		case CENTER:
		default:
			return new Point(bounds.x + ((bounds.width - size.x) / 2), bounds.y
					+ ((bounds.height - size.y) / 2));
		}
	}
}
