/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen.editor;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;

import de.bsvrz.buv.rw.rw.berechtigungen.BerechtigungsParameterDatensatz;

/**
 * Labelprovider fr die Darstellung der Elemente aus dem Parameterdatensatz der
 * Berechtigungen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungsParameterLabelProvider.java 46673 2013-10-16 14:24:41Z peuker $
 */
public class BerechtigungsParameterLabelProvider extends ColumnLabelProvider {

	private final BerechtigungsKlasse klasse;
	private final BerechtigungsParameterDatensatz datensatz;

	/** Standardkonstruktor fr Spalten ohne Berechtigungsklassen. */
	public BerechtigungsParameterLabelProvider() {
		this(null, null);
	}

	/**
	 * Konstruktor, erzeugt einen LabelProvider, der die Informationen zur
	 * bergebenen Klasse aus dem angegebenen Datensatz anzeigen kann.
	 * 
	 * @param datensatz
	 *            der Datensatz (Parameter)
	 * @param klasse
	 *            die Klasse fr die die Spalte des Providers Informationen
	 *            anzeigen soll
	 */
	public BerechtigungsParameterLabelProvider(
			final BerechtigungsParameterDatensatz datensatz,
			final BerechtigungsKlasse klasse) {
		this.datensatz = datensatz;
		this.klasse = klasse;
	}

	@Override
	public void update(final ViewerCell cell) {
		final Object element = cell.getElement();

		if ((klasse == null) || (datensatz == null)) {
			if (element == null) {
				cell.setText("???");
			} else if (element instanceof WildcardFunktion) {
				cell.setText(element.toString());
			} else if ((element instanceof Funktion)
					&& (!((Funktion) element).isAvailable())) {
				cell.setText("* " + element.toString());
			} else if (element instanceof PluginMitBerechtigungen) {
				cell.setText(((PluginMitBerechtigungen) element).getNameOrId());
			} else {
				cell.setText(element.toString());
			}
		} else {
			cell.setText("");
			if (element instanceof Funktion) {
				final Funktion funktion = (Funktion) element;
				final BerechtigungsKey key = new BerechtigungsKey(klasse,
						funktion);
				final Boolean freigabeStatus = datensatz.getFreigabeStatus(key);
				cell.setForeground(cell.getControl().getDisplay()
						.getSystemColor(SWT.COLOR_WHITE));
				if (freigabeStatus == null) {
					cell.setForeground(cell.getControl().getDisplay()
							.getSystemColor(SWT.COLOR_BLACK));
					if (datensatz.isFreigegeben(key)) {
						cell.setBackground(cell.getControl().getDisplay()
								.getSystemColor(SWT.COLOR_GREEN));
						cell.setText("(O)");
					} else {
						cell.setBackground(cell.getControl().getDisplay()
								.getSystemColor(SWT.COLOR_RED));
						cell.setText("(X)");
					}
				} else if (freigabeStatus) {
					cell.setBackground(cell.getControl().getDisplay()
							.getSystemColor(SWT.COLOR_DARK_GREEN));
					cell.setText("O");
				} else {
					cell.setBackground(cell.getControl().getDisplay()
							.getSystemColor(SWT.COLOR_DARK_RED));
					cell.setText("X");
				}
			} else {
				cell.setBackground(cell.getControl().getDisplay()
						.getSystemColor(SWT.COLOR_WHITE));
			}
		}
	}

	@Override
	public String getToolTipText(final Object element) {

		String result = null;

		if ((klasse == null) || (datensatz == null)) {
			if (element instanceof WildcardFunktion) {
				result = "Wildcard - fr alle Funktionen";
			} else if (element instanceof Funktion) {
				final Funktion funktion = (Funktion) element;
				final StringBuffer buffer = new StringBuffer();
				buffer.append(funktion.getFunktionMitBerechtigung()
						.getBezeichnung());
				buffer.append('\n');
				buffer.append(funktion.getId());
				final String beschreibung = funktion
						.getFunktionMitBerechtigung().getBeschreibung();
				if (beschreibung != null) {
					buffer.append('\n');
					buffer.append(beschreibung);
				}
				result = buffer.toString();
			}
		}

		if (result == null) {
			return super.getToolTipText(element);
		}

		return result;
	}

	@Override
	public boolean useNativeToolTip(final Object object) {
		return true;
	}
}
