/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.dav.datenbasis.prelader;

import java.util.Iterator;
import java.util.LinkedList;

import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;

/**
 * Vorladen aller SystemObjekte der DaV-Verbindung. Der Start der
 * Singleton-Instanz erfolgt durch die Klasse Start#earlyStartup(). <br>
 * Mit Hilfe eines {@link IDaVPreLader}-Listeners kann man sich darber
 * informieren lassen, ob die Daten vom DaV als Instanz der Klasse
 * {@link GtmDatenBasis} geladen wurden. Die Methode
 * {@link DaVPreLader#hinzufuegenListenerPreLader(IDaVPreLader)} fgt einen
 * solchen Listener hinzu und die Methode
 * {@link DaVPreLader#entfernenListenerPreLader(IDaVPreLader)} entfernt einen
 * solchen Listener. <br>
 * Zusammen mit einem IDaVVerbindungZuhoerer, der sich bei der Klasse
 * DaVVerbindung registrieren lsst, lsst sich explizit auf einen Abbau der
 * DaV-Verbindung reagieren. Erst wenn nach einem Verbindungsaufbau der
 * {@link IDaVPreLader} wieder grnes Licht gibt, sind die Daten der aktuellen
 * DaV-Verbindung verfgbar. <br>
 * Die Klasse ScihtObjektzusammenstellung aus der Basislib liefert ein schnes
 * Beispiel fr den kombinierten Einsatz von den Listenern {@link IDaVPreLader}
 * sowie IDaVVerbindungZuhoerer.<br>
 * Mit Hilfe der Methode {@link DaVPreLader#istInitialesLadenAbgeschlossen()}
 * kann geprft werden, ob ein initiales Laden der {@link GtmDatenBasis}
 * stattgefunden hat. Ist dies nicht der Fall empfiehlt es sich, einen
 * {@link IDaVPreLader}-Listener bei dieser Klasse anzumelden, bevor die
 * Methoden {@link DaVPreLader#getSystemObjekte()} und
 * {@link DaVPreLader#getGtmDatenBasis()} aufgerufen werden - existiert eine
 * Verbindung zum DaV, lsen diese das Laden der Datenbasis aus. Da dies ein
 * nebenlufiger Vorgang ist, liefern diese Methoden dann <code>null</code> und
 * nur angemeldete {@link IDaVPreLader}-Listener werden ber eine initale
 * Datenbasis informiert.
 * 
 * @author HofmannJ
 * @deprecated wird nicht bentigt, Plug-ins sorgen selbst fr geeignetes
 *             Caching
 * 
 */
@Deprecated
public final class DaVPreLader {
	/**
	 * Liste von Listenern, die ber den Status des {@link DaVPreLader}
	 * informiert werden mchten.
	 */
	private final LinkedList<IDaVPreLader> listeners = new LinkedList<IDaVPreLader>();

	/**
	 * Speichert die {@link GtmDatenBasis}.
	 */
	private GtmDatenBasis gtmDatenBasis;

	/**
	 * Wird <code>true</code>, wenn das initiale Laden abgeschlossen ist. Wird
	 * <code>false</code>, wenn die DaV-Verbindung abgebaut wird.
	 */
	private boolean initialesLadenAbgeschlossen = true;

	/**
	 * Dieser Listener reagiert auf den Auf- und Abbau der Verbindung zum DaV.
	 * Wird die DaV-Verbindung getrennt, so wird das Flag
	 * {@link DaVPreLader#_initialesLadenAbgeschlossen} wieder false. Wird die
	 * Verbindung zum DaV wieder aufgebaut, so wird das initiale Laden durch die
	 * Methode {@link DaVPreLader#systemObjekteInitialLaden()} angestoen, so
	 * dass nach einem Verbingungswiederaufbau die Daten des jeweiligen DaV zur
	 * Verfgung stehen.
	 */
	// private final IDaVVerbindungZuhoerer verbindungZuhoerer;

	private static DaVPreLader instanz;

	/**
	 * Privater Konstruktor.
	 */
	private DaVPreLader() {
		if (RahmenwerkService.getService().getRahmenWerk().isOnline()) {

			final ArgumentList startParameter = RahmenwerkService.getService()
					.getRahmenWerk().getStartParameter();

			boolean vorladen = false;
			if (startParameter.hasArgument("-davSystemObjekteVorladen")) {
				vorladen = startParameter.fetchArgument(
						"-davSystemObjekteVorladen").booleanValue();
			}

			if (vorladen) {
				systemObjekteInitialLaden();
			} else {
				gtmDatenBasis = new GtmDatenBasis(RahmenwerkService
						.getService().getRahmenWerk().getDavVerbindung(),
						false, null);
				initialesLadenAbgeschlossen = true;
			}
		}

		final DavVerbindungsListener listener = new DavVerbindungsListener() {

			@Override
			public void verbindungHergestellt(final DavVerbindungsEvent event) {
				initialesLadenAbgeschlossen = false;
				systemObjekteInitialLaden();
			}

			@Override
			public void verbindungGetrennt(final DavVerbindungsEvent event) {
				initialesLadenAbgeschlossen = false;
			}

			@Override
			public boolean verbindungHalten(final DavVerbindungsEvent event) {
				return false;
			}

		};
		RahmenwerkService.getService().getRahmenWerk()
				.addDavVerbindungsListener(listener);
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link DaVPreLader}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link DaVPreLader}
	 */
	public static synchronized DaVPreLader getInstanz() {

		if (DaVPreLader.instanz == null) {
			DaVPreLader.instanz = new DaVPreLader();
		}
		return DaVPreLader.instanz;
	}

	/**
	 * Fgt einen {@link IDaVPreLader}-Listener zur Liste der Listener, sofern
	 * dort noch nicht vorhanden, hinzu.<br>
	 * <b>Achtung:</b> Hinzugefgte {@link IDaVPreLader}-Listener
	 * <b><i>mssen</i></b> nach Ablauf des Lebenszyklus der jeweiligen Klasse
	 * wieder entfernt werden.
	 * 
	 * @param listener
	 *            Hinzuzufgender {@link IDaVPreLader}-Listener
	 */
	public void hinzufuegenListenerPreLader(final IDaVPreLader listener) {
		synchronized (listeners) {
			if (!listeners.contains(listener)) {
				listeners.add(listener);

				// FIXME Wird das wirklich gebraucht
				// if (gtmDatenBasis != null) {
				// synchronized (gtmDatenBasis) {
				// if (gtmDatenBasis.getSystemObjects().size() > 0) {
				// listener.systemObjekteInitial(gtmDatenBasis
				// .flacheKopie());
				// }
				// }
				// }

			}
		}
	}

	/**
	 * Entfernt einen {@link IDaVPreLader}-Listener von der Liste der Listener.
	 * 
	 * @param listener
	 *            Zu entfernender {@link IDaVPreLader}-Listener.
	 */
	public void entfernenListenerPreLader(final IDaVPreLader listener) {
		synchronized (listeners) {
			if (listeners.contains(listener)) {
				listeners.remove(listener);
			}
		}
	}

	/**
	 * Lst das initiale Laden der {@link GtmDatenBasis} aus.
	 */
	public void systemObjekteInitialLaden() {
		// es werden keine Daten mehr vorgeladen
		// datenLaden(EnumLadeArt.INITIALESLADEN);
	}

	/**
	 * Lst die Aktualisierung einer bereits vorhandenen {@link GtmDatenBasis}
	 * aus.
	 */
	public void systemObjekteAktualisierungsLaden() {
		// es werden keine Daten mehr vorgeladen
		// datenLaden(EnumLadeArt.AKTUALISIERUNGSLADEN);
	}

	/**
	 * Prft, ob das initiale Laden abgeschlossen ist.
	 * 
	 * @return Liefert <code>true</code>, wenn bereits SystemObjkte geladen
	 *         wurden, sonst <code>false</code>.
	 */
	public boolean istInitialesLadenAbgeschlossen() {
		return initialesLadenAbgeschlossen;
	}

	/**
	 * Liefert die Liste der bereits vorgeladenen {@link SystemObject}s. Es
	 * bietet sich an, vorher mit
	 * {@link DaVPreLader#istInitialesLadenAbgeschlossen()} zu prfen, ob ein
	 * initales Laden der Datenbasis durchgefhrt wurde. Ist dies nicht der Fall
	 * und ist eine Verbindung zum DaV aktiv, sollte ein {@link IDaVPreLader}
	 * bei dieser Klasse angemeldet werden. In diesem Fall lst diese Methode
	 * das Laden der Datenbasis aus und alle angemeldeten {@link IDaVPreLader}
	 * werden darber informiert, wenn eine Initiale Datenbasis verfgbar ist.
	 * 
	 * @return Liste der bereits vorgeladenen {@link SystemObject}s oder
	 *         <code>null</code>, wenn noch keine Systemobjekte vorgeladen sind.
	 */
	public LinkedList<SystemObject> getSystemObjekte() {
		// LinkedList<SystemObject> re = null;
		final LinkedList<SystemObject> re = new LinkedList<SystemObject>();

		synchronized (this) {
			if (gtmDatenBasis != null) {
				synchronized (gtmDatenBasis) {
					final Iterator<SystemObject> it = gtmDatenBasis
							.getSystemObjects().iterator();
					while (it.hasNext()) {
						final SystemObject systemObjekt = it.next();
						if (systemObjekt != null) {
							re.add(systemObjekt);
						}
					}
				}
			} else if (!initialesLadenAbgeschlossen) {
				if (RahmenwerkService.getService().getRahmenWerk().isOnline()) {
					systemObjekteInitialLaden();
				}
			}
		}

		return re;
	}

	/**
	 * Liefert ein Kopie der {@link GtmDatenBasis}. Wird <code>null</code>
	 * zurckgeliefert, wurde noch keine Initiale {@link GtmDatenBasis} geladen.
	 * Es bietet sich an, vorher mit
	 * {@link DaVPreLader#istInitialesLadenAbgeschlossen()} zu prfen, ob ein
	 * initales Laden der Datenbasis durchgefhrt wurde. Ist dies nicht der Fall
	 * und ist eine Verbindung zum DaV aktiv, sollte ein {@link IDaVPreLader}
	 * bei dieser Klasse angemeldet werden. In diesem Fall lst diese Methode
	 * das Laden der Datenbasis aus und alle angemeldeten {@link IDaVPreLader}
	 * werden darber informiert, wenn eine Initiale Datenbasis verfgbar ist.
	 * 
	 * @return Kopie der {@link GtmDatenBasis} oder <code>null</code>, wenn noch
	 *         keine {@link GtmDatenBasis} vorgeladen werden konnte.
	 */
	public GtmDatenBasis getGtmDatenBasis() {
		GtmDatenBasis re = null;

		synchronized (this) {
			if (istInitialesLadenAbgeschlossen()) {
				synchronized (gtmDatenBasis) {
					re = gtmDatenBasis.flacheKopie();
				}
			} else if (RahmenwerkService.getService().getRahmenWerk()
					.isOnline()) {
				systemObjekteInitialLaden();
			}
		}

		return re;
	}
}
