/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Abstrakte Definition eines Mens. Das Men kann die gesamte Menstruktur
 * darstellen oder nur einen untergeordneten Teil (SubMenu).
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: Menue.java 45420 2013-08-09 12:12:12Z peuker $
 */
public abstract class Menue extends NamedMenueElement {

	private final List<MenueElement> elemente = new ArrayList<MenueElement>();

	/**
	 * fgt dem Men ein untergeordnetes Element (Aktion, Separator, Submen)
	 * hinzu.
	 * 
	 * @param element
	 *            das Element
	 */
	public void addElement(final MenueElement element) {
		elemente.add(element);
		element.setMenue(this);
		setDirty();
	}

	/**
	 * fgt dem Men ein untergeordnetes Element vor dem bergebenen Element
	 * hinzu.
	 * 
	 * Wenn das angegebene Element nicht gefunden wird, wird das neue Element
	 * als erstes Element in die Liste der untergeordneten Elemente eingefgt.
	 * 
	 * @param element
	 *            das Element
	 * @param relative
	 *            das Element vor dem das neue eingefgt werden soll
	 */
	public void addElementBefore(final MenueElement element,
			final MenueElement relative) {
		final int index = Math.max(0, elemente.indexOf(relative));
		elemente.add(index, element);
		element.setMenue(this);
		setDirty();
	}

	/**
	 * fgt dem Men ein untergeordnetes Element nach dem bergebenen Element
	 * hinzu.
	 * 
	 * Wenn das angegebene Element nicht gefunden wird, wird das neue Element
	 * als letztes Element in die Liste der untergeordneten Elemente eingefgt.
	 * 
	 * @param element
	 *            das Element
	 * @param relative
	 *            das Element vor dem das neue eingefgt werden soll
	 */
	public void addElementAfter(final MenueElement element,
			final MenueElement relative) {
		final int index = elemente.indexOf(relative);
		if ((index < 0) || (index >= elemente.size())) {
			elemente.add(element);
			element.setMenue(this);
		} else {
			elemente.add(index + 1, element);
			element.setMenue(this);
		}
		setDirty();
	}

	/**
	 * entfernt das bergebene Element von der Liste der untergeordneten
	 * Elemente.
	 * 
	 * @param element
	 *            das Element
	 */
	public void removeElement(final MenueElement element) {
		elemente.remove(element);
		element.setMenue(null);
		setDirty();
	}

	/**
	 * liefert die Liste der untergeordneten Elemente des Mens.
	 * 
	 * @return die Liste der Elemente
	 */
	public List<MenueElement> getElemente() {
		return Collections.unmodifiableList(elemente);
	}

	/** setzt den Bearbeitungszustand des Mens. */
	protected void setDirty() {
		final Menue menue = getMenue();
		if (menue != null) {
			menue.setDirty();
		}
	}

	@Override
	public MenueElement createDeepCopy() {
		final Menue menue = createMenuCopy();

		for (final MenueElement element : elemente) {
			menue.addElement(element.createDeepCopy());
		}

		return menue;
	}

	/**
	 * erzeugt eine Kopie des Men-Elements.
	 * 
	 * @return die Kopie
	 */
	protected abstract Menue createMenuCopy();
}
