/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu;

/**
 * Parameterdefinition fr ein Rahmenwerk-Men-Element.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwToolbarElementParameter.java 46673 2013-10-16 14:24:41Z peuker $
 */
public class RwToolbarElementParameter {

	private final String id;
	private final String name;
	private final String defaultValue;
	private final boolean optional;

	/**
	 * erzeugt die Instanz fr eine Parameterdefinition.
	 * 
	 * @param id
	 *            die ID
	 * @param name
	 *            der anzuzeigende Klarname
	 * @param defaultValue
	 *            der Standardwert (kann <code>null</code> sein)
	 * @param optionalStr
	 *            Beschreibung, ob der Parameter optional ist als Boolean-String
	 *            (kann <code>null</code> sein)
	 */
	public RwToolbarElementParameter(final String id, final String name,
			final String defaultValue, final String optionalStr) {
		this.id = id;
		this.name = name;
		this.defaultValue = defaultValue;
		if ((optionalStr == null) || optionalStr.isEmpty()) {
			optional = true;
		} else {
			optional = Boolean.parseBoolean(optionalStr);
		}
	}

	/**
	 * liefert die ID der Definition.
	 * 
	 * @return die Id
	 */
	public String getId() {
		return id;
	}

	/**
	 * liefert den Name der Definition.
	 * 
	 * @return den Name
	 */
	public String getName() {
		return name;
	}

	/**
	 * liefert den Standardwert der Definition oder <code>null</code>.
	 * 
	 * @return den Wert oder <code>null</code>
	 */
	public String getDefaultValue() {
		return defaultValue;
	}

	/**
	 * ermittelt, ob der Parameter optional ist.
	 * 
	 * @return der Zustand
	 */
	public boolean isOptional() {
		return optional;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result)
				+ ((defaultValue == null) ? 0 : defaultValue.hashCode());
		result = (prime * result) + ((id == null) ? 0 : id.hashCode());
		result = (prime * result) + ((name == null) ? 0 : name.hashCode());
		result = (prime * result) + (optional ? 1231 : 1237);
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final RwToolbarElementParameter other = (RwToolbarElementParameter) obj;
		if (defaultValue == null) {
			if (other.defaultValue != null) {
				return false;
			}
		} else if (!defaultValue.equals(other.defaultValue)) {
			return false;
		}
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		if (optional != other.optional) {
			return false;
		}
		return true;
	}
}
