/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.offlinekonfiguration;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;

/**
 * Wertet die Betriebsart ber die Kommandozeilenparameter aus.
 * 
 * @author HofmannJ
 * 
 */
public final class BetriebsArtAuswerten {

	/** Speichert die Singleton-Instanz dieser Klasse. */
	private static BetriebsArtAuswerten instanz;

	/** Betriebsart. */
	private BetriebsArt betriebsart;

	/** Liste der {@link IBetriebsArtZuhoerer}. */
	private final List<IBetriebsArtZuhoerer> zuhoererBetriebsArt = new LinkedList<IBetriebsArtZuhoerer>();

	/**
	 * Liefert die Singleton-Instanz dieser Klasse zurck.
	 * 
	 * @return Singleton-Instanz dieser Klasse
	 */
	public static BetriebsArtAuswerten getInstanz() {
		if (BetriebsArtAuswerten.instanz == null) {
			BetriebsArtAuswerten.instanz = new BetriebsArtAuswerten();
		}
		return BetriebsArtAuswerten.instanz;
	}

	/**
	 * Privater Konstruktor dieser Klasse. <br>
	 * Liest den Kommandozeilenparameter fr die Betriebsart aus und legt diesen
	 * fr den Zurgriff ber die Methode
	 * {@link BetriebsArtAuswerten#getBetriebsArt()} in der Variablein
	 * {@link BetriebsArtAuswerten#_betriebsart} ab.
	 * 
	 */
	private BetriebsArtAuswerten() {

		final ArgumentList argumente = RahmenwerkService.getService()
				.getRahmenWerk().getStartParameter();
		if (argumente.hasArgument("-betriebsart")) {
			if ("online".equalsIgnoreCase(argumente.fetchArgument(
					"-betriebsart").getValue())) {
				betriebsart = BetriebsArt.online;
			} else {
				betriebsart = BetriebsArt.offline;
			}
		}
		benachrichtigeBetriebsArtZuhoerer();
	}

	/**
	 * Liefert die {@link BetriebsArt} des Rahmenwerkes zurck.
	 * 
	 * @return {@link BetriebsArt}
	 */
	public BetriebsArt getBetriebsArt() {
		return betriebsart;
	}

	/**
	 * Setzt die {@link BetriebsArt} des Rahmenwerkes.
	 * 
	 * @param betriebsArt
	 *            Zu setzende {@link BetriebsArt}.
	 */
	public void setBetriebsArt(final BetriebsArt betriebsArt) {
		this.betriebsart = betriebsArt;

		benachrichtigeBetriebsArtZuhoerer();
	}

	/**
	 * Fgt einen {@link IBetriebsArtZuhoerer} zur Liste der Zuhrer hinzu.
	 * 
	 * @param betriebsArtZuhoerer
	 *            Hinzuzufgender {@link IBetriebsArtZuhoerer}.
	 */
	public void hinzufuegenBetriebsArtZuhoerer(
			final IBetriebsArtZuhoerer betriebsArtZuhoerer) {
		this.zuhoererBetriebsArt.add(betriebsArtZuhoerer);
	}

	/**
	 * Entfernt einen {@link IBetriebsArtZuhoerer} von der Liste der Zuhrer.
	 * 
	 * @param betriebsArtZuhoerer
	 *            Zu entfernender {@link IBetriebsArtZuhoerer}.
	 */
	public void entferneBetriebsArtZuhoerer(
			final IBetriebsArtZuhoerer betriebsArtZuhoerer) {
		this.zuhoererBetriebsArt.remove(betriebsArtZuhoerer);
	}

	/**
	 * Benachrichtigt alle {@link IBetriebsArtZuhoerer} in der Liste der Zuhrer
	 * darber, dass sich die Betriebsart der Rahmenwerkes gendert hat.
	 * 
	 */
	private void benachrichtigeBetriebsArtZuhoerer() {
		final Iterator<IBetriebsArtZuhoerer> it = zuhoererBetriebsArt
				.iterator();
		while (it.hasNext()) {
			final IBetriebsArtZuhoerer betriebsArtZuhoerer = it.next();
			betriebsArtZuhoerer.aktuelleBetriebsArt(betriebsart);
		}
	}

	/**
	 * Enumerator, der die Betriebsarten des Rahmenwerkes wiederspiegelt.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum BetriebsArt {
		/** Betriebsart == ONLINE. */
		online,

		/** Betriebsart == OFFLINE. */
		offline
	}
}
