/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.statusleiste;

import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.StatusLineLayoutData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.basislib.menu.AbstractRwMenuControl;
import de.bsvrz.buv.rw.basislib.util.RwColor;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;
import de.bsvrz.dav.daf.main.archive.ArchiveAvailabilityListener;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Statusleistenelement mit dem der Status eines Archivs visualisiert werden
 * kann.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: ArchivStatusControl.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class ArchivStatusControl extends AbstractRwMenuControl {

	/** Listener, der nderungen im Archivstatus verarbeitet. */
	private class ArchiveListener implements ArchiveAvailabilityListener {
		@Override
		public void archiveAvailabilityChanged(final ArchiveRequestManager arg0) {
			updateText();
		}
	}

	/**
	 * Listener, der nderungen des Status der Datenverteilerverbindung
	 * verarbeitet.
	 */
	private class VerbindungsListener implements DavVerbindungsListener {

		@Override
		public void verbindungHergestellt(final DavVerbindungsEvent event) {
			registerArchive();
			updateText();
		}

		@Override
		public void verbindungGetrennt(final DavVerbindungsEvent event) {
			deregisterArchive();
			updateText();
		}

		@Override
		public boolean verbindungHalten(final DavVerbindungsEvent event) {
			return false;
		}
	}

	/** die Parameter des Elements. */
	private static final String PARAM_PID = "pid";
	private static final String PARAM_FARBE_ONLINE = "farbeonline";
	private static final String PARAM_FARBE_OFFINE = "farbeoffline";
	private static final String PARAM_FARBE_UNKNOWN = "farbeunbekannt";
	private static final String PARAM_HINTERGRUND_ONLINE = "hintergrundonline";
	private static final String PARAM_HINTERGRUND_OFFLINE = "hintergrundoffline";
	private static final String PARAM_HINTERGRUND_UNKNOWN = "hintergrundunbekannt";
	private static final String PARAM_FORMAT_ONLINE = "textonline";
	private static final String PARAM_FORMAT_OFFLINE = "textoffline";
	private static final String PARAM_FORMAT_UNKNOWN = "textunbekannt";

	/** die aktuellen Parameter. */
	private String pid;

	private String formatOnline;
	private String formatOffline;
	private String formatUnknown;

	private String foreGroundOnline;
	private String foreGroundOffline;
	private String foreGroundUnknown;

	private String backGroundOnline;
	private String backGroundOffline;
	private String backGroundUnknown;

	private CLabel anzeigeText;
	private VerbindungsListener verbindungsListener;
	private ArchiveRequestManager archiveRequestManager;
	private ArchiveListener archiveListener;

	@Override
	public void fill(final Composite parent) {

		if (anzeigeText == null) {
			anzeigeText = new CLabel(parent, SWT.SHADOW_NONE | SWT.BORDER);
			anzeigeText.setAlignment(SWT.CENTER);
			anzeigeText.addDisposeListener(new DisposeListener() {

				@Override
				public void widgetDisposed(final DisposeEvent e) {

					deregisterArchive();
					anzeigeText = null;
					if (verbindungsListener != null) {
						RahmenwerkImpl.getInstanz()
								.removeDavVerbindungsListener(
										verbindungsListener);
					}
					verbindungsListener = null;
				}

			});

			verbindungsListener = new VerbindungsListener();
			RahmenwerkImpl.getInstanz().addDavVerbindungsListener(
					verbindungsListener);
			if (RahmenwerkImpl.getInstanz().isOnline()) {
				registerArchive();
			}

			updateText();
		}
	}

	private void registerArchive() {
		deregisterArchive();
		if ((pid == null) || pid.trim().isEmpty()) {
			archiveRequestManager = RahmenwerkImpl.getInstanz()
					.getDavVerbindung().getArchive();
		} else {
			archiveRequestManager = RahmenwerkImpl.getInstanz()
					.getDavVerbindung().getArchive(getArchiveObject(pid));
		}
		if (archiveRequestManager != null) {
			archiveListener = new ArchiveListener();
			archiveRequestManager
					.addArchiveAvailabilityListener(archiveListener);
		}
	}

	private void deregisterArchive() {
		if ((archiveListener != null) && (archiveRequestManager != null)) {
			archiveRequestManager
					.removeArchiveAvailabilityListener(archiveListener);
		}
		archiveListener = null;
		archiveRequestManager = null;
	}

	private SystemObject getArchiveObject(final String archivPid) {
		SystemObject archiv = null;
		if (RahmenwerkImpl.getInstanz().isOnline()) {
			archiv = RahmenwerkImpl.getInstanz().getDavVerbindung()
					.getDataModel().getObject(archivPid);
		}
		return archiv;
	}

	private void updateText() {

		if (PlatformUI.getWorkbench().isClosing()) {
			return;
		}

		final UIJob job = new UIJob("Aktualisiere Archivstatus") {

			@Override
			public IStatus runInUIThread(final IProgressMonitor monitor) {

				if ((anzeigeText != null) && (!anzeigeText.isDisposed())) {

					String text = formatUnknown;
					String colorDef = foreGroundUnknown;
					String bkgdDef = backGroundUnknown;

					if (archiveRequestManager != null) {
						if (archiveRequestManager.isArchiveAvailable()) {
							text = formatOnline;
							colorDef = foreGroundOnline;
							bkgdDef = backGroundOnline;
						} else {
							text = formatOffline;
							colorDef = foreGroundOffline;
							bkgdDef = backGroundOffline;
						}
					}

					Color color = new Color(anzeigeText.getDisplay(),
							RwColor.getRgbFromString(colorDef));
					anzeigeText.setForeground(color);
					color.dispose();

					if ((bkgdDef != null) && (!bkgdDef.trim().isEmpty())) {
						color = new Color(anzeigeText.getDisplay(),
								RwColor.getRgbFromString(bkgdDef));
						anzeigeText.setBackground(color);
						color.dispose();
					} else {
						anzeigeText.setBackground((Color) null);
					}

					anzeigeText.setText(text);
					final Point preferredSize = ((Control) anzeigeText)
							.computeSize(SWT.DEFAULT, SWT.DEFAULT);
					final int widthHint = preferredSize.x + 10;
					final StatusLineLayoutData data = new StatusLineLayoutData();
					data.widthHint = widthHint;
					anzeigeText.setLayoutData(data);
					anzeigeText.getParent().layout(true);
				}
				return Status.OK_STATUS;
			}

			@Override
			public boolean belongsTo(final Object family) {
				return Rahmenwerk.JOB_FAMILY.equals(family)
						|| super.belongsTo(family);
			}
		};
		job.schedule();
	}

	@Override
	public void setParameter(final Map<String, String> parameters) {
		for (final Entry<String, String> parameter : parameters.entrySet()) {
			if (ArchivStatusControl.PARAM_PID.equals(parameter.getKey())) {
				pid = parameter.getValue();
			} else if (ArchivStatusControl.PARAM_FORMAT_ONLINE.equals(parameter
					.getKey())) {
				formatOnline = parameter.getValue();
			} else if (ArchivStatusControl.PARAM_FORMAT_OFFLINE
					.equals(parameter.getKey())) {
				formatOffline = parameter.getValue();
			} else if (ArchivStatusControl.PARAM_FORMAT_UNKNOWN
					.equals(parameter.getKey())) {
				formatUnknown = parameter.getValue();
			} else if (ArchivStatusControl.PARAM_FARBE_ONLINE.equals(parameter
					.getKey())) {
				foreGroundOnline = parameter.getValue();
			} else if (ArchivStatusControl.PARAM_FARBE_OFFINE.equals(parameter
					.getKey())) {
				foreGroundOffline = parameter.getValue();
			} else if (ArchivStatusControl.PARAM_FARBE_UNKNOWN.equals(parameter
					.getKey())) {
				foreGroundUnknown = parameter.getValue();
			} else if (ArchivStatusControl.PARAM_HINTERGRUND_ONLINE
					.equals(parameter.getKey())) {
				backGroundOnline = parameter.getValue();
			} else if (ArchivStatusControl.PARAM_HINTERGRUND_OFFLINE
					.equals(parameter.getKey())) {
				backGroundOffline = parameter.getValue();
			} else if (ArchivStatusControl.PARAM_HINTERGRUND_UNKNOWN
					.equals(parameter.getKey())) {
				backGroundUnknown = parameter.getValue();
			}
		}
	}

	@Override
	public boolean useInStatusLine() {
		return true;
	}
}
